
# load libraries
library(ks)

# load data
mapname <- "ShowMap.dat"
mapfile <- read.delim(mapname,header=TRUE)
Show <- as.character(mapfile$Show)
DayPart <- mapfile$Day.Part
Rating <- mapfile$Rating
Regis <- mapfile$Registered
Ideo <- mapfile$Net..Con.
Turnout <- mapfile$Turnout
RepShare <- mapfile$Rep..Share
DemAds <- mapfile$Dem..Ads
RepAds <- mapfile$Rep..Ads
DemGRPs <- mapfile$Dem..GRPs
RepGRPs <- mapfile$Rep..GRPs
TotGRPs <- mapfile$Tot..GRPs
DiffGRPs <- RepGRPs - DemGRPs
P <- length(Show)

############
# figure 1 #
############

# figure 1, panel 1
par(mfrow=c(1,2))
plot(0:1,type='n',xlim=c(-.2,.5),ylim=c(.65,.95),xlab="Conservative Identifiers - Liberal Identifiers",ylab="Precent Registered")
text(Ideo,Regis,'+',cex=10*Rating)

# figure 1, panel 2
plot(0:1,type='n',xlim=c(-.2,.5),ylim=c(.65,.95),xlab="Conservative Identifiers - Liberal Identifiers",ylab="Precent Registered")
for(i in 1:P)
{
	insamp <- FALSE
	if(Show[i] == "WEST WING (NBC)") insamp <- TRUE
	if(Show[i] == "MEET THE PRESS (NBC)") insamp <- TRUE
	if(Show[i] == "60 MINUTES (CBS)") insamp <- TRUE
	if(Show[i] == "DATELINE NBC SUNDAY") insamp <- TRUE
	if(Show[i] == "JAGS (CBS)") insamp <- TRUE
	if(Show[i] == "FOX NEWS SUNDAY") insamp <- TRUE
	if(Show[i] == "ELLEN") insamp <- TRUE
	if(Show[i] == "SATURDAY NIGHT LIVE (NBC)") insamp <- TRUE
	if(Show[i] == "7TH HEAVEN (WB)") insamp <- TRUE
	if(Show[i] == "THE MONTEL WILLIAMS SHOW") insamp <- TRUE
	if(Show[i] == "FRIENDS") insamp <- TRUE
	if(Show[i] == "CHAPELLE'S SHOW (COMEDY CENTRAL)") insamp <- TRUE
	if(Show[i] == "REBA (WB)") insamp <- TRUE
	if(Show[i] == "THE SIMPSONS") insamp <- TRUE
	if(Show[i] == "COPS (FOX)") insamp <- TRUE
	if(Show[i] == "KING OF THE HILL") insamp <- TRUE
	if(insamp)
	{
		text(Ideo[i],Regis[i],Show[i],cex=1.2*(Rating[i])^(1/3))
	}
}

############
# figure 2 #
############

x11()

predname <- matrix(rep("",15),3,5)
fhat <- list()
fhat[[1]] <- list()
fhat[[2]] <- list()
fhat[[3]] <- list()
tbar <- matrix(rep(NA,15),3,5)
vbar <- matrix(rep(NA,15),3,5)
for(j in 1:5)
{
	predname[1,j] <- paste("Pred_60 MINUTES (CBS)_",j,".dat",sep="")
	predname[2,j] <- paste("Pred_CAVUTO ON BUSINESS (FOX NEWS)_",j,".dat",sep="")
	predname[3,j] <- paste("Pred_STEVE HARVEY_",j,".dat",sep="")
}
for(i in 1:3)
{
	for(j in 1:5)
	{
		predfile <- read.delim(predname[i,j],header=TRUE)
		t0Cdf <- predfile$t0Cdf
		v0Cdf <- predfile$v0Cdf
		f <- matrix(c(v0Cdf,t0Cdf),length(t0Cdf),2)
		d1 <- density(v0Cdf,kernel="gaussian")
		d2 <- density(t0Cdf,kernel="gaussian")
		H <- matrix(c(d1$bw,0,0,d2$bw),2,2)
		fhat[[i]][[j]] <- kde(x=f,H=H) # this takes a long time!!!
		tbar[i,j] <- quantile(t0Cdf,0.5)
		vbar[i,j] <- quantile(v0Cdf,0.5)
	}
}

# bounds for plot
lowx <-  -0.1
highx <- 1.1
lowy <-  0.2
highy <- 1.0

# plot it (plot is based on first imputation, but the others look the same)
plot(fhat[[1]][[1]],col="green",xlim=c(lowx,highx),ylim=c(lowy,highy),xlab="Rep. Vote Share",ylab="Turnout",cont=c(50,20))
par(new=TRUE)
plot(fhat[[2]][[1]],col="blue",xlim=c(lowx,highx),ylim=c(lowy,highy),xlab="Rep. Vote Share",ylab="Turnout",cont=c(50,20))
par(new=TRUE)
plot(fhat[[3]][[1]],col="red",xlim=c(lowx,highx),ylim=c(lowy,highy),xlab="Rep. Vote Share",ylab="Turnout",cont=c(50,20))
text(vbar[1,1],tbar[1,1],"60 Mins.",col="green")
text(vbar[2,1],tbar[2,1],"Cavuto Bus.",col="blue")
text(vbar[3,1],tbar[3,1],"Steve Harvey",col="red")

############
# figure 3 #
############

x11()

# figure 3, panel 1
par(mfrow=c(1,2))
plot(0:1,type='n',xlim=c(.2,.8),ylim=c(.4,.7),xlab="Prob. Voting Republican (avg. viewer)",ylab="Prob. Turning Out  (avg. viewer)")
text(RepShare,Turnout,'+',cex=10*Rating)

# figure 3, panel 2
plot(0:1,type='n',xlim=c(.3,.7),ylim=c(.4,.7),xlab="Prob. Voting Republican (avg. viewer)",ylab="Prob. Turning Out  (avg. viewer)")
for(i in 1:P)
{
	insamp <- FALSE
	if(Show[i] == "WEST WING (NBC)") insamp <- TRUE
	if(Show[i] == "MEET THE PRESS (NBC)") insamp <- TRUE
	if(Show[i] == "60 MINUTES (CBS)") insamp <- TRUE
	if(Show[i] == "DATELINE NBC SUNDAY") insamp <- TRUE
	if(Show[i] == "JAGS (CBS)") insamp <- TRUE
	if(Show[i] == "FOX NEWS SUNDAY") insamp <- TRUE
	if(Show[i] == "ELLEN") insamp <- TRUE
	if(Show[i] == "SATURDAY NIGHT LIVE (NBC)") insamp <- TRUE
	if(Show[i] == "7TH HEAVEN (WB)") insamp <- TRUE
	if(Show[i] == "THE MONTEL WILLIAMS SHOW") insamp <- TRUE
	if(Show[i] == "FRIENDS") insamp <- TRUE
	if(Show[i] == "CHAPELLE'S SHOW (COMEDY CENTRAL)") insamp <- TRUE
	if(Show[i] == "REBA (WB)") insamp <- TRUE
	if(Show[i] == "THE SIMPSONS") insamp <- TRUE
	if(Show[i] == "COPS (FOX)") insamp <- TRUE
	if(Show[i] == "KING OF THE HILL") insamp <- TRUE
	if(insamp)
	{
		text(RepShare[i],Turnout[i],Show[i],cex=1.2*(Rating[i])^(1/3))
	}
}

############
# figure 4 #
############

x11()

par(mfrow=c(1,2))
plot(0:1,type='n',xlim=c(.35,.6),ylim=c(.45,.65),xlab="Prob. Voting Republican (avg. viewer)",ylab="Prob. Turning Out  (avg. viewer)")
for(i in 1:P)
{
	if(DemAds[i] > 0) text(RepShare[i],Turnout[i],"D",cex=.05*(DemGRPs[i]),col="blue")
}

plot(0:1,type='n',xlim=c(.35,.6),ylim=c(.45,.65),xlab="Prob. Voting Republican (avg. viewer)",ylab="Prob. Turning Out  (avg. viewer)")
for(i in 1:P)
{
	if(RepAds[i] > 0) text(RepShare[i],Turnout[i],"R",cex=.05*(RepGRPs[i]),col="red")
}

############
# figure 5 #
############

x11()

insamp <- rep(FALSE,P)
for(i in 1:P)
{
	if(Show[i] == "LOCAL NEWS ON YOUR NBC STATION - MORNING") insamp[i] <- TRUE
	if(Show[i] == "TODAY SHOW (NBC)") insamp[i] <- TRUE
	if(Show[i] == "GOOD MORNING AMERICA (ABC)") insamp[i] <- TRUE
	if(Show[i] == "LOCAL NEWS ON YOUR ABC STATION - MORNING") insamp[i] <- TRUE
	if(Show[i] == "LOCAL NEWS ON YOUR ABC STATION - EARLY EVENING") insamp[i] <- TRUE
	if(Show[i] == "LOCAL NEWS ON YOUR NBC STATION - EARLY EVENING") insamp[i] <- TRUE
	if(Show[i] == "LOCAL NEWS ON YOUR CBS STATION - EARLY EVENING") insamp[i] <- TRUE
	if(Show[i] == "THE OPRAH WINFREY SHOW") insamp[i] <- TRUE
	if(Show[i] == "WHEEL OF FORTUNE") insamp[i] <- TRUE
	if(Show[i] == "EVERYBODY LOVES RAYMOND") insamp[i] <- TRUE
	if(Show[i] == "FOX NFL REGULAR SEASON GAMES") insamp[i] <- TRUE
	if(Show[i] == "CBS NFL REGULAR SEASON GAMES") insamp[i] <- TRUE
	if(Show[i] == "ABC COLLEGE FOOTBALL GAMES") insamp[i] <- TRUE
	if(Show[i] == "C.S.I.: CRIME SCENE INVESTIGATION (CBS)") insamp[i] <- TRUE
	if(Show[i] == "GOOD DAY LIVE") insamp[i] <- TRUE
	if(Show[i] == "LOCAL NEWS ON YOUR FOX STATION - MORNING") insamp[i] <- TRUE
	if(Show[i] == "THE MONTEL WILLIAMS SHOW") insamp[i] <- TRUE
	if(Rating[i] >= .40) insamp[i] <- TRUE
}
plot(0:1,type='n',xlim=c(.3,.7),ylim=c(0,150),xlab="Prob. Turning Out  (avg. viewer)",ylab="Total GRPs (in thousands)")
abline(v=quantile(Turnout,probs=.25,na.rm=TRUE),col='cyan',lty=3)
abline(v=quantile(Turnout,probs=.5,na.rm=TRUE),col='cyan',lty=3)
abline(v=quantile(Turnout,probs=.75,na.rm=TRUE),col='cyan',lty=3)
text(subset(Turnout,insamp),subset(TotGRPs,insamp),subset(Show,insamp),cex=.5,col='gray')
text(Turnout,TotGRPs,'+',cex=10*Rating)

############
# figure 6 #
############

x11()

plot(0:1,type='n',xlim=c(.3,.7),ylim=c(0,150),xlab="Prob. Voting Republican (avg. viewer)",ylab="Total GRPs (in thousands)")
abline(v=.5,col='cyan',lty=3)
text(subset(RepShare,insamp),subset(TotGRPs,insamp),subset(Show,insamp),cex=.5,col='gray')
text(RepShare,TotGRPs,'+',cex=10*Rating)

############
# figure 7 #
############

x11()

plot(0:1,type='n',xlim=c(.3,.7),ylim=c(-25,25),xlab="Prob. Voting Republican (avg. viewer)",ylab="Rep. GRPs - Dem. GRPs (in thousands)")
abline(v=.5,col='cyan',lty=3)
abline(h=0,col='cyan',lty=3)
text(subset(RepShare,insamp),subset(DiffGRPs,insamp),subset(Show,insamp),cex=.5,col='gray')
text(RepShare,DiffGRPs,'+',cex=10*Rating)

############
# figure 8 #
############

x11()

par(mfrow=c(2,4))
plot(0:1,type='n',xlim=c(.35,.6),ylim=c(.45,.65),xlab="Prob. Voting Republican (avg. viewer)",ylab="Prob. Turning Out  (avg. viewer)",main="Early Morning")
for(i in 1:P)
{
	if(DayPart[i] == "EM")
	{
		if(DemAds[i] > 0)text(RepShare[i],Turnout[i],"D",cex=.05*(DemGRPs[i]),col="blue")
		if(RepAds[i] > 0)text(RepShare[i],Turnout[i],"R",cex=.05*(RepGRPs[i]),col="red")

	}
}

plot(0:1,type='n',xlim=c(.35,.6),ylim=c(.45,.65),xlab="Prob. Voting Republican (avg. viewer)",ylab="Prob. Turning Out  (avg. viewer)",main="Day Time")
for(i in 1:P)
{
	if(DayPart[i] == "DAY")
	{
		if(DemAds[i] > 0)text(RepShare[i],Turnout[i],"D",cex=.05*(DemGRPs[i]),col="blue")
		if(RepAds[i] > 0)text(RepShare[i],Turnout[i],"R",cex=.05*(RepGRPs[i]),col="red")

	}
}

plot(0:1,type='n',xlim=c(.35,.6),ylim=c(.45,.65),xlab="Prob. Voting Republican (avg. viewer)",ylab="Prob. Turning Out  (avg. viewer)",main="Early Fringe")
for(i in 1:P)
{
	if(DayPart[i] == "EF")
	{
		if(DemAds[i] > 0)text(RepShare[i],Turnout[i],"D",cex=.05*(DemGRPs[i]),col="blue")
		if(RepAds[i] > 0)text(RepShare[i],Turnout[i],"R",cex=.05*(RepGRPs[i]),col="red")

	}
}

plot(0:1,type='n',xlim=c(.35,.6),ylim=c(.45,.65),xlab="Prob. Voting Republican (avg. viewer)",ylab="Prob. Turning Out  (avg. viewer)",main="Early News")
for(i in 1:P)
{
	if(DayPart[i] == "EN")
	{
		if(DemAds[i] > 0)text(RepShare[i],Turnout[i],"D",cex=.05*(DemGRPs[i]),col="blue")
		if(RepAds[i] > 0)text(RepShare[i],Turnout[i],"R",cex=.05*(RepGRPs[i]),col="red")

	}
}

plot(0:1,type='n',xlim=c(.35,.6),ylim=c(.45,.65),xlab="Prob. Voting Republican (avg. viewer)",ylab="Prob. Turning Out  (avg. viewer)",main="Prime Access")
for(i in 1:P)
{
	if(DayPart[i] == "PA")
	{
		if(DemAds[i] > 0)text(RepShare[i],Turnout[i],"D",cex=.05*(DemGRPs[i]),col="blue")
		if(RepAds[i] > 0)text(RepShare[i],Turnout[i],"R",cex=.05*(RepGRPs[i]),col="red")

	}
}

plot(0:1,type='n',xlim=c(.35,.6),ylim=c(.45,.65),xlab="Prob. Voting Republican (avg. viewer)",ylab="Prob. Turning Out  (avg. viewer)",main="Prime Time")
for(i in 1:P)
{
	if(DayPart[i] == "PRIME")
	{
		if(DemAds[i] > 0)text(RepShare[i],Turnout[i],"D",cex=.05*(DemGRPs[i]),col="blue")
		if(RepAds[i] > 0)text(RepShare[i],Turnout[i],"R",cex=.05*(RepGRPs[i]),col="red")

	}
}

plot(0:1,type='n',xlim=c(.35,.6),ylim=c(.45,.65),xlab="Prob. Voting Republican (avg. viewer)",ylab="Prob. Turning Out  (avg. viewer)",main="Late News")
for(i in 1:P)
{
	if(DayPart[i] == "LN")
	{
		if(DemAds[i] > 0)text(RepShare[i],Turnout[i],"D",cex=.05*(DemGRPs[i]),col="blue")
		if(RepAds[i] > 0)text(RepShare[i],Turnout[i],"R",cex=.05*(RepGRPs[i]),col="red")

	}
}

plot(0:1,type='n',xlim=c(.35,.6),ylim=c(.45,.65),xlab="Prob. Voting Republican (avg. viewer)",ylab="Prob. Turning Out  (avg. viewer)",main="Late Fringe")
for(i in 1:P)
{
	if(DayPart[i] == "LF")
	{
		if(DemAds[i] > 0)text(RepShare[i],Turnout[i],"D",cex=.05*(DemGRPs[i]),col="blue")
		if(RepAds[i] > 0)text(RepShare[i],Turnout[i],"R",cex=.05*(RepGRPs[i]),col="red")
	}
}

########################################
# Misc. calculations reported in paper #
########################################

# lowest and highest registered votes
min(Regis)
Show[which(Regis == min(Regis), arr.ind = TRUE)]

max(Regis)
Show[which(Regis == max(Regis), arr.ind = TRUE)]

# lowest and highest net conservative identifiers
min(Ideo)
Show[which(Ideo == min(Ideo), arr.ind = TRUE)]

max(Ideo)
Show[which(Ideo == max(Ideo), arr.ind = TRUE)]

# GRP-Weighted mean turnout and Rep. vote share
weighted.mean(Turnout,DemGRPs,na.rm=TRUE)
weighted.mean(RepShare,DemGRPs,na.rm=TRUE)

weighted.mean(Turnout,RepGRPs,na.rm=TRUE)
weighted.mean(RepShare,RepGRPs,na.rm=TRUE)

# correlation of GRPs by party
cor(DemGRPs,RepGRPs)

# mean turnout of shows
mean(Turnout,na.rm=TRUE)

# percent of GRPs placed on shows with turnout above mean and above top quartile
sum(subset(TotGRPs,Turnout>mean(Turnout,na.rm=TRUE))) / sum(TotGRPs)
sum(subset(TotGRPs,Turnout>quantile(Turnout, probs = 0.75, na.rm = TRUE))) / sum(TotGRPs)

# correlation between GRPs and rating
cor(DemGRPs,Rating)
cor(RepGRPs,Rating)

# registered voters for Good Day Live vs. average show
Regis[which(Show=="GOOD DAY LIVE")]
mean(Regis)
