
#ifndef _GenMatUtil_AddOp
#define _GenMatUtil_AddOp

	/* Add two matrices, C = A + B */
	template <class T> Matrix<T>          operator + (const Matrix<T>          &A, const Matrix<T>          &B);
	template <class T> SymmetricMatrix<T> operator + (const SymmetricMatrix<T> &A, const SymmetricMatrix<T> &B);
	template <class T> LowerMatrix<T>     operator + (const LowerMatrix<T>     &A, const LowerMatrix<T>     &B);


// Add two vectors (operator routine)
template <class T>
inline Vector<T> operator + (const Vector<T> &x, const Vector<T> &y)
{
	int n = x.Size();
	if(y.Size() != n)
	{
		Print("Error in Vector = Vector + Vector");
		Print("z = x + y");
		Print("Incompatible Dimensions");
		Print("x.Name",x.Name);
		Print("y.Name",y.Name);
		Print("x.Size()",x.Size());
		Print("y.Size()",y.Size());
		HaltExecution();
	}

	Vector<T> z(n);
	for(int i=0; i<n; i++) z[i] = x[i] + y[i];
	return z;
}

/* Add a Matrix and a Matrix, C = A + B */
template <class T>
Matrix<T> operator + (const Matrix<T> &A, const Matrix<T> &B)
{
	int i,j;
	int m = A.Rows();
	int n = A.Cols();

	// Check dimensions
	if(B.Rows() != m || B.Cols() != n)
	{
		Print("Error in Matrix = Matrix + Matrix");
		Print("C = A + B");
		Print("Incompatible dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	// Preform calculations
	Matrix<T> C(m,n);
	for(i=0; i<m; i++) for(j=0; j<n; j++) C(i,j) = A(i,j) + B(i,j);
	return C;
}

/* Add a DiagonalMatrix and a DiagonalMatrix, C = A + B */
#ifdef _GenMat_DiagonalMatrix
	template <class T>
	Matrix<T> operator + (const DiagonalMatrix<T> &A, const DiagonalMatrix<T> &B)
	{
		int n = A.Dim();

		/* Check Dimensions */
		if(B.Dim() != n)
		{
			Print("Error in DiagonalMatrix = DiagonalMatrix + DiagonalMatrix");
			Print("C = A + B");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		/* Preform Calculations */
		DiagonalMatrix<T> C(n);
		for(int i=0; i<n; i++) C(i) = A(i) + B(i);
		return C;
	}
#endif

/* Add a SymmetricMatrix and a SymmetricMatrix, C = A + B */
template <class T>
SymmetricMatrix<T> operator + (const SymmetricMatrix<T> &A, const SymmetricMatrix<T> &B)
{
	int i,j;
	int n = A.Dim();

	// Check dimensions
	if(B.Dim() != n)
	{
		Print("Error in SymmetricMatrix = SymmetricMatrix + SymmetricMatrix");
		Print("C = A + B");
		Print("Incompatible dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	SymmetricMatrix<T> C(n);
	for(i=0; i<n; i++) for(j=0; j<=i; j++) C(i,j) = A(i,j) + B(i,j);
	return C;
}

/* Add a LowerMatrix and a LowerMatrix, C = A + B */
template <class T>
LowerMatrix<T> operator + (const LowerMatrix<T> &A, const LowerMatrix<T> &B)
{
	// Check dimensions
	int n = A.Dim();
	if(B.Dim() != n)
	{
		Print("Error in LowerMatrix = LowerMatrix + LowerMatrix");
		Print("C = A + B");
		Print("Incompatible dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	// Preform calculations
	LowerMatrix<T> C(n);
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++) C(i,j) = A(i,j) + B(i,j);
	return C;
}

/* Add a UpperMatrix and a UpperMatrix, C = A + B */
#ifdef _GenMat_UpperMatrix
	template <class T>
	UpperMatrix<T> operator + (const UpperMatrix<T> &A, const UpperMatrix<T> &B)
	{
		/* Check Dimensions */
		int n = A.Dim();
		if(B.Dim() != n)
		{
			Print("Error in UpperMatrix = UpperMatrix + UpperMatrix");
			Print("C = A + B");
			Print("Incompatible dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		/* Preform Calculations */
		UpperMatrix<T> C(n);
		for(int i=0; i<n; i++) for(int j=i; j<n; j++) C(i,j) = A(i,j) + B(i,j);
		return C;
	}
#endif

/* Add a Matrix and a Matrix, C = A + B */
template <class T>
Matrix<T> operator + (const SymmetricMatrix<T> &A, const Matrix<T> &B)
{
	int i,j;
	int n = A.Dim();

	// Check dimensions
	if(B.Rows() != n || B.Cols() != n)
	{
		Print("Error in Matrix = SymmetricMatrix + Matrix");
		Print("C = A + B");
		Print("Incompatible dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	// Preform calculations
	Matrix<T> C(n,n);
	for(i=0; i<n; i++) for(j=0; j<n; j++) C(i,j) = A(i,j) + B(i,j);
	return C;
}

#ifdef _GenMat_SymmetricMatrixUP
	template <class T>
	Matrix<T> operator + (const SymmetricMatrixUP<T> &A, const Matrix<T> &B)
	{
		/* Check Dimensions */
		int n = A.Dim();		
		if(B.Rows() != n || B.Cols() != n)
		{
			Print("Error in Matrix = SymmetricMatrixUP + Matrix");
			Print("C = A + B");
			Print("Incompatible dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}	

		/* Preform Calculations */
		Matrix<T> C(n,n);
		for(int i=0; i<n; i++) for(int j=0; j<n; j++) C(i,j) = A(i,j) + B(i,j);
		return C;
	}
#endif

/* Add a DiagonalMatrix and a SymmetricMatrix, C = A + B */
#ifdef _GenMat_DiagonalMatrix
	template <class T>
	SymmetricMatrix<T> operator + (const DiagonalMatrix<T> &A, const SymmetricMatrix<T> &B)
	{
		/* Check Dimensions */
		int n = A.Dim();
		if(B.Dim() != n)
		{
			Print("Error in SymmetricMatrix = DiagonalMatrix + SymmetricMatrix");
			Print("C = A + B");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		SymmetricMatrix<T> C(n);
		for(int i=0; i<n; i++) for(int j=0; j<=i; j++) C(i,j) = A(i,j) + B(i,j);
		return C;
	}
#endif

/* Add a SymmetricMatrix and a DiagonalMatrix, C = A + B */
#ifdef _GenMat_DiagonalMatrix
	template <class T>
	SymmetricMatrix<T> operator + (const SymmetricMatrix<T> &A, const DiagonalMatrix<T> &B)
	{
		/* Check Dimensions */
		int n = A.Dim();
		if(B.Dim() != n)
		{
			Print("Error in SymmetricMatrix = SymmetricMatrix + DiagonalMatrix");
			Print("C = A + B");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		SymmetricMatrix<T> C(n);
		for(int i=0; i<n; i++) for(int j=0; j<=i; j++) C(i,j) = A(i,j) + B(i,j);
		return C;
	}
#endif

#endif
