
#ifndef _BinarySearch
#define _BinarySearch

/*

Binary search:

Find the index of an element in v equal to a
in O(log(n)) time, where n is the number of
elements in v. Here, v is assumed to be sorted
in increasing order. If an element does not exist
reutrn -1.

*/

template <class T>
int BinarySearchRecur(const GeneralVector<T> &v, int First, int Last, const T &a)
{
   /*

   Searches v[First]..v[Last] for a.

   returns: index of the matching element if it finds a,
            otherwise  -(index where it could be inserted)-1.
    parameters:
      v           in  array of sorted (ascending) values.
      First, Last in  lower and upper subscript bounds
      a           in  value to search for.

	returns:
      index of a, or -insertion_position -1 if key is not
                    in the array. This value can easily be
                    transformed into the position to insert it.

   */

   while(First <= Last)
   {
       int Mid = (First + Last) / 2;  // Compute mid point.

            if(a > v[Mid]) First = Mid + 1;	// Repeat search in top half
       else if(a < v[Mid]) Last  = Mid - 1;	// Repeat search in bottom half
       else	               return Mid;		// Found a, return position
   }
   return -(First + 1);	// Failed to find a
}

template <class T>
int BinarySearch(const GeneralVector<T> &v, const T &a)
{
	int n = v.Size();
	int Test = BinarySearchRecur(v,0,n-1,a);
	return Test;
}

int BinarySearchRecur(const GeneralVector<short> &v, int First, int Last, short a, bool (*LessThan)(short, short))
{
   /*

   Searches v[First]..v[Last] for a.

   returns: index of the matching element if it finds a,
            otherwise  -(index where it could be inserted)-1.
    parameters:
      v           in  array of sorted (ascending) values.
      First, Last in  lower and upper subscript bounds
      a           in  value to search for.

	returns:
      index of a, or -insertion_position -1 if key is not
                    in the array. This value can easily be
                    transformed into the position to insert it.

   */

   while(First <= Last)
   {
       int Mid = (First + Last) / 2;  // Compute mid point.
            if(LessThan(v[Mid],a)) First = Mid + 1; // Repeat search in top half
       else if(LessThan(a,v[Mid])) Last  = Mid - 1; // Repeat search in bottom half
       else	                       return Mid;		 // Found a, return position
   }
   return -(First + 1);	// Failed to find a
}

template <class T>
int BinarySearchRecur(const GeneralVector<T> &v, int First, int Last, const T &a, bool (*LessThan)(const T &, const T &))
{
   /*

   Searches v[First]..v[Last] for a.

   returns: index of the matching element if it finds a,
            otherwise  -(index where it could be inserted)-1.
    parameters:
      v           in  array of sorted (ascending) values.
      First, Last in  lower and upper subscript bounds
      a           in  value to search for.

	returns:
      index of a, or -insertion_position -1 if key is not
                    in the array. This value can easily be
                    transformed into the position to insert it.

   */

   while(First <= Last)
   {
       int Mid = (First + Last) / 2;  // Compute mid point.
            if(LessThan(v[Mid],a)) First = Mid + 1; // Repeat search in top half
       else if(LessThan(a,v[Mid])) Last  = Mid - 1; // Repeat search in bottom half
       else	                       return Mid;		 // Found a, return position
   }
   return -(First + 1);	// Failed to find a
}

template <class T>
int BinarySearch(const GeneralVector<T> &v, const T &a, bool (*LessThan)(const T &, const T &))
{
	int n = v.Size();
	int Test = BinarySearchRecur(v,0,n-1,a,LessThan);
	return Test;
}

int BinarySearch(const GeneralVector<short> &v, short a, bool (*LessThan)(short, short))
{
	int n = v.Size();
	int Test = BinarySearchRecur(v,0,n-1,a,LessThan);
	return Test;
}

template <class T>
int BinarySearch2(const GeneralVector<T> &v, const T &a)
{
	int low = 0;
	int high = v.Size() - 1;
	while (low <= high)
	{
       int mid = (low + high) / 2;
            if(v[mid] > a) high = mid - 1;
       else if(v[mid] < a) low  = mid + 1;
       else                return mid; // Found
	}
	return -(low + 1); // Not Found
}

Vector<int> BinarySearchLowHigh(int Info)
{
	if(Info >= 0)
	{
		return Vector<int>(1,Info);
	}
	else
	{
		Vector<int> Ret(2);
		Ret(0) = -Info - 2;
		Ret(1) = -Info - 1;
		return Ret;		
	}
}

template <class T>
Vector<int> BinarySearchLowHigh(const GeneralVector<T> &v, const T &a)
{
	if(v.Size() == 0) return Vector<int>(0);
	int n = v.Size();
	int Test = BinarySearchRecur(v,0,n-1,a);
	if(Test >= 0)
	{
		return Vector<int>(1,Test);
	}
	else
	{
		Vector<int> Ret(2);
		Ret(0) = -Test - 2;
		Ret(1) = -Test - 1;
		return Ret;		
	}
}

#endif
