
#ifndef _BlasAxpy
#define _BlasAxpy


	/* Preform the operation, y := y + Alpha * x */
	template <class T>
	inline void BlasAxpy(int n, const T &a, const T *x, int incx, T *y, int incy);

/*

Preform the operation,

	y := y + Alpha * x
	
Uses unrolled loops for increments equal to one.	
	
Adapted from the Level-1 Blas routine,

	DAXPY(N, DA, DX, INCX, DY, INCY)
	
*/

template <class T>
inline void BlasAxpy(int n, const T &a, const T *x, int incx, T *y, int incy)
{
			
	if(n <= 0) return;
	if(a == GetZero<T>()) return;
	
	if(incx == 1 && incy == 1)
	{
		/* Both increments equal to 1 */
		int m = n % 4;
		if(m != 0)
		{	
			for(int i=0; i<m; i++) y[i] += a * x[i];
			if(n < 4) return;
		}
		
		for(int i=m; i<n; i+=4)
		{
			y[i  ] += a * x[i  ];
			y[i+1] += a * x[i+1];
			y[i+2] += a * x[i+2];
			y[i+3] += a * x[i+3];
		}
	}
	else
	{
		/* Unequal increments or equal increments not equal to 1 */
		int ix = 0;
		int iy = 0;
		if(incx < 0) ix = (-n + 1) * incx;
		if(incy < 0) iy = (-n + 1) * incy;
		
		for(int i=0; i<n; i++)
		{
			y[iy] += a * x[ix];
			ix += incx;
			iy += incy;
		}
  	} 
}

#endif
