
#ifndef _BlasCopy
#define _BlasCopy

/*

Copies an array x to an array y.
Uses unrolled loops for increments equal to one.

Adapted from the Blas level-1 subroutine,

	DCOPY( N, DX, INCX, DY, INCY)
	
	jack dongarra, linpack, 3/11/78.
	modified 12/3/93, array(1) declarations changed to array(*)
	
*/


template <class Real>
void BlasCopy(int n, const Real *x, int incx, Real *y, int incy)
{

	if(n == 0) return;
	
	if(incx == 1 && incy == 1)
	{
		/* Code for both increments equal to 1 */
		
		/* Clean-up loop */
		int m = n % 7;
		if(m != 0)
		{
			for(int i=0; i<m; i++) y[i] = x[i];
			if(n < 7) return;
		}
		
		for(int i=m; i<n; i+=7)
		{
			y[i    ] = x[i    ];
			y[i + 1] = x[i + 1];
			y[i + 2] = x[i + 2];
			y[i + 3] = x[i + 3];
			y[i + 4] = x[i + 4];
			y[i + 5] = x[i + 5];
			y[i + 6] = x[i + 6];
		}
	}
	else
	{
		/* Code for unequal increments or equal increments
		not equal to 1 */
		
		int ix;
		if(incx < 0) ix = -(n - 1) * incx;
		else         ix = 0;
		
		int iy;
		if(incy < 0) iy = -(n - 1) * incy;
		else         iy = 0;
		
		for(int i=0; i<n; i++)
		{
			y[iy] = x[ix];
			ix += incx;
			iy += incy;
		}
    }
}

#endif
