
#ifndef _BlasDot
#define _BlasDot

	/* Blas routine to form the do product of two vectors */
	template <class Real>
	Real BlasDot(int n, const Real *x, int incx, const Real *y, int incy);

/*

BLAS SUBROUTINE SDOT / DDOT
double precision function ddot(n,dx,incx,dy,incy)

This is a templated version of the Fortran BLAS rountine.
Forms the dot product of two vectors.
Uses unrolled loops for increments equal to one.
Jack Dongarra, linpack, 3/11/78.
modified 12/3/93, array(1) declarations changed to array(*)
Converted to c++ by MP.

*/

template <class Real>
Real BlasDot(int n, const Real *x, int incx, const Real *y, int incy)
{
	/* Quick return if possible (MP) */
	if(n == 0) return Real(Zero);

	/* Check for zero increments (MP) */
	if(n < 0)
	{
		Print("Error in BlasDot");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}
	else if(incx == 0 || incy == 0)
	{
		Print("Error in BlasDot");
		Print("Zero incements");
		Print("incx",incx);
		Print("incy",incy);
		HaltExecution();
	}
	
	Real Ret = Zero;
	if(incx == 1 && incy == 1)
	{
		/* Both increments equal to 1 */
		
		/* Clean-up loop */
		int m = n % 5;
		if(m != 0)
		{
			for(int i=0; i<m; i++) Ret += x[i] * y[i];
			if(n < 5) return Ret;
		}
						
		for(int i=m; i<n; i+=5)
		{
			Ret +=   x[i  ] * y[i  ]
			       + x[i+1] * y[i+1]
			       + x[i+2] * y[i+2]
			       + x[i+3] * y[i+3]
			       + x[i+4] * y[i+4];
		}
	}
	else
	{
		/* Unequal increments or equal increments not equal to 1 */
		
		int ix = 0;
		int iy = 0;
		if(incx < 0) ix = (-n + 1) * incx;
		if(incy < 0) iy = (-n + 1) * incy;
		
		for(int i=0; i<n; i++)
		{
			Ret += x[ix] * y[iy];
			ix += incx;
			iy += incy;
		}
	}
	
 	return Ret;
}

#endif
