
#ifndef _BlasNrm2
#define _BlasNrm2

/*

DOUBLE PRECISION FUNCTION DNRM2(N,X,INCX)

.. Scalar Arguments ..
INTEGER INCX,N

.. Array Arguments ..
DOUBLE PRECISION X(*)

Purpose
=======

DNRM2 returns the euclidean norm of a vector via the function
name, so that

DNRM2 := sqrt( x'*x )

-- This version written on 25-October-1982.
   Modified on 14-October-1993 to inline the call to DLASSQ.
   Sven Hammarling, Nag Ltd.

*     ..
*     .. Local Scalars ..
      DOUBLE PRECISION ABSXI,NORM,SCALE,SSQ
      INTEGER IX

*/

template <class T>
T BlasNrm2(int n, const T* x, int incx)
{
	     if(n < 1 || incx < 1) return Zero;
	else if(n == 1)            return Abs(x[0]);
	
	T Scale = Zero;
	T Ssq = One;
	for(int ix=0; ix<1+(n-1)*incx; ix+=incx)
	{
		if(x[ix] != Zero)
		{
			T Absxi = Abs(x[ix]);
			if(Scale < Absxi)
			{
				Ssq = One + Ssq * Sqr(Scale / Absxi);
				Scale = Absxi;
			}
			else
			{
				Ssq += Sqr(Absxi / Scale);
			}
		}
	}
	return Scale * Sqrt(Ssq);
}

#endif
