
#ifndef _BlasTrmm
#define _BlasTrmm


	void  BlasTrmm(bool Left, bool Upper, bool TransA, bool Diag, int m, int n, double Alpha,
				double *A, int lda, double *B, int ldb);

/*

Performs one of the matrix-matrix operations

    B := Alpha * op(A) * B,   or   B := Alpha * B * op(A),

where  alpha  is a scalar,  B  is an m by n matrix,  A  is a unit, or
non-unit,  upper or lower triangular matrix  and  op( A )  is one  of

    op(A) = A   or   op(A) = A^T.


Parameters
----------

Left   - CHARACTER*1.
         On entry,  SIDE specifies whether  op( A ) multiplies B from
         the left or right as follows:

            SIDE = 'L' or 'l'   B := alpha*op( A )*B.
            SIDE = 'R' or 'r'   B := alpha*B*op( A ).

         Unchanged on exit.

Upper   - CHARACTER*1.
         On entry, UPLO specifies whether the matrix A is an upper or
         lower triangular matrix as follows:

             UPLO = 'U' or 'u'   A is an upper triangular matrix.
             UPLO = 'L' or 'l'   A is a lower triangular matrix.

         Unchanged on exit.

TransA - CHARACTER*1.
         On entry, TRANSA specifies the form of op( A ) to be used in
         the matrix multiplication as follows:

             TRANSA = 'N' or 'n'   op( A ) = A.
             TRANSA = 'T' or 't'   op( A ) = A'.
             TRANSA = 'C' or 'c'   op( A ) = A'.

         Unchanged on exit.

Diag   - CHARACTER*1.
         On entry, DIAG specifies whether or not A is unit triangular
         as follows:

             DIAG = 'U' or 'u'   A is assumed to be unit triangular.
             DIAG = 'N' or 'n'   A is not assumed to be unit
                                 triangular.

         Unchanged on exit.

m      - INTEGER.
         On entry, M specifies the number of rows of B. M must be at
         least zero.
         Unchanged on exit.

n      - INTEGER.
         On entry, N specifies the number of columns of B.  N must be
         at least zero.
         Unchanged on exit.

Alpha  - DOUBLE PRECISION.
         On entry,  ALPHA specifies the scalar  alpha. When  alpha is
         zero then  A is not referenced and  B need not be set before
         entry.
         Unchanged on exit.

A      - DOUBLE PRECISION array of DIMENSION ( LDA, k ), where k is m
         when  SIDE = 'L' or 'l'  and is  n  when  SIDE = 'R' or 'r'.
         Before entry  with  UPLO = 'U' or 'u',  the  leading  k by k
         upper triangular part of the array  A must contain the upper
         triangular matrix  and the strictly lower triangular part of
         A is not referenced.
         Before entry  with  UPLO = 'L' or 'l',  the  leading  k by k
         lower triangular part of the array  A must contain the lower
         triangular matrix  and the strictly upper triangular part of
         A is not referenced.
         Note that when  DIAG = 'U' or 'u',  the diagonal elements of
         A  are not referenced either,  but are assumed to be  unity.
         Unchanged on exit.

lda    - INTEGER.
         On entry, LDA specifies the first dimension of A as declared
         in the calling (sub) program.  When  SIDE = 'L' or 'l'  then
         LDA  must be at least  max( 1, m ),  when  SIDE = 'R' or 'r'
         then LDA must be at least max( 1, n ).
         Unchanged on exit.

B      - DOUBLE PRECISION array of DIMENSION ( LDB, n ).
         Before entry,  the leading  m by n part of the array  B must
         contain the matrix  B,  and  on exit  is overwritten  by the
         transformed matrix.

ldb    - INTEGER.
         On entry, LDB specifies the first dimension of B as declared
         in  the  calling  (sub)  program.   LDB  must  be  at  least
         max( 1, m ).
         Unchanged on exit.


Adapted form the Level 3 Blas subroutine,

	DTRMM ( SIDE, UPLO, TRANSA, DIAG, M, N, ALPHA, A, LDA, B, LDB )

	-- Written on 8-February-1989.
	Jack Dongarra, Argonne National Laboratory.
	Iain Duff, AERE Harwell.
	Jeremy Du Croz, Numerical Algorithms Group Ltd.
	Sven Hammarling, Numerical Algorithms Group Ltd.

*/

void  BlasTrmm(bool Left, bool Upper, bool TransA, bool Diag, int m, int n, double Alpha,
				double *A, int lda, double *B, int ldb)
{

	int i,j,k,nRowA;
	double Temp;
	
	/* Executable Statements */
	
	/* Test the input parameters. */
	if(Left) nRowA = m;
	else     nRowA = n;
	
	if(m < 0)
	{
		Print("Error in BlasTrmm");
		Print("m < 0");
		Print("m",m);
		HaltExecution();
	}
	else if(n < 0)
	{
		Print("Error in BlasTrmm");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}
	else if(lda < Max(1,nRowA))
	{
		Print("lda < Max(1,nRowA)");
		Print("lda",lda);
		Print("nRowA",nRowA);
		HaltExecution();
	}
	else if(ldb < Max(1,m))
	{
		Print("ldb < Max(1,m)");
		Print("ldb",ldb);
		Print("m",m);
		HaltExecution();
	}
	
	/* Quick return if possible. */
	if(n == 0) return;
	
	/* And when Alpha == 0.0. */
	if(Alpha == Zero) BlasSet(m*n,Zero,B,1);
	
    /* Start the operations. */
    if(Left)
    {
    	if(!TransA)
    	{
    		/* Form  B := Alpha * A * B. */
    		if(Upper)
    		{
    			for(i=0; i<m; i++)
    			{
    				for(k=0; k<i; k++)
    				{
    					Temp = Alpha * A[lda*k+i];
						BlasAxpy(n,Temp,&B[ldb*i],1,&B[lda*k],1);
    				}
    				if(Diag)
					{
						Temp = Alpha * A[lda*i+i];
						BlasScal(n,Temp,&B[ldb*i],1);
					}
				}
			}
			else
			{
				for(i=m-1; i>=0; i--)
				{
					for(k=i+1; k<m; k++)
					{
						Temp = Alpha * A[lda*k+i];
						BlasAxpy(n,Temp,&B[ldb*i],1,&B[lda*k],1);
					}
					if(Diag)
					{
						Temp = Alpha * A[lda*i+i];
						BlasScal(n,Temp,&B[ldb*i],1);
					}
				}
			}
		}
		else
		{		
			/* Form  B := Alpha * A^T * B. */
			if(Upper)
			{
				for(i=m-1; i>=0; i--)
				{
					if(Diag)
					{
						Temp = Alpha * A[lda*i+i];
						BlasScal(n,Temp,&B[ldb*i],1);
					}
					for(k=0; k<i; k++)
					{
						Temp = Alpha * A[lda*k+i];
						BlasAxpy(n,Temp,&B[ldb*k],1,&B[lda*i],1);
					}
				}
			}
			else
			{
				for(i=0; i<m; i++)
				{
					if(Diag)
					{
						Temp = Alpha * A[lda*i+i];
						BlasScal(n,Temp,&B[ldb*i],1);
					}
					for(k=i+1; k<m; k++)
					{
						Temp = Alpha * A[lda*k+i];
						BlasAxpy(n,Temp,&B[ldb*k],1,&B[lda*i],1);
					}
				}
			}
		}
	}
	else
	{
		if(!TransA)
		{
			/* Form  B := Alpha * B * A. */
			if(Upper)
			{
				for(i=0; i<m; i++)
				{
					for(j=n-1; j>=0; j--)
					{					
						if(Diag) B[ldb*i+j] *= A[lda*j+j];
						B[ldb*i+j] += BlasDot(j,&A[j],lda,&B[ldb*i],1);	// Not efficient
						B[lda*i+j] *= Alpha;
					}
				}
			}
			else
			{
				for(i=0; i<m; i++)
				{
					for(j=0; j<n; j++)
					{	
						if(Diag) B[ldb*i+j] *= A[lda*j+j];
						B[ldb*i+j] += BlasDot(n-j-1,&A[lda*(j+1)+j],lda,&B[ldb*i+j+1],1);	// Not efficient
						B[ldb*i+j] *= Alpha;
					}
				}
			}
		}
		else
		{
			/* Form  B := Alpha * B * A^T. */
			if(Upper)
			{
				for(i=0; i<m; i++)
				{
					for(k=0; k<n; k++)
					{
						Temp = Alpha * B[ldb*i+k];
						BlasAxpy(k,Temp,&A[k],lda,&B[ldb*i],1);		// Not efficient
						if(Diag) B[ldb*i+k] *= Alpha * A[lda*k+k];
					}
				}
			}
			else
			{
				for(i=0; i<m; i++)
				{
					for(k=n-1; k>=0; k--)
					{
						Temp = Alpha * B[ldb*i+k];
						BlasAxpy(n-k-1,Temp,&A[lda*(k+1)+k],lda,&B[lda*i+k+1],1); // Not efficient
						if(Diag) B[ldb*i+k] *= Alpha * A[lda*k+k];	
					}
				}
			}
		}
	}
}

#endif
