
#ifndef _BlasTrmv
#define _BlasTrmv


	void BlasTrmv(bool Upper, bool Trans, bool Diag, int n, double *A, int lda, double *x, int incx);

/*

Performs one of the matrix-vector operations

    x := A * x,   or   x := A^T * x,

where x is n element vector and A is an n by n unit, or non-unit,
upper or lower triangular matrix.


Parameters
----------

Upper (boolean)	-		On entry, Upper specifies whether the matrix is an upper or
         				lower triangular matrix as follows:

Trans (boolean)	- 		On entry, TRANS specifies the operation to be performed as
         				follows:

         					TRANS = 'N' x := A   * x.
            				TRANS = 'T' x := A^T * x.
            				TRANS = 'C' x := A^T * x.

Diag (boolean)	- 		On entry, Diag specifies whether or not A is unit
         				triangular as follows:

							DIAG = 'U'          A is assumed to be unit triangular.
            				DIAG = 'N'          A is not assumed to be unit
            				                    triangular.

n (integer)		-		On entry, n specifies the order of the matrix A.
         				n must be at least zero.


A    			-		Array of DIMENSION ( LDA, n ).
         				Before entry with  UPLO = 'U', the leading n by n
      					upper triangular part of the array A must contain the upper
         				triangular matrix and the strictly lower triangular part of
         				A is not referenced.
         				Before entry with UPLO = 'L', the leading n by n
         				lower triangular part of the array A must contain the lower
         				triangular matrix and the strictly upper triangular part of
         				A is not referenced.
         				Note that when  DIAG = 'U', the diagonal elements of
         				A are not referenced either, but are assumed to be unity.

lda (integer)	-		On entry, LDA specifies the first dimension of A as declared
        				in the calling (sub) program. LDA must be at least max(n,1).

x  			    -		Array of dimension at least
         				( 1 + ( n - 1 )*abs( INCX ) ).
        				Before entry, the incremented array X must contain the n
         				element vector x. On exit, X is overwritten with the
         				tranformed vector x.

incx 			- 		On entry, INCX specifies the increment for the elements of
         				X.


Adapted from the Level 2 Blas subroutine,

	STRMV(UPLO,TRANS,DIAG,N,A,LDA,X,INCX)

	-- Written on 30-September-1985.
	Sven Hammarling, Nag Central Office.
	REVISED 860623
	REVISED YYMMDD
	BY R. J. HANSON, SANDIA NATIONAL LABS.

*/


void BlasTrmv(bool Upper, bool Trans, bool Diag, int n, double *A, int lda, double *x, int incx)
{
	int i,ix;
	
	if(incx <= 0)
	{
		Print("Error in BlasTrmv");
		Print("incx <= 0");
		Print("incx",incx);
		Print("This routine has not yet been tested for negative increments");
		HaltExecution();
	}
	
	/* Quick return if possible. */
	if(n == 0) return;

	/*

	Set up the start point in x if the increment is not unity. This
	will be (n - 1) * incx too small for descending loops.

	*/

	/* ??? */

	/*

	Start the operations. In this version the elements of A are
	accessed sequentially with one pass through A.

	*/

	if(!Trans)
	{
		/* Form  x := A * x. */
		if(Upper)
		{
			/* Upper triangular */
			ix = 0;
			for(i=0; i<n; i++)
			{
				if(Diag) x[ix] *= A[(lda+1)*i];
				x[ix] += BlasDot(n-i-1,&x[ix+incx],incx,&A[(lda+1)*i+1],1);
				ix += incx;
			}
		}
		else
		{
			/* Lower triangular */
			ix = incx * (n - 1);
			for(i=n-1; i>=0; i--)
			{
				if(Diag) x[ix] *= A[(lda+1)*i];
				x[ix] += BlasDot(i,&x[0],-incx,&A[lda*i],-1);
				ix -= incx;
			}
        }
 	}
 	else
 	{
 		/* Form  x := A^T * x. */
 		if(Upper)
 		{
 			ix = incx * (n - 1);
			for(i=n-1; i>=0; i--)
			{
				if(x[ix] != Zero)
 				{
 					double Temp = x[ix];
 					BlasAxpy(n-i-1,Temp,&A[(lda+1)*i+1],-1,&x[ix+incx],-incx);
	 				if(Diag) x[ix] *= A[(lda+1)*i];
	 			}
	 			ix -= incx;
 			}
		}
		else
		{
			/* Lower triangular */
			ix = 0;
			for(i=0; i<n; i++)
			{
				if(x[ix] != Zero)
				{
					double Temp = x[ix];
					BlasAxpy(i,Temp,&A[lda*i],1,&x[0],incx);
					if(Diag) x[ix] *= A[(lda+1)*i];
				}
				ix += incx;
			}
		}
	}
}

#endif
