
#ifndef _GenMatUtil_CholeskyDecomp
#define _GenMatUtil_CholeskyDecomp



int CholeskyDecomp(Matrix<double> &A)
{
	int n = A.Rows();
	if(A.Cols() != n) return 1;		// Not Square
	if(!IsSymmetric(A)) return 2;	// Not Symmetric
	
	/* Preform Decomposition */
	int Info = LapackPotrf(0,n,A.Begin(),n); 

	/* Set Other Part of Matrix */    
	for(int i=0; i<n; i++) for(int j=i+1; j<n; j++) A(i,j) = Zero;
    
    if(Info) return 3;	// Not Positive Definite, Cholesky Decomposition Failed
    
    return 0;	// Succescully Computed Cholesky Decomposition
}

#ifdef _GenMat_BlasMatrix
	int CholeskyDecomp(BlasMatrix<double> &A)
	{
		int n = A.Rows();
		int ld = A.LeadingDimension();
		if(A.Cols() != n) return 1;		// Not Square
		if(!IsSymmetric(A)) return 2;	// Not Symmetric
		
		/* Preform Decomposition */
		int Info = LapackPotrf(0,n,A.Begin(),ld);

		/* Set Other Part of Matrix */    
		for(int i=0; i<n; i++) for(int j=i+1; j<n; j++) A(i,j) = Zero;
	    
	    if(Info) return 3;	// Not Positive Definite, Cholesky Decomposition Failed
	    
	    return 0;	// Succescully Computed Cholesky Decomposition
	}
#endif

int CholeskyDecomp(const Matrix<double> &A, LowerMatrix<double> &L)
{
	int n = A.Rows();
	
	if(A.Cols() != n)
	{
		Print("Error in CholesDecomp(Matrix,LowerMatrix)");
		Print("A is not square");
		Print("A.Name",A.Name);
		Print("A.Size()",A.Size());
		HaltExecution();
	}
	
	if(L.Dim() != n)
	{
		Print("Error in CholeskyDecomp(Matrix,LowerMatrix)");
		Print("Incompadible Dimensions");
		Print("A.Name",A.Name);
		Print("A.Size()",A.Size());
		Print("L.Name",L.Name);
		Print("L.Size()",L.Size());
		HaltExecution();
	}
	
	for(int i=0; i<n; i++) for(int j=0; j<i; j++) if(Abs(A(i,j) - A(j,i)) > SymmetricTol)
	{
		Print("Error in CholeskyDecomp(Matrix,LowerMatrix");
		Print("A is not symmetric");
		//Print("A(" + String(i) + "," + String(j) + ") != A( " + String(j) + "," + String(i) + "\n");
		Print("SymmetricTol",SymmetricTol);
		Print("A.Name",A.Name);
		Print("A(i,j)",A(i,j));
		Print("A(j,i)",A(j,i));
		Print("Abs(A(i,j) - A(j,i))",Abs(A(i,j) - A(j,i)));
		HaltExecution();
	}
	
	/* Copy.. */
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++) L(i,j) = A(i,j);
	
	/* Preform decomposition */
	int Info = LapackPptrf(0,n,L.Begin());
	
	return Info;
	
	/*
	int j,k;
	double d,s;
	int Spd = 1;
	int n = A.Rows();
	if(A.Cols() != n) return 1;
	if(L.Dim() != n)  return 2;
	
    for(j=0; j<n; j++) 
	{
		d = 0.0;
		for(k=0; k<j; k++) 
		{
			s = 0.0;
			
			/* 
			
			Make the innermost loop fast, we compute
			it using pointers and unrool it
			
			for(i=0; i<k; i++) s += L(k,i) * L(j,i);
			
			*/
	/*					
			s = BlasDot(k,&L(k,0),1,&L(j,0),1);
			
			
			L(j,k) = s = (A(j,k) - s) / L(k,k);
			d = d + s * s;
			Spd = Spd && (A(k,j) == A(j,k)); 
		}
		d = A(j,j) - d;
		Spd = Spd && (d > 0.0);
		L(j,j) = sqrt(d > 0.0 ? d : 0.0);
		for(k=j+1; k<n; k++) L(j,k) = 0.0;
	}
	return 0;
	*/
}   

template <class Real>
int CholeskyDecomp(const SymmetricMatrix<Real> &A, LowerMatrix<Real> &L)
{
	int n = A.Dim();
	if(L.Dim() != n)
	{
		Print("Error in CholeskyDecomp(SymmetricMatrix,LowerMatrix)");
		Print("Incompadible Dimensions");
		Print("A.Name",A.Name);
		Print("L.Name",L.Name);
		Print("A.Size()",A.Size());
		Print("L.Size()",L.Size());
		HaltExecution();
	}
	
	BlasCopy(n*(n+1)/2,A.Begin(),1,L.Begin(),1);
	int Info = LapackPptrf(0,n,L.Begin());
	return Info;	
	
	/*
	int j,k;
	double d,s;
	int Spd = 1;
	int n = A.Dim();
	if(L.Dim() != n)  return 2;
	
    for(j=0; j<n; j++) 
	{
		d = 0.0;
		for(k=0; k<j; k++) 
		{
			s = 0.0;
			
			/* 
			
			Make the innermost loop fast, we compute
			it using pointers and unrool it
			
			for(i=0; i<k; i++) s += L(k,i) * L(j,i);
			
			*/
	/*					
			s = BlasDot(k,&L(k,0),1,&L(j,0),1);
			
			
			L(j,k) = s = (A(j,k) - s) / L(k,k);
			d = d + s * s;
			Spd = Spd && (A(k,j) == A(j,k)); 
		}
		d = A(j,j) - d;
		Spd = Spd && (d > 0.0);
		L(j,j) = sqrt(d > 0.0 ? d : 0.0);
		for(k=j+1; k<n; k++) L(j,k) = 0.0;
	}
	return 0;
	*/
}

#ifdef _GenMat_SymmetricToeplitzMatrix
	int CholeskyDecomp(const SymmetricToeplitzMatrix<double> &A, LowerMatrix<double> &L)
	{
		int n = A.Dim();

		if(L.Dim() != n)
		{
			Print("Error in CholeskyDecomp(SymmetricToeplitzMatrix,LowerMatrix)");
			Print("Incompadible Dimensions");
			Print("A.Name",A.Name);
			Print("L.Name",L.Name);
			Print("A.Size()",A.Size());
			Print("L.Size()",L.Size());
			HaltExecution();
		}

		/* Matrix to Manipulate */
		Matrix<double> G(2,n);
		G.Name = "CholeskyDecomp::G";
		double Fac = sqrt(A(0));
		for(int i=0; i<n; i++) G(0,i) = A(i) / Fac;
		G(1,0) = Zero;
		for(int i=1; i<n; i++) G(1,i) = A(i) / Fac;

		Set(L,Zero);
		
		for(int i=0; i<n; i++) L(i,0) = G(0,i);
		for(int i=n-1; i>=1; i--) G(0,i) = G(0,i-1);
		G(0,0) = Zero;
		
		for(int i=1; i<n; i++)
		{
			if(G(0,i) == Zero) return i + 1; // Not Positive Definite
			double Rho = -G(1,i) / G(0,i);
			if(Rho >= One || Rho <= -One) return i + 1; // Not Positive Definite
			double Delta = One / sqrt((One - Rho) * (One + Rho));
			for(int j=i; j<n; j++)
			{
				double Temp1 = G(0,j);
				double Temp2 = G(1,j);
				G(0,j) = Delta * (Temp1 + Rho * Temp2);
				G(1,j) = Delta * (Rho * Temp1 + Temp2);
			}
			for(int j=i; j<n; j++) L(j,i) = G(0,j);
			for(int j=n-1; j>=i+1; j--) G(0,j) = G(0,j-1);
			G(0,i) = 0;
		}
		
		return 0;	// So Far, Does Not Throw Errors
	}
#endif

#if defined(SymmetricBandToeplitzMatrix) && defined(LowerBandMatrix)
	int CholeskyDecomp(const SymmetricBandToeplitzMatrix<double> &A, LowerBandMatrix<double> &L)
	{
		int n = A.Dim();
		int k = A.Bands();

		if(L.Rows() != n || L.Cols() != n)
		{
			Print("Error in CholeskyDecomp(SymmetricBandToeplitzMatrix,LowerMatrix)");
			Print("Incompadible Dimensions");
			Print("A.Name",A.Name);
			Print("L.Name",L.Name);
			Print("A.Size()",A.Size());
			Print("L.Size()",L.Size());
			HaltExecution();
		}

		/* Matrix to Manipulate */
		Matrix<double> G(2,n);
		G.Name = "CholeskyDecomp::G";
		double Fac = sqrt(A(0));
		for(int i=0; i<n; i++) G(0,i) = A(i) / Fac;
		G(1,0) = Zero;
		for(int i=1; i<n; i++) G(1,i) = A(i) / Fac;

		Set(L,Zero);
		
		for(int i=0; i<k; i++) L(i,0) = G(0,i);
		for(int i=n-1; i>=1; i--) G(0,i) = G(0,i-1);
		G(0,0) = Zero;
		
		for(int i=1; i<n; i++)
		{
			if(G(0,i) == Zero) return i + 1; // Not Positive Definite
			double Rho = -G(1,i) / G(0,i);
			if(Rho >= One || Rho <= -One) return i + 1; // Not Positive Definite
			double Delta = One / sqrt((One - Rho) * (One + Rho));
			//for(int j=i; j<n; j++)
			for(int j=i; j<Min(i+k,n); j++)
			{
				//double Temp1 = G(0,j);
				//double Temp2 = G(1,j);
				//G(0,j) = Delta * (Temp1 + Rho * Temp2);
				//G(1,j) = Delta * (Rho * Temp1 + Temp2);
				double Temp1 = G[j];
				double Temp2 = G[n+j];
				G[j]   = Delta * (Temp1 + Rho * Temp2);
				G[n+j] = Delta * (Rho * Temp1 + Temp2);
			}
			//for(int j=i; j<Min(i+k,n); j++) L(j,i) = G(0,j);
			for(int j=i; j<Min(i+k,n); j++) L[(k+1)*j+k-j+i] = G[j];
			//for(int j=Min(i+k,n-1); j>=i+1; j--) G(0,j) = G(0,j-1);
			for(int j=Min(i+k,n-1); j>=i+1; j--) G[j] = G[j-1];
			G(0,i) = 0;
		}
		
		return 0;	// So Far, Does Not Throw Errors
	}
#endif

template <class Real>
int CholeskyDecompSlow(const SymmetricMatrix<Real> &A, LowerMatrix<Real> &L)
{
	int n = A.Dim();
	if(L.Dim() != n)
	{
		Print("Error in CholeskyDecomp(SymmetricMatrix,LowerMatrix)");
		Print("Incompadible Dimensions");
		Print("A.Name",A.Name);
		Print("L.Name",L.Name);
		Print("A.Size()",A.Size());
		Print("L.Size()",L.Size());
		HaltExecution();
	}
	
	int Spd = 1;
    for(int j=0; j<n; j++) 
	{
		Real d = Zero;
		for(int k=0; k<j; k++) 
		{
			Real s = Zero;					
			for(int i=0; i<k; i++) s += L(k,i) * L(j,i);			
			L(j,k) = s = (A(j,k) - s) / L(k,k);
			d = d + s * s;
			Spd = Spd && (A(k,j) == A(j,k)); 
		}
		d = A(j,j) - d;
		Spd = Spd && (d > Zero);
		L(j,j) = sqrt(d > Zero ? d : Zero);
	}
	return 0;
}

/*
int CholeskyDecomp(const SymmetricBandMatrix<double> &A, LowerBandMatrix<double> &L)
{
	int n = A.Dim();
	int m = A.Bands();
	LowerBandMatrix<double> L(n,m);
	Real* s = A.Store();
	Real* t = L.Store();
	Real* ti = t;

    for(int i=0; i<n; i++)
    {
    	Real* tj = t;
    	int l;
    	if(i < m)
    	{
    		l = m - i;
    		s += l;
    		ti += l;
    		l = i;
    	}
    	else
    	{
    		t += (m + 1);
    		l = m;
    	}
    	
    	for(int j=0; j<l; j++)
    	{
    		Real* tk = ti;
    		double sum = Zero;
    		int k = j;
    		tj += (m - j);
    		
    		while(k--)
    		{
    			sum += *tj++ * *tk++;
    		}
    		*tk = (*s++ - sum) / *tj++;
    	}
    	double sum = 0.0;
    	while(l--) sum += square(*ti++);
    	double d = *s++ - sum;
    	if(d <= Zero) return i + 1;
    	*ti++ = sqrt(d);
    }
}
*/

template <class Real>
int CholeskyDecomp2(const SymmetricMatrix<Real> &A, LowerMatrix<Real> &L)
{
	int n = A.Dim();
	int b = 2;
	if(L.Dim() != n)
	{
		Print("Error in CholeskyDecomp(SymmetricMatrix,LowerMatrix)");
		Print("Incompadible Dimensions");
		Print("A.Name",A.Name);
		Print("L.Name",L.Name);
		Print("A.Size()",A.Size());
		Print("L.Size()",L.Size());
		HaltExecution();
	}
	
	int Spd = 1;
	if(L.Dim() != n)  return 2;
	
    for(int j=0; j<n; j++) 
	{
		double d = 0.0;
		for(int k=0; k<Min(j,b); k++) 
		{
			double s = 0.0;
			
			/* 
			
			Make the innermost loop fast, we compute
			it using pointers and unrool it
					
			*/
					
			//for(int i=0; i<k; i++) s += L(k,i) * L(j,i);
			for(int i=0; i<Min(k,b); i++) s += L(k,i) * L(j,i);
			
			//s = BlasDot(k,&L(k,0),1,&L(j,0),1);
			
			L(j,k) = s = (A(j,k) - s) / L(k,k);
			d = d + s * s;
			Spd = Spd && (A(k,j) == A(j,k)); 
		}
		d = A(j,j) - d;
		Spd = Spd && (d > Zero);
		L(j,j) = sqrt(d > Zero ? d : Zero);
		//for(int k=j+1; k<n; k++) L(j,k) = Zero;
		for(int k=j+1; k<Min(n,k+b); k++) L(j,k) = Zero;
		
	}
	return 0;
}

#if defined(_GenMat_SymmetricBandMatrix) && defined(_GenMat_LowerBandMatrix)
	template <class Real>
	int CholeskyDecomp2(const SymmetricBandMatrix<Real> &A, LowerBandMatrix<Real> &L)
	{
		int n = A.Dim();
		int b = A.Bands();
		if(L.Rows() != n || L.Cols() != n)
		{
			Print("Error in CholeskyDecomp(SymmetricMatrix,LowerMatrix)");
			Print("Incompadible Dimensions");
			Print("A.Name",A.Name);
			Print("L.Name",L.Name);
			Print("A.Size()",A.Size());
			Print("L.Size()",L.Size());
			HaltExecution();
		}
		
		int Spd = 1;	
	    for(int j=0; j<n; j++) 
		{
			double d = 0.0;
			for(int k=0; k<Min(j,b); k++) 
			{
				double s = 0.0;
				
				/* 
				
				Make the innermost loop fast, we compute
				it using pointers and unrool it
						
				*/
						
				//for(int i=0; i<k; i++) s += L(k,i) * L(j,i);
				for(int i=0; i<Min(k,b); i++) s += L(k,i) * L(j,i);
				
				//s = BlasDot(k,&L(k,0),1,&L(j,0),1);
				
				L(j,k) = s = (A(j,k) - s) / L(k,k);
				d = d + s * s;
				Spd = Spd && (A(k,j) == A(j,k)); 
			}
			d = A(j,j) - d;
			Spd = Spd && (d > Zero);
			L(j,j) = sqrt(d > Zero ? d : Zero);
			//for(int k=j+1; k<n; k++) L(j,k) = Zero;
			for(int k=j+1; k<Min(n,k+b); k++) L(j,k) = Zero;
			
		}
		return 0;
	}
#endif

#endif
