
#ifndef _Util_Complex
#define _Util_Complex

const double ComplexTol = 1.0e-15;

// Class for Complex arithmetic
template <class T>
class Complex
{
	public:
	
	T Real;
	T Imag;

	// Constructor 1
	Complex()
	{
		Real = 0;
		Imag = 0;
	}

	// Constructor 2
	Complex(T Real) : Real(Real), Imag(0) {}
	
	// Constructor 3
	Complex(T Real, T Imag) : Real(Real), Imag(Imag) {}
	
	// Destructor
	~Complex() {};
	
	//operatorer för normal räkning med de komplexa talen...
	const Complex& operator += (const Complex &c);
	const Complex& operator += (const T c); //vi vill kunna addera reella tal oxå!
	Complex operator + (const Complex &c) const;
	Complex operator + (const T c) const; //realla tal
	
	friend Complex operator += (T x, const Complex &c);
	friend Complex operator + (T x, const Complex &c);
	
	const Complex& operator -= (const Complex &c);
	const Complex& operator -= (const T c); //vi vill kunna subtrahera reella tal oxå!	
	Complex operator - (const Complex &c) const;
	Complex operator - (const T c) const; //realla tal
	
	friend Complex operator -= (T x, const Complex &c);
	friend Complex operator - (T x, const Complex &c);
	
	const Complex& operator *= (const Complex &c);
	const Complex& operator *= (const T c);
	Complex operator * (const Complex &c) const;
	Complex operator * (const T c) const;
	
	friend Complex operator *= (T x, const Complex &c);
	friend Complex operator * (T x, const Complex &c);
	
	const Complex& operator /= (const Complex& c);
	const Complex& operator /= (const T c);
	Complex operator / (const Complex& c) const;
	Complex operator / (const T c) const;
	
	friend Complex operator /= (T x, const Complex &c);
	friend Complex operator / (T x, const Complex &c);
	
	//unära operatorer, för att addera/subtrahera enklare.
	//vi vill även kunna negera vårt komplexa tal!
	Complex operator - () const; //negerar. En positiv variant verkar onödigt, eller?
	Complex operator ++ (int); //ökar hela vårt komplexa tal med 1 (både realdel och imaginärdel)
	const Complex& operator ++ ();
	Complex operator -- (int); //ökar hela vårt komplexa tal med 1 (både realdel och imaginärdel)
	const Complex& operator -- ();
	
	//en extra tilldelningsoperator för realla tal
	const Complex& operator = (const T c);

	//jämförelseoperatorer 
	//(ska bara behöva definiera två av dem eftersom vi inkluderat utility.h)
	bool operator == (const Complex& c) const;
	bool operator < (const Complex& c) const;
	
	const double GetR() const
	{
		return sqrt(Sqr(Real)+Sqr(Imag));
	}
	
	const double GetTheta() const
	{
		if(Real > Zero) return atan(Imag/Real);
		else if(Real < Zero)
		{
			if(Imag >= Zero) return atan(Imag/Real)+Pi;
			else             return atan(Imag/Real)-Pi;
		}
		else
		{
			if(Imag > Zero) return Pi / Two;
			else if(Imag < Zero) return -Pi / Two;
			else return Nan;
		}
	}	
};

// Function Prototypes:
template <class T> void Sum(Complex<T> &x, const Complex<T> &y, const Complex<T> &z);
template <class T> void Subtract(Complex<T> &x, const Complex<T> &y, const Complex<T> &z);
template <class T> void Prod(Complex<T> &x, const Complex<T> &y, const Complex<T> &z);
template <class T> void Divide(Complex<T> &x, const Complex<T> &y, const Complex<T> &z);
template <class T> void Set(Complex<T> &x, const Complex<T> &y);
template <class T> void Set(Complex<T> &x, T y);
template <class T> Complex<T> Exp(const Complex<T> &x);
template <class T> T Abs(const Complex<T> &x);
template <class T> Complex<T> Pow(const Complex<T> &x, int n);
template <class T> Vector< Complex<T> > ComplexSqrt(T a);
template <class T> void Convert(Vector< Complex<T> > &v, const Vector<T> &vr, const Vector<T> &vc);
template <class T> ostream & operator << (ostream &os, Complex<T> x);
template <class T> bool IsReal(const Complex<T> &c);
template <class T> T ToReal(const Complex<T> &c);

// Standard Operations:

template <class T>
void Sum(Complex<T> &x, const Complex<T> &y, const Complex<T> &z)
{
	x.Real = y.Real + z.Real;
	x.Imag = y.Imag + z.Imag;
	return;
}

template <class T>
void Subtract(Complex<T> &x, const Complex<T> &y, const Complex<T> &z)
{
	x.Real = y.Real - z.Real;
	x.Imag = y.Imag - z.Imag;
	return;
}

template <class T>
void Prod(Complex<T> &x, const Complex<T> &y, const Complex<T> &z)
{
	x.Real = y.Real * z.Real - y.Imag * z.Imag;
	x.Imag = y.Real * z.Imag + y.Real * z.Imag;
	return;
}

template <class T>
void Divide(Complex<T> &x, const Complex<T> &y, const Complex<T> &z)
{
	T k = 1 / (z.Real * z.Real + z.Imag * z.Imag);
	x.Real = (y.Real * z.Real - y.Imag * z.Imag) * k;
	x.Imag = (y.Imag * z.Real - y.Real * z.Imag) * k;
	return;
}

template <class T>
void Set(Complex<T> &x, const Complex<T> &y)
{
	x.Real = y.Real;
	x.Imag = y.Imag;
	return;
}

template <class T>
void Set(Complex<T> &x, T y)
{
	x.Real = y;
	x.Imag = 0;
	return;
}	

template <class T>
Complex<T> Exp(const Complex<T> &x)
{
	// evaluate the exponential of a Complex number
	// using the formula, e^(c+di)=cos(s)*e^c+i*sin(d)*e^c
	Complex<T> y;
	T exp1 = exp(x.Real);
	y.Real = exp1 * cos(x.Imag);
	y.Imag = exp1 * sin(x.Imag);
	return y;
}

template <class T>
T Abs(const Complex<T> &x)
{
	/*
	
	Evaluate the modulos of a Complex number,
	
	m = Sqrt((x.real)^2 + (x.imag)^2)
	
	*/
	
	return sqrt(Sqr(x.Real) + Sqr(x.Imag));

}

/*
template <class T>
Complex<T> Pow(const Complex<T> &x, int n)
{
	// Evaluate integer powers of a Complex number
	// int O(log(n)^2) time

	if(x.Real == Zero) return Nan;
	if(n == 0) return One;
	else if(n == 1) return x;
	else if(n == -1) return -x;
		
	Complex<T> y;
	y.Real = 1;
	y.Imag = 0;
	T log2 = log(2);
	int k = floor(log(abs(n)) / log2);
	Vector< Complex<T> > pw(k);

	Prod(pw[0],x,x);
	for(int i=1; i<k; i++) Prod(pw[i],pw[i-1],pw[i-1]);

	while(n > 0)
	{
		k = floor(log(abs(n)) / log2);
		Prod(y,y,pw[k]);
		n /= pow(2,k);
	}
	if(n > 0 || n % 2 == 0) return y;
	else                    return -y;
}
*/

template <class T>
Complex<T> Pow(const Complex<T> &x, double y)
{
	return Exp(Log(x) * y);
}

template <class T>
Vector< Complex<T> > ComplexSqrt(T a)
{
	// Solve x^2=a
	Vector<Complex> Roots(2);
	if(a>=0)
	{
		Roots[0].Real = sqrt(a);
		Roots[1].Real = -Roots[0].Real;
	}
	else
	{
		Roots[0].Imag = sqrt(-a);
		Roots[1].Imag = -Roots[0].Imag;
	}
	return Roots;
}

template <class T>
void Convert(Vector< Complex<T> > &v, const Vector<T> &vr, const Vector<T> &vc)
{
	int n = vr.Size();
	if(vc.Size() != n)
	{
		Print("Error in Complex(Comples Vector, Real Vector, Real Vector): Imcompadible Dimensions");
		return;
	}
	//if(v.Size() != n) v.Resize(n);
	
	if(v.Size() != n)
	{
		Print("Error in Convert(Vector,Vector,Vectr)");
		Print("Incompadible Dimensions");
		HaltExecution();
	}	
	
	for(int i=0; i<n; i++)
	{
		v[i].Real = vr[i];
		v[i].Imag = vc[i];
	}
	return;
}

template <class T>
ostream & operator << (ostream &os, Complex<T> x)
{
  	os << x.Real;
  	if(x.Imag > 0) os << " + " << x.Imag << "*i";
  	else if(x.Imag < 0) os << " - " << -x.Imag << "*i";
  	return os;
}


// Add two complex numbers
template <class T>
inline Complex<T> operator + (const Complex<T> &x, const Complex<T> &y)
{
	return Complex(x.real + y.real,x.imag + y.imag);
}

// Add a real and a complex
template <class T>
inline Complex<T> operator + (double x, const Complex<T> &y)
{
	return Complex(x + y.real,y.imag);
}

// Add a complex and a real
template <class T>
inline Complex<T> operator + (const Complex<T> &x, double y)
{
	return Complex(x.real + y,x.imag);
}

// Overloaded Operators
/*
template <class T>
Complex<T> & Complex<T>::operator += (const Complex<T> &x)
{
	real += x.real;
	imag += x.imag;
	return *this;
}
*/

/*
template <class T>
inline Complex<T> operator += (double x, const Complex<T> &y)
{
	return Complex(x + y.real, y.imag);
}

template <class T>
inline Complex<T> operator += (const Complex<T> &x, double y)
{
	return Complex(x.real + y, x.imag);
}

// Subtract two complex numbers
template <class T>
inline Complex<T> operator - (const Complex<T> &x, const Complex<T> &y)
{
	return Complex(x.real - y.real,x.imag - y.imag);
}

// Subtract a real and a complex
template <class T>
inline Complex<T> operator - (double x, const Complex<T> &y)
{
	return Complex(x - y.real,-y.imag);
}

// Subtract a complex and a real
template <class T>
inline Complex<T> operator - (const Complex<T> &x, double y)
{
	return Complex(x.real - y,x.imag);
}

// Multiply two complex numbers
template <class T>
inline Complex<T> operator * (const Complex<T> &x, const Complex<T> &y)
{
	return Complex(x.real * y.real - x.imag * y.imag,x.real * y.imag + x.imag * y.real);
}

// Multiply a real by a complex
template <class T>
inline Complex<T> operator * (double x, const Complex<T> &y)
{
	return Complex(x * y.real,x * y.imag);
}

// Multiply a complex by a real
template <class T>
inline Complex<T> operator * (const Complex<T> &x, double y)
{
	return Complex(x.real * y,x.imag * y);
}
*/

// Overloaded Operators (Taken From Somewhere Else
template <class T>
const Complex<T> & Complex<T>::operator += (const Complex& c)
{
	Real += c.Real;
	Imag += c.Imag;
	return *this;
}

template <class T>
const Complex<T> & Complex<T>::operator += (const T c)
{
	Real += c;
	return *this;
}

template <class T>
Complex<T> Complex<T>::operator + (const Complex<T> &c) const
{
	Complex<T> e(*this);
	e += c;
	return e;
}

template <class T>
Complex<T> Complex<T>::operator + (const T c) const
{
	Complex<T> e(Real + c,Imag);
	return e;
}

template <class T>
Complex<T> operator += (T x, const Complex<T> &c)
{
	Complex<T> e(x + c.Real, c.Imag);
	return e;
}

template <class T>
Complex<T> operator + (T x, const Complex<T> &c)
{
	Complex<T> e(x + c.Real, c.Imag);
	return e;
}

template <class T>
const Complex<T> & Complex<T>::operator -= (const Complex<T> &c)
{
	Real -= c.Real;
	Imag -= c.Imag;
	return *this;
}

template <class T>
const Complex<T> & Complex<T>::operator -= (const T c)
{
	Real -= c;
	return *this;
}

template <class T>
Complex<T> Complex<T>::operator - (const Complex<T> &c) const
{
	Complex e(*this);
	e -= c;
	return e;
}

template <class T>
Complex<T> Complex<T>::operator - (const T c) const
{
	Complex<T> e(Real - c,Imag);
	return e;
}

template <class T>
Complex<T> operator -= (T x, const Complex<T> &c)
{
	Complex<T> e(x - c.Real, -c.Imag);
	return e;
}

template <class T>
Complex<T> operator - (T x, const Complex<T> &c)
{
	Complex<T> e(x - c.Real, -c.Imag);
	return e;
}

template <class T>
const Complex<T> & Complex<T>::operator *= (const Complex<T> &c)
{
	T ac, bd;
	
	if(Real != 0 && Imag != 0)
	{
		ac = Real * c.Real;
		bd = Imag * c.Imag;
		
		// the order here is important since z might be the same as x or y
		Imag = (Real + Imag) * (c.Real + c.Imag) - ac - bd;
		Real = ac - bd;
		return *this;
	}
	
	if(Real==0)
	{
		ac = -Imag * c.Imag;
		Imag = Imag * c.Real;
		Real = ac;
		return *this;
	}
	
	if(Imag==0)
	{
		ac = Real * c.Real;
		Imag = Real * c.Imag;
		Real = ac;
		return *this;
	}
	
	return *this;
}

template <class T>
const Complex<T> & Complex<T>::operator *= (const T c)
{
	Real *= c;
	Imag *= c;
	return *this;
}

template <class T>
Complex<T> Complex<T>::operator * (const Complex<T> &c) const
{
	Complex<T> e(*this);
	e *= c;
	return e;
}

template <class T>
Complex<T> Complex<T>::operator * (const T c) const
{
	Complex<T> e(Real * c,Imag * c);
	return e;
}

template <class T>
Complex<T> operator *= (T x, const Complex<T> &c)
{
	Complex<T> e(x * c.Real, x * c.Imag);
	return e;
}

template <class T>
Complex<T> operator * (T x, const Complex<T> &c)
{
	Complex<T> e(x * c.Real, x * c.Imag);
	return e;
}

template <class T>
const Complex<T> & Complex<T>::operator /= (const Complex<T> &c)
{
	T ac,bd,den;
	
	if(c.Real == 0 && c.Imag == 0)
	{
		Real = NAN;
		Imag = NAN;
		return *this;
	}
	
	if(Real == 0 && Imag == 0)
		return *this;
	
	if(c.Real != 0 && c.Imag != 0)
	{
		den = c.Real * c.Real + c.Imag * c.Imag;
	
		ac = Real * c.Real;
		bd = Imag * c.Imag;
		
		// the order here is important since z might be the same as x or y
		Imag = ((Real + Imag) * (c.Real - c.Imag) - ac + bd) / den;
		Real = (ac + bd)/den;
		
		return *this;
	}
	
	if(c.Imag == 0)
	{
		Real = Real / c.Real;
		Imag = Imag / c.Real;
	}
	
	if(c.Real == 0)
	{
		ac = Real;
		Real = Imag / c.Imag;
		Imag = -ac / c.Imag;
	}
	
	return *this;
}

template <class T>
const Complex<T> & Complex<T>::operator /= (const T c)
{
	if(c == 0)
	{
		Real = NAN;
		Imag = NAN;
		return *this;
	}
	
	Real /= c;
	Imag /= c;
	
	return *this;
}

template <class T>
Complex<T> Complex<T>::operator / (const Complex<T> &c) const
{
	Complex<T> e(*this);
	e /= c;
	return e;
}

template <class T>
Complex<T> Complex<T>::operator / (const T c) const
{
	if(c == 0)
	{
		Complex<T> e(NAN, NAN);
		return e;
	}
	
	Complex<T> e(Real / c,Imag / c);
	return e;
}

template <class T>
Complex<T> operator /= (T x, const Complex<T> &c)
{
	T den;
	
	if(c.Real == 0 && c.Imag == 0)
	{
		Complex<T> e(NAN, NAN);
		return e;
	}
	
	den = c.Real * c.Real + c.Imag * c.Imag;
	
	Complex<T> e(x * c.Real / den, -x * c.Imag / den);
	return e;
}

template <class T>
Complex<T> operator / (T x, const Complex<T> &c)
{
	T den;
	
	if(c.Real == 0 && c.Imag == 0)
	{
		Complex<T> e(NAN, NAN);
		return e;
	}
	
	den = c.Real * c.Real + c.Imag * c.Imag;
	
	Complex<T> e(x * c.Real / den, -x * c.Imag / den);
	return e;
}

template <class T>
Complex<T> Complex<T>::operator - () const
{
	Complex<T> e(*this);
	e.Real = -Real;
	e.Imag = -Imag;
	return e;
}

template <class T>
const Complex<T> & Complex<T>::operator ++ ()
{
	Complex<T> e(1,1);
	return (*this) += e;
}

template <class T>
Complex<T> Complex<T>::operator ++ (int)
{
	Complex<T> e(1,1);
	(*this) += e;
	return e;
}

template <class T>
const Complex<T> & Complex<T>::operator -- ()
{
	Complex<T> e(1,1);
	return (*this) -= e;
}

template <class T>
Complex<T> Complex<T>::operator -- (int)
{
	Complex<T> e(1,1);
	(*this) -= e;
	return e;
}

template <class T>
const Complex<T> & Complex<T>::operator = (const T c)
{
	Real = c;
	Imag = 0;
	return *this;
}

template <class T>
bool Complex<T>::operator == (const Complex<T> &c) const
{
	if(c.Real == Real && c.Imag == Imag) return true;
	else								 return false;
}

template <class T>
bool Complex<T>::operator < (const Complex<T> &c) const
{
	if(Real < c.Real && Imag < c.Imag) return true;
	else							   return false;
}

template <class T>
bool IsReal(const Complex<T> &c)
{
	if(Abs(c.Imag) >= ComplexTol) return true;
	else                          return false;
}

template <class T>
T ToReal(const Complex<T> &c)
{
	if(!IsReal(c))
	{
		Print("Error in ToReal");
		Print("c is not Real");
		Print("c",c);
		Print("ComplexTol",ComplexTol);
		HaltExecution();
	}
	
	return c.Real;
}

template <class T>
Complex<T> Log(const Complex<T> &c)
{
	return Complex<T>(log(c.GetR()),c.GetTheta());	
}

template <class T>
Complex<T> Sqrt(const Complex<T> &c)
{
	return Pow(c,0.5);
}

#endif

