
#ifndef _GenMatUtil_Copy
#define _GenMatUtil_Copy

	/* Set B = A */
	template <class T> void Copy(GeneralVector<T>    &y, const GeneralVector<T>   &x);
	template <class T> void Copy(Matrix<T>           &B, const Matrix<T>          &A);
	template <class T1, class T2> void Copy(SymmetricMatrix<T1>  &B, const SymmetricMatrix<T2> &A);
	template <class T> void Copy(LowerMatrix<T>      &B, const LowerMatrix<T>     &A);
	template <class T> void Copy(Matrix<T>           &B, const SymmetricMatrix<T> &A);
	
	/* Set the elements of the matrix A equal to the elements of the matrix B,
	and do not check for data loss */
	template <class T> void Copy(SymmetricMatrix<T> &B, const Matrix<T>          &A);
	template <class T> void Copy(LowerMatrix<T>     &B, const Matrix<T>          &A);

	template <class T> void Copy(GeneralAssignableMatrix<T> &B, const GeneralMatrix<T>   &A);



template <class T>
void Copy(GeneralVector<T> &y, const GeneralVector<T> &x)
{
	int n = x.Size();

	/* Check for Incompadible Dimensions */
	if(y.Size() != n)
	{
		Print("Error in Copy(GeneralVector,GeneralVector)");
		Print("y = x");
		Print("Incompadible Dimensions");
		Print("y.Name",y.Name);
		Print("x.Name",x.Name);
		Print("y.Size()",y.Size());
		Print("x.Size()",x.Size());
		HaltExecution();
	}

	/* Preform Calculations */
	for(int i=0; i<n; i++) y[i] = x[i];
}

template <class T>
void Copy(Matrix<T> &B, const Matrix<T> &A)
{
	/* Check for Incompadible Dimensions */
	int m = A.Rows();
	int n = A.Cols();
	if(B.Rows() != m || B.Cols() != n)
	{
		Print("Error in Copy(Matrix,Matrix)");
		Print("B = A");
		Print("Incompadible Dimensions");
		Print("B.Name",B.Name);
		Print("A.Name",A.Name);
		Print("B.Size()",B.Size());
		Print("A.Size()",A.Size());
		HaltExecution();
	}

	/* Preform Calculations */
	for(int i=0; i<m; i++) for(int j=0; j<n; j++) B(i,j) = A(i,j);
}

template <class T1, class T2>
void Copy(SymmetricMatrix<T1> &B, const SymmetricMatrix<T2> &A)
{
	/* Check for Incompadible Dimensions */
	int n = A.Dim();
	if(B.Dim() != n)
	{
		Throw(ToString("Error in Copy(SymmetricMatrix,SymmetricMatrix)\n")
		             + "B = A\n"
		             + "Incompadible Dimensions\n"
		             + "B.Name: " + B.Name + "\n"
		             + "A.Name: " + A.Name + "\n"
		             + "B.Size(): " + ToString(B.Size()) + "\n"
		             + "A.Size(): " + ToString(A.Size()) + "\n");
	}

	/* Preform Calculations */
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++) B(i,j) = A(i,j);
}

// Set the elements or DiagonalMatrix A to the elements of DiagonalMatrix B
#ifdef _GenMat_DiagonalMatrix
	template <class T>
	void Copy(DiagonalMatrix<T> &B, const DiagonalMatrix<T> &A)
	{
		int n = A.Dim();

		/* Check for Incompadible Dimensions */
		if(B.Dim() != n)
		{
			Print("Error in Copy(DiagonalMatrix,DiagonalMatrix)")
			Print("B = A");
			Print("Incompadible Dimensions");
			Print("B.Name",B.Name);
			Print("A.Name",A.Name);
			Print("B.Size()",B.Size());
			Print("A.Size()",A.Size());
			HaltExecution();
		}

		/* Preform Calculations */
		for(int i=0; i<n; i++) B(i) = A(i);
	}
#endif

// Set the elements or Matrix A to the elements of Matrix B
template <class T>
void Copy(LowerMatrix<T> &B, const LowerMatrix<T> &A)
{
	/* Check for Incompadible Dimensions */
	int n = A.Dim();
	if(B.Dim() != n)
	{
		Print("Error in Copy(LowerMatrix,LowerMatrix)");
		Print("B = A");
		Print("Incompadible Dimensions");
		Print("B.Name",B.Name);
		Print("A.Name",A.Name);
		Print("B.Size()",B.Size());
		Print("A.Size()",A.Size());
		HaltExecution();
	}

	/* Preform Calculations */
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++) B(i,j) = A(i,j);
}

// Set the elements or Matrix A to the elements of Matrix B
#ifdef _GenMat_UpperMatrix
	template <class T>
	void Copy(UpperMatrix<T> &B, const UpperMatrix<T> &A)
	{
		/* Check for Incompadible Dimensions */
		int n = A.Dim();
		if(B.Dim() != n)
		{
			Print("Error in Copy(UpperMatrix,UpperMatrix)");
			Print("B = A");
			Print("Incompadible Dimensions");
			Print("B.Name",B.Name);
			Print("A.Name",A.Name);
			Print("B.Size()",B.Size());
			Print("A.Size()",A.Size());
			HaltExecution();
		}

		/* Preform Calculations */
		for(int i=0; i<n; i++) for(int j=i; j<n; j++) B(i,j) = A(i,j);
	}
#endif

// Set the elements or Matrix A to the elements of Matrix B
#ifdef _GenMat_BlockMatrix
	template <class T>
	void Copy(BlockMatrix<T> &B, const BlockMatrix<T> &A)
	{
		/* Check for Incompadible Dimensions */
		int m = B.Blocks();
		if(A.Blocks() != m)
		{
			Print("Error in Copy(BlockMatrix,BlockMatrix)");
			Print("B = A");
			Print("Incompadible dimensions");
			Print("B.Name",B.Name);
			Print("A.Name",A.Name);
			Print("B.Blocks()",B.Blocks());
			Print("A.Blocks()",A.Blocks());
			HaltExecution();
		}

		/* Preform Calculations */
		for(int b=0; b<m; b++)
		{
			int r = B.Block(b);
			if(A.Block(b) != r)
			{
				Print("Error in Copy(BlockMatrix,BlockMatrix)");
				Print("B = A");
				Print("Incompadible Dimensions");
				Print("B.Name",B.Name);
				Print("A.Name",A.Name);
				Print("b",b);
				Print("B.Block(b)",B.Block(b));
				Print("A.Block(b)",r);
				HaltExecution();
			}

			for(int i=0; i<r; i++) for(int j=0; j<r; j++) B(b,i,j) = A(b,i,j);
		}
	}
#endif

template <class T>
void Copy(Matrix<T> &B, const SymmetricMatrix<T> &A)
{
	/* Check for Incompadible Dimensions */
	int n = A.Dim();
	if(B.Rows() != n || B.Cols() != n)
	{
		Print("Error in Copy(Matrix,SymmetricMatrix)");
		Print("B = A");
		Print("Incompadible Dimensions");
		Print("B.Name",B.Name);
		Print("A.Name",A.Name);
		Print("B.Size()",B.Size());
		Print("A.Size()",A.Size());
		HaltExecution();
	}

	/* Preform Calculations */
	for(int i=0; i<n; i++) for(int j=0; j<n; j++) B(i,j) = A(i,j);
}

#ifdef _GenMat_BandMatrix
	template <class T>
	void Copy(Matrix<T> &B, const BandMatrix<T> &A)
	{
		/* Check for Incompadible Dimensions */
		int n = A.Dim();
		if(B.Rows() != n || B.Cols() != n)
		{
			Print("Error in Copy(Matrix,BandMatrix)");
			Print("B = A");
			Print("Incompadible Dimensions");
			Print("B.Name",B.Name);
			Print("A.Name",A.Name);
			Print("B.Size()",B.Size());
			Print("A.Size()",A.Size());
			HaltExecution();
		}

		/* Preform Calculations */
		for(int i=0; i<n; i++) for(int j=0; j<n; j++) B(i,j) = A(i,j);
	}
#endif

#ifdef _GenMat_SparseMatrix
	template <class T>
	void Copy(Matrix<T> &B, const SparseMatrix<T> &A)
	{
		/* Check for Incompadible Dimensions */
		int m = A.Rows();
		int n = A.Cols();
		if(B.Rows() != m || B.Cols() != n)
		{
			Print("Error in Copy(Matrix,SparseMatrix)");
			Print("B = A");
			Print("Incompadible Dimensions");
			Print("B.Name",B.Name);
			Print("A.Name",A.Name);
			Print("B.Size()",B.Size());
			Print("A.Size()",A.Size());
			HaltExecution();
		}

		/* Preform Calculations */
		for(int i=0; i<m; i++)
		{
			for(int j=0; j<n; j++) B(i,j) = Zero;
			for(int j=A.r_[i]; j<A.r_[i+1]; j++) B(i,A.Col(j)) = A.Val(j);
		}
	}
#endif

template <class T>
void Copy(SymmetricMatrix<T> &B, const Matrix<T> &A)
{
	/* Check for Incompadible Dimensions */
	int n = A.Rows();
	if(A.Cols() != n || B.Dim() != n)
	{
		Print("Error in Copy(SymmetricMatrix,Matrix)");
		Print("B = A");
		Print("Incompadible Dimensions");
		Print("B.Name",B.Name);
		Print("A.Name",A.Name);
		Print("B.Size()",B.Size());
		Print("A.Size()",A.Size());
		HaltExecution();
	}

	#ifdef _CheckSymmetric
		if(!IsSymmetric(A))
		{
			Print("Error in Copy(SymmetricMatrix,Matrix)");
			Print("B = A");
			Print("Resulting Matrix is Not Symmetric");
			Print("B.Name",B.Name);
			Print("A.Name",A.Name);
			double MaxDiff = Zero;
			for(int i=0; i<n; i++) for(int j=0; j<i; j++)
			{
				double Temp = abs(A(i,j) - A(j,i)) / Max(Min(abs(A(i,j)),abs(A(i,j))),One);
				if(Temp > MaxDiff) MaxDiff = Temp;
			}
			Print("MaxDiff",MaxDiff);
			Print("SymmetricTol",SymmetricTol);
			if(n <= 12) Print("A",A);
			HaltExecution();
		}
	#endif

	/* Preform Calculations */
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++) B(i,j) = A(i,j);
}

template <class T>
void Copy(SymmetricMatrix<T> &B, const Matrix<T> &A, double Tol)
{
	/* Check for incompadible dimensions */
	int n = A.Rows();
	if(A.Cols() != n || B.Dim() != n)
	{
		Print("Error in Copy(SymmetricMatrix,Matrix,Real)");
		Print("B = A");
		Print("Incompadible Dimensions");
		Print("B.Name",B.Name);
		Print("A.Name",A.Name);
		Print("B.Size()",B.Size());
		Print("A.Size()",A.Size());
		HaltExecution();
	}

	#ifdef _CheckSymmetric
		if(!IsSymmetric(A,Tol))
		{
			Print("Error in Copy(SymmetricMatrix,Matrix,Real)");
			Print("B = A");
			Print("Resulting Matrix is Not Symmetric");
			Print("B.Name",B.Name);
			Print("A.Name",A.Name);
			double MaxDiff = Zero;
			for(int i=0; i<n; i++) for(int j=0; j<i; j++)
			{
				double Temp = abs(A(i,j) - A(j,i)) / Max(Min(abs(A(i,j)),abs(A(i,j))),One);
				if(Temp > MaxDiff) MaxDiff = Temp;
			}
			Print("MaxDiff",MaxDiff);
			Print("Tol",Tol);
			if(n <= 12) Print("A",A);
			HaltExecution();
		}
	#endif

	/* Preform Calculations */
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++) B(i,j) = (A(i,j) + A(j,i)) / Two;
}

#ifdef _GenMat_SymmetricMatrixUP
	template <class T>
	void Copy(SymmetricMatrixUP<T> &B, const Matrix<T> &A)
	{
		/* Check for incompadible dimensions */
		int n = A.Rows();
		if(A.Cols() != n || B.Dim() != n)
		{
			Print("Error in Copy(SymmetricMatrixUP,Matrix)");
			Print("B = A");
			Print("Incompadible Dimensions");
			Print("B.Name",B.Name);
			Print("A.Name",A.Name);
			Print("B.Size()",B.Size());
			Print("A.Size()",A.Size());
			HaltExecution();
		}

		#ifdef _CheckSymmetric
			for(int i=0; i<n; i++) for(int j=0; j<i; j++) if(abs(A(i,j) - A(j,i)) > SymmetricTol)
			{
				Print("Error in Copy(SymmetricMatrixUP,Matrix)");
				Print("B = A");
				Print("Resulting matrix is not symmetric");
				Print("B.Name",B.Name);
				Print("A.Name",A.Name);
				Print("i",i);
				Print("j",j);
				Print("A(i,j)",A(i,j));
				Print("A(j,i)",A(j,i));
				Print("abs(A(i,j) - A(j,i))",abs(A(i,j) - A(j,i)));
				HaltExecution();
			}
		#endif

		/* Preform Calculations */
		for(int i=0; i<n; i++) for(int j=0; j<=i; j++) B(i,j) = A(i,j);
	}
#endif

template <class T>
void Copy(LowerMatrix<T> &B, const Matrix<T> &A)
{
	/* Check for incompadible dimensions */
	int n = A.Rows();
	if(A.Cols() != n || B.Dim() != n)
	{
		Print("Error in Copy(LowerMatrix,Matrix)");
		Print("B = A");
		Print("Incompadible Dimensions");
		Print("B.Name",B.Name);
		Print("A.Name",A.Name);
		Print("B.Size()",B.Size());
		Print("A.Size()",A.Size());
		HaltExecution();
	}

	#ifdef _CheckLower
		// Check if resulting matrix is lower-triangular
		for(int i=0; i<n; i++) for(int j=i+1; j<n; j++) if(abs(A(i,j)) > LowerTol)
		{
			Print("Error in Copy(Lower,Matrix)");
			Print("B = A");
			Print("Resulting matrix is not lower-triangular");
			Print("B.Name",B.Name);
			Print("A.Name",A.Name);
			Print("i",i);
			Print("j",j);
			Print("A(i,j)",A(i,j));
			HaltExecution();
		}
	#endif

	/* Preform Calculations */
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++) B(i,j) = A(i,j);
}

#ifdef _GenMat_LowerMatrixUN
	template <class T>
	void Copy(LowerMatrixUN<T> &B, const Matrix<T> &A)
	{
		/* Check for Incompadible Dimensions */
		int n = A.Rows();
		if(A.Cols() != n || B.Dim() != n)
		{
			Print("Error in Copy(LowerMatrixUN,Matrix)");
			Print("B = A");
			Print("Incompadible Dimensions");
			Print("B.Name",B.Name);
			Print("A.Name",A.Name);
			Print("B.Size()",B.Size());
			Print("A.Size()",A.Size());
			HaltExecution();
		}

		if(!IsLower(A))
		{
			Print("Error in Copy(LowerMatrixUN,Matrix)");
			Print("B = A");
			Print("B is not Lower");
			Print("B.Name",B.Name);
			Print("A.Name",A.Name);
			HaltExecution();
		}

		/* Preform Calculations */
		for(int i=0; i<n; i++) for(int j=0; j<=i; j++) B(i,j) = A(i,j);
	}
#endif

#ifdef _GenMat_UpperMatrixUN
	template <class T>
	void Copy(UpperMatrixUN<T> &B, const Matrix<T> &A)
	{
		/* Check for Incompadible Dimensions */
		int n = A.Rows();
		if(A.Cols() != n || B.Dim() != n)
		{
			Print("Error in Copy(UpperMatrixUN,Matrix)");
			Print("B = A");
			Print("Incompadible Dimensions");
			Print("B.Name",B.Name);
			Print("A.Name",A.Name);
			Print("B.Size()",B.Size());
			Print("A.Size()",A.Size());
			HaltExecution();
		}

		if(!IsUpper(A))
		{
			Print("Error in Copy(UpperMatrixUN,Matrix)");
			Print("B = A");
			Print("B is not Lower");
			Print("B.Name",B.Name);
			Print("A.Name",A.Name);
			HaltExecution();
		}

		/* Preform Calculations */
		for(int i=0; i<n; i++) for(int j=i; j<n; j++) B(i,j) = A(i,j);
	}
#endif

#ifdef _GenMat_BandMatrix
	template <class T>
	void Copy(BandMatrix<T> &B, const Matrix<T> &A)
	{
		/* Check dimensions */
		int n = A.Rows();
		if(A.Cols() != n || B.Dim() != n)
		{
			Print("Error in Copy(BandMatrix,Matrix)");
			Print("B = A");
			Print("Incompadible Dimensions");
			Print("B.Name",B.Name);
			Print("A.Name",A.Name);
			Print("B.Size()",B.Size());
			Print("A.Size()",A.Size());
			HaltExecution();
		}

		/* Preform Calculations */
		/* Note that this code is not efficient */
		for(int i=0; i<n; i++) for(int j=0; j<n; j++) B(i,j) = A(i,j);
	}
#endif

template <class T>
void Copy(Matrix<T> &B, const LowerMatrix<T> &A)
{
	/* Check for incompadible dimensions */
	int n = A.Dim();
	if(A.Rows() != n || B.Rows() != n)
	{
		Print("Error in Copy(Matrix,LowerMatrix)");
		Print("B = A");
		Print("Incompadible Dimensions");
		Print("B.Name",B.Name);
		Print("A.Name",A.Name);
		Print("B.Size()",B.Size());
		Print("A.Size()",A.Size());
		HaltExecution();
	}

	/* Preform Calculations */
	for(int i=0; i<n; i++) for(int j=0; j<n; j++) B(i,j) = A(i,j);
}

#ifdef _GenMat_LowerMatrixUN
	template <class T>
	void Copy(LowerMatrix<T> &B, const LowerMatrixUN<T> &A)
	{
		/* Check for incompadible dimensions */
		int n = A.Dim();
		if(B.Dim() != n)
		{
			Print("Error in Copy(LowerMatrix,LowerMatrixUN)");
			Print("B = A");
			Print("Incompadible Dimensions");
			Print("B.Name",B.Name);
			Print("A.Name",A.Name);
			Print("B.Size()",B.Size());
			Print("A.Size()",A.Size());
			HaltExecution();
		}

		/* Preform Calculations */
		for(int i=0; i<n; i++) for(int j=0; j<=i; j++) B(i,j) = A(i,j);
	}
#endif

#ifdef _GenMat_LowerMatrixUN
	template <class T>
	void Copy(Matrix<T> &B, const LowerMatrixUN<T> &A)
	{
		/* Check for incompadible dimensions */
		int n = A.Dim();
		if(B.Rows() != n || B.Cols() != n)
		{
			Print("Error in Copy(Matrix,LowerMatrixUN)");
			Print("B = A");
			Print("Incompadible Dimensions");
			Print("B.Name",B.Name);
			Print("A.Name",A.Name);
			Print("B.Size()",B.Size());
			Print("A.Size()",A.Size());
			HaltExecution();
		}

		/* Preform Calculations */
		Set(B,Zero);
		for(int i=0; i<n; i++) for(int j=0; j<=i; j++) B(i,j) = A(i,j);
	}
#endif

template <class T>
void Copy(Matrix<T> &B, const GeneralMatrix<T> &A)
{
	/* Check for Incompadible Dimensions */
	int m = A.Rows();
	int n = A.Cols();
	if(B.Rows() != m || B.Cols() != n)
	{
		Print("Error in Copy(Matrix,GeneralMatrix)");
		Print("B = A");
		Print("Incompadible Dimensions");
		Print("B.Name",B.Name);
		Print("A.Name",A.Name);
		Print("B.Size()",B.Size());
		Print("A.Size()",A.Size());
		HaltExecution();
	}

	/* Preform Calculations */
	for(int i=0; i<m; i++) for(int j=0; j<n; j++) B(i,j) = A(i,j);
}

#ifdef _GenMat_SymmetricMatrixLU
	template <class T>
	void Copy(SymmetricMatrixLU<T> &B, const Matrix<T> &A)
	{
		/* Check for incompadible dimensions */
		int n = A.Rows();
		if(A.Cols() != n || B.Dim() != n)
		{
			Print("Error in Copy(SymmetricMatrixLU,Matrix)");
			Print("B = A");
			Print("Incompadible Dimensions");
			Print("B.Name",B.Name);
			Print("A.Name",A.Name);
			Print("B.Size()",B.Size());
			Print("A.Size()",A.Size());
			HaltExecution();
		}

		#ifdef _CheckSymmetric
			for(int i=0; i<n; i++) for(int j=0; j<i; j++) if(abs(A(i,j) - A(j,i)) > SymmetricTol)
			{
				Print("Error in Copy(SymmetricMatrixLU,Matrix)");
				Print("B = A");
				Print("Resulting matrix is not symmetric");
				Print("B.Name",B.Name);
				Print("A.Name",A.Name);
				Print("i",i);
				Print("j",j);
				Print("A(i,j)",A(i,j));
				Print("A(j,i)",A(j,i));
				Print("abs(A(i,j) - A(j,i))",abs(A(i,j) - A(j,i)));
				HaltExecution();
			}
		#endif

		/* Preform Calculations */
		for(int i=0; i<n; i++) for(int j=0; j<=i; j++) B(i,j) = A(i,j);
	}
#endif

template <class T> void Copy(GeneralAssignableMatrix<T> &B, const GeneralMatrix<T> &A)
{
	/* Check for Incompadible dimensions */
	int m = A.Rows();
	int n = A.Cols();
	if(B.Rows() != m || B.Cols() != n)
	{
		Print("Error in Copy(GeneralMatrix,GeneralMatrix)");
		Print("B = A");
		Print("Incompadible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	/* Preform Calculations */
	for(int i=0; i<m; i++) for(int j=0; j<n; j++) B(i,j) = A(i,j);
}

#ifdef _GenMat_SparseMatrixCC
	cs* ToCSparseMatrix(const SparseMatrixCC<double> &A)
	{
		Print("A.c",A.c);
		int m = A.Rows();
		int n = A.Cols();
		int len = A.Length();
		cs* B = cs_spalloc(m,n,len,1,-1);
		Print("A.c",A.c);
		BlasCopy(n+1,A.c.Begin(),1,B->i,1);
		Print("A.c",A.c);
		BlasCopy(len,A.r.Begin(),1,B->p,1);
		Print("A.c",A.c);
		BlasCopy(len,A.v.Begin(),1,B->x,1);
		B->nz = -1; // Compresed Column Format
		Print("A.c",A.c);
		return B;
	}
#endif

#ifdef _GenMat_SparseMatrixCC
	SparseMatrixCC<double> ToSparseMatrixCC(cs *A)
	{
		if(A->nz != -1)
		{
			Print("Error in ToSparseMatrixCC(cs)");
			Print("A->nz",A->nz);
			HaltExecution();
		}

		int m = A->m;
		int n = A->n;
		int len = A->nzmax;
		Vector<int> r(len);
		Vector<int> c(n+1);
		Vector<double> v(len);
		for(int j=0; j<len; j++) r[j] = A->i[j];
		for(int j=0; j<n+1; j++) c[j] = A->p[j];
		for(int j=0; j<len; j++) v[j] = A->x[j];
		SparseMatrixCC<double> B(m,n,r,c,v);
		return B;
	}
#endif

#endif
