
#ifndef _Stat_Count
#define _Stat_Count



template <class T>
Vector<int> CountBy(const GeneralVector<T> &y)
{
	int N = y.Size();
	Vector<T> List = ListValues(y);
	int K = List.Size();	
	Vector<int> Ret(K,0);
	for(int n=0; n<N; n++)
	{
		int Curr = BinarySearch(List,y[n]);
		if(Curr < 0)
		{
			Print("Error in CountBy(y)");
			Print("Curr < 0");
			Print("Curr",Curr);
			HaltExecution();
		}
		Ret(Curr)++;
	}
	return Ret;
}

template <class T>
Vector<int> CountBy(const Vector< DataPoint<double> > &x, const Vector<T> &y)
{
	int N = x.Size();
	if(y.Size() != N)
	{
		Print("Error in CountBy(x,y)");
		Print("Incompadible Dimensions");
		Print("x.Name",x.Name);
		Print("y.Name",y.Name);
		Print("x.Size()",x.Size());
		Print("y.Size()",y.Size());
	}	
	
	Vector<T> List = ListValues(y);
	int K = List.Size();	
	Vector< Stack<double> > x_By_y(K);
	
	for(int n=0; n<N; n++)
	{
		if(!x(n).IsMissing())
		{
			int Curr = BinarySearch(List,y[n]);
			if(Curr < 0)
			{
				Print("Error in MeanBy(x,y)");
				Print("Curr < 0");
				Print("Curr",Curr);
				HaltExecution();
			}
			x_By_y[Curr].Push(x[n].Data);
		}
	}
	
	Vector<int> Ret(K);
	for(int k=0; k<K; k++) Ret[k] = x_By_y[k].Size();
	return Ret;
}

template <class T>
Vector<double> FreqBy(const GeneralVector<T> &y)
{
	int N = y.Size();
	Vector<T> List = ListValues(y);
	int K = List.Size();	
	Vector<double> Ret(K,Zero);
	for(int n=0; n<N; n++)
	{
		int Curr = BinarySearch(List,y[n]);
		if(Curr < 0)
		{
			Print("Error in FreqBy(y)");
			Print("Curr < 0");
			Print("Curr",Curr);
			HaltExecution();
		}
		Ret(Curr)++;
	}
	for(int k=0; k<K; k++) Ret(k) /= (double)N;
	return Ret;
}

template <class T>
Vector< Vector<double> > FreqBy(const Matrix<T> &X)
{
	int N = X.Rows();
	int K = X.Cols();
	Vector< Vector<double> > Ret(K);
	for(int k=0; k<K; k++)
	{
		Vector<T> Temp(N);
		for(int n=0; n<N; n++) Temp(n) = X(n,k); // Unnecesary Copying Step
		Vector<T> List = ListValues(Temp);
		int J = List.Size();
		Ret(k) = Vector<double>(J,Zero);
		for(int n=0; n<N; n++)
		{
			int Curr = BinarySearch(List,Temp(n));
			if(Curr < 0)
			{
				Print("Error in FreqBy(X)");
				Print("Curr < 0");
				Print("Curr",Curr);
				HaltExecution();
			}
			Ret(k)(Curr) += One;
		}
		for(int j=0; j<J; j++) Ret(k)(j) /= (double)N;
	}
	return Ret;
}

template <class T>
Vector< Vector<double> > FreqBy(const Matrix<T> &X, const Vector<double> &W)
{
	int N = X.Rows();
	int K = X.Cols();
	
	if(W.Size() != N)
	{
		Print("Error in FreqBy");
		Print("Incompadible Dimensions");
		Print("X.Size()",X.Size());
		Print("W.Size()",W.Size());
		HaltExecution();
	}
	
	Vector< Vector<double> > Ret(K);
	for(int k=0; k<K; k++)
	{
		Vector<T> Temp(N);
		for(int n=0; n<N; n++) Temp(n) = X(n,k); // Unnecesary Copying Step
		Vector<T> List = ListValues(Temp);
		int J = List.Size();
		Ret(k) = Vector<double>(J,Zero);
		for(int n=0; n<N; n++)
		{
			int Curr = BinarySearch(List,Temp(n));
			if(Curr < 0)
			{
				Print("Error in FreqBy(X)");
				Print("Curr < 0");
				Print("Curr",Curr);
				HaltExecution();
			}
			Ret(k)(Curr) += W(n);
		}
		for(int j=0; j<J; j++) Ret(k)(j) /= (double)N;
	}
	return Ret;
}

bool FreqCheck(const Vector<double> &Freq1)
{
	int K = Freq1.Size();	
	for(int k=0; k<K; k++) if(Freq1(k) < Zero) return True;
	if(Abs(Sum(Freq1) - One) > 0.000000000001) return True;
	return False;
}

int FreqCheck(const Vector< Vector<double> > &Freq1)
{
	int K = Freq1.Size();
	for(int k=0; k<K; k++) if(FreqCheck(Freq1(k))) return k + 1; // Return Index of Problem
	return 0;
}

#endif

