
#ifndef _Stat_DataPoint
#define _Stat_DataPoint

/* String Representation of Missing Value */
string MissingString = "";

template <class T>
class DataPoint
{

	public:

	/* Eventually, these should be private, and the "safe" operators,
	IsMissing, SetToMissing, and GetData, should be used */

	T Data;			// Data Value
	bool Missing;	// Missing Value?

	public:

	/* Default Constructor */
	DataPoint()
	{
		Missing = True;
	}

	/* Constructor 2: Set to Value a */
	DataPoint(const T &a)
	{
		Data = a;
		Missing = False;
	}

	/* Copy Constructor */
	DataPoint(const DataPoint &d)
	{
		Data    = d.Data;
		Missing = d.Missing;
	}

	~DataPoint() {}

	DataPoint Copy()
	{
		DataPoint d;
		d.Data = Data;
		d.Missing = Missing;
		return d;
	}

	const DataPoint& operator = (const DataPoint &d)
	{
		Data = d.Data;
		Missing = d.Missing;
	}

	const DataPoint& operator = (const T &a)
	{
		Data = a;
		Missing = False;
	}

	const bool IsMissing() const
	{
		return Missing;
	}

	void SetToMissing()
	{
		Missing = True;
	}

	const T GetData() const
	{
		if(Missing) Throw((string)"Error in DataPoint::GetData\nData Is Missing");
		return Data;
	}

	DataPoint<T> operator += (const T &y) { if(!Missing) Data += y; }
	DataPoint<T> operator -= (const T &y) { if(!Missing) Data -= y; }
	DataPoint<T> operator *= (const T &y) { if(!Missing) Data *= y; }
	DataPoint<T> operator /= (const T &y) { if(!Missing) Data /= y; }

	DataPoint<T> operator += (const DataPoint<T> &y)
	{
		if(!Missing)
		{
			if(y.Missing) Missing = True;
			else          Data += y.Data;
		}
	}

	DataPoint<T> operator -= (const DataPoint<T> &y)
	{
		if(!Missing)
		{
			if(y.Missing) Missing = True;
			else          Data -= y.Data;
		}
	}

	DataPoint<T> operator *= (const DataPoint<T> &y)
	{
		if(!Missing)
		{
			if(y.Missing) Missing = True;
			else          Data *= y.Data;
		}
	}

	DataPoint<T> operator /= (const DataPoint<T> &y)
	{
		if(!Missing)
		{
			if(y.Missing) Missing = True;
			else          Data /= y.Data;
		}
	}

	const DataPoint<T> & operator ++ ()
	{
		if(!Missing) Data++;
	}

	DataPoint<T> operator ++ (int)
	{
		if(!Missing) Data++;
	}

	const DataPoint<T> & operator -- ()
	{
		if(!Missing) Data--;
	}

	DataPoint<T> operator -- (int)
	{
		if(!Missing) Data--;
	}
};

/* Overload the cout operator for DataVector objects */
template <class T>
ostream & operator << (ostream &os, const DataPoint<T> &d)
{

	if(d.Missing) os << MissingString;
	else          os << d.Data;
   	return os;
}

template <class T>
bool operator == (const DataPoint<T> &a, const DataPoint<T> &b)
{
	     if(a.Missing && b.Missing)   return True;
	else if(!a.Missing && !b.Missing) return (a.Data == b.Data);
	else                              return False;
}

template <class T>
bool operator == (const DataPoint<T> &a, const T &b)
{
	if(a.IsMissing()) return False;
	else              return (a.Data == b);
}

template <class T>
bool operator == (const T &a, const DataPoint<T> &b)
{
	if(b.IsMissing()) return False;
	else              return (a == b.Data);
}

template <class T>
bool operator != (const DataPoint<T> &a, const DataPoint<T> &b)
{
	     if(a.Missing && b.Missing)   return False;
	else if(!a.Missing && !b.Missing) return (a.Data != b.Data);
	else                              return True;
}

template <class T>
bool operator != (const DataPoint<T> &a, const T &b)
{
	if(a.Missing) return False;
	else          return (a.Data != b);
}

template <class T>
bool operator != (const T &a, const DataPoint<T> &b)
{
	if(b.Missing) return False;
	else          return (a != b.Data);
}

/*
template <class T>
DataPoint<bool> operator <= (const DataPoint<T> &a, const DataPoint<T> &b)
{
	if(a.Missing)
	{
		if(b.Missing) return True;
		else          return DataPoint<bool>();
	}
	else
	{
		if(b.Missing) return DataPoint<bool>();
		else          return (a.Data <= b.Data);
	}
}

template <class T>
DataPoint<bool> operator <= (const DataPoint<T> &a, const T &b)
{
	if(a.Missing) return DataPoint<bool>();
	else          return (a.Data <= b);
}

template <class T>
DataPoint<bool> operator >= (const DataPoint<T> &a, const T &b)
{
	if(a.Missing) return DataPoint<bool>();
	else          return (a.Data >= b);
}

template <class T>
DataPoint<bool> operator > (const DataPoint<T> &a, const DataPoint<T> &b)
{
	if(a.Missing)
	{
		if(b.Missing) return False;
		else          return DataPoint<bool>();
	}
	else
	{
		if(b.Missing) return DataPoint<bool>();
		else          return (a.Data > b.Data);
	}
}

template <class T>
DataPoint<bool> operator > (const DataPoint<T> &a, const T &b)
{
	if(a.Missing) return DataPoint<bool>();
	else          return (a.Data > b);
}
*/

template <class T>
DataPoint<bool> operator < (const DataPoint<T> &a, const DataPoint<T> &b)
{
	if(b.Missing)
	{
		if(a.Missing) return False;
		else          return DataPoint<bool>();
	}
	else
	{
		if(a.Missing) return DataPoint<bool>();
		else          return (a.Data < b.Data);
	}
}

template <class T>
DataPoint<bool> operator < (const DataPoint<T> &a, const T &b)
{
	if(a.Missing) return DataPoint<bool>();
	else          return (a.Data < b);
}

template <class T>
DataPoint<T> operator + (const DataPoint<T> &x, const DataPoint<T> &y)
{
	if(x.IsMissing() || y.IsMissing())
	{
		DataPoint<T> z;	// Missing Data Point By Default
		return z;
	}
	else
	{
		DataPoint<T> z(x.Data + y.Data);
		return z;
	}
}

template <class T>
DataPoint<T> operator + (const T &x, const DataPoint<T> &y)
{
	if(y.IsMissing())
	{
		DataPoint<T> z;	// Missing Data Point By Default
		return z;
	}
	else
	{
		DataPoint<T> z(x + y.Data);
		return z;
	}
}

template <class T>
DataPoint<T> operator + (const DataPoint<T> &x, const T &y)
{
	if(x.IsMissing())
	{
		DataPoint<T> z;	// Missing Data Point By Default
		return z;
	}
	else
	{
		DataPoint<T> z(x.Data + y);
		return z;
	}
}

template <class T>
DataPoint<T> operator - (const DataPoint<T> &x)
{
	if(x.IsMissing())
	{
		return x;
	}
	else
	{
		DataPoint<T> z(-x.Data);
		return z;
	}
}

template <class T>
DataPoint<T> operator - (const DataPoint<T> &x, const DataPoint<T> &y)
{
	if(x.IsMissing() || y.IsMissing())
	{
		DataPoint<T> z;	// Missing Data Point By Default
		return z;
	}
	else
	{
		DataPoint<T> z(x.Data - y.Data);
		return z;
	}
}

template <class T>
DataPoint<T> operator - (const T &x, const DataPoint<T> &y)
{
	if(y.IsMissing())
	{
		DataPoint<T> z;	// Missing Data Point By Default
		return z;
	}
	else
	{
		DataPoint<T> z(x - y.Data);
		return z;
	}
}

template <class T>
DataPoint<T> operator - (const DataPoint<T> &x, const T &y)
{
	if(x.IsMissing())
	{
		DataPoint<T> z;	// Missing Data Point By Default
		return z;
	}
	else
	{
		DataPoint<T> z(x.Data - y);
		return z;
	}
}

template <class T>
DataPoint<T> operator * (const DataPoint<T> &x, const DataPoint<T> &y)
{
	if(x.IsMissing() || y.IsMissing()) return DataPoint<T>(); // Missing Data Point By Default
	else							   return DataPoint<T>(x.Data * y.Data);
}

template <class T>
DataPoint<T> operator * (const T &x, const DataPoint<T> &y)
{
	if(y.IsMissing()) return DataPoint<T>(); // Missing Data Point By Default
	else              return DataPoint<T>(x * y.Data);
}

template <class T>
DataPoint<T> operator * (const DataPoint<T> &x, const T &y)
{
	if(x.IsMissing()) return DataPoint<T>(); // Missing Data Point By Default
	else              return DataPoint<T>(x.Data * y);
}

template <class T>
DataPoint<T> operator / (const DataPoint<T> &x, const T &y)
{
	if(x.IsMissing())
	{
		DataPoint<T> z;	// Missing Data Point By Default
		return z;
	}
	else
	{
		DataPoint<T> z(x.Data / y);
		return z;
	}
}

template <class T>
DataPoint<T> operator / (const T &x, const DataPoint<T> &y)
{
	if(y.IsMissing())
	{
		return DataPoint<T>(); // Missing
	}
	else
	{
		if(y.Data == Zero) return DataPoint<T>(); // Missing (can't divide by zero)
		DataPoint<T> z(x / y.Data);
		return z;
	}
}

template <class T>
DataPoint<T> operator / (const DataPoint<T> &x, const DataPoint<T> &y)
{
	if(x.IsMissing() || y.IsMissing())
	{
		return DataPoint<T>(); // Missing
	}
	else
	{
		if(y.Data == Zero) return DataPoint<T>(); // Missing (can't divide by zero)
		DataPoint<T> z(x.Data / y.Data);
		return z;
	}
}

DataPoint<double> ToDataPointDouble(const DataPoint<int> &x);
DataPoint<double> ToDataPointDouble(const DataPoint<int> &x)
{
	if(x.IsMissing())
	{
		DataPoint<double> y;
		return y;
	}
	else
	{
		DataPoint<double> y;
		y = (double)x.Data;
		return y;
	}
}

DataPoint<double> ToDataPointDouble(const string &s);
DataPoint<double> ToDataPointDouble(const string &s)
{
	if(s == "")
	{
		return DataPoint<double>();
	}
	else if(s == ".")
	{
		return DataPoint<double>();
	}
	else
	{
		double x = ToDouble(s);
		return DataPoint<double>(x);
	}
}

DataPoint<double> Log(const DataPoint<double> &x);
DataPoint<double> Log(const DataPoint<double> &x)
{
	if(x.IsMissing())
	{
		DataPoint<double> y;
		return y;
	}
	else if(x.Data <= Zero)
	{
		DataPoint<double> y;
		return y;
	}
	else
	{
		DataPoint<double> y;
		y = Log(x.Data);
		return y;
	}
}

DataPoint<double> Exp(const DataPoint<double> &x);
DataPoint<double> Exp(const DataPoint<double> &x)
{
	if(x.IsMissing())
	{
		DataPoint<double> y;
		return y;
	}
	else
	{
		DataPoint<double> y;
		y = exp(x.Data);
		return y;
	}
}

DataPoint<double> Sqrt(const DataPoint<double> &x);
DataPoint<double> Sqrt(const DataPoint<double> &x)
{
	if(x.IsMissing())
	{
		DataPoint<double> y;
		return y;
	}
	else if(x.Data < Zero)
	{
		DataPoint<double> y;
		return y;
	}
	else
	{
		DataPoint<double> y;
		y = sqrt(x.Data);
		return y;
	}
}

inline DataPoint<double> Abs(const DataPoint<double> &x)
{
	DataPoint<double> y;
	if(!x.IsMissing()) y = fabs(x.GetData());
	return y;
}

/* Convert a string to a DataPoint<int> */
bool Convert(DataPoint<int> &x, const string &s)
{
	if(s == "" || s == "." || s == " " || s == "N/A" || s == "NA")
	{
		x.SetToMissing();
		return 0;
	}

	std::istringstream i(s);
	int x1;
	if(!(i >> x1))
	{
		x = 0;	// Set x to default value (0)
		return 1;	// Return error code
	}
	x = x1;
	return 0;		// Return no error code
}

/* Convert a string to a double */
bool Convert(DataPoint<int> &x, char ch)
{
	string s = "" + ch;
	if(s == "" || s == "." || s == " " || s == "N/A" || s == "NA")
	{
		x.SetToMissing();
		return 0;
	}

	std::istringstream i(s);
	int x1;
	if(!(i >> x1))
	{
		x = 0;	// Set x to default value (0)
		return 1;	// Return error code
	}
	x = x1;
	return 0;		// Return no error code
}

/* Convert a string to a DataPoint<int> */
bool Convert(DataPoint<float> &x, const string &s)
{
	if(s == "" || s == "." || s == " " || s == "N/A" || s == "NA")
	{
		x.SetToMissing();
		return 0;
	}

	std::istringstream i(s);
	int x1;
	if(!(i >> x1))
	{
		x = 0;	// Set x to default value (0)
		return 1;	// Return error code
	}
	x = x1;
	return 0;		// Return no error code
}

/* Convert a string to a double */
bool Convert(DataPoint<double> &x, const string &s)
{
	//Print("Copnf",s);
	if(s == "" || s == "." || s == " " || s == "N/A" || s == "NA")
	{
		x.SetToMissing();
		return 0;
	}

	std::istringstream i(s);
	double x1;
	if(!(i >> x1))
	{
		x = Zero;	// Set x to default value (0.0)
		return 1;	// Return error code
	}
	x = x1;
	return 0;		// Return no error code
}

/* Convert a string to a double */
bool Convert(DataPoint<string> &x, const string &s)
{
	x = s;
	return 0;		// Return no error code
}

/* Convert a string to a double */
bool Convert(DataPoint<string> &x, char ch)
{
	string s = "" + ch;
	x = s;
	return 0;		// Return no error code
}

template<class T>
DataPoint<T> ToDataPoint(const string &s)
{
	DataPoint<T> x;
	bool Info = Convert(x,s);
	if(Info) Throw("Error in ToDataPoint<T>");
	return x;
}

template <class T>
bool Convert(T &y, const DataPoint<T> &x)
{
	if(x.IsMissing()) return True;
	y = x.GetData();
	return False; // No Error
}

template <class T>
bool Convert(DataPoint<T> &y, const T &x)
{
	y = x;
	return False; // No Error
}

bool Convert(DataPoint<double> &y, const int x)
{
	y = (double)x;
	return False; // No Error
}

bool Convert(DataPoint<double> &y, const DataPoint<int> x)
{
	if(!x.IsMissing()) y = (double)(x.GetData());
	else               y.SetToMissing();
	return False; // No Error
}

template <class T>
bool Convert(string &s, const T &x)
{
	std::ostringstream StreamOut;
    StreamOut << x;
	s = StreamOut.str();
	return False;
}

/*
Vector<double> ToVector(const Vector< DataPoint<double> > &x);
Vector<double> ToVector(const Vector< DataPoint<double> > &x)
{
	int n = x.Size();
	Vector<double> y(n);
	for(int i=0; i<n; i++)
	{
		if(x(i).IsMissing())
		{
			Print("Error in ToVector(Vector<DataPoint>)");
			Print("Missing Observation");
			HaltExecution();
		}
		y(i) = x(i).GetData();
	}
	return y;
}
*/

template <class T>
Vector< DataPoint<T> > ToDataPointVector(const GeneralVector<T> &x)
{
	int n = x.Size();
	Vector< DataPoint<T> > y(n);
	for(int i=0; i<n; i++) y(i) = x(i);
	return y;
}

template <class T>
Matrix< DataPoint<T> > ToDataPointMatrix(const Matrix<T> &A)
{
	int m = A.Rows();
	int n = A.Cols();
	Matrix< DataPoint<T> > B(m,n);
	for(int i=0; i<m; i++) for(int j=0; j<n; j++) B(i,j) = A(i,j);
	return B;
}

template <class T>
SymmetricMatrix< DataPoint<T> > ToDataPointSymmetricMatrix(const SymmetricMatrix<T> &A)
{
	int n = A.Dim();
	SymmetricMatrix< DataPoint<T> > B(n);
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++) B(i,j) = A(i,j);
	return B;
}

template <class T>
SymmetricMatrix<T> ToSymmetricMatrix(const SymmetricMatrix< DataPoint<T> > &A)
{
	int n = A.Dim();
	SymmetricMatrix<T> B(n);
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++) B(i,j) = A(i,j).GetData();
	return B;
}

template <class T>
DataPoint<T> exp(const DataPoint<T> &x)
{
	if(x.Missing) return DataPoint<T>();
	else          return DataPoint<T>(exp(x.Data));
}

template <class T>
DataPoint<T> NormalCdf(const DataPoint<T> &x)
{
	if(x.Missing) return DataPoint<T>();
	else          return DataPoint<T>(NormalCdf(x.Data));
}

// Overload the cout operator for vectors
template <class T>
istream & operator >> (istream &is, DataPoint<T> &x)
{
	string s;
	is >> s;
	bool Info = Convert(x,s);
	if(Info)
	{
		Print("Error in >>(DataPoint)");
		HaltExecution();
	}
	return is;
}

template <class T>
bool LessThan(const DataPoint<T> &a, const DataPoint<T> &b)
//bool LessThan(const DataPoint<double> &a, const DataPoint<double> &b)
{
	if(b.Missing)
	{
		if(a.Missing) return False;
		else          return True;
	}
	else
	{
		if(a.Missing) return False;
		else          return (a.Data < b.Data);
	}
}

/*
bool LessThan(short a, short b)
{
	return (a < b);
}
*/

template <class T>
bool LessThan(const T &a, const T &b)
{
	return (a < b);
}

#endif
