
#ifndef _Data_Date
#define _Data_Date

/*

Utilities for Dates

Date Object
Convert string to Date Object

*/

class Date
{

	public:
	
	int Julian;
	
	private:
	
	void Convert(int &Julian, int Month, int Day, int Year)
	{
		/* Use NR in C to Covert */
		const int IGREG = 15 + 31 * (10 + 12 * 1582);
		int ja,jul,jy=Year,jm;
		if(jy == 0)
		{
			Print("Error in Date");
			Print("There is No Year Zero");
			HaltExecution();
		}
		if(jy < 0) ++jy;
		if(Month > 2)
		{
			jm = Month + 1;
		}
		else
		{
			--jy;
			jm = Month + 13;
		}
		Julian = int(floor(365.25 * jy) + floor(30.6001 * jm) + Day + 1720995);
		if(Day + 31 * (Month + 12 * Year) >= IGREG)
		{
			ja = int(0.01 * jy);
			Julian += 2 - ja + int(0.25 * ja);
		}
	}
	
	public:
	
	Date()
	{
		Julian = -1; // Missing Date
	}
	
	Date(int Julian)
	{
		this->Julian = (int)(Julian);
	}
	
	Date(int Month, int Day, int Year)
	{
		int Julian1;
		Convert(Julian1,Month,Day,Year);
		Julian = (int)(Julian1);
	}
	
	bool IsMissing()
	{
		return (Julian < 0);
	}
	
	int GetYear()
	{
		if(Julian < 0)
		{
			Print("Error in GetYear::Missing Date");
			HaltExecution();
		}
	
		
		/* Use NR in C to Covert */
		const int IGREG = 2299161;
		int ja,jalpha,jb,jc,jd,je;
		if(Julian >= IGREG)
		{
			jalpha = int((double(Julian - 1867216) - 0.25) / 36524.25);
			ja = Julian + 1 + jalpha - int(0.25 * jalpha);
		}
		else if(Julian < 0)
		{
			ja = Julian + 36525 * (1 - Julian / 36525);
		}
		else
		{
			ja = Julian;
		}
		jb = ja + 1524;
		jc = int(6680.0 + (double(jb - 2439870) - 122.1) / 365.25);
		jd = int(365 * jc + (0.25 * jc));
		je = int((jb - jd) / 30.6001);
		int Day = jb - jd - int(30.6001 * je);
		int Month = je - 1;
		if(Month > 12) Month -= 12;
		int Year = jc - 4715;
		if(Month > 2) --Year;
		if(Year <= 0) --Year;
		if(Julian < 0) Year -= 100 * (1 - Julian / 36525);
		return Year;
	}
	
	int GetMonth()
	{
		if(Julian < 0)
		{
			Print("Error in GetYear::Missing Date");
			HaltExecution();
		}
	
		
		/* Use NR in C to Covert */
		const int IGREG = 2299161;
		int ja,jalpha,jb,jc,jd,je;
		if(Julian >= IGREG)
		{
			jalpha = int((double(Julian - 1867216) - 0.25) / 36524.25);
			ja = Julian + 1 + jalpha - int(0.25 * jalpha);
		}
		else if(Julian < 0)
		{
			ja = Julian + 36525 * (1 - Julian / 36525);
		}
		else
		{
			ja = Julian;
		}
		jb = ja + 1524;
		jc = int(6680.0 + (double(jb - 2439870) - 122.1) / 365.25);
		jd = int(365 * jc + (0.25 * jc));
		je = int((jb - jd) / 30.6001);
		int Day = jb - jd - int(30.6001 * je);
		int Month = je - 1;
		if(Month > 12) Month -= 12;
		int Year = jc - 4715;
		if(Month > 2) --Year;
		if(Year <= 0) --Year;
		if(Julian < 0) Year -= 100 * (1 - Julian / 36525);
		return Month;
	}
	
	int GetDay()
	{
		if(Julian < 0)
		{
			Print("Error in GetYear::Missing Date");
			HaltExecution();
		}
	
		
		/* Use NR in C to Covert */
		const int IGREG = 2299161;
		int ja,jalpha,jb,jc,jd,je;
		if(Julian >= IGREG)
		{
			jalpha = int((double(Julian - 1867216) - 0.25) / 36524.25);
			ja = Julian + 1 + jalpha - int(0.25 * jalpha);
		}
		else if(Julian < 0)
		{
			ja = Julian + 36525 * (1 - Julian / 36525);
		}
		else
		{
			ja = Julian;
		}
		jb = ja + 1524;
		jc = int(6680.0 + (double(jb - 2439870) - 122.1) / 365.25);
		jd = int(365 * jc + (0.25 * jc));
		je = int((jb - jd) / 30.6001);
		int Day = jb - jd - int(30.6001 * je);
		int Month = je - 1;
		if(Month > 12) Month -= 12;
		int Year = jc - 4715;
		if(Month > 2) --Year;
		if(Year <= 0) --Year;
		if(Julian < 0) Year -= 100 * (1 - Julian / 36525);
		return Day;
	}
};

/* Overload output stream */
ostream & operator << (ostream &os, const Date &d);

/* Number of Days Between Two Dates */
int operator - (const Date &d1, const Date &d2);
	
/* Compare Two Dates */
bool operator <  (const Date &d1, const Date &d2);
bool operator >  (const Date &d1, const Date &d2);
bool operator <= (const Date &d1, const Date &d2);
bool operator >= (const Date &d1, const Date &d2);
bool operator == (const Date &d1, const Date &d2);

/* Overload output stream */
ostream & operator << (ostream &os, const Date &d)
{

	/* Missing Date */
	int Julian1 = (int)(d.Julian);
	if(Julian1 < 0)
	{
		os << ".";
		return os;
	}

	/* Use NR in C to Covert */
	const int IGREG = 2299161;
	int ja,jalpha,jb,jc,jd,je;
	if(Julian1 >= IGREG)
	{
		jalpha = int((double(Julian1 - 1867216) - 0.25) / 36524.25);
		ja = Julian1 + 1 + jalpha - int(0.25 * jalpha);
	}
	else if(Julian1 < 0)
	{
		ja = Julian1 + 36525 * (1 - Julian1 / 36525);
	}
	else
	{
		ja = Julian1;
	}
	jb = ja + 1524;
	jc = int(6680.0 + (double(jb - 2439870) - 122.1) / 365.25);
	jd = int(365 * jc + (0.25 * jc));
	je = int((jb - jd) / 30.6001);
	int Day = jb - jd - int(30.6001 * je);
	int Month = je - 1;
	if(Month > 12) Month -= 12;
	int Year = jc - 4715;
	if(Month > 2) --Year;
	if(Year <= 0) --Year;
	if(Julian1 < 0) Year -= 100 * (1 - Julian1 / 36525);

	os << Month << "/" << Day << "/" << Year;
	return os;
}

/* Number of Days Between Two Dates */
int operator - (const Date &d1, const Date &d2)
{
	if(d1.Julian < 0 || d2.Julian < 0) Throw("Error in Date - Date\nMissing Date");
	return d1.Julian - d2.Julian;
}

/*
Date ParseDate1(string s)
{
	s = Trim(s);
	Stack<string> s2 = ParseBy(s,' ');
	
	int Month;
	     if(s2(0) == "January"  ) Month =  1;
	else if(s2(0) == "February" ) Month =  2;
	else if(s2(0) == "March"    ) Month =  3;
	else if(s2(0) == "April"    ) Month =  4;
	else if(s2(0) == "May"      ) Month =  5;
	else if(s2(0) == "June"     ) Month =  6;
	else if(s2(0) == "July"     ) Month =  7;
	else if(s2(0) == "August"   ) Month =  8;
	else if(s2(0) == "September") Month =  9;
	else if(s2(0) == "October"  ) Month = 10;
	else if(s2(0) == "November" ) Month = 11;
	else if(s2(0) == "December" ) Month = 12;
	else
	{
		Throw("Error in ParseDate1");
	}
	int Day = ToInteger(DeleteFirst(s2(1),','));
	int Year = ToInteger(s2(2));
	return Date(Month,Day,Year);
}

Date ParseDate2(string s)
{
	s = Trim(s);
	Stack<string> s2 = ParseBy(s,'-');
	
	int Month;
	     if(s2(0) == "Jan") Month =  1;
	else if(s2(0) == "Feb") Month =  2;
	else if(s2(0) == "Mar") Month =  3;
	else if(s2(0) == "Apr") Month =  4;
	else if(s2(0) == "May") Month =  5;
	else if(s2(0) == "Jun") Month =  6;
	else if(s2(0) == "Jul") Month =  7;
	else if(s2(0) == "Aug") Month =  8;
	else if(s2(0) == "Sep") Month =  9;
	else if(s2(0) == "Oct") Month = 10;
	else if(s2(0) == "Nov") Month = 11;
	else if(s2(0) == "Dec") Month = 12;
	else
	{
		Throw("Error in ParseDate2:\nUnreconnized Date: " + s);
	}
	int Day = ToInteger(s2(0));
	int Year = ToInteger(s2(2));
	if(Year <= 99) Year += 2000;
	return Date(Month,Day,Year);
}

Date ParseDate3(string s)
{
	s = Trim(s);
	Stack<string> s2 = ParseBy(s,' ');
	
	int Month;
	     if(s2(0) == "January,"  ) Month =  1;
	else if(s2(0) == "February," ) Month =  2;
	else if(s2(0) == "March,"    ) Month =  3;
	else if(s2(0) == "April,"    ) Month =  4;
	else if(s2(0) == "May,"      ) Month =  5;
	else if(s2(0) == "June,"     ) Month =  6;
	else if(s2(0) == "July,"     ) Month =  7;
	else if(s2(0) == "August,"   ) Month =  8;
	else if(s2(0) == "September,") Month =  9;
	else if(s2(0) == "October,"  ) Month = 10;
	else if(s2(0) == "November," ) Month = 11;
	else if(s2(0) == "December," ) Month = 12;
	else
	{
		Throw("Error in ParseDate3:\nUnreconnized Date: " + s);
	}
	int Year = ToInteger(s2(1));
	return Date(Month,15,Year);
}
*/

bool operator < (const Date &d1, const Date &d2)
{
	return (d1.Julian < d2.Julian);
}

bool operator > (const Date &d1, const Date &d2)
{
	return (d1.Julian > d2.Julian);
}

bool operator <= (const Date &d1, const Date &d2)
{
	return (d1.Julian <= d2.Julian);
}

bool operator >= (const Date &d1, const Date &d2)
{
	return (d1.Julian >= d2.Julian);
}

bool operator == (const Date &d1, const Date &d2)
{
	return (d1.Julian == d2.Julian);
}

/* Convert a string to a Date */
Date ToDate(string s)
{

	/* Check for Missing Date */
	if(s == "")
	{
		Date d(-1);
		return d;
	}

	/* Format 1: mm/dd/yyyy */
	Stack<string> s1 = ParseBy(s,"/");
	if(s1.Size() == 3)
	{
		Date d(ToInteger(s1(0)),ToInteger(s1(1)),ToInteger(s1(2)));
		return d;
	}	
	
	/* Format 2: d-m-y */
	Stack<string> s2 = ParseBy(s,"-");
	if(s2.Size() == 3)
	{
		int Month;
		     if(s2(1) == "Jan") Month =  1;
		else if(s2(1) == "Feb") Month =  2;
		else if(s2(1) == "Mar") Month =  3;
		else if(s2(1) == "Apr") Month =  4;
		else if(s2(1) == "May") Month =  5;
		else if(s2(1) == "Jun") Month =  6;
		else if(s2(1) == "Jul") Month =  7;
		else if(s2(1) == "Aug") Month =  8;
		else if(s2(1) == "Sep") Month =  9;
		else if(s2(1) == "Oct") Month = 10;
		else if(s2(1) == "Nov") Month = 11;
		else if(s2(1) == "Dec") Month = 12;
		else
		{
			Throw("Error in ToDate\nUnrecognized Format (dd-mmm-yy)\nDate: " + s);
		}
		try
		{
			int Day = ToInteger(s2(0));
			int Year = ToInteger(s2(2));
			if(Year <= 99) Year += 2000;
			return Date(Month,Day,Year);
		}
		catch(int e)
		{
			Throw("Error in ToDate\nUnrecognized Format (dd-mmm-yy)\nDate: " + s);
		}
	}
	
	/* m* dd, yyyy */
	Stack<string> s3 = ParseBy(Trim(DeleteAll(s,',')),' ');
	if(s3.Size() == 3)
	{
		int Month;
		     if(s3(0) == "January"   || s3(0) == "Jan." ) Month =  1;
		else if(s3(0) == "February"  || s3(0) == "Feb." ) Month =  2;
		else if(s3(0) == "March"     || s3(0) == "Mar." ) Month =  3;
		else if(s3(0) == "April"     || s3(0) == "Apr." ) Month =  4;
		else if(s3(0) == "May"                          ) Month =  5;
		else if(s3(0) == "June"      || s3(0) == "Jun." ) Month =  6;
		else if(s3(0) == "July"      || s3(0) == "Jul." ) Month =  7;
		else if(s3(0) == "August"    || s3(0) == "Aug." ) Month =  8;
		else if(s3(0) == "September" || s3(0) == "Sept.") Month =  9;
		else if(s3(0) == "October"   || s3(0) == "Oct." ) Month = 10;
		else if(s3(0) == "November"  || s3(0) == "Nov." ) Month = 11;
		else if(s3(0) == "December"  || s3(0) == "Dec." ) Month = 12;
		else
		{
			/* The other order */
			     if(s3(1) == "January"   || s3(1) == "Jan." ) Month =  1;
			else if(s3(1) == "February"  || s3(1) == "Feb." ) Month =  2;
			else if(s3(1) == "March"     || s3(1) == "Mar." ) Month =  3;
			else if(s3(1) == "April"     || s3(1) == "Apr." ) Month =  4;
			else if(s3(1) == "May"                          ) Month =  5;
			else if(s3(1) == "June"      || s3(1) == "Jun." ) Month =  6;
			else if(s3(1) == "July"      || s3(1) == "Jul." ) Month =  7;
			else if(s3(1) == "August"    || s3(1) == "Aug." ) Month =  8;
			else if(s3(1) == "September" || s3(1) == "Sept.") Month =  9;
			else if(s3(1) == "October"   || s3(1) == "Oct." ) Month = 10;
			else if(s3(1) == "November"  || s3(1) == "Nov." ) Month = 11;
			else if(s3(1) == "December"  || s3(1) == "Dec." ) Month = 12;
			else
			{
				Throw("Error in ToDate\nUnrecognized Format (m* dd, yyyy)\nDate: " + s);
			}
			try
			{
				int Year = ToInteger(s3(2));
				int Day = ToInteger(s3(0));
				return Date(Month,Day,Year);
			}
			catch(int e)
			{
				Throw("Error in ToDate (1)\nUnrecognized Format (m* dd, yyyy)\nDate: " + s);
			}
		}
		try
		{
			int Year = ToInteger(s3(2));
			int Day = ToInteger(s3(1));
			return Date(Month,Day,Year);
		}
		catch(int e)
		{
			Throw("Error in ToDate (2)\nUnrecognized Format (m* dd, yyyy)\nDate: " + s);
		}
	}
	else if(s3.Size() == 2)
	{
		int Month;
		     if(s3(0) == "January"   || s3(0) == "Jan." ) Month =  1;
		else if(s3(0) == "February"  || s3(0) == "Feb." ) Month =  2;
		else if(s3(0) == "March"     || s3(0) == "Mar." ) Month =  3;
		else if(s3(0) == "April"     || s3(0) == "Apr." ) Month =  4;
		else if(s3(0) == "May"                          ) Month =  5;
		else if(s3(0) == "June"      || s3(0) == "Jun." ) Month =  6;
		else if(s3(0) == "July"      || s3(0) == "Jul." ) Month =  7;
		else if(s3(0) == "August"    || s3(0) == "Aug." ) Month =  8;
		else if(s3(0) == "September" || s3(0) == "Sept.") Month =  9;
		else if(s3(0) == "October"   || s3(0) == "Oct." ) Month = 10;
		else if(s3(0) == "November"  || s3(0) == "Nov." ) Month = 11;
		else if(s3(0) == "December"  || s3(0) == "Dec." ) Month = 12;
		else
		{
			Throw("Error in ToDate\nUnrecognized Format (m*, yyyy)\nDate: " + s);
		}
		try
		{
			int Year = ToInteger(s3(1));
			return Date(Month,15,Year);
		}
		catch(int e)
		{
			Throw("Error in ToDate\nUnrecognized Format (m*, yyyy)\nDate: " + s);
		}	
	}
	else
	{
		try
		{
			int Year = ToInteger(s3(0));
			return Date(6,15,Year);
		}
		catch(int e)
		{
			Throw("Error in ToDate\nUnrecognized Format (yyyy)\nDate: " + s);
		}
	}
		
	Throw("Error in ToDate\nUnrecognized Format\nDate: " + s);
}

bool Convert(Date &d, const string &s)
{
	try
	{
		d = ToDate(s);
		return False;
	}
	catch(int e)
	{
		return True;	
	}
}

#endif