
#ifndef _Opt_Deriv1D
#define _Opt_Deriv1D


/*

Utility for computing numerical derivatives:
Compute the derivative of a fucntion 'func' at the point 'x' using the step size 'h'
Function value at 'x' is returned in 'f'
First derivative is returned in 'df'
When applicable, second derivative is returned in 'ddf'
Type = 1: calculates a right-sided derivative
Type = 2: calculates a left-sided derivative
Type = 3: calculates a two-sided derivative
Type = 4: calculates a normalized right-sided derivative
Type = 5: calculates a normalized left-sided derivative
Type = 6: calculates a normalized two-sided derivative

*/

/* Function Prototypes: */

	/* Compute the numerical first derivative of a one-dimensional function */
	void Deriv1(double (*func)(double), double x, double &f, double &df, double h, int Type);

	/* Compute the numerical second derivative of a one-dimensional function */
	void Deriv2(double (*func)(double), double x, double &f, double &df, double &ddf, double h, int Type);


/* Compute the numerical first derivative of a one-dimensional function */
void Deriv1(double (*func)(double), double x, double &f, double &df, double h, int Type)
{
	f = func(x);
	double vh;
	if(Type == 1 || Type == 2 || Type == 3) vh = h;
	else
	{
		vh = h * fabs(x);
		if(vh == Zero) vh = h;
	}
	if(Type == 1 || Type == 4) df = (func(x+vh) - f) / vh;
	if(Type == 2 || Type == 5) df = (f - func(x - vh)) / vh;
	if(Type == 3 || Type == 6) df = (func(x + vh) - func(x - vh)) / (Two * vh);
}

/*

Compute the numerical second derivative of a one-dimensional function
Note: for the two-sided first derivaitve, the step size will be '2*h'

*/

void Deriv2(double (*func)(double), double x, double &f, double &df, double &ddf, double h, int Type)
{
	f = func(x);
	double fp,fpp;
	double fm,fmm;
	double vh;

	if(Type == 1 || Type == 2 || Type == 3) vh = h;
	else
	{
		vh = h * fabs(x);
		if(vh == Zero) vh = h;
	}

	if(Type == 1 || Type == 4)
	{
		fp = func(x + vh);
		fpp = func(x + Two * vh);
		df = (fp - f) / vh;
		ddf = (fpp - Two * fp + f) / (vh * vh);
	}
	if(Type == 2 || Type == 5)
	{
		fm = func(x - vh);
		fmm = func(x - Two * vh);
		df = (f - fm) / vh;
		ddf = (f - Two * fm + fmm) / (vh * vh);
	}
	if(Type == 3 || Type == 6)
	{
		fpp = func(x + Two * vh);
		fmm = func(x - Two * vh);
		df = (fpp - fmm) / (Four * vh);
		ddf = (fpp - Two * f + fmm) / (Four * vh * vh);
	}
}

#endif