

#ifndef _Prob_DistChiSquare
#define _Prob_DistChiSquare




	double ChiSquareCdf      (double x,   double df);
	double ChiSquareCdfInv   (double cdf, double df);
	void   ChiSquareCdfValues(int *n_data, int *df, double *x, double *fx);
	bool   ChiSquareCheck    (            double df);
	double ChiSquareMean     (            double df);
	double ChiSquarePdf      (double x,   double df);
	double ChiSquareSample   (            double df, long &RandomSeed);
	double ChiSquareVariance (            double df);

	void   ChiSquareNoncentralCdfValues(int *n_data, int *df, double *lambda, double *x, double *cdf);
	double ChiSquareNoncentralCheck    (double df, double Lambda);
	double ChiSquareNoncentralMean     (double df, double Lambda);
	double ChiSquareNoncentralSample   (double df, double Lambda, long &RandomSeed);
	double ChiSquareNoncentralVariance (double df, double Lambda);

/*

Purpose:

	ChiSquareCdf evaluates the Chi squared CDF.

Modified:

	12 October 2004

Author:

	John Burkardt

Parameters:

	Input, double X, the value of the random deviate.
	Input, double A, the parameter of the distribution, usually
	the number of degrees of freedom.

	Output, double CDF, the value of the CDF.

*/

double ChiSquareCdf(double x, double df)
{
	return GammaCdf(Half * x,Zero,One,Half * df);
}


/*

Purpose:

	ChiSquareCdfInv inverts the Chi squared PDF.

Modified:

	12 October 2004

Reference:

	Best and Roberts,
	The Percentage Points of the Chi-Squared Distribution,
	Algorithm AS 91,
	Applied Statistics,
	Volume 24, Number ?, pages 385-390, 1975.

Parameters:

	Input, double CDF, a value of the chi-squared cumulative
	probability density function.
	0.000002 <= CDF <= 0.999998.

	Input, double A, the parameter of the chi-squared
	probability density function.  0 < A.

	Output, double X, the value of the chi-squared random deviate
	with the property that the probability that a chi-squared random
	deviate with parameter A is less than or equal to X is CDF.

*/

double ChiSquareCdfInv(double cdf, double df)
{
	double a2;
	double aa = 0.6931471806;
	double b;
	double c;
	double c1 = 0.01;
	double c2 = 0.222222;
	double c3 = 0.32;
	double c4 = 0.4;
	double c5 = 1.24;
	double c6 = 2.2;
	double c7 = 4.67;
	double c8 = 6.66;
	double c9 = 6.73;
	double c10 = 13.32;
	double c11 = 60.0;
	double c12 = 70.0;
	double c13 = 84.0;
	double c14 = 105.0;
	double c15 = 120.0;
	double c16 = 127.0;
	double c17 = 140.0;
	double c18 = 175.0;
	double c19 = 210.0;
	double c20 = 252.0;
	double c21 = 264.0;
	double c22 = 294.0;
	double c23 = 346.0;
	double c24 = 420.0;
	double c25 = 462.0;
	double c26 = 606.0;
	double c27 = 672.0;
	double c28 = 707.0;
	double c29 = 735.0;
	double c30 = 889.0;
	double c31 = 932.0;
	double c32 = 966.0;
	double c33 = 1141.0;
	double c34 = 1182.0;
	double c35 = 1278.0;
	double c36 = 1740.0;
	double c37 = 2520.0;
	double c38 = 5040.0;
	double cdf_max = 0.999998;
	double cdf_min = 0.000002;
	double ch;
	double e = 0.0000005;
	double g;
	int i;
	int it_max = 20;
	double p1;
	double p2;
	double q;
	double s1;
	double s2;
	double s3;
	double s4;
	double s5;
	double s6;
	double t;
	double x;
	double x2;
	double xx;

	if(cdf < cdf_min)
	{
		x = -1.0;
		Print("Error in ChiSquareCdfInv");
		Print("Cdf < CdfMin");
		Print("Cdf",cdf);
		Print("CdfMin",cdf_min);
		HaltExecution();
	}

	if(cdf_max < cdf)
	{
		x = -1.0;
		Print("Error in ChiSquareCdfInv");
		Print("Cdf > CdfMax");
		Print("Cdf",cdf);
		Print("CdfMax",cdf_max);
		HaltExecution();
	}

	xx = 0.5 * df;
	c = xx - 1.0;

	/* Compute Log ( Gamma ( A/2 ) ). */
	g = gamma_log(df / 2.0);

	if(df < -c5 * log(cdf))
	{
		/* Starting approximation for small chi-squared. */
		ch = pow((cdf * xx * exp(g + xx * aa)),(1.0 / xx));
		if(ch < e)
		{
			x = ch;
			return x;
		}
	}
	else if(df <= c3)
	{
		/* Starting approximation for A less than or equal to 0.32. */
		ch = c4;
		a2 = log(1.0 - cdf);
		for(;;)
		{
			q = ch;
			p1 = 1.0 + ch * (c7 + ch);
			p2 = ch * (c9 + ch * (c8 + ch));
			t = - 0.5 + (c7 + 2.0 * ch) / p1 - (c9 + ch * (c10 + 3.0 * ch)) / p2;
			ch = ch - (1.0 - exp(a2 + g + 0.5 * ch + c * aa) * p2 / p1) / t;
			if(fabs(q / ch - 1.0) <= c1)
			{
				break;
			}
		}
	}
	else
	{
		/* Call to algorithm AS 111.
		   Note that P has been tested above.
		   AS 241 could be used as an alternative. */
		x2 = NormalCdfInv(cdf);

		/* Starting approximation using Wilson and Hilferty estimate. */
		p1 = c2 / df;
		ch = df * pow((x2 * sqrt(p1) + 1.0 - p1), 3);

		/* Starting approximation for P tending to 1. */
		if(c6 * df + 6.0 < ch)
		{
			ch = -2.0 * (log(1.0 - cdf) - c * log(0.5 * ch) + g);
		}
	}

	/* Call to algorithm AS 239 and calculation of seven term Taylor series. */
	for(i=1; i<=it_max; i++)
	{
		q = ch;
		p1 = 0.5 * ch;
		p2 = cdf - gamma_inc(xx, p1);
		t = p2 * exp(xx * aa + g + p1 - c * log(ch));
		b = t / ch;
		a2 = 0.5 * t - b * c;
		s1 = (c19 + a2 * (c17 + a2 * (c14 + a2 * (c13 + a2 * (c12 + a2 * c11))))) / c24;
		s2 = (c24 + a2 * (c29 + a2 * (c32 + a2 * (c33 + a2 * c35)))) / c37;
		s3 = (c19 + a2 * (c25 + a2 * (c28 + a2 * c31))) / c37;
		s4 = (c20 + a2 * (c27 + a2 * c34) + c * (c22 + a2 * (c30 + a2 * c36))) / c38;
		s5 = (c13 + c21 * a2 + c * (c18 + c26 * a2)) / c37;
		s6 = (c15 + c * (c23 + c16 * c)) / c38;
		ch = ch + t * (1.0 + 0.5 * t * s1 - b * c * (s1 - b * (s2 - b * (s3 - b * (s4 - b * (s5 - b * s6))))));
		if(fabs(q / ch - 1.0) < e) // MP: I FIXED AN ERROR HERE
		{
			x = ch;
			return x;
		}
	}

	x = ch;

	Print("Error in ChiSquaredCdfInv");
	Print("Convergence not Reached");
	Print("cdf",cdf);
	Print("df",df);
	Print("ChiSquareCdf(x,df)",ChiSquareCdf(x,df));
	Print("Tol",e);
	Print("Error",fabs(q / ch - 1.0));
	HaltExecution();

	return x;
}


/*

Purpose:

	ChiSquareCdfValues returns some values of the Chi-Square CDF.

Discussion:

	In Mathematica, the function can be evaluated by:

	   Needs["Statistics`ContinuousDistributions`"]
	   dist = ChiSquareDistribution [ df ]
	   CDF [ dist, x ]

Modified:

	09 August 2004

Author:

	John Burkardt

Reference:

	Milton Abramowitz and Irene Stegun,
	Handbook of Mathematical Functions,
	US Department of Commerce, 1964.

	Stephen Wolfram,
	The Mathematica Book,
	Fourth Edition,
	Wolfram Media / Cambridge University Press, 1999.

Parameters:

	Input/output, int *N_DATA.  The user sets N_DATA to 0 before the
	first call.  On each call, the routine increments N_DATA by 1, and
	returns the corresponding data; when there is no more data, the
	output value of N_DATA will be 0 again.

	Output, int *A, the parameter of the function.

	Output, double *X, the argument of the function.

	Output, double *FX, the value of the function.

*/

void ChiSquareCdfValues(int *n_data, int *df, double *x, double *fx)
{
	#define N_MAX 21

  int df_vec[N_MAX] = {
     1,  2,  1,  2,
     1,  2,  3,  4,
     1,  2,  3,  4,
     5,  3,  3,  3,
     3,  3, 10, 10,
    10 };

  double fx_vec[N_MAX] = {
     0.7965567455405796E-01,
     0.4987520807317687E-02,
     0.1124629160182849E+00,
     0.9950166250831946E-02,
     0.4729107431344619E+00,
     0.1812692469220181E+00,
     0.5975750516063926E-01,
     0.1752309630642177E-01,
     0.6826894921370859E+00,
     0.3934693402873666E+00,
     0.1987480430987992E+00,
     0.9020401043104986E-01,
     0.3743422675270363E-01,
     0.4275932955291202E+00,
     0.6083748237289110E+00,
     0.7385358700508894E+00,
     0.8282028557032669E+00,
     0.8883897749052874E+00,
     0.1721156299558408E-03,
     0.3659846827343712E-02,
     0.1857593622214067E-01 };

  double x_vec[N_MAX] = {
     0.01E+00,
     0.01E+00,
     0.02E+00,
     0.02E+00,
     0.40E+00,
     0.40E+00,
     0.40E+00,
     0.40E+00,
     1.00E+00,
     1.00E+00,
     1.00E+00,
     1.00E+00,
     1.00E+00,
     2.00E+00,
     3.00E+00,
     4.00E+00,
     5.00E+00,
     6.00E+00,
     1.00E+00,
     2.00E+00,
     3.00E+00 };

	if(*n_data < 0)
	{
		*n_data = 0;
	}

	*n_data = *n_data + 1;

	if(N_MAX < *n_data)
	{
		*n_data = 0;
		*df = 0;
		*x = 0.0;
		*fx = 0.0;
	}
	else
	{
		*df = df_vec[*n_data-1];
		*x = x_vec[*n_data-1];
		*fx = fx_vec[*n_data-1];
	}

	return;

	#undef N_MAX
}

/*

Purpose:

	CHI_SQUARE_CHECK checks the parameter of the central Chi squared PDF.

Modified:

	12 October 2004

Author:

	John Burkardt

Parameters:

	Input, double A, the parameter of the distribution.
	A >= 1.

	Output, bool CHI_SQUARE_CHECK, is true if the parameters are legal.

*/

bool ChiSquareCheck(double df)
{
	if(df < One)
	{
		Print("Error in ChiSquareCheck");
		Print("df < 1");
		return false;
	}
	return true;
}


/*

Purpose:

	CHI_SQUARE_MEAN returns the mean of the central Chi squared PDF.

Modified:

	12 October 2004

Author:

	John Burkardt

Parameters:

	Input, double A, the parameter of the distribution.
	1 <= A.

	Output, double MEAN, the mean value.

*/

double ChiSquareMean(double df)
{
	return df;
}


/*

Purpose:

	CHI_SQUARE_PDF evaluates the central Chi squared PDF.

Formula:

	PDF(A;X) =
	  EXP ( - X / 2 ) * X**((A-2)/2) / ( 2**(A/2) * GAMMA ( A/2 ) )

Modified:

	12 October 2004

Author:

	John Burkardt

Parameters:

	Input, double X, the argument of the PDF.
	0.0 <= X

	Input, double A, the parameter of the PDF.
	1 <= A.

	Output, double PDF, the value of the PDF.

*/

double ChiSquarePdf(double x, double df)
{
	if(x < Zero)
	{
		return Zero;
	}
	else
	{
		double b = df / Two;
		return exp(-Half * x) * pow(x,(b - One)) / (pow(Two,b) * gamma(b));
	}
}


/*

Purpose:

	CHI_SQUARE_SAMPLE samples the central Chi squared PDF.

Modified:

	12 October 2004

Author:

	John Burkardt

Parameters:

	Input, double A, the parameter of the PDF.
	1 <= A.

	Input/output, int *SEED, a seed for the random number generator.

	Output, double X, a sample of the PDF.

*/

double ChiSquareSample(double df, long &RandomSeed)
{
	int n = (int)df;
	int it_max = 100;
	if((double)(n) == df && n <= it_max)
	{
		double x = Zero;
		for(int i=1; i<=n; i++)
		{
			double x2 = NormalSample(RandomSeed);
			x += Sqr(x2);
		}
		return x;
	}
	else
	{
		return Two * GammaSample(Zero,One,df / Two,RandomSeed);
	}
}


/*

Purpose:

	CHI_SQUARE_VARIANCE returns the variance of the central Chi squared PDF.

Modified:

	12 October 2004

Author:

	John Burkardt

Parameters:

	Input, double A, the parameter of the distribution.
	1 <= A.

	Output, double VARIANCE, the variance of the PDF.

*/

double ChiSquareVariance(double df)
{
	return Two * df;
}


/*

Purpose:

	CHI_SQUARE_NONCENTRAL_CDF_VALUES returns values of the noncentral chi CDF.

Discussion:

	In Mathematica, the function can be evaluated by:
	Needs["Statistics`ContinuousDistributions`"]
	dist = NoncentralChiSquareDistribution [ df, lambda ]
	CDF [ dist, x ]

Modified:

	30 August 2004

Author:

	John Burkardt

Reference:

	Stephen Wolfram,
	The Mathematica Book,
	Fourth Edition,
	Wolfram Media / Cambridge University Press, 1999.

Parameters:

	Input/output, int *N_DATA.  The user sets N_DATA to 0 before the
	first call.  On each call, the routine increments N_DATA by 1, and
	returns the corresponding data; when there is no more data, the
	output value of N_DATA will be 0 again.

	Output, int *DF, the number of degrees of freedom.

	Output, double *LAMBDA, the noncentrality parameter.

	Output, double *X, the argument of the function.

	Output, double *CDF, the noncentral chi CDF.

*/

void ChiSquareNoncentralCdfValues(int *n_data, int *df, double *lambda, double *x, double *cdf)
{

	#define N_MAX 28

  double cdf_vec[N_MAX] = {
     0.8399444269398261E+00,
     0.6959060300435139E+00,
     0.5350879697078847E+00,
     0.7647841496310313E+00,
     0.6206436532195436E+00,
     0.4691667375373180E+00,
     0.3070884345937569E+00,
     0.2203818092990903E+00,
     0.1500251895581519E+00,
     0.3071163194335791E-02,
     0.1763982670131894E-02,
     0.9816792594625022E-03,
     0.1651753140866208E-01,
     0.2023419573950451E-03,
     0.4984476352854074E-06,
     0.1513252400654827E-01,
     0.2090414910614367E-02,
     0.2465021206048452E-03,
     0.2636835050342939E-01,
     0.1857983220079215E-01,
     0.1305736595486640E-01,
     0.5838039534819351E-01,
     0.4249784402463712E-01,
     0.3082137716021596E-01,
     0.1057878223400849E+00,
     0.7940842984598509E-01,
     0.5932010895599639E-01,
     0.2110395656918684E+00 };

  int df_vec[N_MAX] = {
      1,   2,   3,
      1,   2,   3,
      1,   2,   3,
      1,   2,   3,
     60,  80, 100,
      1,   2,   3,
     10,  10,  10,
     10,  10,  10,
     10,  10,  10,
      8 };

  double lambda_vec[N_MAX] = {
     0.5E+00,
     0.5E+00,
     0.5E+00,
     1.0E+00,
     1.0E+00,
     1.0E+00,
     5.0E+00,
     5.0E+00,
     5.0E+00,
    20.0E+00,
    20.0E+00,
    20.0E+00,
    30.0E+00,
    30.0E+00,
    30.0E+00,
     5.0E+00,
     5.0E+00,
     5.0E+00,
     2.0E+00,
     3.0E+00,
     4.0E+00,
     2.0E+00,
     3.0E+00,
     4.0E+00,
     2.0E+00,
     3.0E+00,
     4.0E+00,
     0.5E+00 };

  double x_vec[N_MAX] = {
      3.000E+00,
      3.000E+00,
      3.000E+00,
      3.000E+00,
      3.000E+00,
      3.000E+00,
      3.000E+00,
      3.000E+00,
      3.000E+00,
      3.000E+00,
      3.000E+00,
      3.000E+00,
     60.000E+00,
     60.000E+00,
     60.000E+00,
      0.050E+00,
      0.050E+00,
      0.050E+00,
      4.000E+00,
      4.000E+00,
      4.000E+00,
      5.000E+00,
      5.000E+00,
      5.000E+00,
      6.000E+00,
      6.000E+00,
      6.000E+00,
      5.000E+00 };

	if(*n_data < 0)
	{
		*n_data = 0;
	}

	*n_data = *n_data + 1;

	if(N_MAX < *n_data)
	{
		*n_data = 0;
		*x = 0.0;
		*lambda = 0.0;
		*df = 0;
		*cdf = 0.0;
	}
	else
	{
		*x = x_vec[*n_data-1];
		*lambda = lambda_vec[*n_data-1];
		*df = df_vec[*n_data-1];
		*cdf = cdf_vec[*n_data-1];
	}

	return;

	#undef N_MAX
}


/*

Purpose:

	CHI_SQUARE_NONCENTRAL_CHECK checks the parameters of the noncentral Chi Squared PDF.

Modified:

	13 October 2004

Author:

	John Burkardt

Parameters:

	Input, int A, the parameter of the PDF.
	1.0 <= A.

	Input, double B, the noncentrality parameter of the PDF.
	0.0 <= B.

	Output, bool CHI_SQUARE_NONCENTRAL_CHECK, is true if the parameters
	are legal.

*/

double ChiSquareNoncentralCheck(double df, double Lambda)
{
	if(df < One)
	{
		Print("CHI_SQUARE_NONCENTRAL_CHECK - Fatal error!");
		Print("  df < 1.");
		return false;
	}
	if(Lambda < Zero)
	{
		Print("CHI_SQUARE_NONCENTRAL_CHECK - Fatal error!");
		Print("  Lambda < 0.");
		return false;
	}
	return true;
}


/*

Purpose:

	CHI_SQUARE_NONCENTRAL_MEAN returns the mean of the noncentral Chi squared PDF.

Modified:

	13 October 2004

Author:

	John Burkardt

Parameters:

	Input, int A, the parameter of the PDF.
	1.0 <= A.

	Input, double B, the noncentrality parameter of the PDF.
	0.0 <= B.

	Output, double MEAN, the mean value.

*/

double ChiSquareNoncentralMean(double df, double Lambda)
{
	return df + Lambda;
}


/*

Purpose:

	CHI_SQUARE_NONCENTRAL_SAMPLE samples the noncentral Chi squared PDF.

Modified:

	30 November 1999

Author:

	John Burkardt

Parameters:

	Input, int A, the parameter of the PDF.
	1.0 <= A.

	Input, double B, the noncentrality parameter of the PDF.
	0.0 <= B.

	Input/output, int *SEED, a seed for the random number generator.

	Output, double X, a sample of the PDF.

*/

double ChiSquareNoncentralSample(double df, double Lambda, long &RandomSeed)
{
	double x1 = ChiSquareSample(df - One,RandomSeed);
	double x2 = NormalSample(Sqrt(Lambda),One,RandomSeed);
	return x1 + Sqr(x2);
}


/*

Purpose:

	CHI_SQUARE_NONCENTRAL_VARIANCE returns the variance of the noncentral Chi squared PDF.

Modified:

	13 October 2004

Author:

	John Burkardt

Parameters:

	Input, double A, the parameter of the PDF.
	1 <= A.

	Input, double B, the noncentrality parameter of the PDF.
	0.0 <= B.

	Output, double VARIANCE, the variance value.

*/

double ChiSquareNoncentralVariance(double df, double Lambda)
{
	return Two * (df + Two * Lambda);
}

double ChiSquareScaledCdf(double x, double df, double a, double b)
{
	return (ChiSquareCdf(x,df) - a) / b;
}

double ChiSquareScaledCdfInv(double cdf, double df, double a, double b)
{
	return a + b * ChiSquareCdfInv(cdf,df);
}

double ChiSquareScaledMean(double df, double a, double b)
{
	return a + b * ChiSquareMean(df);
}

double ChiSquareScaledPdf(double x, double df, double a, double b)
{
	return ChiSquarePdf((x - a) / b,df);
}

double ChiSquareScaledSample(double df, double a, double b, long &RandomSeed)
{
	return a + b * ChiSquareSample(df,RandomSeed);
}

double ChiSquareScaledVariance(double df, double a, double b)
{
	return Sqr(b) * ChiSquareVariance(df);
}

/*
 *  Mathlib : A C Library of Special Functions
 *  Copyright (C) 1998 Ross Ihaka
 *  Copyright (C) 2000--2009 The R Development Core Team
 *  Copyright (C) 2004--2009 The R Foundation
 *  based on AS 91 (C) 1979 Royal Statistical Society
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, a copy is available at
 *  http://www.r-project.org/Licenses/
 *
 *  DESCRIPTION
 *
 *	Compute the quantile function of the gamma distribution.
 *
 *  NOTES
 *
 *	This function is based on the Applied Statistics
 *	Algorithm AS 91 ("ppchi2") and via pgamma(.) AS 239.
 *
 *	R core improvements:
 *	o  lower_tail, log_p
 *      o  non-trivial result for p outside [0.000002, 0.999998]
 *	o  p ~ 1 no longer gives +Inf; final Newton step(s)
 *
 *  REFERENCES
 *
 *	Best, D. J. and D. E. Roberts (1975).
 *	Percentage Points of the Chi-Squared Distribution.
 *	Applied Statistics 24, page 385.  */

/* give_log in "d";  log_p in "p" & "q" : */
/*
#define M_LN2 0.69314718055994530942

#define give_log log_p
							/* "DEFAULT" */
							/* --------- */
/*#define R_D__0	(log_p ? ML_NEGINF : 0.)		/* 0 */
/*#define R_D__1	(log_p ? 0. : 1.)			/* 1 */
/*#define R_DT_0	(lower_tail ? R_D__0 : R_D__1)		/* 0 */
//#define R_DT_1	(lower_tail ? R_D__1 : R_D__0)		/* 1 */

/* Use 0.5 - p + 0.5 to perhaps gain 1 bit of accuracy */
//#define R_D_Lval(p)	(lower_tail ? (p) : (0.5 - (p) + 0.5))	/*  p  */
//#define R_D_Cval(p)	(lower_tail ? (0.5 - (p) + 0.5) : (p))	/*  1 - p */

//#define R_D_val(x)	(log_p	? log(x) : (x))		/*  x  in pF(x,..) */
//#define R_D_qIv(p)	(log_p	? exp(p) : (p))		/*  p  in qF(p,..) */
//#define R_D_exp(x)	(log_p	?  (x)	 : exp(x))	/* exp(x) */
//#define R_D_log(p)	(log_p	?  (p)	 : log(p))	/* log(p) */
//#define R_D_Clog(p)	(log_p	? log1p(-(p)) : (0.5 - (p) + 0.5)) /* [log](1-p) */

/* log(1 - exp(x))  in more stable form than log1p(- R_D_qIv(x))) : */
//#define R_Log1_Exp(x)   ((x) > -M_LN2 ? log(-expm1(x)) : log1p(-exp(x)))

/* log(1-exp(x)):  R_D_LExp(x) == (log1p(- R_D_qIv(x))) but even more stable:*/
//#define R_D_LExp(x)     (log_p ? R_Log1_Exp(x) : log1p(-x))

//#define R_DT_val(x)	(lower_tail ? R_D_val(x)  : R_D_Clog(x))
//#define R_DT_Cval(x)	(lower_tail ? R_D_Clog(x) : R_D_val(x))

/*#define R_DT_qIv(p)	R_D_Lval(R_D_qIv(p))		 *  p  in qF ! */
//#define R_DT_qIv(p)	(log_p ? (lower_tail ? exp(p) : - expm1(p)) \
//			       : R_D_Lval(p))

/*#define R_DT_CIv(p)	R_D_Cval(R_D_qIv(p))		 *  1 - p in qF */
//#define R_DT_CIv(p)	(log_p ? (lower_tail ? -expm1(p) : exp(p)) \
//			       : R_D_Cval(p))

//#define R_DT_exp(x)	R_D_exp(R_D_Lval(x))		/* exp(x) */
//#define R_DT_Cexp(x)	R_D_exp(R_D_Cval(x))		/* exp(1 - x) */

//#define R_DT_log(p)	(lower_tail? R_D_log(p) : R_D_LExp(p))/* log(p) in qF */
//#define R_DT_Clog(p)	(lower_tail? R_D_LExp(p): R_D_log(p))/* log(1-p) in qF*/
//#define R_DT_Log(p)	(lower_tail? (p) : R_Log1_Exp(p))
/* ==   R_DT_log when we already "know" log_p == TRUE :*/

/* Compute  log(gamma(a+1))  accurately also for small a (0 < a < 0.5). */
/*double lgamma1p (double a)
{
    const double eulers_const =	 0.5772156649015328606065120900824024;

    /* coeffs[i] holds (zeta(i+2)-1)/(i+2) , i = 0:(N-1), N = 40 : */
/*    const int N = 40;
    static const double coeffs[40] = {
	0.3224670334241132182362075833230126e-0,/* = (zeta(2)-1)/2 */
/*	0.6735230105319809513324605383715000e-1,/* = (zeta(3)-1)/3 */
/*	0.2058080842778454787900092413529198e-1,
	0.7385551028673985266273097291406834e-2,
	0.2890510330741523285752988298486755e-2,
	0.1192753911703260977113935692828109e-2,
	0.5096695247430424223356548135815582e-3,
	0.2231547584535793797614188036013401e-3,
	0.9945751278180853371459589003190170e-4,
	0.4492623673813314170020750240635786e-4,
	0.2050721277567069155316650397830591e-4,
	0.9439488275268395903987425104415055e-5,
	0.4374866789907487804181793223952411e-5,
	0.2039215753801366236781900709670839e-5,
	0.9551412130407419832857179772951265e-6,
	0.4492469198764566043294290331193655e-6,
	0.2120718480555466586923135901077628e-6,
	0.1004322482396809960872083050053344e-6,
	0.4769810169363980565760193417246730e-7,
	0.2271109460894316491031998116062124e-7,
	0.1083865921489695409107491757968159e-7,
	0.5183475041970046655121248647057669e-8,
	0.2483674543802478317185008663991718e-8,
	0.1192140140586091207442548202774640e-8,
	0.5731367241678862013330194857961011e-9,
	0.2759522885124233145178149692816341e-9,
	0.1330476437424448948149715720858008e-9,
	0.6422964563838100022082448087644648e-10,
	0.3104424774732227276239215783404066e-10,
	0.1502138408075414217093301048780668e-10,
	0.7275974480239079662504549924814047e-11,
	0.3527742476575915083615072228655483e-11,
	0.1711991790559617908601084114443031e-11,
	0.8315385841420284819798357793954418e-12,
	0.4042200525289440065536008957032895e-12,
	0.1966475631096616490411045679010286e-12,
	0.9573630387838555763782200936508615e-13,
	0.4664076026428374224576492565974577e-13,
	0.2273736960065972320633279596737272e-13,
	0.1109139947083452201658320007192334e-13/* = (zeta(40+1)-1)/(40+1) */
 /*   };

    const double c = 0.2273736845824652515226821577978691e-12;/* zeta(N+2)-1 */
 /*   const double tol_logcf = 1e-14;
    double lgam;
    int i;

    if (fabs (a) >= 0.5)
	return lgammafn (a + 1);

    /* Abramowitz & Stegun 6.1.33 : for |x| < 2,
     * <==> log(gamma(1+x)) = -(log(1+x) - x) - gamma*x + x^2 * \sum_{n=0}^\infty c_n (-x)^n
     * where c_n := (Zeta(n+2) - 1)/(n+2)  = coeffs[n]
     *
     * Here, another convergence acceleration trick is used to compute
     * lgam(x) :=  sum_{n=0..Inf} c_n (-x)^n
     */
  /*  lgam = c * logcf(-a / 2, N + 2, 1, tol_logcf);
    for (i = N - 1; i >= 0; i--)
	lgam = coeffs[i] - a * lgam;

    return (a * lgam - eulers_const) * a - log1pmx (a);
} /* lgamma1p */

/*
#ifdef DEBUG_qgamma
# define DEBUG_q
#endif

double qchisq_appr(double p, double nu, double g/* = log Gamma(nu/2) *//*,
	/*	   int lower_tail, int log_p, double tol /* EPS1 *//*)
/*{
#define C7	4.67
#define C8	6.66
#define C9	6.73
#define C10	13.32

    double p_, alpha, a, c, ch, p1;
    double p2, q, t, x;

    /* test arguments and initialise */
/*
#ifdef IEEE_754
    if (ISNAN(p) || ISNAN(nu))
	return p + nu;
#endif
    //R_Q_P01_check(p);
    if ((log_p	&& p > 0) || (!log_p && (p < 0 || p > 1)) )
    {
    	Print("Error in qchisq_appr");
    	Print("Invalid p");
    	Print("p",p);
    	HaltExecution();
    }

    //if (nu <= 0) ML_ERR_return_NAN;
    if (nu <= 0)
    {
		Print("Error in qchisq_appr");
    	Print("Invalid nu");
    	HaltExecution();
	}

    p_ = R_DT_qIv(p);/* lower_tail prob (in any case) */
/*
    alpha = 0.5 * nu;/* = [pq]gamma() shape */
 /*   c = alpha-1;

    if(nu < (-1.24)*(p1 = R_DT_log(p))) {	/* for small chi-squared */
	/* log(alpha) + g = log(alpha) + log(gamma(alpha)) =
	 *        = log(alpha*gamma(alpha)) = lgamma(alpha+1) suffers from
	 *  catastrophic cancellation when alpha << 1
	 */
/*	double lgam1pa = (alpha < 0.5) ? lgamma1p(alpha) : (log(alpha) + g);
	ch = exp((lgam1pa + p1)/alpha + M_LN2);
#ifdef DEBUG_qgamma
	REprintf(" small chi-sq., ch0 = %g\n", ch);
#endif

    } else if(nu > 0.32) {	/*  using Wilson and Hilferty estimate */
/*
	x = qnorm(p, 0, 1, lower_tail, log_p);
	p1 = 2./(9*nu);
	ch = nu*pow(x*sqrt(p1) + 1-p1, 3);

#ifdef DEBUG_qgamma
	REprintf(" nu > .32: Wilson-Hilferty; x = %7g\n", x);
#endif
	/* approximation for p tending to 1: */
/*	if( ch > 2.2*nu + 6 )
	    ch = -2*(R_DT_Clog(p) - c*log(0.5*ch) + g);

    } else { /* "small nu" : 1.24*(-log(p)) <= nu <= 0.32 */
/*
	ch = 0.4;
	a = R_DT_Clog(p) + g + c*M_LN2;
#ifdef DEBUG_qgamma
	REprintf(" nu <= .32: a = %7g\n", a);
#endif
	do {
	    q = ch;
	    p1 = 1. / (1+ch*(C7+ch));
	    p2 = ch*(C9+ch*(C8+ch));
	    t = -0.5 +(C7+2*ch)*p1 - (C9+ch*(C10+3*ch))/p2;
	    ch -= (1- exp(a+0.5*ch)*p2*p1)/t;
	} while(fabs(q - ch) > tol * fabs(ch));
    }

    return ch;
}

double qgamma(double p, double alpha, double scale, int lower_tail, int log_p)
/*			shape = alpha */
/*{
#define EPS1 1e-2
#define EPS2 5e-7/* final precision of AS 91 */
/*#define EPS_N 1e-15/* precision of Newton step / iterations */
/*#define LN_EPS -36.043653389117156 /* = log(.Machine$double.eps) iff IEEE_754 */
/*
#define MAXIT 1000/* was 20 */
/*
#define pMIN 1e-100   /* was 0.000002 = 2e-6 */
/*#define pMAX (1-1e-14)/* was (1-1e-12) and 0.999998 = 1 - 2e-6 */
/*
    const static double
	i420  = 1./ 420.,
	i2520 = 1./ 2520.,
	i5040 = 1./ 5040;

    double p_, a, b, c, g, ch, ch0, p1;
    double p2, q, s1, s2, s3, s4, s5, s6, t, x;
    int i, max_it_Newton = 1;

    /* test arguments and initialise */
/*
#ifdef IEEE_754
    if (ISNAN(p) || ISNAN(alpha) || ISNAN(scale))
	return p + alpha + scale;
#endif
    R_Q_P01_boundaries(p, 0., ML_POSINF);

    if (alpha < 0 || scale <= 0) ML_ERR_return_NAN;

    if (alpha == 0) /* all mass at 0 : */ /*return 0.;

    if (alpha < 1e-10) {
    /* Warning seems unnecessary now: */
/*#ifdef _DO_WARN_qgamma_
	MATHLIB_WARNING("value of shape (%g) is extremely small: results may be unreliable", alpha);
#endif
	max_it_Newton = 7;/* may still be increased below */
/*    }

    p_ = R_DT_qIv(p);/* lower_tail prob (in any case) */
/*
#ifdef DEBUG_qgamma
    REprintf("qgamma(p=%7g, alpha=%7g, scale=%7g, l.t.=%2d, log_p=%2d): ",
	     p,alpha,scale, lower_tail, log_p);
#endif
    g = lgammafn(alpha);/* log Gamma(v/2) */

    /*----- Phase I : Starting Approximation */
 /*   ch = qchisq_appr(p, /* nu= 'df' =  *//* 2*alpha, /* lgamma(nu/2)= */ /*g,
		     lower_tail, log_p, /* tol= */ /*EPS1);
    if(!R_FINITE(ch)) {
	/* forget about all iterations! */
	/*max_it_Newton = 0; goto END;
    }
    if(ch < EPS2) {/* Corrected according to AS 91; MM, May 25, 1999 */
/*	max_it_Newton = 20;
	goto END;/* and do Newton steps */
  /*  }

    /* FIXME: This (cutoff to {0, +Inf}) is far from optimal
     * -----  when log_p or !lower_tail, but NOT doing it can be even worse */
  /*  if(p_ > pMAX || p_ < pMIN) {
	/* did return ML_POSINF or 0.;	much better: */
/*	max_it_Newton = 20;
	goto END;/* and do Newton steps */
 /*   }

#ifdef DEBUG_qgamma
    REprintf("\t==> ch = %10g:", ch);
#endif

/*----- Phase II: Iteration
 *	Call pgamma() [AS 239]	and calculate seven term taylor series
 */
   /* c = alpha-1;
    s6 = (120+c*(346+127*c)) * i5040; /* used below, is "const" */

    /*ch0 = ch;/* save initial approx. */
    /*for(i=1; i <= MAXIT; i++ ) {
	q = ch;
	p1 = 0.5*ch;
	p2 = p_ - pgamma_raw(p1, alpha, /*lower_tail*//*TRUE, /*log_p*//*FALSE);
#ifdef DEBUG_qgamma
	if(i == 1) REprintf(" Ph.II iter; ch=%g, p2=%g\n", ch, p2);
	if(i >= 2) REprintf("     it=%d,  ch=%g, p2=%g\n", i, ch, p2);
#endif
#ifdef IEEE_754
	if(!R_FINITE(p2) || ch <= 0)
#else
	if(errno != 0 || ch <= 0)
#endif
	    { ch = ch0; max_it_Newton = 27; goto END; }/*was  return ML_NAN;*/
/*
	t = p2*exp(alpha*M_LN2+g+p1-c*log(ch));
	b = t/ch;
	a = 0.5*t - b*c;
	s1 = (210+ a*(140+a*(105+a*(84+a*(70+60*a))))) * i420;
	s2 = (420+ a*(735+a*(966+a*(1141+1278*a)))) * i2520;
	s3 = (210+ a*(462+a*(707+932*a))) * i2520;
	s4 = (252+ a*(672+1182*a) + c*(294+a*(889+1740*a))) * i5040;
	s5 = (84+2264*a + c*(1175+606*a)) * i2520;

	ch += t*(1+0.5*t*s1-b*c*(s1-b*(s2-b*(s3-b*(s4-b*(s5-b*s6))))));
	if(fabs(q - ch) < EPS2*ch)
	    goto END;
	if(fabs(q - ch) > 0.1*ch) {/* diverging? -- also forces ch > 0 */
/*	    if(ch < q) ch = 0.9 * q; else ch = 1.1 * q;
	}
    }
/* no convergence in MAXIT iterations -- but we add Newton now... */
/*#ifdef DEBUG_q
    MATHLIB_WARNING3("qgamma(%g) not converged in %d iterations; rel.ch=%g\n",
		     p, MAXIT, ch/fabs(q - ch));
#endif
/* was
 *    ML_ERROR(ME_PRECISION, "qgamma");
 * does nothing in R !*/
/*
END:
/* PR# 2214 :	 From: Morten Welinder <terra@diku.dk>, Fri, 25 Oct 2002 16:50
   --------	 To: R-bugs@biostat.ku.dk     Subject: qgamma precision

   * With a final Newton step, double accuracy, e.g. for (p= 7e-4; nu= 0.9)
   *
   * Improved (MM): - only if rel.Err > EPS_N (= 1e-15);
   *		    - also for lower_tail = FALSE	 or log_p = TRUE
   * 		    - optionally *iterate* Newton
   */
 /*   x = 0.5*scale*ch;
    if(max_it_Newton) {
	/* always use log scale */
	/*if (!log_p) {
	    p = log(p);
	    log_p = TRUE;
	}
	if(x == 0) {
	    const double _1_p = 1. + 1e-7;
	    const double _1_m = 1. - 1e-7;
	    x = DBL_MIN;
	    p_ = pgamma(x, alpha, scale, lower_tail, log_p);
	    if(( lower_tail && p_ > p * _1_p) ||
	       (!lower_tail && p_ < p * _1_m))
		return(0.);
	    /* else:  continue, using x = DBL_MIN instead of  0  */
/*	}
	else
	    p_ = pgamma(x, alpha, scale, lower_tail, log_p);
	for(i = 1; i <= max_it_Newton; i++) {
	    p1 = p_ - p;
#ifdef DEBUG_qgamma
	    if(i == 1) REprintf("\n it=%d: p=%g, x = %g, p.=%g; p1=d{p}=%g\n",
				i, p, x, p_, p1);
	    if(i >= 2) REprintf("          x{it= %d} = %g, p.=%g, p1=d{p}=%g\n",
				i,    x, p_, p1);
#endif
	    if(fabs(p1) < fabs(EPS_N * p))
		break;
	    /* else */
/*	    if((g = dgamma(x, alpha, scale, log_p)) == R_D__0) {
#ifdef DEBUG_q
		if(i == 1) REprintf("no final Newton step because dgamma(*)== 0!\n");
#endif
		break;
	    }
	    /* else :
	     * delta x = f(x)/f'(x);
	     * if(log_p) f(x) := log P(x) - p; f'(x) = d/dx log P(x) = P' / P
	     * ==> f(x)/f'(x) = f*P / P' = f*exp(p_) / P' (since p_ = log P(x))
	     */
/*	    t = log_p ? p1*exp(p_ - g) : p1/g ;/* = "delta x" */
/*	    t = lower_tail ? x - t : x + t;
	    p_ = pgamma (t, alpha, scale, lower_tail, log_p);
	    if (fabs(p_ - p) > fabs(p1) ||
		(i > 1 && fabs(p_ - p) == fabs(p1)) /* <- against flip-flop *//*) {
		/* no improvement */
/*#ifdef DEBUG_qgamma
		if(i == 1 && max_it_Newton > 1)
		    REprintf("no Newton step done since delta{p} >= last delta\n");
#endif
		break;
	    } /* else : */
/*#ifdef Harmful_notably_if_max_it_Newton_is_1
	    /* control step length: this could have started at
	       the initial approximation */
/*	    if(t > 1.1*x) t = 1.1*x;
	    else if(t < 0.9*x) t = 0.9*x;
#endif
	    x = t;
	}
    }

    return x;
}

double qchisq(double p, double df, int lower_tail, int log_p)
{
    return qgamma(p, 0.5 * df, 2.0, lower_tail, log_p);
}

double qchisq(double p, double df)
{
    return qchisq(p,df,1,0)
}
*/

#endif
