
#ifndef _Prob_DistNormal
#define _Prob_DistNormal


	/* The Normal and Standard Normal Distributions */

		template <class Real> Real NormalPdf(Real x);
		template <class Real> Real NormalCdf(Real x);
		double NormalMean();
		double NormalVariance();
		template <class Real> Real NormalCdfInv(const Real &p);
		double NormalCdfInv2(double p);
		double NormalSample(long &RandomSeed, bool Reset = False);
		void   NormalCdfValues(int *n_data, double *x, double *fx);
		
		double NormalPdf(double x, double Mu, double Sigma);
		double NormalCdf(double x, double Mu, double Sigma);
		//bool NormalCheck(double Mu, double Sigma);
		void NormalCdfValues(int *n_data, double *Mu, double *Sigma, double *x, double *fx);
		double NormalCdfInv(double cdf, double Mu, double Sigma);
		//double NormalMean(double a, double b);
		//double NormalVariance(double a, double b);
		//double NormalSample(double a, double b, int *seed);
		double NormalSample(double Mu, double Sigma, long &RandomSeed);


/*

Purpose:		 
	
	Evaluates the Normal 01 CDF.

Modified:

	10 February 1999

Author:

	John Burkardt

Reference:

	A G Adams,
    Areas Under the Normal Curve,
    Algorithm 39, 
    Computer j., 
    Volume 12, pages 197-198, 1969.

Parameters:

    Input, double X, the argument of the CDF.
    Output, double CDF, the value of the CDF.
    
*/

template <class Real>
Real NormalCdf(Real x)
{
	static Real a1  = 0.398942280444E+00;
	static Real a2  = 0.399903438504E+00;
	static Real a3  = 5.75885480458E+00;
	static Real a4  = 29.8213557808E+00;
	static Real a5  = 2.62433121679E+00;
	static Real a6  = 48.6959930692E+00;
	static Real a7  = 5.92885724438E+00;
	static Real b0  = 0.398942280385E+00;
	static Real b1  = 3.8052E-08;
	static Real b2  = 1.00000615302E+00;
	static Real b3  = 3.98064794E-04;
	static Real b4  = 1.98615381364E+00;
	static Real b5  = 0.151679116635E+00;
	static Real b6  = 5.29330324926E+00;
	static Real b7  = 4.8385912808E+00;
	static Real b8  = 15.1508972451E+00;
	static Real b9  = 0.742380924027E+00;
	static Real b10 = 30.789933034E+00;
	static Real b11 = 3.99019417011E+00;
	
	Real cdf;
	Real q;
	Real y;
	
	Real absx = Abs(x);
	if(absx <= 1.28)
	{
		/* |X| <= 1.28. */
		y = 0.5 * x * x;
		q = 0.5 - absx * (a1 - a2 * y / (y + a3 - a4 / (y + a5 + a6 / (y + a7))));
	}
	else if(absx <= 12.7)
	{
		/* 1.28 < |X| <= 12.7 */
	    y = 0.5 * x * x;
		q =  exp(-y) * b0 / (absx - b1 + b2 / (absx + b3 + b4 / (absx - b5
				+ b6 / (absx + b7 - b8 / (absx + b9 + b10 / (absx + b11))))));
	}
	else
	{
		/* 12.7 < |X| */
		q = Zero;
	}
	
	/* Take account of negative X. */
	if(x < Zero) cdf = q;
	else         cdf = One - q;

	return cdf;
}

/*

Purpose:
	
	Evaluates the Normal CDF.

Modified:

	19 September 2004

Author:

	John Burkardt

Parameters:

	Input, double X, the argument of the CDF.
	
	Input, double Mu, Sigma, the parameters of the PDF.
	Sigma > 0.0.
	
	Output, double CDF, the value of the CDF.
	
*/

double NormalCdf(double x, double Mu, double Sigma)
{
	double y   = (x - Mu) / Sigma;
	double cdf = NormalCdf(y);
	return cdf;
}

/*

Purpose:

	Inverts the standard normal CDF.

Discussion:

	The result is accurate to about 1 part in 10**16.

Modified:

	27 December 2004

Author:

	Michael Wichura.
	C++ version by John Burkardt.
	
Reference:
	
	Michael Wichura,
	The Percentage Points of the Normal Distribution,
	Algorithm AS 241,
	Applied Statistics,
	Volume 37, Number 3, pages 477-484, 1988.
	
Parameters:

	Input, real ( kind = 8 ) P, the value of the cumulative probability 
	densitity function.  0 < P < 1.  If P is outside this range, an
	"infinite" value is returned.
	
	Output, real ( kind = 8 ) NORMAL_01_CDF_INVERSE, the normal deviate value 
	with the property that the probability of a standard normal deviate being 
	less than or equal to this value is P.
	
*/

template <class Real>
Real NormalCdfInv(const Real &p)
{
	static double a[8] = {
  		3.3871328727963666080,     1.3314166789178437745e+2,
    	1.9715909503065514427e+3,  1.3731693765509461125e+4,
    	4.5921953931549871457e+4,  6.7265770927008700853e+4,
    	3.3430575583588128105e+4,  2.5090809287301226727e+3 };
  	static double b[8] = {
    	1.0,                       4.2313330701600911252e+1,
    	6.8718700749205790830e+2,  5.3941960214247511077e+3,
    	2.1213794301586595867e+4,  3.9307895800092710610e+4,
    	2.8729085735721942674e+4,  5.2264952788528545610e+3 };
	static double c[8] = {
    	1.42343711074968357734,     4.63033784615654529590,
    	5.76949722146069140550,     3.64784832476320460504,
    	1.27045825245236838258,     2.41780725177450611770e-1,
    	2.27238449892691845833e-2,  7.74545014278341407640e-4 };
    	
    double const1 = 0.180625;
    double const2 = 1.6;
    
	static double d[8] = {
    	1.0,                        2.05319162663775882187,
    	1.67638483018380384940,     6.89767334985100004550e-1,
    	1.48103976427480074590e-1,  1.51986665636164571966e-2,
    	5.47593808499534494600e-4,  1.05075007164441684324e-9 };
	static double e[8] = {
    	6.65790464350110377720,     5.46378491116411436990,
    	1.78482653991729133580,     2.96560571828504891230e-1,
    	2.65321895265761230930e-2,  1.24266094738807843860e-3,
    	2.71155556874348757815e-5,  2.01033439929228813265e-7 };
	static double f[8] = {
    	1.0,                        5.99832206555887937690e-1,
    	1.36929880922735805310e-1,  1.48753612908506148525e-2,
    	7.86869131145613259100e-4,  1.84631831751005468180e-5,
    	1.42151175831644588870e-7,  2.04426310338993978564e-15 };

	Real q;
	Real r;
	double split1 = 0.425;
	double split2 = 5.0;
	Real value;
    
    if(p <= Zero)
    {
    	value = -d_huge();
    	return value;
	}
	
	if(One <= p)
	{
		value = d_huge();
		return value;
	}
	
	q = p - Half;
	
	if(abs(q) <= split1)
	{
		r = const1 - q * q;
		value = q * dpoly_value(8,a,r) / dpoly_value(8,b,r);
	}
	else
	{
		if(q < Zero) r = p;
		else        r = One - p;
		
		if(r <= Zero)
		{
			value = -One;
			Print("Error in NormalCdfInv");
			Print("Fatal Error");
			HaltExecution();
		}
		
		r = sqrt(-log(r));

   		if(r <= split2)
   		{
   			r = r - const2;
   			value = dpoly_value(8,c,r) / dpoly_value(8,d,r);
   		}
   		else
   		{
   			r = r - split2;
   			value = dpoly_value(8,e,r) / dpoly_value(8,f,r);
   		}
   		
   		if(q < Zero) value = -value;
   	}
   	
   	return value;
}

/*

This is an alternative routine for evaluateign
the inverse of the normal cdf. It is somewaht faster
than the above routine, presumably becuase it uses a
shorter series.

Computes the Inverse Normal CDF. The Algorithm is taken from
Kennedy and Gentle, Statistical Computing, pg. 95.

Hajivassiliou: This algorithm seems to be very accurate.  It is
rated to be accurate to 1e-7.  However, it appears to be
substantially more accurate -- approximately 1e-10.

(Input)		p:	probability
(Output)	xp: the corresponding value such that NormalCdf(xp) = p

*/

double NormalCdfInv2(double p)
{
	/* Constants */
	static double lim = 1.0e20;
	static double p0 = -0.322232431088;				static double q0 = 0.0993484626060;
	static double p1 = -1.0;						static double q1 = 0.588581570495;
	static double p2 = -0.342242088547;				static double q2 = 0.531103462366;
	static double p3 = -0.0204231210245;			static double q3 = 0.103537752850;
	static double p4 = -0.453642210148 * 1.0e-4;	static double q4 = 0.38560700634 * 1.0e-2;

	/* Check for Extreme Values */
	if(p < 0.00000001) return -Inf;
	if(p > 0.99999999) return Inf;	
	
	/* Handle p = 0.5 and p > 0.5 */
	bool gt = false;
	if(p == 0.5) return 0.0;
	else if(p > 0.5)
	{
		p = 1.0 - p;
		gt = true;
	}
	
	/* Computation of function for p < 0.5 */
	double y = sqrt(-2.0 * log(p));
	double xp = y + ((((y * p4 + p3) * y + p2) * y + p1) * y + p0)
		/ ((((y * q4 + q3) * y + q2) * y + q1) * y + q0);

	/* Convert results for p > 0.5  */
	if(gt) return xp;
	else return - xp;
}


/*

Purpose:

	Returns some values of the Normal 01 CDF.

Discussion: 

	In Mathematica, the function can be evaluated by:
	
	Needs["Statistics`ContinuousDistributions`"]
	dist = NormalDistribution [ 0, 1 ]
	CDF [ dist, x ]

Modified:	28 August 2004

Author:	John Burkardt

Reference:

	Milton Abramowitz and Irene Stegun,
	Handbook of Mathematical Functions,
	US Department of Commerce, 1964.
	
	Stephen Wolfram,
	The Mathematica Book,
	Fourth Edition,
	Wolfram Media / Cambridge University Press, 1999.

Parameters:

	Input/output, int *N_DATA.  The user sets N_DATA to 0 before the
	first call.  On each call, the routine increments N_DATA by 1, and
	returns the corresponding data; when there is no more data, the
	output value of N_DATA will be 0 again.
	
	Output, double *X, the argument of the function.
	
	Output, double *FX, the value of the function.
	
*/

void NormalCdfValues(int *n_data, double *x, double *fx)
{
	#define N_MAX 17
	
	double fx_vec[N_MAX] = {
		0.5000000000000000E+00,
		0.5398278372770290E+00,
		0.5792597094391030E+00,
		0.6179114221889526E+00,
		0.6554217416103242E+00,
		0.6914624612740131E+00,
		0.7257468822499270E+00,
		0.7580363477769270E+00,
		0.7881446014166033E+00,
		0.8159398746532405E+00,
		0.8413447460685429E+00,
		0.9331927987311419E+00,
		0.9772498680518208E+00,
		0.9937903346742239E+00,
		0.9986501019683699E+00,
		0.9997673709209645E+00,
		0.9999683287581669E+00 };
		
	double x_vec[N_MAX] = {
		0.0000000000000000E+00,
		0.1000000000000000E+00,
		0.2000000000000000E+00,
		0.3000000000000000E+00,
		0.4000000000000000E+00,
		0.5000000000000000E+00,
		0.6000000000000000E+00,
		0.7000000000000000E+00,
		0.8000000000000000E+00,
		0.9000000000000000E+00,
		0.1000000000000000E+01,
		0.1500000000000000E+01,
		0.2000000000000000E+01,
		0.2500000000000000E+01,
		0.3000000000000000E+01,
		0.3500000000000000E+01,
		0.4000000000000000E+01 };
		
	if(*n_data < 0)
	{
		*n_data = 0;
	}
	
	*n_data = *n_data + 1;
	
	if(N_MAX < *n_data)
	{
		*n_data = 0;
		*x = 0.0;
		*fx = 0.0;
	}
	else
	{
		*x = x_vec[*n_data-1];
		*fx = fx_vec[*n_data-1];
	}
	
	return;
	
	#undef N_MAX
}

/*

Purpose:

	NORMAL_01_MEAN returns the mean of the Normal 01 PDF.

Modified:

	18 September 2004

Author:

	John Burkardt

Parameters:

	Output, double MEAN, the mean of the PDF.

*/

double NormalMean()
{
	return Zero;
}


/*

Purpose:

	NormalPdf(x) evaluates the Normal 01 PDF.

Discussion:

	The Normal 01 PDF is also called the "Standard Normal" PDF, or
	the Normal PDF with 0 mean and variance 1.
	
Formula:
	
	PDF(X) = exp (-0.5 * X^2) / sqrt(2 * Pi)

Modified:

	18 September 2004

Author:

	John Burkardt

Parameters:

	Input, double X, the argument of the PDF.
	
	Output, double PDF, the value of the PDF.

*/

template <class Real>
Real NormalPdf(Real x)
{
	return exp(-0.5 * x * x) / Sqrt2Pi;
}


/*

Purpose: NORMAL_01_SAMPLE samples the standard normal probability distribution.

Discussion:

	The standard normal probability distribution function (PDF) has 
	mean 0 and standard deviation 1.
	
Method:

	The Box-Muller method is used, which is efficient, but 
	generates two values at a time.
	
Modified: 18 September 2004

Author: John Burkardt

Parameters:

	Input/output, int *SEED, a seed for the random number generator.
	
	Output, double NORMAL_01_SAMPLE, a normally distributed random value.

*/

double NormalSample(long &RandomSeed, bool Reset)
{
	double r1;
	double r2;
	static int used = -1;
	double x;
	static double y = 0.0;
	
	if(used == -1) used = 0;
	
	if(Reset)
	{
		used = -1;
		return Zero;
	}
	
	/*
	
	If we've used an even number of values so far, generate two more, return one,
	and save one.
	
	*/
	
	if((used % 2) == 0)
	{
		for(;;)
		{
			r1 = UniformSample(RandomSeed);
			if(r1 != 0.0) break;
		}
		
		r2 = UniformSample(RandomSeed);
		
		x = sqrt(-2.0 * log(r1)) * cos(2.0 * Pi * r2);
		y = sqrt(-2.0 * log(r1)) * sin(2.0 * Pi * r2);
	}
	else
	{
		x = y;
	}
	
	used = used + 1;
	
	return x;
}


/*

Purpose:

	Returns the variance of the Normal 01 PDF.

Modified:

	18 September 2004

Author:

	John Burkardt

Parameters:

	Output, double VARIANCE, the variance of the PDF.

*/

double NormalVariance()
{
	return One;
}


/*

Purpose:

	NORMAL_01_VECTOR samples the standard normal probability distribution.

Discussion:

	The standard normal probability distribution function (PDF) has
	mean 0 and standard deviation 1.
	
	This routine can generate a vector of values on one call.  It
	has the feature that it should provide the same results
	in the same order no matter how we break up the task.
	
	Before calling this routine, the user may call RANDOM_SEED
	in order to set the seed of the random number generator.
	
Method:

	The Box-Muller method is used, which is efficient, but
	generates an even number of values each time.  On any call
	to this routine, an even number of new values are generated.
	Depending on the situation, one value may be left over.
	In that case, it is saved for the next call.
	
Modified:

	18 October 2004

Author:

	John Burkardt

Parameters:

	Input, int N, the number of values desired.  If N is negative,
	then the code will flush its internal memory; in particular,
	if there is a saved value to be used on the next call, it is
	instead discarded.  This is useful if the user has reset the
	random number seed, for instance.
	
	Input/output, int *SEED, a seed for the random number generator.
	
	Output, double X(N), a sample of the standard normal PDF.
	
Local parameters:

	Local, int MADE, records the number of values that have
	been computed.  On input with negative N, this value overwrites
	the return value of N, so the user can get an accounting of
	how much work has been done.
	
	Local, real R(N+1), is used to store some uniform random values.
	Its dimension is N+1, but really it is only needed to be the
	smallest even number greater than or equal to N.
	
	Local, int SAVED, is 0 or 1 depending on whether there is a
	single saved value left over from the previous call.
	
	Local, int X_LO, X_HI, records the range of entries of
	X that we need to compute.  This starts off as 1:N, but is adjusted
	if we have a saved value that can be immediately stored in X(1),
	and so on.
	
	Local, real Y, the value saved from the previous call, if
	SAVED is 1.
	
*/

/*
double *NormalVector(int n, int *seed)
{
	int i;
	int m;
	static int made = 0;
	double *r;
	static int saved = 0;
	double *x;
	int x_hi;
	int x_lo;
	static double y = 0.0;

	x = new double[n];
	
	/*
	
	I'd like to allow the user to reset the internal data.
	But this won't work properly if we have a saved value Y.
	I'm making a crock option that allows the user to signal
	explicitly that any internal memory should be flushed,
	by passing in a negative value for N.
	
	*/
/*
	if(n < 0)
	{
		made = 0;
		saved = 0;
		y = 0.0;
		return NULL;
	}
	else if(n == 0)
	{
		return NULL;
	}
	
	/* Record the range of X we need to fill in. */
/*	x_lo = 1;
	x_hi = n;
	
	/* Use up the old value, if we have it. */
/*	if(saved == 1)
	{
		x[0] = y;
		saved = 0;
		x_lo = 2;
	}
		
	if(x_hi - x_lo + 1 == 0)
	{
		/* Maybe we don't need any more values. */
/*	
	}
	else if(x_hi - x_lo + 1 == 1)
	{
		/* If we need just one new value, do that here to avoid null arrays. */
/*		r = dvec_uniform_01(2,seed);
		
		x[x_hi-1] = sqrt(-2.0 * log(r[0])) * cos(2.0 * Pi * r[1]);
		y         = sqrt(-2.0 * log(r[0])) * sin(2.0 * Pi * r[1]);

		saved = 1;
		made = made + 2;
		delete[] r;
	}
	else if((x_hi - x_lo + 1) % 2 == 0)
	{
		/* If we require an even number of values, that's easy. */
/*		m = (x_hi - x_lo + 1) / 2;

		r = dvec_uniform_01(2*m,seed);

		for(i=0; i<=2*m-2; i=i+2)
		{
			x[x_lo+i-1] = sqrt(-2.0 * log(r[i])) * cos(2.0 * Pi * r[i+1]);
			x[x_lo+i  ] = sqrt(-2.0 * log(r[i])) * sin(2.0 * Pi * r[i+1]);
		}
		
		made = made + x_hi - x_lo + 1;
		delete [] r;
	}
	else
	{
	
		/*
	
		If we require an odd number of values, we generate an even number,
		and handle the last pair specially, storing one in X(N), and
		saving the other for later.
		
		*/
/*	
		x_hi = x_hi - 1;
		
		m = (x_hi - x_lo + 1) / 2 + 1;
		
		r = dvec_uniform_01(2*m,seed);
		
		for(i=0; i<=2*m-4; i=i+2)
		{
			x[x_lo+i-1] = sqrt(-2.0 * log(r[i])) * cos(2.0 * Pi * r[i+1]);
			x[x_lo+i  ] = sqrt(-2.0 * log(r[i])) * sin(2.0 * Pi * r[i+1]);
		}

		i = 2*m - 2;
		
		x[x_lo+i-1] = sqrt(-2.0 * log(r[i])) * cos(2.0 * Pi * r[i+1]);
		y           = sqrt(-2.0 * log(r[i])) * sin(2.0 * Pi * r[i+1]);

		saved = 1;
		
		made = made + x_hi - x_lo + 2;
		
		delete[] r;
	}
	
	return x;
}
*/

/*

Purpose:

	NORMAL_CDF_INV inverts the Normal CDF.

Modified:

	19 September 2004

Author:

	John Burkardt

Reference:

	Algorithm AS 111, 
	Applied Statistics, 
	Volume 26, pages 118-121, 1977.

Parameters:

	Input, double CDF, the value of the CDF.
	0.0 <= CDF <= 1.0.
	
	Input, double A, B, the parameters of the PDF.
	0.0 < B.
	
	Output, double X, the corresponding argument.

*/

double NormalCdfInv(double cdf, double a, double b)
{
	
	if(cdf < Zero || cdf > One)
	{
		Print("Error in NormalCdfInv");
		Print("Cdf < 0 Or Cdf > 1");
		Print("Cdf",cdf);
		HaltExecution();
	}
    
	double x2 = NormalCdfInv(cdf);
	double x = a + b * x2;
	return x;
}


/*

Purpose:

	NORMAL_CDF_VALUES returns some values of the Normal CDF.

Discussion: 

	In Mathematica, the function can be evaluated by:
	
	Needs["Statistics`ContinuousDistributions`"]
	dist = NormalDistribution [ mu, sigma ]
	CDF [ dist, x ]
	
Modified:

	05 August 2004

Author:

	John Burkardt

Reference:

	Milton Abramowitz and Irene Stegun,
	Handbook of Mathematical Functions,
	US Department of Commerce, 1964.
	
	Stephen Wolfram,
	The Mathematica Book,
	Fourth Edition,
	Wolfram Media / Cambridge University Press, 1999.
	
Parameters:

	Input/output, int *N_DATA.  The user sets N_DATA to 0 before the
	first call.  On each call, the routine increments N_DATA by 1, and
	returns the corresponding data; when there is no more data, the
	output value of N_DATA will be 0 again.
	
	Output, double *MU, the mean of the distribution.
	
	Output, double *SIGMA, the variance of the distribution.
	
	Output, double *X, the argument of the function.
	
	Output, double *FX, the value of the function.
	
*/

void NormalCdfValues(int *n_data, double *mu, double *sigma, double *x, double *fx)
{
	#define N_MAX 12
	
	double fx_vec[N_MAX] = {
		0.5000000000000000E+00,
		0.9772498680518208E+00,
		0.9999683287581669E+00,
		0.9999999990134124E+00,
		0.6914624612740131E+00,
		0.6305586598182364E+00,
		0.5987063256829237E+00,
		0.5792597094391030E+00,
		0.6914624612740131E+00,
		0.5000000000000000E+00,
		0.3085375387259869E+00,
		0.1586552539314571E+00 };
		
	double mu_vec[N_MAX] = {
		0.1000000000000000E+01,
		0.1000000000000000E+01,
		0.1000000000000000E+01,
		0.1000000000000000E+01,
		0.1000000000000000E+01,
		0.1000000000000000E+01,
		0.1000000000000000E+01,
		0.1000000000000000E+01,
		0.2000000000000000E+01,
		0.3000000000000000E+01,
		0.4000000000000000E+01,
		0.5000000000000000E+01 };
		
	double sigma_vec[N_MAX] = {
		0.5000000000000000E+00,
		0.5000000000000000E+00,
		0.5000000000000000E+00,
		0.5000000000000000E+00,
		0.2000000000000000E+01,
		0.3000000000000000E+01,
		0.4000000000000000E+01,
		0.5000000000000000E+01,
		0.2000000000000000E+01,
		0.2000000000000000E+01,
		0.2000000000000000E+01,
		0.2000000000000000E+01 };
		
	double x_vec[N_MAX] = {
		0.1000000000000000E+01,
		0.2000000000000000E+01,
		0.3000000000000000E+01,
		0.4000000000000000E+01,
		0.2000000000000000E+01,
		0.2000000000000000E+01,
		0.2000000000000000E+01,
		0.2000000000000000E+01,
		0.3000000000000000E+01,
		0.3000000000000000E+01,
		0.3000000000000000E+01,
		0.3000000000000000E+01 };
		
	if(*n_data < 0)
	{
		*n_data = 0;
	}
	
	*n_data = *n_data + 1;
	
	if(N_MAX < *n_data)
	{
		*n_data = 0;
		*mu = 0.0;
		*sigma = 0.0;
		*x = 0.0;
		*fx = 0.0;
	}
	else
	{
		*mu = mu_vec[*n_data-1];
		*sigma = sigma_vec[*n_data-1];
		*x = x_vec[*n_data-1];
		*fx = fx_vec[*n_data-1];
	}
	
	return;
	
	#undef N_MAX
}


/*

Purpose: NORMAL_CHECK checks the parameters of the Normal PDF.

Modified: 19 September 2004

Author: John Burkardt

Parameters:

	Input, double A, B, the parameters of the PDF.
	0.0 < B.
	
	Output, bool NORMAL_CHECK, is true if the parameters are legal.

*/

/*
bool NormalCheck(double Mu, double Sigma)
{
	if(Sigma <= 0.0)
	{
		Print("Error in NormalCheck");
		Print("Sigma <= 0.");
		Print("Sigma",Sigma);
		return false;
	}
	return true;
}
*/


/*

Purpose: NORMAL_MEAN returns the mean of the Normal PDF.

Modified: 19 September 2004

Author: John Burkardt

Parameters:

	Input, double A, B, the parameters of the PDF.
	0.0 < B.
	
	Output, double MEAN, the mean of the PDF.
	
*/

/*
double NormalMean(double a, double b)
{
	double mean;
	mean = a;
	return mean;
}
*/

/*

Purpose: NORMAL_PDF evaluates the Normal PDF.

Formula:

	PDF(A,B;X) 
		=   exp ( - 0.5 * ( ( X - A ) / B )**2 )
		  / ( B * SQRT ( 2 * PI ) )
		  
Discussion: The normal PDF is also known as the Gaussian PDF.

Modified: 19 September 2004

Author: John Burkardt

Parameters:

	Input, double X, the argument of the PDF.
	
	Input, double A, B, the parameters of the PDF.
	0.0 < B.
	
	Output, double PDF, the value of the PDF.
	
*/

double NormalPdf(double x, double Mu, double Sigma)
{
	return exp(-0.5 * Sqr((x - Mu) / Sigma)) / (Sigma * Sqrt2Pi);
}

/*

Purpose: NORMAL_SAMPLE samples the Normal PDF.

Modified: 20 November 2004

Author: John Burkardt

Parameters:

	Input, double A, B, the parameters of the PDF.
	0.0 < B.
	
	Input/output, int *SEED, a seed for the random number generator.
	
	Output, double NORMAL_SAMPLE, a sample of the PDF.
	
*/

double NormalSample(double Mu, double Sigma, long &RandomSeed)
{
	double x;
	x = NormalSample(RandomSeed);
	x = Mu + Sigma * x;
	return x;
}


/*

Purpose: NORMAL_VARIANCE returns the variance of the Normal PDF.

Modified: 19 September 2004

Author: John Burkardt

Parameters:

	Input, double A, B, the parameters of the PDF.
	0.0 < B.
	
	Output, double VARIANCE, the variance of the PDF.

*/

/*
double NormalVariance(double a, double b)
{
	double variance;
	variance = b * b;
	return variance;
}
*/


/*

Purpose: NORMAL_VECTOR samples the normal probability distribution.

Discussion:

	The normal probability distribution function (PDF) has
	a user-specified mean and standard deviation.
	
	This routine can generate a vector of values on one call.  It
	has the feature that it should provide the same results
	in the same order no matter how we break up the task.
	
	Before calling this routine, the user may call RANDOM_SEED
	in order to set the seed of the random number generator.
	
Method:

	The Box-Muller method is used, which is efficient, but
	generates an even number of values each time.  On any call
	to this routine, an even number of new values are generated.
	Depending on the situation, one value may be left over.
	In that case, it is saved for the next call.
	
Modified: 18 October 2004

Author: John Burkardt

Parameters:

	Input, int N, the number of values desired.  If N is negative,
	then the code will flush its internal memory; in particular,
	if there is a saved value to be used on the next call, it is
	instead discarded.  This is useful if the user has reset the
	random number seed, for instance.
	
	Input, double MEAN, the desired mean value.
	
	Input, double DEV, the desired standard deviation.
	
	Input/output, int *SEED, a seed for the random number generator.
	
	Output, double NORMAL_VECTOR[N], a sample of the normal PDF.
	
*/

/*
double *NormalVector(int n, double mean, double dev, int *seed)
{
	int i;
	double *x;
	
	x = NormalVector(n,seed);

	for(i=0; i<n; i++) x[i] = mean + dev * x[i];
	
	return x;
}
*/

double dNormalPdf(double x)
{	
	return -x * NormalPdf(x);
}



double ddNormalPdf(double x)
{
	return (Sqr(x) - One) * NormalPdf(x);
}

template <class Real>
Real NormalCdfRatio(Real x1, Real x2)
{
	if(x1 >= -1.28 || x2 >= -1.28) return NormalCdf(x1) / NormalCdf(x2);

	static Real a1  = 0.398942280444E+00;
	static Real a2  = 0.399903438504E+00;
	static Real a3  = 5.75885480458E+00;
	static Real a4  = 29.8213557808E+00;
	static Real a5  = 2.62433121679E+00;
	static Real a6  = 48.6959930692E+00;
	static Real a7  = 5.92885724438E+00;
	static Real b0  = 0.398942280385E+00;
	static Real b1  = 3.8052E-08;
	static Real b2  = 1.00000615302E+00;
	static Real b3  = 3.98064794E-04;
	static Real b4  = 1.98615381364E+00;
	static Real b5  = 0.151679116635E+00;
	static Real b6  = 5.29330324926E+00;
	static Real b7  = 4.8385912808E+00;
	static Real b8  = 15.1508972451E+00;
	static Real b9  = 0.742380924027E+00;
	static Real b10 = 30.789933034E+00;
	static Real b11 = 3.99019417011E+00;
	
	Real absx1 = Abs(x1);
	Real absx2 = Abs(x2);
	
	Real y1 = 0.5 * x1 * x1;
	Real z1 =   b0 / (absx1 - b1 + b2 / (absx1 + b3 + b4 / (absx1 - b5
	          + b6 / (absx1 + b7 - b8 / (absx1 + b9 + b10 / (absx1 + b11))))));
	   
	Real y2 = 0.5 * x2 * x2;
	Real z2 =   b0 / (absx2 - b1 + b2 / (absx2 + b3 + b4 / (absx2 - b5
	     	  + b6 / (absx2 + b7 - b8 / (absx2 + b9 + b10 / (absx2 + b11))))));
	
	return exp(-y1 + y2) * z1 / z2;
}

double NormalPVal(double ZStat)
{
	return Two * NormalCdf(-abs(ZStat));
}

double NormalPVal(double Est, double Se)
{
	return NormalPVal(Est / Se);
}

string NormalStars(double ZStat)
{
	double pval = NormalPVal(ZStat);
	     if(pval <= 0.001) return "***";
	else if(pval <= 0.01)  return "**";
	else if(pval <= 0.05)  return "*";
	else if(pval <= 0.1)   return "+";
	else                   return "";
}

string NormalStars(double Est, double Se)
{
	return NormalStars(Est / Se);
}

#endif
