
#ifndef DistStudent
#define DistStudent



	double StudentCdf(double x, double a, double b, double c);
	void StudentCdfValues( int *n_data, int *a, double *x, double *fx );
	//bool StudentCheck(double a, double b, double c);
	//double StudentMean(double a, double b, double c);
	double StudentPdf(double x, double a, double b, double c);
	double StudentSample(double a, double b, double c, long &RandomSeed);
	//double StudentVariance(double a, double b, double c);
	double StudentCdfInv(double p, int df);
	

/*

Purpose:

	STUDENT_CDF evaluates the central Student T CDF.
	
Modified:

	18 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double X, the argument of the CDF.
	
	Input, double A, B, C, the parameters of the PDF.
	0 < B,
	0 < C.
	
	Output, double STUDENT_CDF, the value of the CDF.
	
*/

double StudentCdf(double x, double a, double b, double c)
{
	double y = (x - a) / b;
	double a2 = 0.5 * c;
	double b2 = 0.5;
	double c2 = c / (c + y * y);
	
	if(y <= Zero) return       Half * BetaInc(a2,b2,c2);
	else          return One - Half * BetaInc(a2,b2,c2);
}

/*

Purpose:

	TFN calculates the T function of Owen.
	
Discussion:

	Owen's T function is useful for computation of the bivariate normal
	distribution and the distribution of a skewed normal distribution.
	
	Although it was originally formulated in terms of the bivariate
	normal function, the function can be defined more directly as
	
    T(H,A) = 1 / ( 2 * PI ) *
       Integral ( 0 <= X <= A ) e^( -H^2 * (1+X^2) / 2 ) / (1+X^2) dX

Modified:

	10 December 2004

Reference:

	D B Owen,
	Tables for computing the bivariate normal distribution,
	Annals of Mathematical Statistics,
	Volume 27, pages 1075-1090, 1956.
	
	J C Young and C E Minder,
	Algorithm AS 76,
	An Algorithm Useful in Calculating Non-Central T and
	Bivariate Normal Distributions,
	Applied Statistics,
	Volume 23, Number 3, 1974, pages 455-457.
	
Parameters:

	Input, double H, A, the arguments of the T function.
	
	Output, double TFN, the value of the T function.
	
*/

double tfn(double h, double a)
{
	#define NGAUSS 10
	
	double as;
	double h1;
	double h2;
	double hs;
	int i;
	double r1;
	double r2;
	double rt;
	double two_pi_inverse = 0.1591549430918953;
	double tv1 = 1.0E-35;
	double tv2 = 15.0;
	double tv3 = 15.0;
	double tv4 = 1.0E-05;
	double value;
	double weight[NGAUSS] = {
		0.666713443086881375935688098933E-01,
		0.149451349150580593145776339658E+00,
		0.219086362515982043995534934228E+00,
		0.269266719309996355091226921569E+00,
		0.295524224714752870173892994651E+00,
		0.295524224714752870173892994651E+00,
		0.269266719309996355091226921569E+00,
		0.219086362515982043995534934228E+00,
		0.149451349150580593145776339658E+00,
		0.666713443086881375935688098933E-01 };
	double x;
	double xtab[NGAUSS] = {
		-0.973906528517171720077964012084E+00,
		-0.865063366688984510732096688423E+00,
		-0.679409568299024406234327365115E+00,
		-0.433395394129247190799265943166E+00,
		-0.148874338981631210884826001130E+00,
		0.148874338981631210884826001130E+00,
		0.433395394129247190799265943166E+00,
		0.679409568299024406234327365115E+00,
		0.865063366688984510732096688423E+00,
		0.973906528517171720077964012084E+00 };
		
	if(fabs(h) < tv1)
	{
		/* Test for H near zero. */
		value = atan(a) * two_pi_inverse;
	}
	else if(tv2 < fabs(h))
	{
		/* Test for large values of abs(H). */
		value = 0.0;
	}
	else if(fabs(a) < tv1)
	{
		/* Test for A near zero. */
		value = 0.0;
	}
	else
	{
		/* Test whether abs(A) is so large that it must be truncated.
		   If so, the truncated value is H2. */
		hs = - 0.5 * h * h;
		h2 = a;
		as = a * a;
		
		/* Computation of truncation point by Newton iteration. */
		if(tv3 <= log(1.0 + as) - hs * as)
		{
			h1 = 0.5 * a;
			as = 0.25 * as;
			
			for(;;)
			{
				rt = as + 1.0;
				h2 = h1 + (hs * as + tv3 - log(rt)) / (2.0 * h1 * (1.0 / rt - hs));
				as = h2 * h2;
				
				if(fabs(h2 - h1) < tv4)
				{
					break;
				}
				h1 = h2;
			}
		}
		
		/* Gaussian quadrature on the interval [0,H2]. */
		rt = 0.0;
		for(i=0; i<NGAUSS; i++)
		{
			x = 0.5 * h2 * (xtab[i] + 1.0);
			rt = rt + weight[i] * exp(hs * (1.0 + x * x)) / (1.0 + x * x);
		}
		value = rt * 0.5 * h2 * two_pi_inverse;
	}
	
	return value;
	#undef NGAUSS
}

/*

Purpose:

	STUDENT_NONCENTRAL_CDF evaluates the noncentral Student T CDF.
	
Modified:

	18 October 2004
	
Reference:
	
	Algorithm AS 5,
	Applied Statistics,
	Volume 17, 1968, page 193.
	
Parameters:

	Input, double X, the argument of the CDF.
	
	Input, int IDF, the number of degrees of freedom.
	
	Input, double D, the noncentrality parameter.
	
	Output, double STUDENT_NONCENTRAL_CDF, the value of the CDF.
	
*/

double StudentNoncentralCdf(double x, int idf, double d)
{
	
	#define PI 3.141592653589793
	
	double a;
	int a_max = 100;
	double ak;
	double b;
	double cdf;
	double cdf2;
	double drb;
	double emin = 12.5;
	double f;
	double fk;
	double fmkm1;
	double fmkm2;
	int k;
	double sum2;
	double temp;
	
	f = ( double ) idf;
	
	if ( idf == 1 )
	{
		a = x / sqrt ( f );
		b = f / ( f + x * x );
		drb = d * sqrt ( b );
		
		cdf2 = NormalCdf( drb );
		cdf = 1.0 - cdf2 + 2.0 * tfn ( drb, a );
	}
	else if ( idf <= a_max )
	{
		a = x / sqrt ( f );
		b = f / ( f + x * x );
		drb = d * sqrt ( b );
		sum2 = 0.0;
		
		fmkm2 = 0.0;
		if ( fabs ( drb ) < emin )
		{
			cdf2 = NormalCdf ( a * drb );
			fmkm2 = a * sqrt ( b ) * exp ( - 0.5 * drb * drb ) * cdf2 / sqrt ( 2.0 * PI );
		}
		
		fmkm1 = b * d * a * fmkm2;
		if ( fabs ( d ) < emin )
		{
			fmkm1 = fmkm1 + 0.5 * b * a * exp ( - 0.5 * d * d ) / PI;
		}
		
		if ( idf % 2 == 0 )
		{
			sum2 = fmkm2;
		}
		else
		{
			sum2 = fmkm1;
		}
		
		ak = 1.0;
		
		for ( k = 2; k <= idf - 2; k = k + 2 )
		{
			fk = ( double ) ( k );
			
			fmkm2 = b * ( d * a * ak * fmkm1 + fmkm2 ) * ( fk - 1.0 ) / fk;
			ak = 1.0 / ( ak * ( fk - 1.0 ) );
			fmkm1 = b * ( d * a * ak * fmkm2 + fmkm1 ) * fk / ( fk + 1.0 );
			
			if ( idf % 2 == 0 )
			{
				sum2 = sum2 + fmkm2;
			}
			else
			{
				sum2 = sum2 + fmkm1;
			}
			
			ak = 1.0 / ( ak * fk );
		}
		
		if ( idf % 2 == 0 )
		{
			cdf2 = NormalCdf( d );
			cdf = 1.0 - cdf2 + sum2 * sqrt ( 2.0 * PI );
		}
		else
		{
			cdf2 = NormalCdf ( drb );
			cdf = 1.0 - cdf2 + 2.0 * ( sum2 + tfn ( drb, a ) );
		}
	}
	else
	{		
		/* Normal approximation. */
		a = sqrt(0.5 * f) * exp(gamma_log(0.5 * (f - 1.0)) - gamma_log(0.5 * f)) * d;
		temp = (x - a) / sqrt (f * (1.0 + d * d) / (f - 2.0) - a * a);
		cdf2 = NormalCdf(temp);
		cdf = cdf2;
	}
	
	return cdf;
	#undef PI
}

double StudentCdf(double x, double df)
{
	return StudentNoncentralCdf(x,df,Zero);
}

//******************************************************************************
//
//  Purpose:
//
//    STUDENT_NONCENTRAL_CDF_VALUES returns values of the noncentral Student CDF.
//
//  Discussion:
//
//    In Mathematica, the function can be evaluated by:
//
//      Needs["Statistics`ContinuousDistributions`"]
//      dist = NoncentralStudentTDistribution [ df, lambda ]
//      CDF [ dist, x ]
//
//    Mathematica seems to have some difficulty computing this function
//    to the desired number of digits.
//
//  Modified:
//
//    01 September 2004
//
//  Author:
//
//    John Burkardt
//
//  Reference:
//
//    Milton Abramowitz and Irene Stegun,
//    Handbook of Mathematical Functions,
//    US Department of Commerce, 1964.
//
//    Stephen Wolfram,
//    The Mathematica Book,
//    Fourth Edition,
//    Wolfram Media / Cambridge University Press, 1999.
//
//  Parameters:
//
//    Input/output, int *N_DATA.  The user sets N_DATA to 0 before the
//    first call.  On each call, the routine increments N_DATA by 1, and
//    returns the corresponding data; when there is no more data, the
//    output value of N_DATA will be 0 again.
//
//    Output, int *DF, double *LAMBDA, the parameters of the
//    function.
//
//    Output, double *X, the argument of the function.
//
//    Output, double *FX, the value of the function.
//


void student_noncentral_cdf_values ( int *n_data, int *df, double *lambda,
  double *x, double *fx )
{
# define N_MAX 30

  int df_vec[N_MAX] = {
     1,  2,  3,
     1,  2,  3,
     1,  2,  3,
     1,  2,  3,
     1,  2,  3,
    15, 20, 25,
     1,  2,  3,
    10, 10, 10,
    10, 10, 10,
    10, 10, 10 };

  double fx_vec[N_MAX] = {
     0.8975836176504333E+00,
     0.9522670169E+00,
     0.9711655571887813E+00,
     0.8231218864E+00,
     0.9049021510E+00,
     0.9363471834E+00,
     0.7301025986E+00,
     0.8335594263E+00,
     0.8774010255E+00,
     0.5248571617E+00,
     0.6293856597E+00,
     0.6800271741E+00,
     0.20590131975E+00,
     0.2112148916E+00,
     0.2074730718E+00,
     0.9981130072E+00,
     0.9994873850E+00,
     0.9998391562E+00,
     0.168610566972E+00,
     0.16967950985E+00,
     0.1701041003E+00,
     0.9247683363E+00,
     0.7483139269E+00,
     0.4659802096E+00,
     0.9761872541E+00,
     0.8979689357E+00,
     0.7181904627E+00,
     0.9923658945E+00,
     0.9610341649E+00,
     0.8688007350E+00 };

  double lambda_vec[N_MAX] = {
     0.0E+00,
     0.0E+00,
     0.0E+00,
     0.5E+00,
     0.5E+00,
     0.5E+00,
     1.0E+00,
     1.0E+00,
     1.0E+00,
     2.0E+00,
     2.0E+00,
     2.0E+00,
     4.0E+00,
     4.0E+00,
     4.0E+00,
     7.0E+00,
     7.0E+00,
     7.0E+00,
     1.0E+00,
     1.0E+00,
     1.0E+00,
     2.0E+00,
     3.0E+00,
     4.0E+00,
     2.0E+00,
     3.0E+00,
     4.0E+00,
     2.0E+00,
     3.0E+00,
     4.0E+00 };

  double x_vec[N_MAX] = {
      3.00E+00,
      3.00E+00,
      3.00E+00,
      3.00E+00,
      3.00E+00,
      3.00E+00,
      3.00E+00,
      3.00E+00,
      3.00E+00,
      3.00E+00,
      3.00E+00,
      3.00E+00,
      3.00E+00,
      3.00E+00,
      3.00E+00,
     15.00E+00,
     15.00E+00,
     15.00E+00,
      0.05E+00,
      0.05E+00,
      0.05E+00,
      4.00E+00,
      4.00E+00,
      4.00E+00,
      5.00E+00,
      5.00E+00,
      5.00E+00,
      6.00E+00,
      6.00E+00,
      6.00E+00 };
//
  if ( *n_data < 0 )
  {
    *n_data = 0;
  }

  *n_data = *n_data + 1;

  if ( N_MAX < *n_data )
  {
    *n_data = 0;
    *df = 0;
    *lambda = 0.0;
    *x = 0.0;
    *fx = 0.0;
  }
  else
  {
    *df = df_vec[*n_data-1];
    *lambda = lambda_vec[*n_data-1];
    *x = x_vec[*n_data-1];
    *fx = fx_vec[*n_data-1];
  }

  return;
# undef N_MAX
}




/*

Purpose:

	STUDENT_CDF_VALUES returns some values of the Student CDF.
	
Discussion:

	In Mathematica, the function can be evaluated by:
	
		Needs["Statistics`ContinuousDistributions`"]
		dist = StudentTDistribution [ df ]
		CDF [ dist, x ]
		
Modified:

	21 August 2004
	
Author:

	John Burkardt
	
Reference:

	Milton Abramowitz and Irene Stegun,
	Handbook of Mathematical Functions,
	US Department of Commerce, 1964.
	
	Stephen Wolfram,
	The Mathematica Book,
	Fourth Edition,
	Wolfram Media / Cambridge University Press, 1999.
	
Parameters:

	Input/output, int *N_DATA.  The user sets N_DATA to 0 before the
	first call.  On each call, the routine increments N_DATA by 1, and
	returns the corresponding data; when there is no more data, the
	output value of N_DATA will be 0 again.
	
	Output, int *A, the parameter of the function.
	Output, double *X, the argument of the function.
	Output, double *FX, the value of the function.
	
*/

void StudentCdfValues(int *n_data, int *a, double *x, double *fx)
{
	const int N_MAX = 13;
	
	int a_vec[N_MAX] = {
    1, 2, 3, 4,
    5, 2, 5, 2,
    5, 2, 3, 4,
    5 };

	double fx_vec[N_MAX] = {
     0.6000231200328521E+00,
     0.6001080279134390E+00,
     0.6001150934648930E+00,
     0.6000995134721354E+00,
     0.5999341989834830E+00,
     0.7498859393137811E+00,
     0.7500879487671045E+00,
     0.9500004222186464E+00,
     0.9499969138365968E+00,
     0.9900012348724744E+00,
     0.9900017619355059E+00,
     0.9900004567580596E+00,
     0.9900007637471291E+00 };

 	double x_vec[N_MAX] = {
     0.325E+00,
     0.289E+00,
     0.277E+00,
     0.271E+00,
     0.267E+00,
     0.816E+00,
     0.727E+00,
     2.920E+00,
     2.015E+00,
     6.965E+00,
     4.541E+00,
     3.747E+00,
     3.365E+00 };

	if(*n_data < 0) *n_data = 0;
	
	*n_data = *n_data + 1;

	if(N_MAX < *n_data)
	{
		*n_data = 0;
		*a = 0;
		*x = 0.0;
		*fx = 0.0;
	}
	else
	{
		*a = a_vec[*n_data-1];
		*x = x_vec[*n_data-1];
		*fx = fx_vec[*n_data-1];
	}
}


/*

Purpose:

	STUDENT_CHECK checks the parameter of the central Student T CDF.
	
Modified:

	14 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double A, B, C, the parameters of the PDF.
	0 < B,
	0 < C.
	
*/

/*
bool StudentCheck(double a, double b, double c)
{
	if(b <= 0.0)
	{
		cout << " \n";
		cout << "STUDENT_CHECK - Fatal error!\n";
		cout << "  B must be greater than 0.\n";
		return false;
	}
	
	if(c <= 0.0)
	{
		cout << " \n";
		cout << "STUDENT_CHECK - Fatal error!\n";
		cout << "  C must be greater than 0.\n";
		return false;
	}
	return true;
}
*/


/*

Purpose:

	STUDENT_MEAN returns the mean of the central Student T PDF.
	
Modified:

	18 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double X, the argument of the PDF.
	
	Input, double A, B, C, the parameter of the PDF.
	0 < B,
	0 < C.
	
	Output, double STUDENT_MEAN, the mean of the PDF.
	
*/

/*
double StudentMean(double a, double b, double c)
{
	double mean;
	mean = a;
	return mean;
}
*/

/*

Purpose:

	STUDENT_PDF evaluates the central Student T PDF.
	
Formula:
	
	PDF(A,B,C;X) = Gamma ( (C+1)/2 ) /
		( Gamma ( C / 2 ) * Sqrt ( PI * C )
		* ( 1 + ((X-A)/B)**2/C )**(C + 1/2 ) )
		
Modified:
	
	18 October 2004
		
Author:
	
	John Burkardt
		
Parameters:
	
	Input, double X, the argument of the PDF.
		
	Input, double A, B, C, the parameters of the PDF.
	0 < B,
	0 < C.
	
	Output, double STUDENT_PDF, the value of the PDF.
	
*/

double StudentPdf(double x, double a, double b, double c)
{
	double y = (x - a) / b;
	double pdf =   gamma(Half * (c + One)) / (sqrt(Pi * c)
          		 * gamma(Half * c)
		         * sqrt(pow((One + y * y / c),(Two * c + One))));
	return pdf;
}


/*

Purpose:

	STUDENT_SAMPLE samples the central Student T PDF.
	
Modified:

	18 October 2004
	
Author:

	John Burkardt

Parameters:

	Input, double A, B, C, the parameters of the PDF.
	0 < B,
	0 < C.
	
	Input/output, int *SEED, a seed for the random number generator.
	
	Output, double STUDENT_SAMPLE, a sample of the PDF.
	
*/

double StudentSample(double a, double b, double c, long &RandomSeed)
{
	if(c < Three)
	{
		Print("Error in StudentSample");
		Print("Sampling Fails for C < 3");
		HaltExecution();
	}
	
	double a2 = Zero;
	double b2 = c  / (c - Two);
	double x2 = NormalSample(a2,b2,RandomSeed);
	double x3 = ChiSquareSample(c,RandomSeed) * c / (c - Two);
	double x = a + b * x2 * sqrt(c) / x3;
	return x;
}


/*

Purpose:

	STUDENT_VARIANCE returns the variance of the central Student T PDF.
	
Modified:

	18 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double A, B, C, the parameter of the PDF.
	0 < B,
	0 < C.
	However, the variance is not defined unless 2 < C
	
	Output, double STUDENT_VARIANCE, the variance of the PDF.
	
*/

/*
double StudentVariance(double a, double b, double c)
{
	double variance;
	
	if(c < 3.0)
	{
		cout << " \n";
		cout << "STUDENT_VARIANCE - Fatal error!\n";
		cout << "  Variance not defined for C <= 2.\n";
		HaltExecution();
	}
	
	variance = b * b * c / ( c - 2.0 );
	
	return variance;
}
*/

/*

Purpose:

	CDFT evaluates the CDF of the T distribution.
	
Discussion:

	This routine calculates any one parameter of the T distribution 
	given the others.
	
	The value P of the cumulative distribution function is calculated 
	directly.
	
	Computation of other parameters involve a seach for a value that
	produces the desired value of P.   The search relies on the
	monotonicity of P with respect to the other parameters.
	
	The original version of this routine allowed the search interval
	to extend from -1.0E+300 to +1.0E+300, which is fine until you 
	try to evaluate a function at such a point!
	
Reference:

	Abramowitz and Stegun,  
	Handbook of Mathematical Functions 
	1966, Formula 26.5.27.
	
Parameters:

	Input, int *WHICH, indicates which argument is to be calculated
	from the others.
	1 : Calculate P and Q from T and DF;
	2 : Calculate T from P, Q and DF;
	3 : Calculate DF from P, Q and T.
	
	Input/output, double *P, the integral from -infinity to T of 
	the T-density.  Whether an input or output value, this will lie in the
	range [0,1].
	
	Input/output, double *Q, equal to 1-P.  If Q is an input
	value, it should lie in the range [0,1].  If Q is an output value,
	it will lie in the range [0,1].
	
	Input/output, double *T, the upper limit of integration of 
	the T-density.  If this is an input value, it may have any value.
	It it is an output value, it will be searched for in the range
	[ -1.0D+30, 1.0D+30 ].
	
	Input/output, double *DF, the number of degrees of freedom
	of the T distribution.  If this is an input value, it should lie
	in the range: (0 , +infinity).  If it is an output value, it will be
	searched for in the range: [1, 1.0D+10].
	
	Output, int *STATUS, reports the status of the computation.
	  0, if the calculation completed correctly;
	 -I, if the input parameter number I is out of range;
	 +1, if the answer appears to be lower than lowest search bound;
	 +2, if the answer appears to be higher than greatest search bound;
	 +3, if P + Q /= 1.
	 
	Output, double *BOUND, is only defined if STATUS is nonzero.
	If STATUS is negative, then this is the value exceeded by parameter I.
	if STATUS is 1 or 2, this is the search bound that was exceeded.

*/

/*
void cdft(int *which, double *p, double *q, double *t, double *df, int *status, double *bound )
{
	#define tol (1.0e-8)
	#define atol (1.0e-50)
	#define zero (1.0e-300)
	#define inf 1.0e30
	#define maxdf 1.0e10
	static int K1 = 1;
	static double K4 = 0.5e0;
	static double K5 = 5.0e0;
	static double fx,cum,ccum,pq;
	static unsigned long qhi,qleft,qporq;
	static double T2,T3,T6,T7,T8,T9,T10,T11;
	
	// Check arguments
	
	
S30:
   
   	// P, Q, and DF
   	
   	if(*p < Zero || *p > One || *q < Zero || *q > One || *df < Zero)
   	{	
   		Print("Err");
   		HaltExecution();
   	}
   	
  	// P + Q
  	
  	pq = *p + *q;
  	if(abs(pq - One) > 0.0000000001)
  	{
  		Print("Err2");
  		HaltExecution();
  	}
  	
  	/*
  	
  	Computing T
  	.. Get initial approximation for T
  	
  	*/
/*    	
    *t = dt1(p,q,df);
	T2 = -inf;
	T3 = inf;
	T6 = atol;
	T7 = tol;
	dstinv(&T2,&T3,&K4,&K4,&K5,&T6,&T7);
	*status = 0;
	dinvr(status,t,&fx,&qleft,&qhi);
S180:
	if(!(*status == 1)) goto S210;
	cumt(t,df,&cum,&ccum);
	if(!qporq) goto S190;
	fx = cum-*p;
	goto S200;
S190:
    fx = ccum-*q;
S200:
    dinvr(status,t,&fx,&qleft,&qhi);
    goto S180;
S210:
        if(!(*status == -1)) goto S240;
        if(!qleft) goto S220;
        *status = 1;
        *bound = -inf;
        goto S230;
S220:
        *status = 2;
        *bound = inf;
S240:
S230:
        ;
    }
   
S310:
    return;
#undef tol
#undef atol
#undef zero
#undef inf
#undef maxdf
}
*/

/*

Taken from Goose stdtri

*/

double StudentCdfInv(double p, int df)
{

	Print("Warning: StudentCdfInv not owlking properly");
	PauseExecution();

	double t, rk, z;
	int rflg;
	
	if(df <= 0 || p <= Zero || p >= One)
	{
		Print("Error in StudentCdfInv");
		HaltExecution();
	}
	
	rk = df;
	
	if(p > 0.25 && p < 0.75)
	{
		if(p == Half) return Zero;
		z = One - Two * p;
		z = BetaInc(Half,Half * rk,fabs(z));
		t = sqrt(rk * z / (One - z));
		if(p < Half) t = -t;
		return t;
	}
	
	rflg = -1;
	
	if(p >= 0.5)
	{
		p = One - p;
		rflg = 1;
	}
	z = BetaInc(Half * rk,Half,Two * p);
	
	double MAXNUM = 1.0e80;	// MP
	
	if(MAXNUM * z < rk)
    return(rflg* MAXNUM);

 	t = sqrt(rk / z - rk);
 	return (rflg * t);
}

double StudentPVal(double TStat, double df)
{
	return Two * StudentCdf(-abs(TStat),df);
}

double StudentPVal(double Est, double Se, double df)
{
	return StudentPVal(Est / Se,df);
}

string StudentStars(double TStat, double df)
{
	double pval = StudentPVal(TStat,df);
	     if(pval <= 0.001) return "***";
	else if(pval <= 0.01)  return "**";
	else if(pval <= 0.05)  return "*";
	else if(pval <= 0.1)   return "+";
	else                   return "";
}

string StudentStars(double Est, double Se, double df)
{
	return StudentStars(Est / Se,df);
}

#endif
