
#ifndef _DistUniform
#define _DistUniform


	template <class Real> Real UniformCdf(const Real &x);
	double UniformCdfInv(double cdf);
	double UniformMean();
	double UniformPdf(double x);
	double UniformVariance();
	double *uniform_01_order_sample(int n, int *seed);
	double UniformCdf(double x, double a, double b);
	double UniformCdfInv(double cdf, double a, double b);
	bool UniformCheck(double a, double b);
	double UniformMean(double a, double b);
	double UniformPdf(double x, double a, double b);
	double UniformVariance(double a, double b);
	double UniformSample(double a, double b, long &RandomSeed);

/*

Purpose:

	UNIFORM_01_CDF evaluates the Uniform 01 CDF.
	
Modified:

	18 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double X, the argument of the CDF.
	
	Output, double UNIFORM_01_CDF, the value of the CDF.
	
*/

template <class Real>
Real UniformCdf(const Real &x)
{
		 if(x   < Zero) return Zero;
	else if(One < x   ) return One;
	else				return x;
}


/*

Purpose:

	UNIFORM_01_CDF_INV inverts the Uniform 01 CDF.
	
Modified:

	18 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double CDF, the value of the CDF.
	0.0 <= CDF <= 1.0.
	
	Output, double UNIFORM_01_CDF_INV, the corresponding argument.
	
*/

double UniformCdfInv(double cdf)
{
	double x;
	
	if(cdf < 0.0 || 1.0 < cdf)
	{
		Print("Error in UniformCdfInv(double cdf)");
		Print("Cdf < 0 Or Cdf > 1");
		HaltExecution();
	}
    
    x = cdf;
    
    return x;
}


/*

Purpose:

	UNIFORM_01_MEAN returns the mean of the Uniform 01 PDF.
	
Modified:

	18 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Output, double UNIFORM_01_MEAN, the mean of the discrete uniform PDF.
	
*/

double UniformMean()
{
	double mean;
	mean = 0.5;
	return mean;
}


/*

Purpose:

	UNIFORM_01_PDF evaluates the Uniform 01 PDF.
	
Formula:

	PDF(X) = 1 for 0 <= X <= 1
		   = 0 otherwise

Modified:

	14 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double X, the argument of the PDF.
	0.0 <= X <= 1.0.
	
	Output, double PDF, the value of the PDF.
	
*/

double UniformPdf(double x)
{
	double pdf;
	
	if(x < 0.0 || 1.0 < x) pdf = 0.0;
	else                   pdf = 1.0;

	return pdf;
}


/*

Purpose:

	UNIFORM_01_SAMPLE is a random number generator.
	
Formula:

	SEED = SEED * (7**5) mod (2**31 - 1)
	UNIFORM_01_SAMPLE = SEED * / ( 2**31 - 1 )
	
Parameters:

	Input/output, int *SEED, the integer "seed" used to generate
	the output random number, and updated in preparation for the
	next one.  *SEED should not be zero.
	
	Output, double UNIFORM_01_SAMPLE, a random value between 0 and 1.
	
Local Parameters:

	Local, int IA = 7**5.
	
	Local, int IB = 2**15.
	
	Local, int IB16 = 2**16.
	
	Local, int IP = 2**31 - 1.
	
*/

inline double UniformSample(long &RandomSeed)
{
	return Random2(RandomSeed);
}	

/*
double uniform_01_sample(int *seed)
{
	static int ia = 16807;
	static int ib15 = 32768;
	static int ib16 = 65536;
	static int ip = 2147483647;
	int iprhi;
	int ixhi;
	int k;
	int leftlo;
	int loxa;
	double temp;
	
	// Don't let SEED be 0.
	
	if(*seed == 0) *seed = ip;
	
	// Get the 15 high order bits of SEED.
	
	ixhi = *seed / ib16;
	
	// Get the 16 low bits of SEED and form the low product.
	
	loxa = ( *seed - ixhi * ib16 ) * ia;
	
	// Get the 15 high order bits of the low product.
	
	leftlo = loxa / ib16;
	
	// Form the 31 highest bits of the full product.
	
	iprhi = ixhi * ia + leftlo;
	
	// Get overflow past the 31st bit of full product.
	
	k = iprhi / ib15;
	
	// Assemble all the parts and presubtract IP.  The parentheses are essential.
	
	*seed = ( ( ( loxa - leftlo * ib16 ) - ip ) + ( iprhi - k * ib15 ) * ib16 ) + k;
	        
	// Add IP back in if necessary.
	
	if(*seed < 0) *seed = *seed + ip;
	
	// Multiply by 1 / (2**31-1).
	
	temp = ((double)*seed) * 4.656612875E-10;
	
	return temp;
}
*/


/*

Purpose:

	UNIFORM_01_VARIANCE returns the variance of the Uniform 01 PDF.
	
Modified:

	18 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Output, double UNIFORM_01_VARIANCE, the variance of the PDF.
	
*/

double UniformVariance()
{
	double variance;
	variance = 1.0 / 12.0;
	return variance;
}


/*

Purpose:

	UNIFORM_01_ORDER_SAMPLE samples the Uniform 01 Order PDF.
	
Discussion:

	In effect, this routine simply generates N samples of the
	Uniform 01 PDF; but it generates them in order.  (Actually,
	it generates them in descending order, but stores them in
	the array in ascending order).  This saves the work of
	sorting the results.  Moreover, if the order statistics
	for another PDF are desired, and the inverse CDF is available,
	then the desired values may be generated, presorted, by
	calling this routine and using the results as input to the
	inverse CDF routine.
	
Modified:

	27 October 2004
	
Author:

	John Burkardt
	
Reference:

	Jerry Banks, editor,
	Handbook of Simulation,
	Engineering and Management Press Books, 1998, page 168.
	
Parameters:

	Input, int N, the number of elements in the sample.
	
	Input/output, int *SEED, a seed for the random number generator.
	
	Output, double UNIFORM_01_ORDER_SAMPLE[N], N samples of the Uniform 01 PDF, in
	ascending order.
	
*/

/*
double *uniform_01_order_sample(int n, int *seed)
{
	int i;
	double u;
	double v;
	double *x;
	
	x = new double[n];
	
	v = 1.0;
	
	for(i=n-1; 0<=i; i--)
	{
		u = d_uniform_01(seed);
		v = v * pow(u,1.0 / (double)(i + 1));
		x[i] = v;
	}
	
	return x;
}
*/

/*

Purpose:

	UNIFORM_CDF evaluates the Uniform CDF.
	
Modified:

	17 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double X, the argument of the CDF.
	
	Input, double A, B, the parameters of the PDF.
	A < B.
	
	Output, double UNIFORM_CDF, the value of the CDF.
	
*/

double UniformCdf(double x, double a, double b)
{
	double cdf;
	
	if(x < a)       cdf = 0.0;
	else if(b < x)  cdf = 1.0;
	else            cdf = (x - a) / (b - a);
		
	return cdf;
}


/*

Purpose:

	UNIFORM_CDF_INV inverts the Uniform CDF.
	
Modified:

	17 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double CDF, the value of the CDF.
	0.0 <= CDF <= 1.0.
	
	Input, double A, B, the parameters of the PDF.
	A < B.
	
	Output, double UNIFORM_CDF_INV, the corresponding argument.
	
*/


double UniformCdfInv(double cdf, double a, double b)
{
	double x;
	
	if(cdf < 0.0 || 1.0 < cdf)
	{
		Print("Error in UniformCdfInv(double cdf, double a, double b)");
		Print("Cdf < 0 or Cdf > 1");
		HaltExecution();
	}
    
    x = a + (b - a) * cdf;
    
    return x;
}


/*

Purpose:

	UNIFORM_CHECK checks the parameters of the Uniform CDF.
	
Modified:

	17 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double A, B, the parameters of the PDF.
	A < B.
	
	Output, bool UNIFORM_CHECK, is true if the parameters are legal.
	
*/

bool UniformCheck(double a, double b)
{
	if(b <= a) return false;
	else       return true;
}


/*

Purpose:

	UNIFORM_MEAN returns the mean of the Uniform PDF.
	
Modified:

	17 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double A, B, the parameters of the PDF.
	A < B.
	
	Output, double UNIFORM_MEAN, the mean of the discrete uniform PDF.

*/

double UniformMean(double a, double b)
{
	double mean;
	mean = 0.5 * (a + b);
	return mean;
}


/*

Purpose:

	UNIFORM_PDF evaluates the Uniform PDF.
	
Discussion:

	The Uniform PDF is also known as the "Rectangular" or "de Moivre" PDF.
	
Formula:

	PDF(A,B;X) = 1 / ( B - A ) for A <= X <= B
	           = 0 otherwise
	           
Moified:

	17 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double X, the argument of the PDF.
	
	Input, double A, B, the parameters of the PDF.
	A < B.
	
	Output, double UNIFORM_PDF, the value of the PDF.
	
*/

double UniformPdf(double x, double a, double b)
{
	double pdf;
	
	if(x < a || b < x) pdf = 0.0;
	else               pdf = 1.0 / (b - a);

	return pdf;
}


/*

Purpose:

	UNIFORM_SAMPLE samples the Uniform PDF.
	
Modified:

	17 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double A, B, the parameters of the PDF.
	A < B.
	
	Input/output, int *SEED, a seed for the random number generator.
	
	Output, double UNIFORM_SAMPLE, a sample of the PDF.
	
*/

double UniformSample(double a, double b, long &RandomSeed)
{
	double cdf = UniformSample(RandomSeed);
	double x   = UniformCdfInv(cdf,a,b);
	return x;
}


/*

Purpose:

	UNIFORM_VARIANCE returns the variance of the Uniform PDF.
	
Modified:

	17 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double A, B, the parameters of the PDF.
	A < B.
	
	Output, double UNIFORM_VARIANCE, the variance of the PDF.
	
*/

double UniformVariance(double a, double b)
{
	double variance;
	variance = (b - a) * (b - a) / 12.0;
	return variance;
}

#endif
