
#ifndef _GenMat_GeneralMatrix
#define _GenMat_GeneralMatrix

/*

Define a General Matrix.
The following is defined for general matrices.


T (i,j)		-	Access the elements of a GeneralMatrix

int Rows()	-	The number of rows of a GeneralMatrix
int Cols()	-	The number of columns of a GeneralMatrix

GeneralMatrix can be used for generic matrix algorithms

for example, Cmd can have a function pointer,

int (*F)(const Vector<double>, Vector<double>, GeneralMatrix<double>)

then, we setting the weighting matrix, we can use,

W = V.Copy();
Chk = Invert(W);

This algorithm will know to use a differnent algorithm to inverse
a SymmetricMatrix vs. a BlockMatrix.

*/

namespace MatrixType
{

	/* Matrix Types */
	const short int Matrix                           =  1;
	const short int DiagonalMatrix                   =  2;
	const short int IdentityMatrix                   =  3;
	const short int BlockMatrix                      =  4;
	const short int BlasMatrix                       =  5;
	const short int SymmetricMatrix                  =  6;
	const short int SymmetricMatrixUP                =  7;
	const short int SymmetricMatrixLU                =  8;
	const short int SymmetricMatrixUU                =  9;
 	const short int LowerMatrix                      = 10;
	const short int LowerMatrixUN                    = 11;
	const short int LowerMatrixUU                    = 12;
	const short int UpperMatrix                      = 13;
	const short int UpperMatrixUN                    = 14;
	const short int UpperMatrixUU                    = 15;
	const short int BandMatrix                       = 16;
	const short int SymmetricBandMatrix              = 17;
	const short int LowerBandMatrix                  = 18;
	const short int UpperBandMatrix                  = 19;
	const short int ToeplitzMatrix                   = 20;
	const short int SymmetricToeplitzMatrix          = 21;
	const short int SymmetricBandToeplitzMatrix      = 22;
	const short int SimpleConstraintMatrix           = 23;
	const short int SimpleConstraintProjectionMatrix = 24;
	const short int SparseMatrix                     = 25;
	const short int SparseMatrixCC                   = 26;
	const short int SparseMatrixRcv                  = 27;
	const short int SparseSymmetricMatrix            = 28;
	const short int SparseUpdatedMatrix              = 29;
	const short int SparseUpdatedMatrixCC            = 30;
	const short int SparseSymmetricUpdatedMatrix     = 31;
	const short int SparseSymmetricUpdatedMatrixCC   = 32;
	const short int MultipliedSparseMatrix           = 33;
	const short int SparseMatrixCCView               = 34;
	const short int SparseMatrixRcvView              = 35;
	const short int SparseSymmetricMatrixView        = 36;
	const short int SparseSymmetricMatrixRcvView     = 37;
	const short int Stack                            = 38;
	const short int Vector                           = 39;
	const short int RowVector                        = 40;
	const short int BlasVector                       = 41;
}

template <class T>
class GeneralMatrix
{

	public:
	
	/* Name of the General Matrix */
	string Name;
	
	/* Access Array Elements with Bounds Checking */
	virtual inline const T& operator()(int i, int j) const = 0;
	
	/* Properties of the Matrix */
	virtual inline int Type() const = 0;
	virtual inline const int Rows() const = 0;
	virtual inline const int Cols() const = 0;
	virtual inline const int Length() const = 0;
	virtual inline const Vector<int> Size() const = 0;
};

#endif
