
#ifndef _GenMat_GeneralVector
#define _GenMat_GeneralVector

/*

Define a General Matrix.
The following is defined for general matrices.


T (i,j)		-	Access the elements of a GeneralMatrix

int Rows()	-	The number of rows of a GeneralMatrix
int Cols()	-	The number of columns of a GeneralMatrix

GeneralMatrix can be used for generic matrix algorithms

for example, Cmd can have a function pointer,

int (*F)(const Vector<double>, Vector<double>, GeneralMatrix<double>)

then, we setting the weighting matrix, we can use,

W = V.Copy();
Chk = Invert(W);

This algorithm will know to use a differnent algorithm to inverse
a SymmetricMatrix vs. a BlockMatrix.

*/

template <class T>
class GeneralVector
{

	public:

	string Name;	// Name of the GeneralMatrix
//	int n_;			// Number of columns in the GeneralMatrix
//	int len_;		// Size of underlying array
	//T   *v_;		// Underlying array
	/*

	Type of Vector

	1	-	Vector
	2	-	Stack

	*/

	/* Constructor */
	//GeneralVector() {};

	/* Destructor */
	//~GeneralVector() {};

	/* Access array elements with bounds checking */
	virtual inline T& operator()(LongInt i) = 0;
	virtual inline const T& operator()(LongInt i) const = 0;

	/* Access array elements without bounds checking */
	virtual inline T* Begin()        const = 0;
	//virtual inline T* End()          const = 0;
	virtual inline T& operator[](LongInt i) = 0;
	virtual inline const T& operator[](LongInt i) const = 0;

	/* Properties of the matrix */
	virtual inline int Type() const = 0;
	virtual inline LongInt Size() const = 0;
	virtual inline LongInt Length() const = 0;
	virtual inline int Increment() const = 0;
};

#endif
