
#ifndef _Opt_GradientCheck
#define _Opt_GradientCheck

	
	Vector< Vector<double> > GradientCheck(int (*Func)(const Vector<double> &, double &, Vector<double> &, int),
										   const Vector<double> &x, double Tol);


namespace GradientCheckSpace
{

	int n;
	int m;
	int len;
	int (*Func1)(const Vector<double> &, double &, Vector<double> &, int);
	int Func2(const Vector<double> &x, double &f);
	int Func2(const Vector<double> &x, double &f)
	{
		Vector<double> df(n);
		return Func1(x,f,df,0);
	}
	
	int (*Func3)(const Vector<double> &, double &, Vector<double> &, Vector<double> &, Matrix<double> &, int);
	
	int Func4(const Vector<double> &x, double &f, Vector<double> &cf);
	int Func4(const Vector<double> &x, double &f, Vector<double> &cf)
	{
		Vector<double> df(n);
		Matrix<double> dcf(m,n);
		return Func3(x,f,df,cf,dcf,0);
	}
	
	int (*Func3S)(const Vector<double> &, double &, Vector<double> &, Vector<double> &, Vector<double> &, int);
	
	int Func4S(const Vector<double> &x, double &f, Vector<double> &cf);
	int Func4S(const Vector<double> &x, double &f, Vector<double> &cf)
	{
		Vector<double> df(n);
		Vector<double> dcf(len);
		return Func3S(x,f,df,cf,dcf,0);
	}
	
	#ifdef _AutoDeriv
		int (*Func5)(const Vector< BDeriv<double> > &, BDeriv<double> &);
		int Func6(const Vector<double> &x, double &f);
		int Func6(const Vector<double> &x, double &f)
		{
			int n = x.Size();
			Vector< BDeriv<double> > xB(n);
			for(int i=0; i<n; i++) xB(i) = x(i);
			BDeriv<double> fB;
			int Info = Func3(xB,fB);
			f = fB.val();
			return Info;
		}
	#endif
}

Vector< Vector<double> > GradientCheck(int (*Func)(const Vector<double> &, double &, Vector<double> &, int),
                                       const Vector<double> &x, double Tol)
{
	/* Compute the gradient */
	int n = x.Size();
	double f;
	Vector<double> df1(n);
	Vector<double> df2(n);
	Vector<double> TypX(n,One);

	GradientCheckSpace::n = n;
	GradientCheckSpace::Func1 = Func;

	Func(x,f,df1,1);
	int Info = Gradient(GradientCheckSpace::Func2,x,f,df2,MachEps3,6,TypX);

	Vector<double> Diff(n);
	for(int i=0; i<n; i++) Diff[i] = abs(df1[i] - df2[i]);
	
 	Vector<double> RelDiff(n);
	for(int i=0; i<n; i++) RelDiff[i] = Diff[i] / Max(abs(df2[i]),One);

	Vector<double> Prob(n,1.0);

	if(!Info) for(int i=0; i<n; i++) if(RelDiff[i] <= Tol) Prob[i] = 0.0;
	
	Vector< Vector<double> > Ret(5);
	Ret(0) = df1;
	Ret(1) = df2;
	Ret(2) = RelDiff;
	Ret(3) = Prob;
	Ret(4) = Vector<double>(1,NormLInf(RelDiff));
	return Ret;
}

Vector< Vector<double> > GradientCheck(int (*Func)(const Vector<double> &, double &, Vector<double> &, Vector<double> &, Matrix<double> &, int),
                                       const Vector<double> &x, int m, double Tol)
{
	/* Compute the gradient */
	int n = x.Size();
	double f;
	Vector<double> cf(m);;
	Vector<double> df1(n);
	Vector<double> df2(n);
	Matrix<double> dcf1(m,n);
	Matrix<double> dcf2(m,n);
	Vector<double> TypX(n,One);

	GradientCheckSpace::n = n;
	GradientCheckSpace::m = m;
	GradientCheckSpace::Func3 = Func;

	Func(x,f,df1,cf,dcf1,1);
	int Info = Gradient(GradientCheckSpace::Func4,x,f,df2,cf,dcf2,MachEps3,6,TypX);

	Vector<double> Diff(n+n*m);
	int Curr = 0;
	for(int i=0; i<n; i++) Diff[Curr++] = abs(df1[i] - df2[i]);
	for(int i=0; i<m; i++) for(int j=0; j<n; j++) Diff[Curr++] = abs(dcf1(i,j) - dcf2(i,j));
	
 	Vector<double> RelDiff(n+n*m);
	Curr = 0;
	for(int i=0; i<n; i++) RelDiff[Curr++] = Diff[i] / Max(abs(df2[i]),One);
	for(int i=0; i<m; i++) for(int j=0; j<n; j++)
	{
		RelDiff[Curr] = Diff[Curr] / Max(abs(dcf2(i,j)),One);
		Curr++;
	}	

	Vector<double> Prob(n+n*m,1.0);

	if(!Info) for(int i=0; i<n+n*m; i++) if(RelDiff[i] <= Tol) Prob[i] = 0.0;
	
	Vector< Vector<double> > Ret(5);
	Ret(0) = df1;
	Ret(1) = df2;
	Ret(2) = RelDiff;
	Ret(3) = Prob;
	Ret(4) = Vector<double>(1,NormLInf(RelDiff));
	return Ret;
}

#ifdef _GenMat_SparseMatrixRcvView
	Vector< Vector<double> > GradientCheck(int (*Func)(const Vector<double> &, double &, Vector<double> &, Vector<double> &, Vector<double> &, int),
	                                       const SparseMatrixRcvView &Cs, const Vector<double> &x, int m, double Tol)
	{
		Print("Gradient Check");

		/* Compute the gradient */
		int n = x.Size();
		int len = Cs.Length();
		double f;
		Vector<double> cf(m);;
		Vector<double> df1(n);
		Vector<double> df2(n);
		Vector<double> dcf1Sparse(len);
		Matrix<double> dcf1(m,n);			dcf1.Name = "GradientCheck::dcf1";
		Matrix<double> dcf2(m,n);			dcf2.Name = "GradientCheck::dcf2";		
		Vector<double> TypX(n,One);

		GradientCheckSpace::n = n;
		GradientCheckSpace::m = m;
		GradientCheckSpace::len = len;
		GradientCheckSpace::Func3S = Func;
		
		Print("G.1");

		Func(x,f,df1,cf,dcf1Sparse,1);
		
		Print("G.2");
		
		Set(dcf1,Zero);
		Vector<double> NonZero(n+m*n,0.0);
		NonZero.Name = "NonZero";
		for(int i=0; i<len; i++)
		{
			dcf1(Cs.r(i),Cs.c(i)) = dcf1Sparse(i);
			NonZero(n+n*Cs.r(i)+Cs.c(i)) = 1.0;
		}
		
		Print("G.3");
			
		int Info = Gradient(GradientCheckSpace::Func4S,x,f,df2,cf,dcf2,MachEps3,6,TypX);

		Print("G.4");

		Vector<double> Diff(n+n*m);
		int Curr = 0;
		for(int i=0; i<n; i++) Diff[Curr++] = abs(df1[i] - df2[i]);
		for(int i=0; i<m; i++) for(int j=0; j<n; j++) Diff[Curr++] = abs(dcf1(i,j) - dcf2(i,j));
		
	 	Vector<double> RelDiff(n+n*m);
		Curr = 0;
		for(int i=0; i<n; i++) RelDiff[Curr++] = Diff[i] / Max(abs(df2[i]),One);
		for(int i=0; i<m; i++) for(int j=0; j<n; j++)
		{
			RelDiff[Curr] = Diff[Curr] / Max(abs(dcf2(i,j)),One);
			Curr++;
		}	

		Vector<double> Prob(n+n*m,1.0);

		if(!Info) for(int i=0; i<n+n*m; i++) if(RelDiff[i] <= Tol) Prob[i] = 0.0;
		
		Vector<double> d1(n+n*m);
		Vector<double> d2(n+n*m);
		Curr = 0;
		for(int i=0; i<n; i++)
		{
			d1(Curr) = df1(i);
			d2(Curr) = df2(i);
			Curr++;
		}
		for(int i=0; i<m; i++) for(int j=0; j<n; j++)
		{
			d1(Curr) = dcf1(i,j);
			d2(Curr) = dcf2(i,j);
			Curr++;
		}
		
		Vector< Vector<double> > Ret(6);
		Ret(0) = d1;
		Ret(1) = d2;
		Ret(2) = RelDiff;
		Ret(3) = Prob;
		Ret(4) = NonZero;
		Ret(5) = Vector<double>(1,NormLInf(RelDiff));
		return Ret;
	}
#endif

#ifdef _AutoDeriv
	Vector<int> GradientCheck(int (*Func)(const Vector< BDeriv<double> > &, BDeriv<double> &),
								const Vector<double> &x, double Tol)
	{
		/* Compute the gradient */
		int n = x.Size();
		double f;
		Vector<double> df1(n);
		Vector<double> df2(n);
		Vector<double> TypX(n,One);

		GradientCheckSpace::n = n;
		GradientCheckSpace::Func5 = Func;

		Vector< BDeriv<double> > xB(n);
		for(int i=0; i<n; i++) xB(i) = x(i);
		BDeriv<double> fB;
		int Info = Func(xB,fB);
		for(int i=0; i<n; i++) fB.diff(0,1);
		f = fB.x();
		for(int i=0; i<n; i++) df1(i) = xB(i).d(0);
		
		Info = Gradient(GradientCheckSpace::Func6,x,f,df2,MachEps3,6,TypX);
	
		Vector<double> Diff(n);
		for(int i=0; i<n; i++) Diff[i] = abs(df1[i] - df2[i]);
		
		Vector<double> RelDiff(n);
		for(int i=0; i<n; i++) RelDiff[i] = Diff[i] / abs(df2[i]);

		Vector<int> Ret(n,1);

		if(Info) return Ret;

		for(int i=0; i<n; i++) if(RelDiff[i] <= Tol) Ret[i] = 0;
		
		return Ret;
	}
#endif

#endif
