
#ifndef _Grid
#define _Grid

/*

Create a grid of n evenly spaced points between low and high
where n is the length of grid. We do not reuiqre low<high,
the reverse will create a grid of points in decreasing order

*/

void Grid(Vector<double> &g, double Low, double High);
Vector<double> Grid(double Low, double High, int K);
Matrix<double> Grid(const Vector<double> &Low, const Vector<double> &High, const Vector<int> &K);

void Grid(Vector<double> &Grid1, double Low, double High)
{
	int K = Grid1.Size();
	if(K == 1)
	{
		Grid1[0] = Half * (Low + High);
	}
	else
	{
		for(int k=0; k<K; k++) Grid1[k] = Low + (High - Low) * k / (K - One);
	}
}

Vector<double> Grid(double Low, double High, int K)
{
	Vector<double> Grid1(K);
	Grid1.Name = "Grid1(" + ToString(Low) + "," + ToString(High) + "," + ToString(K) + ")";
	if(K == 1)
	{
		Grid1[0] = Half * (Low + High);
	}
	else
	{
		for(int k=0; k<K; k++) Grid1[k] = Low + (High - Low) * k / (K - One);
	}
	return Grid1;
}

Matrix<double> Grid(const Vector<double> &Low, const Vector<double> &High, const Vector<int> &K)
{
	int D = Low.Size();
	if(High.Size() != D || K.Size() != D)
	{
		Print("Error in Grid");
		Print("Incompadible Dimensions");
		Print("Low.Size()",Low.Size());
		Print("High.Size()",High.Size());
		Print("K.Size()",K.Size());
		HaltExecution();
	}
	
	int KProd = 1;
	for(int d=0; d<D; d++) KProd *= K(d);
	
	Matrix<double> Grid1(KProd,D);
	
	if(D == 1)
	{
		for(int k=0; k<K(0); k++) Grid1(k,0) = Low(0) + (High(0) - Low(0)) * k / (K(0) - One);
	}
	else if(D == 2)
	{
		int Curr = 0;
		for(int k1=0; k1<K(0); k1++) for(int k2=0; k2<K(1); k2++)
		{
			Grid1(Curr,0) = Low(0) + (High(0) - Low(0)) * k1 / (K(0) - One);
			Grid1(Curr,1) = Low(1) + (High(1) - Low(1)) * k2 / (K(1) - One);
			Curr++;
		}
	}
	else
	{
		Print("Error in Grid");
		Print("This Part Not Done Yet");
		HaltExecution();
	}
	
	return Grid1;
}

#endif
