
#ifndef _Data_Input
#define _Data_Input

/*

Output to a file

*/

	/* Function Prototypes */
	void Input(string FileName, Vector<double> &x);
	void Input(string FileName, Vector<double> &x1, Vector<double> x2);
	
void Input(string FileName, Vector<double> &x)
{
	ifstream in1(FileName.c_str());
	if(!in1.is_open())
	{	
		Print("Error in Input");
		Print("File cannot be opened");
		Print("FileName",FileName);
		HaltExecution();
	}
	int n = x.Size();
	for(int i=0; i<n; i++) in1 >> x[i];
}

void Input(string FileName, Vector<double> &x1, Vector<double> x2)
{
	ifstream in1(FileName.c_str());
	if(!in1.is_open())
	{	
		Print("Error in Input");
		Print("File cannot be opened");
		Print("FileName",FileName);
		HaltExecution();
	}
	int n = x1.Size();
	in1 >> x1.Name >> x2.Name;
	for(int i=0; i<n; i++) in1 >> x1[i] >> x2[i];
}

template <class T1, class T2, class T3, class T4>
void Input(string FileName, Vector<T1> &x1, Vector<T2> &x2, Vector<T3> &x3, Vector<T4> &x4)
{
	ifstream in1(FileName.c_str());
	if(!in1.is_open())
	{	
		Print("Error in Input");
		Print("File cannot be opened");
		Print("FileName",FileName);
		HaltExecution();
	}
	int n = x1.Size();
	in1 >> x1.Name >> x2.Name >> x3.Name >> x4.Name;
	for(int i=0; i<n; i++) in1 >> x1[i] >> x2[i] >> x3[i] >> x4[i];
}

template <class T>
void Input(string FileName, Matrix<T> &A)
{

	ifstream in1(FileName.c_str());
	if(!in1.is_open())
	{	
		Print("Error in Input(Matrix)");
		Print("File Cannot Be Opened");
		Print("FileName",FileName);
		HaltExecution();
	}

	int m = A.Rows();
	int n = A.Cols();
	for(int i=0; i<m; i++) for(int j=0; j<n; j++) in1 >> A(i,j);
}

template <class T>
void Input(string FileName, SymmetricMatrix<T> &A)
{

	ifstream in1(FileName.c_str());
	if(!in1.is_open())
	{	
		Print("Error in Input(SymmetricMatrix)");
		Print("File Cannot Be Opened");
		Print("FileName",FileName);
		HaltExecution();
	}

	double Temp;
	int n = A.Dim();
	
	for(int i=0; i<n; i++)
	{
		for(int j=0; j<i; j++)
		{
			in1 >> Temp;
			if(abs(Temp - A(j,i) >= SymmetricTol))
			{
				Print("Error in Input(SymmetricMatrix)");
				Print("Matrix is Not Symmetric");
				Print("A.Name",A.Name);
				Print("i",i);
				Print("j",j);
				Print("A(i,j)",Temp);
				Print("A(j,i)",A(j,i));
				Print("abs(A(i,j) - A(j,i))",abs(Temp - A(j,i)));
				HaltExecution();
			}
		}		
		for(int j=i; j<n; j++) in1 >> A(i,j);
	}
}	

template <class T>
void Input(string FileName, LowerMatrix<T> &A)
{

	ifstream in1(FileName.c_str());
	if(!in1.is_open())
	{	
		Print("Error in Input(LowerMatrix)");
		Print("File Cannot Be Opened");
		Print("FileName",FileName);
		HaltExecution();
	}

	double Temp;
	int n = A.Dim();
	
	for(int i=0; i<n; i++)
	{
		
		for(int j=0; j<=i; j++) in1 >> A(i,j);
	
		for(int j=i+1; j<n; j++)
		{
			in1 >> Temp;
			if(abs(Temp) >= LowerTol)
			{
				Print("Error in Input(LowerMatrix)");
				Print("Matrix is Not Lower Triangular");
				Print("A.Name",A.Name);
				Print("i",i);
				Print("j",j);
				Print("A(i,j)",Temp);
				HaltExecution();
			}
		}		
	}
}

/*
Matrix<double> InputMatrix(string FileName)
{

	ifstream in1(FileName.c_str());
	if(!in1.is_open())
	{	
		Print("Error in InputMatrix(FileName)");
		Print("File Cannot Be Opened");
		Print("FileName",FileName);
		HaltExecution();
	}

	int m = A.Rows();
	int n = A.Cols();
	for(int i=0; i<m; i++) for(int j=0; j<n; j++) in1 >> A(i,j);
}
*/

template <class T1, class T2>
void Input(string FileName, Vector<T1> &x, Matrix<T2> &A)
{

	ifstream in1(FileName.c_str());
	if(!in1.is_open())
	{	
		Print("Error in Input(Matrix)");
		Print("File Cannot Be Opened");
		Print("FileName",FileName);
		HaltExecution();
	}

	int m = A.Rows();
	int n = A.Cols();
	
	if(x.Size() != m)
	{
		Print("Error in Input");
		HaltExecution();
	}
	
	for(int i=0; i<m; i++)
	{
		in1 >> x(i);
		for(int j=0; j<n; j++) in1 >> A(i,j);
	}
}

template <class T>
Vector<T> InputVector(string FileName)
{
	const char Delim = '\t';

	ifstream in1(FileName.c_str());
	if(!in1.is_open())
	{	
		Print("Error in InputVector(FileName)");
		Print("File Cannot Be Opened");
		Print("FileName",FileName);
		HaltExecution();
	}
	
	/* Open the file */
	char ch;
	FILE *f1;	// Pointer to the file
	f1 = fopen(FileName.c_str(),"r"); 
	if(f1 == NULL)
	{
		Print("Error in InputMatrix");
		Print("Unable to Open File");
		Print("FileName",FileName);
		HaltExecution();
	}
		
	int m = 0;
	string Data = "";
	do
	{
		ch = getc(f1);	// Get one character from the file
		if(ch == '\n' || ch == '\r') m++;
		Data += ch;	// Push the character to a Stack
	}
	while(ch != EOF);	// repeat until EOF (end of file)
	
	/* Close the File */
	fclose(f1);

	Data = Data.substr(0,Data.length()-1); // get rid of EOF
	if(Data[Data.length()-1] == '\n') Data = Data.substr(0,Data.length()-1); // get rid of last '\n'
	//if(Data[Data.length()-1] == '\n') Data = Data.substr(0,Data.length()-1); // get rid of second to last '\n'
	Stack<string> Temp = ParseBy(Data,'\n');
	Temp.Name = "InputVector::Temp";
	int n = Temp.Size();
	
	Vector<T> Ret(n);
	Ret.Name = "InputVector::Ret";
	for(int i=0; i<n; i++)
	{
		bool Info = Convert(Ret(i),Temp(i));
		if(Info)
		{
			Print("Error in InputVector");
			Print("Cannot Convert");
			HaltExecution();
		}
	}
	return Ret;
}

template <class T>
Matrix<T> InputMatrix(string FileName)
{
	const char Delim = '\t';

	ifstream in1(FileName.c_str());
	if(!in1.is_open())
	{	
		Print("Error in Input(Matrix)");
		Print("File Cannot Be Opened");
		Print("FileName",FileName);
		HaltExecution();
	}
	
	/* Open the file */
	char ch;
	FILE *f1;	// Pointer to the file
	f1 = fopen(FileName.c_str(),"r"); 
	if(f1 == NULL)
	{
		Print("Error in InputMatrix");
		Print("Unable to Open File");
		Print("FileName",FileName);
		HaltExecution();
	}
		
	int m = 0;
	Stack<char> Data;
	do
	{
		ch = getc(f1);	// Get one character from the file
		if(ch == '\n' || ch == '\r') m++;
		Data.Push(ch);	// Push the character to a Stack
	}
	while(ch != EOF);	// repeat until EOF (end of file)
			
	/* Trim if Possible */
	try { Data.Trim(); }
	catch(int i) { }
	
	/* Close the File */
	fclose(f1);
							
	
				
	/*
			
	The rest of the file contains the data, each entry separated by
	tabs. Mark the positions of each entry in a Stack and replace
	the tabs and enlines with the end of string character, '\0'.
	Throughout, make sure that each row of the data matrix contains
	K entries, where K is the number of variable names in the first
	line of the data file.
			
	*/

	//k = 1;
	//int m = 0;
	Stack<char*> EntryStack;
	//EntryStack.Reset(m*n+1); // Initialize Stack
	//T Temp2;
	//int Info;
	
	/* First Line */
	//Stack<char*> Temp;
	EntryStack.Push(&Data[0]);
	int k = 1;
	while(Data[k] != '\n' && Data[k] != '\r')
	{
		if(Data[k] == Delim)
		{
			Data[k] = '\0';
			EntryStack.Push(&Data[k+1]);
		}
		k++;
	}
	Data[k] = '\0';
	k++;
	int n = EntryStack.Size();
	
	/* Start w/ Second Line */
	EntryStack.Push(&Data[k]);
	int CurrLine = 0;
	int LineStart = k;
	bool Paren = 0;	
	int Curr = 0;
	int n2 = 0;	
	while(&Data[k] < Data.End())
	{
		if(Data[k] == '\n' || Data[k] == '\r')
		{
			Data[k] = '\0';
			n2++;
			k++;
			EntryStack.Push(&Data[k]);
							
			/* Check column index */
			if(n2 != n)
			{
				Print("Error in DataFileDelim (1)");
				Print("Columns are not conformant");
				Print("Data[k] == '\\n' or '\\r'");
				Print("FileName",FileName);
				Print("CurrLine",CurrLine);
				Print("k",k);
				Print("n2",n2);
				Print("n",n);
				HaltExecution();
			}
									
			n2 = 0;	// Reset column index
			CurrLine++;
			LineStart = k+1;
		}
		else if(Data[k] == Delim)
		{
			Data[k] = '\0';
			n2++;
			k++;
			EntryStack.Push(&Data[k]);
					
			/* Check clumn index */
			if(n2 == n)
			{
				Print("Error in DataFileDelim (2)");
				Print("Columns are not conformant");
				Print("Data[k] == Delim");
				Print("FileName",FileName);
				Print("k",k);
				Print("n2",n2);
				Print("n",n);
			
				/* Print the current line */
				int kLow = k;
				while(Data[kLow] != '\n' && Data[kLow] != '\r') kLow--;
				Print("kLow",kLow);
				char* s1 = &Data[kLow+1];
				Print("Current line",s1);
				Print("CurrLine",CurrLine);
				string s2 = "";
				k = LineStart;
				while(Data[k] != '\n' && Data[k] != '\r')
				{
					if(Data[k] != '\0') s2 += Data[k];
					k++;
				}
				Print("Line Text",s2);
				HaltExecution();
			}
		}
		else k++;
	}
				
	/* Matrix of pointers allows accesing elements using the (i,j) notation */
	Matrix<char*> Ret(m,n,EntryStack.Begin());
	Matrix<T> Ret2(m,n);
	for(int i=0; i<m; i++) for(int j=0; j<n; j++)
	{
		T Temp;
		int Info = Convert(Temp,Ret(i,j));
		if(Info)
		{
			Print("Error in InputMatrix");
			Print("Convert Failed");
			Print("i",i);
			Print("j",j);
			Print("Ret(i,j)",Ret(i,j));
			HaltExecution();
		}		
		Ret2(i,j) = Temp;
	}
	return Ret2;
}

#endif
