
#ifndef _GenMatUtil_Inverse
#define _GenMatUtil_Inverse



/* Find the inverse of a Matrix using Guass-Jordan elimination (efficient) */
/*
template <class T>
int Invert(Matrix<T> &A)
{
	int icol,irow;
	T big,dum,pivinv,temp;
	int n = A.Rows();
	
	/* Check if Matrix is Square */
/*	if(A.Cols() != n) return 1;
	
	/* Check is Matrix Contains Nan */
	//if(IsNan(A)) return 2;
/*	
	if(n == 1)
	{
		if(A(0,0) != T(0))
		{
			A(0,0) = T(1) / A(0,0);
			return 0;
		}
		else return 3;
	}
	
	Vector<int> indxc(n);		indxc.Name = "MatrixUtil::Invert::indxc";
	Vector<int> indxr(n);		indxr.Name = "MatrixUtil::Invert::indxr";
	Vector<int> ipiv(n);		ipiv.Name  = "MatrixUtil::Invert::ipiv";
	for(int j=0; j<n; j++) ipiv[j] = 0;
	for(int i=0; i<n; i++)
	{
		big = T(0);
		for(int j=0; j<n; j++) 	if(ipiv[j] != 1) for(int k=0; k<n; k++)
		{
			if(ipiv[k] == 0)
			{
				if(fabs(A(j,k)) >= big)
				{
					big = fabs(A(j,k));
					irow = j;
					icol = k;
				}
			}
			else if(ipiv[k] > 1) return 4;
		}
		++(ipiv[icol]);
		if(irow != icol) for(int l=0; l<n; l++)
		{
			temp = A(irow,l);
			A(irow,l) = A(icol,l);
			A(icol,l) = temp;
		}
		indxr[i] = irow;
		indxc[i] = icol;
		if(A(icol,icol) == T(0)) return 5;
		pivinv = T(1) / A(icol,icol);
		A(icol,icol) = T(1);
		for(int l=0;l<n;l++) A(icol,l) *= pivinv;
		for(int ll=0; ll<n; ll++) if(ll != icol)
		{
			dum = A(ll,icol);
			A(ll,icol) = T(0);
			for(int l=0; l<n; l++) A(ll,l) -= A(icol,l) * dum;
		}
	}
	for(int l=n-1; l>=0; l--)
	{
		if(indxr[l] != indxc[l]) for(int k=0; k<n; k++)
		{
			temp = A(k,indxr[l]);
			A(k,indxr[l]) = A(k,indxc[l]);
			A(k,indxc[l]) = temp;
		}
	}
	
	return 0;
}
*/

template <class T>
int Invert(Matrix<T> &A)
{
	return InvertLU(A);
}

template <class T>
Matrix<T> Inverse(const Matrix<T> &A)
{
	int n = A.Rows();
	
	/* Check if the Matrix is Square */
	if(A.Cols() != n)
	{
		Print("Error in Matrix = Inverse(Matrix)");
		Print("B = A^-1");
		Print("Matrix is not square");
		Print("A.Name",A.Name);
		Print("A.Size()",A.Size());
		HaltExecution();
	}
	
	Matrix<T> B = A.Copy();	
	B.Name = "Inverse(" + A.Name + ")";
	int Info = Invert(B);
	
	if(Info)
	{
		Throw(  ToString("Error in Matrix = Inverse(Matrix)\n")
		      + "B = A^-1\n"
		      + "Matrix is not Invertable\n"
		      + "A.Name: " + A.Name + "\n"
		      + "A.Size(): " + ToString(A.Size()) + "\n");
	}
	
	return B;
}

template <class T>
int Invert(SymmetricMatrix<T> &A)
{
	/*
	int n = A.Dim();
	Matrix<T> B(n,n);
	for(int i=0; i<n; i++) for(int j=0; j<n; j++) B(i,j) = A(i,j);
	int Info = Invert(B);
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++) A(i,j) = B(i,j);
	return Info;
	*/
	//return InvertBK(A);
	return InvertLU(A);
}

template <class T>
SymmetricMatrix<T> Inverse(const SymmetricMatrix<T> &A)
{
	/*
	int n = A.Dim();
	Matrix<T> B(n,n);
	for(int i=0; i<n; i++) for(int j=0; j<n; j++) B(i,j) = A(i,j);
	int Info = Invert(B);
	
	if(Info)
	{
		Print("Error in SymmetricMatrix = Inverse(SymmetricMatrix)");
		Print("B = A^-1");
		Print("A is not Invertable");
		Print("A.Name",A.Name);
		Print("A.Size()",A.Size());
		if(A.Rows() <= 12) Print("A",A);
		HaltExecution();
	}
	
	SymmetricMatrix<T> C(n);
	C.Name = "Inverse(" + A.Name + ")";
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++) C(i,j) = B(i,j);
	return C;
	*/
	//return InverseBK(A);
	return InverseLU(A);
}

#ifdef _GenMat_SymmetricMatrixLU
	template <class T>
	int Invert(SymmetricMatrixLU<T> &A)
	{
		int n = A.Dim();
		Matrix<T> B(n,n);
		for(int i=0; i<n; i++) for(int j=0; j<n; j++) B(i,j) = A(i,j);
		int Info = Invert(B);
		for(int i=0; i<n; i++) for(int j=0; j<=i; j++) A(i,j) = B(i,j);
		return Info;
	}
#endif

#ifdef _GenMat_SymmetricMatrixLU
	template <class T>
	SymmetricMatrixLU<T> Inverse(const SymmetricMatrixLU<T> &A)
	{
		int n = A.Dim();
		Matrix<T> B(n,n);
		for(int i=0; i<n; i++) for(int j=0; j<n; j++) B(i,j) = A(i,j);
		int Info = Invert(B);
		
		if(Info)
		{
			Print("Error in SymmetricMatrix = Inverse(SymmetricMatrixLU)");
			Print("B = A^-1");
			Print("A is not Invertable");
			Print("A.Name",A.Name);
			Print("A.Size()",A.Size());
			if(A.Rows() <= 12) Print("A",A);
			HaltExecution();
		}
		
		SymmetricMatrixLU<T> C(n);
		C.Name = "Inverse(" + A.Name + ")";
		for(int i=0; i<n; i++) for(int j=0; j<=i; j++) C(i,j) = B(i,j);
		return C;
	}
#endif

#ifdef _GenMat_SymmetricMatrixUP
	template <class T>
	int Invert(SymmetricMatrixUP<T> &A)
	{
		int n = A.Dim();
		Matrix<T> B(n,n);
		for(int i=0; i<n; i++) for(int j=0; j<n; j++) B(i,j) = A(i,j);
		int Info = Invert(B);
		for(int i=0; i<n; i++) for(int j=0; j<=i; j++) A(i,j) = B(i,j);
		return Info;
	}
#endif

#ifdef _GenMat_SymmmetricMatrixUP
	template <class T>
	SymmetricMatrixUP<T> Inverse(const SymmetricMatrixUP<T> &A)
	{
		int n = A.Dim();
		Matrix<T> B(n,n);
		for(int i=0; i<n; i++) for(int j=0; j<n; j++) B(i,j) = A(i,j);
		int Info = Invert(B);
		
		if(Info)
		{
			Print("Error in SymmetricMatrix = Inverse(SymmetricMatrixUP)");
			Print("B = A^-1");
			Print("A is not Invertable");
			Print("A.Name",A.Name);
			Print("A.Size()",A.Size());
			if(A.Rows() <= 12) Print("A",A);
			HaltExecution();
		}
		
		SymmetricMatrixUP<T> C(n);
		C.Name = "Inverse(" + A.Name + ")";
		for(int i=0; i<n; i++) for(int j=0; j<=i; j++) C(i,j) = B(i,j);
		return C;
	}
#endif

#ifdef _GetMat_SymmetricMatrixUU
	template <class T>
	int Invert(SymmetricMatrixUU<T> &A)
	{
		int n = A.Dim();
		Matrix<T> B(n,n);
		for(int i=0; i<n; i++) for(int j=0; j<n; j++) B(i,j) = A(i,j);
		int Info = Invert(B);
		for(int i=0; i<n; i++) for(int j=0; j<=i; j++) A(i,j) = B(i,j);
		return Info;
	}
#endif

#ifdef _GenMat_SymmetricMatrixUU
	template <class T>
	SymmetricMatrixUU<T> Inverse(const SymmetricMatrixUU<T> &A)
	{
		int n = A.Dim();
		Matrix<T> B(n,n);
		for(int i=0; i<n; i++) for(int j=0; j<n; j++) B(i,j) = A(i,j);
		int Info = Invert(B);
		
		if(Info)
		{
			Print("Error in SymmetricMatrix = Inverse(SymmetricMatrixUU)");
			Print("B = A^-1");
			Print("A is not Invertable");
			Print("A.Name",A.Name);
			Print("A.Size()",A.Size());
			if(A.Rows() <= 12) Print("A",A);
			HaltExecution();
		}
		
		SymmetricMatrixUU<T> C(n);
		C.Name = "Inverse(" + A.Name + ")";
		for(int i=0; i<n; i++) for(int j=0; j<=i; j++) C(i,j) = B(i,j);
		return C;
	}
#endif

/* Invert a DiagonalMatrix */
#ifdef _GenMat_DiagonalMatrix
	template <class T>
	int Invert(DiagonalMatrix<T> &A)
	{
		for(int i=0; i<A.Dim(); i++)
		{
			if(A(i) == Zero) return 1;
			else A(i) = One / A(i);
		}
		return 0;
	}

	template <class T>
	DiagonalMatrix<T> Inverse(const DiagonalMatrix<T> &A)
	{
		DiagonalMatrix<T> AInv = A.Copy();
		for(int i=0; i<A.Dim(); i++)
		{
			if(A(i) == Zero)
			{
				Print("Error in Inverse(DiagonalMatrix): Not Invertable)");
				Print("i",i);
				Print("A(i)",A(i));
				HaltExecution();
			}
			else A(i) = One / A(i);
		}
		return AInv;
	}
#endif

/*

Invert a LowerMatrix efficiently

Computational cost: 1 + 2^2 + 3^2 + 4^2 + ... n^2 = O(n^3)

*/

template <class T>
int Invert(LowerMatrix<T> &A)
{
	int n = A.Dim();
	return LapackTptri(0,1,n,A.Begin());
}

template <class T>
LowerMatrix<T> Inverse(const LowerMatrix<T> &A)
{
	LowerMatrix<T> AInv = A.Copy();
	int Info = Invert(AInv);
	
	if(Info)
	{
		Print("Error in Inverse(LowerMatrix)");
		Print("A is not invertable");
		Print("A.Name",A.Name);
		Print("A.Size()",A.Size());
		if(A.Rows() <= 12) Print("A",A);
		HaltExecution();
	}
	
	AInv.Name = "Inverse(" + A.Name + ")";
	
	return AInv;
}

#ifdef _GenMat_UpperMatrix
	template <class T>
	int Invert(UpperMatrix<T> &A)
	{
		int n = A.Dim();
		return LapackTptri(1,1,n,A.Begin());
	}

	template <class T>
	UpperMatrix<T> Inverse(const UpperMatrix<T> &A)
	{
		UpperMatrix<T> AInv = A.Copy();
		int Info = Invert(AInv);
		
		if(Info)
		{
			Print("Error in Inverse(UpperMatrix)");
			Print("A is not invertable");
			Print("A.Name",A.Name);
			Print("A.Size()",A.Size());
			if(A.Rows() <= 12) Print("A",A);
			HaltExecution();
		}
		
		AInv.Name = "Inverse(" + A.Name + ")";
	
		return AInv;
	}
#endif

/* Find the inverse of a BlockMatrix using Guass-Jordan elimination (efficient) */
#ifdef _GenMat_BlockMatrix
	template <class T>
	int Invert(BlockMatrix<T> &A)
	{
		int i,icol,irow,j,k,l,ll,b,n;
		T big,dum,pivinv,temp;

		int B = A.Blocks();
		for(b=0; b<B; b++)
		{
			n = A.Block(b);
			Vector<int> indxc(n);
			Vector<int> indxr(n);
			Vector<int> ipiv(n);
			for(j=0; j<n; j++) ipiv[j] = 0;
			for(i=0; i<n; i++)
			{
				big = Zero;
				for(j=0; j<n; j++) if(ipiv[j] != 1) for(k=0; k<n; k++)
				{
					if(ipiv[k] == 0)
					{
						if(fabs(A(b,j,k)) >= big)
						{
							big = fabs(A(b,j,k));
							irow = j;
							icol = k;
						}
					}
					else if(ipiv[k] > 1) return 2;
				}
				ipiv[icol]++;
				if(irow != icol) for (l=0; l<n; l++)
				{
					temp = A(b,irow,l);
					A(b,irow,l) = A(b,icol,l);
					A(b,icol,l) = temp;
				}
				indxr[i] = irow;
				indxc[i] = icol;
				if(A(icol,icol) == Zero) return 2;
				pivinv = One / A(b,icol,icol);
				A(b,icol,icol) = One;
				for(l=0;l<n;l++) A(b,icol,l) *= pivinv;
				for(ll=0;ll<n;ll++) if(ll != icol)
				{
					dum = A(b,ll,icol);
					A(b,ll,icol) = Zero;
					for(l=0;l<n;l++) A(b,ll,l) -= A(b,icol,l) * dum;
				}
			}
			for(l=n-1; l>=0; l--)
			{
				if(indxr[l] != indxc[l]) for(k=0; k<n; k++)
				{
					temp = A(b,k,indxr[l]);
					A(b,k,indxr[l]) = A(b,k,indxc[l]);
					A(b,k,indxc[l]) = temp;
				}
			}
		}
		return 0;
	}

	template <class T>
	BlockMatrix<T> Inverse(const BlockMatrix<T> &A)
	{
		BlockMatrix<T> Ainv = A.Copy();
		int Info = Invert(Ainv);
		if(Info)
		{
			Print("Error in BlockMatrix = Inverse(BlockMatrix)");
			Print("BlockMatrix is not ivetable");
			Print("A.Name",A.Name);
			Print("A.Size()",A.Size());
			if(A.Rows() + A.Cols() <= 25) Print("A",A);
			HaltExecution();
		}
		AInv.Name = "Inverse(" + A.Name + ")";
		return Ainv;
	}
#endif

/*
template <class T>
int Invert(LowerBandMatrix<T> &A)
{
	int n = A.Rows();
	
	if(A.Cols() != n)
	{
		Print("Error in Inverse(LowerBandMatrix)");
		Print("LowerBandMatrix is not Square");
		Print("A.Name",A.Name);
		Print("A.Size()",A.Size());
		HaltExecution();
	}
	
	int b = A.Lower();
	
	//LowerMatrix<double> L(n);
	//for(int i=0; i<n; i++) for(int j=0; j<=i; j++) L(i,j) = A(i,j);
	
	//Vector<double> Temp(n);
	//for(int i=0; i<n; i++) Temp(i) = A(i,i);
	//for(int i=0; i<n; i++) A(i,i) = One;
	
	for(int i=0; i<n; i++)
	{
		A(i,i) = One / A(i,i);
		for(int j=i+1; j<n; j++)
		//for(int j=Max(i-b,i+1); j<n; j++)
		{
			double sm = Zero;
			for(int k=i; k<j; k++) sm -= A(j,k) * A(k,i);
			//for(int k=Max(j-b,i); k<j; k++) sm -= A(j,k) * A(k,i);
			//A(j,i) = sm / Temp(j);
			A(j,i) = sm / A(j,j);
		}
		
		
	}
	
	//for(int i=0; i<n; i++) for(int j=0; j<=i; j++) A(i,j) = L(i,j);
	 //
        // Compute inverse of lower triangular matrix.
        //
        /*
        for(int j=n; j>=1; j--)
        {
            //if(nounit)
            //{
                if(L(j,j) == Zero)
                {
                	return -1;
                }
                L(j,j) = One / L(j,j);
				double ajj = -L(j,j);
            //}
            //else
            //{
            //    ajj = -1;
            //}
            if(j < n)
            {
                
                //
                // Compute elements j+1:n of j-th column.
                //
                
                nmj = n-j;
                jp1 = j+1;
                ap::vmove(t.getvector(jp1, n), a.getcolumn(j, jp1, n));
                for(i = j+1; i <= n; i++)
                {
                    if( i>j+1 )
                    {
                        v = ap::vdotproduct(a.getrow(i, jp1, i-1), t.getvector(jp1, i-1));
                    }
                    else
                    {
                        v = 0;
                    }
                    if( nounit )
                    {
                        a(i,j) = v+a(i,i)*t(i);
                    }
                    else
                    {
                        a(i,j) = v+t(i);
                    }
                }
                ap::vmul(a.getcolumn(j, jp1, n), ajj);
            }
        }
        */
/*    return 0;
}
*/

/*

Note that the inverse of a lower band matrix is not necesairly a
lower band matrix

*/

#ifdef _GenMat_LowerBandMatrix
	template <class T>
	LowerMatrix<T> Inverse(const LowerBandMatrix<T> &A)
	{
		int n = A.Rows();
		int b = A.Lower();
		
		/* Check Dimensions */
		if(A.Cols() != n)
		{
			Print("Error in Inverse(LowerBandMatrix)");
			Print("LowerBandMatrix is not Square");
			Print("A.Name",A.Name);
			Print("A.Size()",A.Size());
			HaltExecution();
		}
		
		LowerMatrix<T> AInv(n);
		Copy(AInv,A);
		int Info = Invert(AInv);
		if(Info)
		{
			Print("Error in Inverse(LowerBandMatrix)");
			Print("A is Not Invertable");
			Print("A.Name",A.Name);
			Print("A.Size()",A.Size());
			if(A.Rows() <= 12) Print("A",A);
			HaltExecution();
		}
		
		AInv.Name = "Inverse(" + A.Name + ")";
		
		return AInv;
	}
#endif

#endif