
#ifndef _GenMatUtil_InverseCholesky
#define _GenMatUtil_InverseCholesky



template <class T>
int InvertCholesky(Matrix<T> &A)
{
	int n = A.Rows();
	if(A.Cols() != n) return -1;

	if(!IsSymmetric(A)) return -2;	
	
	int Info = LapackPotrf(0,n,A.Begin(),n);
	if(Info > 0) return Info;
	
	Info = LapackPotri(0,n,A.Begin(),n);
	if(Info > 0) return Info + n;
	
	return 0;
}

#ifdef _GenMat_BlasMatrix
	template <class T>
	int InvertCholesky(BlasMatrix<T> &A)
	{
		int n = A.Rows();
		int lda = A.LeadingDimension();
		if(A.Cols() != n) return -1;

		if(!IsSymmetric(A)) return -2;	
		
		int Info = LapackPotrf(0,n,A.Begin(),lda);
		if(Info > 0) return Info;
		
		Info = LapackPotri(0,n,A.Begin(),lda);
		if(Info > 0) return Info + n;
		
		return 0;
	}
#endif

template <class T>
int InvertCholesky(SymmetricMatrix<T> &A)
{
	int n = A.Dim();
	
	Timer tm;
	
	int Info = LapackPptrf(0,n,A.Begin());
	if(Info > 0) return Info;
	
	Info = LapackPptri(0,n,A.Begin());
	if(Info > 0) return Info + n;
	
	return 0;
}

#ifdef _GenMat_SymmetricMatrixUP
	template <class T>
	int InvertCholesky(SymmetricMatrixUP<T> &A)
	{
		int n = A.Dim();
		int Info = LapackPptrf(1,n,A.Begin());
		if(Info > 0) return Info;
		Info = LapackPptri(1,n,A.Begin());
		if(Info > 0) return Info;
		return 0;
	}
#endif

#ifdef _GetMat_SymmetricMatrixLU
	template <class T>
	int InvertCholesky(SymmetricMatrixLU<T> &A)
	{
		int n = A.Dim();
		int Info = LapackPotrf(0,n,A.Begin(),n);
		if(Info > 0) return Info;
		Info = LapackPotri(0,n,A.Begin(),n);
		if(Info > 0) return Info;
		return 0;
	}
#endif

#ifdef _GenMat_SymmetricMatrixUU
	template <class T>
	int InvertCholesky(SymmetricMatrixUU<T> &A)
	{
		int n = A.Dim();
		int Info = LapackPotrf(1,n,A.Begin(),n);
		if(Info > 0) return Info;
		Info = LapackPotri(1,n,A.Begin(),n);
		if(Info > 0) return Info;
		return 0;
	}
#endif

template <class T>
Matrix<T> InverseCholesky(const Matrix<T> &A)
{
	Matrix<T> AInv = A.Copy();
	int Info = InvertCholesky(AInv);
	if(Info > 0)
	{
		Print("Error in InverseCholesky(Matrix)");
		Print("Matrix is Not Invertable");
		Print("A.Name",A.Name);
		Print("Info",Info);
		HaltExecution();
	}
	return AInv;
}

template <class T>
SymmetricMatrix<T> InverseCholesky(const SymmetricMatrix<T> &A)
{
	SymmetricMatrix<T> AInv = A.Copy();
	int Info = InvertCholesky(AInv);
	if(Info > 0)
	{
		Print("Error in InverseCholesky(SymmetricMatrix)");
		Print("Matrix is Not Invertable");
		Print("A.Name",A.Name);
		Print("Info",Info);
		HaltExecution();
	}
	return AInv;
}

#ifdef _GenMat_SymmetricMatrixUP
	template <class T>
	SymmetricMatrixUP<T> InverseCholesky(const SymmetricMatrixUP<T> &A)
	{
		SymmetricMatrixUP<T> AInv = A.Copy();
		int Info = InvertCholesky(AInv);
		if(Info > 0)
		{
			Print("Error in InverseCholesky(SymmetricMatrixUP)");
			Print("Matrix is Not Invertable");
			Print("A.Name",A.Name);
			HaltExecution();
		}
		return AInv;
	}
#endif

#ifdef _GenMat_SymmetricMatrixLU
	template <class T>
	SymmetricMatrixLU<T> InverseCholesky(const SymmetricMatrixLU<T> &A)
	{
		SymmetricMatrixLU<T> AInv = A.Copy();
		int Info = InvertCholesky(AInv);
		if(Info > 0)
		{
			Print("Error in InverseCholesky(SymmetricMatrixLU)");
			Print("Matrix is Not Invertable");
			Print("A.Name",A.Name);
			HaltExecution();
		}
		return AInv;
	}
#endif

#ifdef _GenMat_SymmetricMatrixUU
	template <class T>
	SymmetricMatrixUU<T> InverseCholesky(const SymmetricMatrixUU<T> &A)
	{
		SymmetricMatrixUU<T> AInv = A.Copy();
		int Info = InvertCholesky(AInv);
		if(Info > 0)
		{
			Print("Error in InverseCholesky(SymmetricMatrixUU)");
			Print("Matrix is Not Invertable");
			Print("A.Name",A.Name);
			HaltExecution();
		}
		return AInv;
	}
#endif

#endif
