
#ifndef _GenMatUtil_InverseLU
#define _GenMatUtil_InverseLU

	

int InvertLU(Matrix<double> &A)
{
	int n = A.Rows();
	
	if(A.Cols() != n)
	{
		Print("Error in InvertLU");
		Print("Matrix is Not Square");
		Print("A.Size()",A.Size());
		HaltExecution();
	}

	Vector<int> Piv(n);

	int Info = LapackGetrf(n,n,A.Begin(),n,Piv.Begin());
	if(Info > 0) return Info;

	int nb = 1;
	int nWork = n * nb;
	Vector<double> Work(nWork);
	Info = LapackGetri(n,A.Begin(),n,Piv.Begin(),Work.Begin(),nWork);
	if(Info > 0) return Info + n;
	
	return 0;
}

#ifdef _GenMat_BlasMatrix
	int InvertLU(BlasMatrix<double> &A)
	{
		int n = A.Rows();
		int lda = A.LeadingDimension();
		
		if(A.Cols() != n)
		{
			Print("Error in InvertLU");
			Print("Matrix is Not Square");
			Print("A.Size()",A.Size());
			HaltExecution();
		}

		Vector<int> Piv(n);

		int Info = LapackGetrf(n,n,A.Begin(),lda,Piv.Begin());
		if(Info > 0) return Info;
		
		int nb = 1;
		int nWork = n * nb;
		Vector<double> Work(nWork);	
		Info = LapackGetri(n,A.Begin(),lda,Piv.Begin(),Work.Begin(),nWork);
		if(Info > 0) return Info + n;
		
		return 0;
	}
#endif

int InvertLU(SymmetricMatrix<double> &A)
{
	int n = A.Dim();
	Matrix<double> B(n,n);
	Copy(B,A);
	
	Vector<int> Piv(n);
	
	int Info = LapackGetrf(n,n,B.Begin(),n,Piv.Begin());
	if(Info > 0) return Info;
	
	int nb = 1;
	int nWork = n * nb;
	Vector<double> Work(nWork);
	Info = LapackGetri(n,B.Begin(),n,Piv.Begin(),Work.Begin(),nWork);
	if(Info > 0) return Info + n;
	Copy(A,B);
	return 0;
}

Matrix<double> InverseLU(const Matrix<double> &A)
{

	/*
	int n = A.Rows();
	
	if(A.Cols() != n)
	{
		Print("Error in InvertLU");
		Print("Matrix is Not Square");
		Print("A.Size()",A.Size());
		HaltExecution();
	}

	//Vector<int> Piv(n);
	Matrix<double> B = A.Copy();

	//int Info = LapackGetrf(n,n,B.Begin(),n,Piv.Begin());
	/*
	int Info = LapackGetf2(n,n,B.Begin(),n,Piv.Begin());
	if(Info > 0)
	{
		Print("Error in InverseLU");
		Print("LapackGetrf Failed");
		Print("B",B);
		Print("Info",Info);
		HaltExecution();
	}
	
	int nb = 1;
	int nWork = n * nb;
	Vector<double> Work(nWork);
	Info = LapackGetri(n,B.Begin(),n,Piv.Begin(),Work.Begin(),nWork);
	if(Info > 0)
	{
		Print("Error in InverseLU");
		Print("LapackGetri Failed");
		Print("Info",Info);
		HaltExecution();
	}
	*/
	
	Matrix<double> B = A.Copy();
	int Info = InvertLU(B);
	if(Info)
	{
		Throw(  ToString("Error in Matrix = InverseLU(Matrix)\n")
		      + "B = A^-1\n"
		      + "Matrix is not Invertable\n"
		      + "A.Name: " + A.Name + "\n"
		      + "A.Size(): " + ToString(A.Size()) + "\n");
	}
	return B;
}

SymmetricMatrix<double> InverseLU(const SymmetricMatrix<double> &A)
{
	int n = A.Dim();
	if(n == 0) return SymmetricMatrix<double>(0);
	
	Matrix<double> B(n,n);
	Copy(B,A);
	int Info = InvertLU(B);
	if(Info)
	{
		Throw(  ToString("Error in SymmetricMatrix = InverseLU(SymmetricMatrix)\n")
		      + "B = A^-1\n"
		      + "Matrix is not Invertable\n"
		      + "A.Name: " + A.Name + "\n"
		      + "A.Size(): " + ToString(A.Size()) + "\n");
	}
	SymmetricMatrix<double> C(n);
	Copy(C,B);
	return C;

	//int Info = LapackGetrf(n,n,B.Begin(),n,Piv.Begin());
	/*
	int Info = LapackGetf2(n,n,B.Begin(),n,Piv.Begin());
	if(Info > 0)
	{
		Print("Error in InverseLU");
		Print("LapackGetrf Failed");
		Print("B",B);
		Print("Info",Info);
		HaltExecution();
	}
	
	int nb = 1;
	int nWork = n * nb;
	Vector<double> Work(nWork);
	Info = LapackGetri(n,B.Begin(),n,Piv.Begin(),Work.Begin(),nWork);
	if(Info > 0)
	{
		Print("Error in InverseLU");
		Print("LapackGetri Failed");
		Print("Info",Info);
		HaltExecution();
	}
	*/
	/*
	Matrix<double> B = A.Copy();
	int Info = InvertLU(B);
	if(Info)
	{
		Print("Error in InverseLU");
		Print("Info",Info);
		HaltExecution();
	}
	return B;
	*/
}

#endif
