
#ifndef _GenMatUtil_IsSymmetric
#define _GenMatUtil_IsSymmetric


/* Check if a GeneralMatrix is symmetric */
template <class T>
bool IsSymmetric(const GeneralMatrix<T> &A)
{
	int n = A.Rows();
	if(A.Cols() != n) return 0;
	for(int i=0; i<n; i++) for(int j=0; j<i; j++)
		if(abs(A(i,j) - A(j,i)) / Max(Min(abs(A(i,j)),abs(A(i,j))),One) >= SymmetricTol) return 0;
	return 1;
}

/* Check if a Matrix is symmetric */
template <class T>
bool IsSymmetric(const Matrix<T> &A)
{
	int n = A.Rows();
	if(A.Cols() != n) return 0;
	for(int i=0; i<n; i++) for(int j=0; j<i; j++)
		if(abs(A(i,j) - A(j,i)) / Max(Min(abs(A(i,j)),abs(A(j,i))),One) >= SymmetricTol) return 0;
	return 1;
}


/* Check if a Matrix is symmetric */
template <class T>
bool IsSymmetric(const Matrix<T> &A, const T &Tol)
{
	int n = A.Rows();
	if(A.Cols() != n) return 0;
	for(int i=0; i<n; i++) for(int j=0; j<i; j++)
		if(abs(A(i,j) - A(j,i)) / Max(Min(abs(A(i,j)),abs(A(j,i))),One) >= Tol) return 0;
	return 1;
}

/* Check if a LowerMatrix is symmetric */
template <class T>
bool IsSymmetric(const LowerMatrix<T> &A)
{
	int n = A.Dim();
	for(int i=0; i<n; i++) for(int j=0; j<i; j++) if(A(i,j) != 0) return 0;
	return 1;
}

/* Check if a SymmetricBandMatrix is symmetric */
#ifdef _GenMat_SymmetricBandMatrix
	template <class T>
	bool IsSymmetric(const SymmetricBandMatrix<T> &A)
	{
		return 1;
	}
#endif

#endif

