
#ifndef _LapackGetri
#define _LapackGetri


	int LapackGetri(int n, double *A, int lda, int *Piv, double *Work, int nWork);

/*

Compute the inverse of a matrix using the LU factorization
computed by LapackGetrf

This method inverts U and then computes inv(A) by solving the system
A^-1 * L = U^-1 for A^-1.

Arguments
---------

n (integer)		The dimension of the matrix A.  N >= 0.

A (array)		Array of dimension (LDA,N).
				On entry, the factors L and U from the factorization
				A = P*L*U as computed by DGETRF.
				On exit, if INFO = 0, the inverse of the original matrix A.

lda				(input) INTEGER
          		The leading dimension of the array A.  LDA >= max(1,N).

Piv (array)		(input) INTEGER array, dimension (N)
				The pivot indices from DGETRF; for 1<=i<=N, row i of the
				matrix was interchanged with row IPIV(i).

Work			(workspace/output) DOUBLE PRECISION array, dimension (LWORK)
				On exit, if INFO=0, then WORK(1) returns the optimal LWORK.

				LWORK   (input) INTEGER
				The dimension of the array WORK.  LWORK >= max(1,N).
				For optimal performance LWORK >= N*NB, where NB is
				the optimal blocksize returned by ILAENV.

				If LWORK = -1, then a workspace query is assumed; the routine
				only calculates the optimal size of the WORK array, returns
				this value as the first entry of the WORK array, and no error
				message related to LWORK is issued by XERBLA.

Info			(output) INTEGER
				= 0:  successful exit
				< 0:  if INFO = -i, the i-th argument had an illegal value
				> 0:  if INFO = i, U(i,i) is exactly zero; the matrix is
                singular and its inverse could not be computed.

Adapted from the Lapack subroutine,

	SUBROUTINE DGETRI( N, A, LDA, IPIV, WORK, LWORK, INFO )

  -- LAPACK routine (version 3.0) --
     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
     Courant Institute, Argonne National Lab, and Rice University
     June 30, 1999

*/

int LapackGetri(int n, double *A, int lda, int *Piv, double *Work, int nWork)
{
	
	/* Local Scalars */
	
	//bool LQuery;
	//int i,iws,j,jb,jj,jp,LDWork,LWorkOpt,nb,nbMin,nn;
	
	/* Test the input parameters. */
	
	//nb = 1; /* NB = ILAENV( 1, 'DGETRI', ' ', N, -1, -1, -1 ) */
    //LWorkOpt = n * nb;
	//Work[0] = LWorkOpt;
	//LQuery = (LWork == -1);
	
	if(n < 0)
	{
		Print("Error in LapackGetri");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}
	else if(n > 0 && lda < Max(1,n))
	{
		Print("Error in LapackGetri");
		Print("lda < Max(1,n)");
		Print("lda",lda);
		Print("n",n);
		HaltExecution();
	}
	//else if(LWork < Max(1,n) && !LQuery)
	else if(n > 0 && nWork < Max(1,n))
    {
    	Print("Error in LapackGetri");
    	Print("abc");
    	HaltExecution();
    }
    
	//if(LQuery) return 0;

	/* Quick return if possible */
	if(n == 0) return 0;
	
	/*
	
	Form inv(U). If Info > 0 from LapackTrtri, then U is singular,
	and the inverse is not computed.
	
	*/
	
	int Info = LapackTrtri(1,1,n,A,lda);
	
	if(Info > 0) return Info;
	
	int ldw = n;
	/*
	nbMin = 2;
    LDWork = n;
    if(nb > 1 && nb < n)
    {
    	iws = Max(LDWork * nb,1);
    	if(LWork < iws)
    	{
    		nb = LWork / LDWork;
    		nbMin = Max(2,32); /* NBMIN = MAX( 2, ILAENV( 2, 'DGETRI', ' ', N, -1, -1, -1 ) ) */
    /*	}
	}
	else
	{
		iws = n;
	}
	*/
	int nbMin = 2;
	int nb = 1;
	
	/* Solve the equation A^-1 * L = U^-1 for A^-1. */
	if(nb < nbMin || nb >= n)
	{
		/* Use unblocked code. */
		for(int j=n-1; j>=0; j--)
		{
			/* Copy current column of L to WORK and replace with zeros. */
			for(int i=j+1; i<n; i++)
			{
				Work[i] = A[lda*i+j];
				A[lda*i+j] = Zero;
			}
			
			/* Compute current column of inv(A). */
			if(j < n - 1) BlasGemv(0,n,n-j-1,-One,&A[j+1],lda,&Work[j+1],1,One,&A[j],lda);
		
			// CALL DGEMV( 'No transpose', N, N-J, -ONE, A( 1, J+1 ),	LDA, WORK( J+1 ), 1, ONE, A( 1, J ), 1 )
		}
	}
	else
	{
	
		Print("Error in LapackGetri");
		Print("Blocked code not ready yet");
		HaltExecution();
		
		/* Use blocked code. */
		int nn = ((n - 1) / nb) * nb + 1;
        for(int j=nn-1; j>=0; j-=nb)
        {
        	int jb = Min(nb,n-j+1);
        	
        	/* 
        	
        	Copy current block column of L to WORK and replace with
			zeros.
			
			*/
			
			for(int jj=j; jj<=j+jb-1; jj++)
			{
				for(int i=jj+1; i<n; i++)
				{
					Work[i+(jj-j)*ldw] = A[lda*i+j];
					A[lda*i+j] = Zero;
				}
			}
			
			/* Compute current block column of A^-1. */
			if(j + jb <= n)
			{
				BlasGemm(0,0,n,jb,n-j-jb+1,-One,&A[j+jb],lda,&Work[j+jb],ldw,One,&A[j],lda);
				BlasTrsm(0,0,0,0,n,jb,One,&Work[j],ldw,&A[j],lda);
			}
		}
	}
	
	/* Apply column interchanges. */
	for(int j=n-2; j>=0; j--)
	{
		int jp = Piv[j];
		if(jp != j) BlasSwap(n,&A[j],lda,&A[jp],lda);
	}
	
	//Work[0] = iws;
	return 0;
}

#endif