
#ifndef _LapackLaswp
#define _LapackLaswp


	void LapackLaswp(int n, double *A, int lda, int k1, int k2, int *Piv, int incx);
	
/*

Perform a series of row interchanges on the matrix A.
One row interchange is initiated for each of rows k1 through k2 of A.


Arguments
---------

n (integer)		-	The number of columns of the matrix A.

A (array)		-	Array, dimension (LDA,N)
					On entry, the matrix of column dimension N to which the row
					interchanges will be applied.
					On exit, the permuted matrix.

lda (integer)	-	The leading dimension of the array A.

k1 (integer)	-	The first element of Piv for which a row interchange will
					be done.
					
k2 (integer)	-	The last element of Piv for which a row interchange will
					be done.

Piv (array)		-	Array, dimension (m * abs(incx))
					The vector of pivot indices.  Only the elements in positions
					k1 through k2 of IPIV are accessed.
					Piv(k) = l implies rows k and l are to be interchanged.
					
incx (integer)	-	The increment between successive values of Piv.  If Piv
					is negative, the pivots are applied in reverse order.


Adapted from the Lapack routine,

	DLASWP( N, A, LDA, K1, K2, IPIV, INCX )
	
	-- LAPACK auxiliary routine (version 3.0) --
    Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
    Courant Institute, Argonne National Lab, and Rice University
    June 30, 1999
     
    Modified by
	R. C. Whaley, Computer Science Dept., Univ. of Tenn., Knoxville, USA

*/


void LapackLaswp(int n, double *A, int lda, int k1, int k2, int *Piv, int incx)
{
	int i,ip,ix,j,k,n32;
	double Temp;
	
	/*
	
	Interchange row i with row Piv(i) for each of rows k1 through k2.
	The following code works in blocks of size 32
	
	*/
	
	if(incx > 0)
	{
		n32 = (n / 32) * 32;
		
		if(n32 > 0)
		{
			for(j=0; j<n32; j+=32)
			{
				ix = k1;
				for(i=k1; i<=k2; i++)
				{
					ip = Piv[ix];
					if(ip != i)
					{
						for(k=j; k<j+32; k++)
						{
							Temp        = A[lda*i+k];
							A[lda*i+k]  = A[lda*ip+k];
							A[lda*ip+k] = Temp;
						}
					}
					ix += incx;
				}
			}
		}
			
		if(n32 != n)
		{
			ix = k1;
			for(i=k1; i<=k2; i++)
			{
				ip = Piv[ix];
				if(ip != i)
				{
					for(k=n32; k<n; k++)
					{
						Temp        = A[lda*i+k];
						A[lda*i+k]  = A[lda*ip+k];
						A[lda*ip+k] = Temp;
					}
				}
				ix += incx;
			}
		}
	}
	else if(incx < 0)
	{	
		n32 = (n / 32) * 32;
	
		if(n32 < 0)
		{
			for(j=0; j<n32; j+=32)
			{
				ix = k2;
				for(i=k2; i>=k1; i--)
				{
					ip = Piv[ix];
					if(ip != i)
					{
						for(k=j; k<j+32; k++)
						{
							Temp        = A[lda*i+k];
							A[lda*i+k]  = A[lda*ip+k];
							A[lda*ip+k] = Temp;
						}
					}
					ix += incx;
				}
			}
		}
	
		if(n32 != n)
		{
			ix = k2;
			for(i=k2; i>=k1; i--)
			{
				ip = Piv[ix];
				if(ip != i)
				{
					for(k=n32; k<n; k++)
					{
						Temp        = A[lda*i+k];
						A[lda*i+k]  = A[lda*ip+k];
						A[lda*ip+k] = Temp;
					}
				}
				ix += incx;
			}
		}
	}
}

#endif
