

#ifndef _LapackPotrs
#define _LapackPotrs


	void LapackPotrs(bool Upper, int n, int nRhs, double *A, int lda, double *B, int ldb);

/*

Solves a system of linear equations A * X = B with a symmetric
positive definite matrix A using the Cholesky factorization
A = U^T * U or A = L * L^T computed by LapackPotrf


Arguments
---------

UPLO    (input) CHARACTER*1
        = 'U':  Upper triangle of A is stored;
        = 'L':  Lower triangle of A is stored.

N       (input) INTEGER
        The order of the matrix A.  N >= 0.

NRHS    (input) INTEGER
        The number of right hand sides, i.e., the number of columns
        of the matrix B.  NRHS >= 0.

A       (input) DOUBLE PRECISION array, dimension (LDA,N)
        The triangular factor U or L from the Cholesky factorization
        A = U**T*U or A = L*L**T, as computed by DPOTRF.

LDA     (input) INTEGER
        The leading dimension of the array A.  LDA >= max(1,N).

B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
        On entry, the right hand side matrix B.
        On exit, the solution matrix X.

LDB     (input) INTEGER
        The leading dimension of the array B.  LDB >= max(1,N).

INFO    (output) INTEGER
        = 0:  successful exit
        < 0:  if INFO = -i, the i-th argument had an illegal value


Adapted form the Lapack subroutine,

	DPOTRS( UPLO, N, NRHS, A, LDA, B, LDB, INFO )
	
	-- LAPACK routine (version 3.0) --
	Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
	Courant Institute, Argonne National Lab, and Rice University
	March 31, 1993

*/

void LapackPotrs(bool Upper, int n, int nRhs, double *A, int lda, double *B, int ldb)
{

	/* Quick return if possible */
	if(n == 0 || nRhs == 0) return;

	/* Test the input parameters. */
	if(n < 0)
	{
		Print("Error in LapackPotrs");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}
	else if(nRhs < 0)
	{
		Print("Error in LapackPotrs");
		Print("nRhs < 0");
		Print("nRhs",nRhs);
		HaltExecution();
	}
	else if(lda < Max(1,n))
	{
		Print("Error in LapackPotrs");
		Print("lda < Max(1,n)");
		Print("lda",lda);
		Print("n",n);
		HaltExecution();
	}
	else if(ldb < Max(1,nRhs))
	{
		Print("Error in LapackPotrs");
		Print("ldb < Max(1,nRhs)");
		Print("ldb",ldb);
		Print("nRhs",nRhs);
		HaltExecution();
	}
		
	if(Upper)
	{
		/* Solve A * X = B where A = U^T * U. */
		
		/* Solve U^T * X = B, overwriting B with X. */
		
		//CALL DTRSM( 'Left', 'Upper', 'Transpose', 'Non-unit', N, NRHS, ONE, A, LDA, B, LDB )
		BlasTrsm(1,1,1,1,n,nRhs,One,A,lda,B,ldb);
		
		/* Solve U * X = B, overwriting B with X. */
		
		//CALL DTRSM( 'Left', 'Upper', 'No transpose', 'Non-unit', N, NRHS, ONE, A, LDA, B, LDB )
		BlasTrsm(1,1,0,1,n,nRhs,One,A,lda,B,ldb);
	}
	else
	{
		/* Solve A*X = B where A = L * L^T. */

		/* Solve L * X = B, overwriting B with X. */
		
		//CALL DTRSM( 'Left', 'Lower', 'No transpose', 'Non-unit', N, NRHS, ONE, A, LDA, B, LDB )
		BlasTrsm(1,0,0,1,n,nRhs,One,A,lda,B,ldb);
		
		/* Solve L^T * X = B, overwriting B with X. */
		//CALL DTRSM( 'Left', 'Lower', 'Transpose', 'Non-unit', N, NRHS, ONE, A, LDA, B, LDB )
		BlasTrsm(1,0,1,1,n,nRhs,One,A,lda,B,ldb);
	}
}

#endif
