
#ifndef _LapackPptri
#define _LapackPptri

	/* Compute the inverse of a symmetric positive definite matris using the
	cholesky factorization computed by LapackPptri */
	int LapackPptri(bool Upper, int n, double *A);

/*

Compute the inverse of a real symmetric positive definite
matrix A using the Cholesky factorization A = U^T * U or A = L * L^T
computed by LapackPptrf


Arguments
---------

Upper (boolean)		-	= 1:  Upper triangular factor is stored in A;
						= 0:  Lower triangular factor is stored in A.

n (integer)			-	The dimension of the matrix A. n >= 0.

A (arrray)			-	Array, dimension (n * (n + 1) / 2)
						On entry, the triangular factor U or L from the Cholesky
						factorization A = U^T * U or A = L * L^T, packed columnwise as
 						a linear array.  The j-th column of U or L is stored in the
						array AP as follows:
						if Upper = 1, A(i + (j-1)*j/2) = U(i,j) for 1<=i<=j;
						if Upper = 0, A(i + (j-1)*(2n-j)/2) = L(i,j) for j<=i<=n.

						On exit, the upper or lower triangle of the (symmetric)
						inverse of A, overwriting the input factor U or L.

Info (integer)		-	= 0:  successful exit
						> 0:  if Info = i, the (i-1,i-1) element of the factor U or L is
						zero, and the inverse could not be computed.


Adapted from the Lapack routine,

	SUBROUTINE DPPTRI( UPLO, N, AP, INFO )

	-- LAPACK routine (version 3.0) --
    Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
    Courant Institute, Argonne National Lab, and Rice University
    March 31, 1993

*/

int LapackPptri(bool Upper, int n, double *A)
{
	int j,jc,jj,jjn;
	double Ajj;
	double One = 1.0;

	/* Test the input parameters. */
	if(n < 0)
	{
		Print("Error in LapackPptri");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}

	/* Quick return if possible */
	if(n == 0) return 0;

	/* Invert the triangular Cholesky factor U or L. */
	int Info = LapackTptri(Upper,1,n,A);
	if(Info > 0) return Info;

	if(Upper)
	{
		/* Compute the product inv(U) * inv(U)^T. */
		jj = 0;
		for(j=0; j<n; j++)
		{
			jjn = jj + n - j;
			A[jj] = BlasDot(n-j,&A[jj],1,&A[jj],1);
			if(j < n - 1) BlasTpmv(1,0,1,n-j-1,&A[jjn],&A[jj+1],1);
			jj = jjn;
		}
	}
	else
	{
		/* Compute the product inv(L)^T * inv(L). */
		jj = -1;
		for(j=0; j<n; j++)
		{
			//Print("LapackPppri, j",j);

			jc = jj + 1;
			jj += j + 1;
			if(j > 0) BlasSpr(0,j,One,&A[jc],1,A);
			Ajj = A[jj];
			BlasScal(j+1,Ajj,&A[jc],1);
		}
	}

	return 0;	// Successful exit
}

#endif
