
#ifndef _LapackTptri
#define _LapackTptri

	/* Compute the inverse of of an upper or lower triangular
	matrix A stored in packed format. */
	int LapackTptri(bool Upper, bool Diag, int n, double *A);

/*

Compute the inverse of of an upper or lower triangular
matrix A stored in packed format.


Arguments
---------

Upper (boolean)		-		Specifies if the matrix in upper-triangular

Diag (boolean)		-		Specifies whether the matrix has a non-unit
							diagonal

n (integer)			-		The dimension of the matrix. n >= 0.

A (array)			-		Array of dimension n * (n + 1) / 2 representing
							an upper or lower triangular matrix in packed
							strorage. On exit, contains the inverse of the
							matrix.

Info (integer)		-		= 0:  successful exit
							< 0:  if INFO = -i, the i-th argument had an illegal value
							> 0:  if INFO = i, A(i,i) is exactly zero.  The triangular
							matrix is singular and its inverse can not be computed.


Adapted from the Lapack routine,

	DTPTRI( UPLO, DIAG, N, AP, INFO )

	LAPACK routine (version 3.0) --
    Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
    Courant Institute, Argonne National Lab, and Rice University
    September 30, 1994

*/


int LapackTptri(bool Upper, bool Diag, int n, double *A)
{
	
	//int j,jc,jcLast,jj;
    //double Ajj;

	/* Test the Input Parameters. */
	if(n < 0)
	{
		Print("Error in LapackTptri");
		Print("n <= 0");
		Print("n",n);
		HaltExecution();
	}

	/* Check for Singularity if Non-unit. */
    if(Diag)
    {
    	if(Upper)
    	{
        	int jj = 0;
        	for(int j=0; j<n; j++)
        	{
        		if(A[jj] == Zero) return j + 1;
        		jj += n - j;
        	}
        }
        else
        {
    		int jj = 0;
    		for(int j=0; j<n; j++)
    		{
    			if(A[jj] == Zero) return j + 1;
    			jj += j + 2;
    		}
        }
    }

	if(Upper)
	{
		/* Compute Inverse of Upper Triangular Matrix. */
		int jcLast;
		double Ajj;
		int jc = n * (n + 1) / 2 - 1;
        for(int j=n-1; j>=0; j--)
        {
         	if(Diag)
        	{
        		A[jc] = One / A[jc];
            	Ajj = -A[jc];
            }
            else Ajj = -One;

            if(j < n - 1)
            {
            	/* Compute Elements j+1:n of j-th Column. */
				BlasTpmv(1,1,Diag,n-j-1,&A[jcLast],&A[jc+1],1);
				BlasScal(n-j-1,Ajj,&A[jc+1],1);
			}
			jcLast = jc;
			jc -= n - j + 1;
		}
	}
	else
	{
    	/* Compute Inverse of Lower Triangular Matrix. */
       	double Ajj;
       	int jc = 0;
    	for(int j=0; j<n; j++)
    	{
	   		if(Diag)
    		{
    			A[jc+j] = One / A[jc+j];
    			Ajj = -A[jc+j];
    		}
    		else Ajj = -One;

			/* Compute Elements 1:j-1 of j-th Column. */
			if(j > 0)
			{
				BlasTpmv(0,1,Diag,j,A,&A[jc],1);
				BlasScal(j,Ajj,&A[jc],1);
			}

			jc += j + 1;
		}
	}

	return 0;
}

#endif
