
#ifndef _LapackTrti2
#define _LapackTrti2


	int LapackTrti2(bool Upper, bool Diag, int n, double *A, int lda);

/*

Compute the inverse of a real upper or lower triangular
matrix.

This is the Level 2 BLAS version of the algorithm.


Arguments
---------


UPLO    (input) CHARACTER*1
        Specifies whether the matrix A is upper or lower triangular.
        = 'U':  Upper triangular
        = 'L':  Lower triangular

DIAG    (input) CHARACTER*1
        Specifies whether or not the matrix A is unit triangular.
        = 'N':  Non-unit triangular
        = 'U':  Unit triangular

N       (input) INTEGER
        The order of the matrix A.  N >= 0.

A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
        On entry, the triangular matrix A.  If UPLO = 'U', the
        leading n by n upper triangular part of the array A contains
        the upper triangular matrix, and the strictly lower
        triangular part of A is not referenced.  If UPLO = 'L', the
        leading n by n lower triangular part of the array A contains
        the lower triangular matrix, and the strictly upper
        triangular part of A is not referenced.  If DIAG = 'U', the
        diagonal elements of A are also not referenced and are
        assumed to be 1.

        On exit, the (triangular) inverse of the original matrix, in
        the same storage format.

LDA     (input) INTEGER
        The leading dimension of the array A.  LDA >= max(1,N).

INFO    (output) INTEGER
        = 0: successful exit
        < 0: if INFO = -k, the k-th argument had an illegal value


Adapted from the Lapack subroutine,

	DTRTI2( UPLO, DIAG, N, A, LDA, INFO )

	-- LAPACK routine (version 3.0) --
    Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
	Courant Institute, Argonne National Lab, and Rice University
	February 29, 1992

*/


int LapackTrti2(bool Upper, bool Diag, int n, double *A, int lda)
{
	/* Parameters */
	double One = 1.0;
	double Aii;	
	
	/* Test the input parameters. */
	if(n < 0)
	{
		Print("Error in LapackTrti2");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}
	else if(lda < Max(1,n))
	{
		Print("Error in LapackTrti2");
		Print("lda < Max(1,n)");
		Print("lda",lda);
		Print("n",n);
		HaltExecution();
	}
	
    if(Upper)
	{
		/* Compute inverse of upper triangular matrix. */
		for(int i=0; i<n; i++)
		{
			if(Diag)
			{
				A[lda*i+i] = One / A[lda*i+i];
				Aii = -A[lda*i+i];
			}
			else Aii = -One;
			
			
			/* Compute elements 1:j-1 of j-th column. */
			BlasTrmv(1,0,Diag,i,A,lda,&A[i],lda);
			BlasScal(i,Aii,&A[i],lda);
		}
	}
	else
	{
		/* Compute inverse of lower triangular matrix. */
		
		for(int i=0; i<n; i++)
		{
			if(Diag)
			{
				A[lda*i+i] = One / A[lda*i+i];
				Aii = -A[lda*i+i];
			}
			else Aii = -One;
			
			BlasTrmv(0,1,Diag,i,A,lda,&A[lda*i],1);
			BlasScal(i,Aii,&A[lda*i],1);	
		}
	}
	
	return 0;
}

#endif