
#ifndef _LapackTrtri
#define _LapackTrtri


	int LapackTrtri(bool Upper, bool Diag, int n, double *A, int lda);
	
/*

Compute the inverse of a real upper or lower triangular
matrix A.

This is the Level 3 BLAS version of the algorithm.

Arguments
=========

Upper   (input) CHARACTER*1
        = 'U':  A is upper triangular;
        = 'L':  A is lower triangular.

Diag    (input) CHARACTER*1
        = 'N':  A is non-unit triangular;
        = 'U':  A is unit triangular.

n       (input) INTEGER
        The order of the matrix A.  N >= 0.

A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
        On entry, the triangular matrix A.  If UPLO = 'U', the
        leading N-by-N upper triangular part of the array A contains
        the upper triangular matrix, and the strictly lower
        triangular part of A is not referenced.  If UPLO = 'L', the
        leading N-by-N lower triangular part of the array A contains
        the lower triangular matrix, and the strictly upper
        triangular part of A is not referenced.  If DIAG = 'U', the
        diagonal elements of A are also not referenced and are
        assumed to be 1.
        On exit, the (triangular) inverse of the original matrix, in
        the same storage format.

lda     (input) INTEGER
        The leading dimension of the array A.  LDA >= max(1,N).

Info    (output) INTEGER
        = 0: successful exit
        < 0: if INFO = -i, the i-th argument had an illegal value
        > 0: if INFO = i, A(i,i) is exactly zero.  The triangular
             matrix is singular and its inverse can not be computed.


Adapted from the Lapack subroutine,

	DTRTRI( UPLO, DIAG, N, A, LDA, INFO )

    -- LAPACK routine (version 3.0) --
    Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
    Courant Institute, Argonne National Lab, and Rice University
    March 31, 1993

*/

int LapackTrtri(bool Upper, bool Diag, int n, double *A, int lda)
{
	
	/* Parameters */
	double One = 1.0;
	double Zero = 0.0;
	
	int j,jb,nb,nn;
	
	/* Test the input parameters. */
	if(n < 0)
	{
		Print("Error in LapackTrtri");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}
	else if(lda < Max(1,n))
	{
		Print("lda < Max(1,n)");
		Print("lda",lda);
		Print("n",n);
		HaltExecution();
	}
	
	/* Quick return if possible */
	if(n == 0) return 0;
	
	/* Check for singularity if non-unit. */
	if(Diag)
	{
		for(int Info=1; Info<=n; Info++)
		{
			if(A[lda*(Info-1)+Info-1] == Zero) return Info;
		}
	}
	
	int Info;
	
	/* Determine the block size for this environment. */
	nb = 1; /* NB = ILAENV( 1, 'DTRTRI', UPLO // DIAG, N, -1, -1, -1 ) */
    
    if(nb <= 1 || nb >= n)
    {
    	/* Use unblocked code */
    	Info = LapackTrti2(Upper,Diag,n,A,lda);
    }
    else
    {
       	/* Use blocked code */
    	if(Upper)
    	{
    		/* Compute inverse of upper triangular matrix */
    		for(j=0; j<n; j+=nb)
    		{
    			jb = Min(nb,n-j);
    			
    			/* Compute rows 1:j-1 of current block column */
    			BlasTrmm(1,1,0,Diag,j,jb, One, A         ,lda,&A[j],lda);
				BlasTrsm(0,1,0,Diag,j,jb,-One,&A[lda*j+j],lda,&A[j],lda);

				/* Compute inverse of current diagonal block */
				Info = LapackTrti2(1,Diag,jb,&A[lda*j+j],lda);
			}
		}
		else
		{
			/* Compute inverse of lower triangular matrix */
			nn = ((n - 1) / nb) * nb + 1;
            for(int j=nn-1; j>=0; j-=nb)
            {
            	jb = Min(nb,n-j);
            		
            	if(j + jb < n)
            	{
            		/* Compute rows j+jb:n of current block column */
            		BlasTrmm(1,0,0,Diag,n-j-jb,jb, One,&A[lda*(j+jb)+j+jb],lda,&A[lda*(j+jb)+j],lda);
 					BlasTrsm(0,0,0,Diag,n-j-jb,jb,-One,&A[lda*j+j]        ,lda,&A[lda*(j+jb)+j],lda);
               	}
              
              	/* Compute inverse of current diagonal block */
				Info = LapackTrti2(0,Diag,jb,&A[lda*j+j],lda);
            }
        }
    }
   
	return Info;
}

#endif