
#ifndef _Stat_LinRegObj
#define _Stat_LinRegObj


/* Object-oriented version of linear regression */
class DummyObj
{
	public:
	
	Vector<string> Dummy;
	DataPoint<string> Exclude;
	
	DummyObj() {}
	
	DummyObj(const Vector<string> &Dummy, const DataPoint<string> &Exclude)
	{
		this->Dummy = Dummy;
		this->Exclude = Exclude;	
	}
	
	~DummyObj() {}
};

class LinRegObj
{

	public:

	string Name;
	bool NoConst;
	bool RobustSe;
	int N;
	int K;
	Vector<double> Y;
	//Matrix<double> X;
	Matrix<double> XTrans;
	Vector<double> Beta;
	Vector< DataPoint<double> > BetaSe;
	Vector< DataPoint<double> > BetaLower;
	Vector< DataPoint<double> > BetaUpper;
	Vector<string> BetaLabels;
	double Sigma2;
	double Sigma2Se;
	SymmetricMatrix<double> BetaVar;
	double RSquared;
	double AdjRSquared;
	Vector<double> Resid;
	Vector<double> Pred;
	Stack<DummyObj> Dummies;
	double LogLikelihood;
	
	/*Default Constructor */
	LinRegObj() {};
	
	/* Contructor */
	LinRegObj(const GeneralVector<double> &Y, const GeneralVector<double> &X)
	{
		this->Y.Name = "LinRegObj::Y";
		this->Y = Vector<double>(Y.Size());
		Copy(this->Y,Y);

		Name = "LinRegObj";
		NoConst  = False;
		RobustSe = True;

		N = Y.Size();
		if(X.Size() != N)
		{
			Print("Error in LinRegObj");
			Print("Incompadible Dimensions");
			HaltExecution();
		}
		
		//this->X = Matrix<double>(N,1);
		//this->X.Name = "X";
		//for(int n=0; n<N; n++) this->X(n,0) = X(n);
		XTrans = Matrix<double>(1,N);
		XTrans.Name = "XTrans";
		for(int n=0; n<N; n++) XTrans(0,n) = X(n);
		
		Sigma2 = Zero;
		Sigma2Se = Zero;
		LogLikelihood = Zero;
	}
	
	/* Contructor */
	LinRegObj(const GeneralVector< DataPoint<double> > &Y, const GeneralVector< DataPoint<double> > &X)
	{
		Name = "LinRegObj";
		NoConst  = False;
		RobustSe = True;

		N = Y.Size();
		if(X.Size() != N)
		{
			Print("Error in LinRegObj");
			Print("Incompadible Dimensions");
			HaltExecution();
		}
		
		int N1 = 0;
		for(int n=0; n<N; n++) if(!Y(n).IsMissing() && !X(n).IsMissing()) N1++;
		
		//this->X = Matrix<double>(N1,1);
		//this->X.Name = "X";
		XTrans = Matrix<double>(1,N1);
		XTrans.Name = "XTrans";
		this->Y = Vector<double>(N1);
		this->Y.Name = "Y";
		int Curr = 0;
		for(int n=0; n<N; n++) if(!Y(n).IsMissing() && !X(n).IsMissing())
		{
			//this->X(Curr,0) = X(n).GetData();
			XTrans(0,Curr) = X(n).GetData();
			this->Y(Curr)   = Y(n).GetData();
			Curr++;
		}
		N = N1;
		Sigma2 = Zero;
		Sigma2Se = Zero;
		LogLikelihood = Zero;
	}

	/* Contructor */
	LinRegObj(const GeneralVector<double> &Y, const Matrix<double> &X)
	{
		Print("Linregobj constructor");
		Print("Y.Size()",Y.Size());
		this->Y.Name = "LinRegObj::Y";
		this->Y = Vector<double>(Y.Size());
		Copy(this->Y,Y);
		XTrans = Matrix<double>(X.Cols(),X.Rows());
		for(int n=0; n<X.Rows(); n++) for(int k=0; k<X.Cols(); k++) XTrans(k,n) = X(n,k);
		 
		Name = "LinRegObj";
		NoConst  = True;
		RobustSe = True;

		N = Y.Size();
		if(X.Rows() != N)
		{
			Print("Error in LinRegObj");
			Print("Incompadible Dimensions");
			HaltExecution();
		}
		
		if(N == 0)
		{
			Print("Error in LinRegObj");
			Print("N = 0");
			HaltExecution();
		}
		
		Sigma2 = Zero;
		Sigma2Se = Zero;
		LogLikelihood = Zero;
	}
	
	/* Contructor */
	LinRegObj(const GeneralVector< DataPoint<double> > &Y, const Matrix< DataPoint<double> > &X)
	{		
		Name = "LinRegObj";
		NoConst  = True;
		RobustSe = True;

		N = Y.Size();
		int K = X.Cols();
		if(X.Rows() != N)
		{
			Print("Error in LinRegObj");
			Print("Incompadible Dimensions");
			HaltExecution();
		}
		
		int N1 = 0;
		for(int n=0; n<N; n++)
		{
			int Info = True;
			if(Y(n).IsMissing()) Info = False;
			for(int k=0; k<K; k++) if(X(n,k).IsMissing()) Info = False;
			if(Info) N1++;
		}
		
		//this->X = Matrix<double>(N1,K);
		//this->X.Name = "X";
		XTrans = Matrix<double>(K,N1);
		XTrans.Name = "XTrans";
		this->Y = Vector<double>(N1);
		this->Y.Name = "Y";
		int Curr = 0;
		for(int n=0; n<N; n++)
		{
			int Info = True;
			if(Y(n).IsMissing()) Info = False;
			for(int k=0; k<K; k++) if(X(n,k).IsMissing()) Info = False;
			if(Info)
			{
				//for(int k=0; k<K; k++) this->X(Curr,k) = X(n,k).GetData();
				for(int k=0; k<K; k++) XTrans(k,Curr) = X(n,k).GetData();
				this->Y(Curr)   = Y(n).GetData();
				Curr++;
			}
		}
		N = N1;
		Sigma2 = Zero;
		Sigma2Se = Zero;
		LogLikelihood = Zero;
	}
	
	void AddDummy(const DummyObj &dum)
	{
		if(dum.Dummy.Size() != N)
		{
			Print("Error in LinRegObj::AddDummy");
			Print("Incompadible Dimensions");
			Print("N",N);
			Print("dum.Dummy.Size()",dum.Dummy.Size());
			HaltExecution();
		}
		if(!dum.Exclude.IsMissing())
		{
			int Info = Search(dum.Dummy,dum.Exclude.GetData());
			if(Info < 0)
			{
				Print("Error in AddDummy");
				Print("Excluded Cateory Does Not Exist");
				Print("dum.Exclude",dum.Exclude);
				HaltExecution();
			}
		}		
		Dummies.Push(dum);
	}

	int Solve()
	{
		int CountConst = 0;
		if(!NoConst) CountConst++;
		//int K1 = X.Cols() + (!NoConst);
		int K1 = XTrans.Rows() + (!NoConst);
		Vector< Vector<string> > DummyList(Dummies.Size());
		K = K1;
		for(int i=0; i<Dummies.Size(); i++)
		{
			DummyList(i) = ListValues(Dummies(i).Dummy);
			if(Dummies(i).Exclude.IsMissing())
			{
				K += DummyList(i).Size();
				CountConst++;
			}
			else
			{
				K += DummyList(i).Size() - 1;
			}
		}
		if(CountConst > 1)
		{
			Print("Error in LinRegObj::Solve()");
			Print("Too Many Constants!");
			HaltExecution();
		}
		
		Beta = Vector<double>(K);

		/* Form XXdN and XYdN */
		Vector<double> XYdN(K);
		SymmetricMatrix<double> XXdN(K);
		if(NoConst)
		{
			/* Do Not Add a Constant */
			{
				/* XY Part */
				for(int k1=0; k1<K1; k1++)
				{
					XYdN(k1) = Zero;
					//for(int n=0; n<N; n++) XYdN(k1) += X(n,k1) * Y(n);
					//for(int n=0; n<N; n++) XYdN(k1) += XTrans(k1,n) * Y(n);
					XYdN(k1) += BlasDot(N,&XTrans(k1,0),1,Y.Begin(),1);
					XYdN(k1) /= (double)N;
				}
				
				/* DY Part */
				int Curr = K1;
				for(int i=0; i<Dummies.Size(); i++)
				{
					int KCurr;
					if(Dummies(i).Exclude.IsMissing()) KCurr = DummyList(i).Size();
					else							   KCurr = DummyList(i).Size() - 1;
						
					for(int k1=Curr; k1<Curr+KCurr; k1++) XYdN(k1) = Zero;
					for(int n=0; n<N; n++)
					{
						int k1 = BinarySearch(DummyList(i),Dummies(i).Dummy(n));
						if(k1 < 0)
						{
							Print("Error QQQ");
							HaltExecution();
						}
						if(!Dummies(i).Exclude.IsMissing()) if(Dummies(i).Dummy(n) > Dummies(i).Exclude.GetData()) k1--;
						
						if(Dummies(i).Exclude.IsMissing()) XYdN(Curr+k1) += Y(n);
						else                               if(Dummies(i).Dummy(n) != Dummies(i).Exclude) XYdN(Curr+k1) += Y(n);
					}
					for(int k1=Curr; k1<Curr+KCurr; k1++) XYdN(k1) /= (double)N;
					Curr += KCurr;
				}
				
				/* XX Diag Part */
				for(int k1=0; k1<K1; k1++)
				{
					XXdN(k1,k1) = Zero;
					//for(int n=0; n<N; n++) XXdN(k1,k1) += Sqr(X(n,k1));
					XXdN(k1,k1) += BlasDot(N,&XTrans(k1,0),1,&XTrans(k1,0),1);
					XXdN(k1,k1) /= (double)N;
				}
				
				/* XX Off-Diag Part */
				for(int k1=0; k1<K1; k1++) for(int k2=0; k2<k1; k2++)
				{
					XXdN(k1,k2) = Zero;
					//for(int n=0; n<N; n++) XXdN(k1,k2) += X(n,k1) * X(n,k2);
					XXdN(k1,k2) += BlasDot(N,&XTrans(k1,0),1,&XTrans(k2,0),1);
					XXdN(k1,k2) /= (double)N;
				}
						
				/* DX Part */
				Curr = K1;
				for(int i=0; i<Dummies.Size(); i++)
				{
					int KCurr;
					if(Dummies(i).Exclude.IsMissing()) KCurr = DummyList(i).Size();
					else							   KCurr = DummyList(i).Size() - 1;
						
					for(int k1=Curr; k1<Curr+KCurr; k1++) for(int k2=0; k2<K1; k2++) XXdN(k1,k2) = Zero;
					
					for(int n=0; n<N; n++)
					{
						int k1 = BinarySearch(DummyList(i),Dummies(i).Dummy(n));
						if(k1 < 0)
						{

							HaltExecution();
						}
						if(!Dummies(i).Exclude.IsMissing()) if(Dummies(i).Dummy(n) > Dummies(i).Exclude.GetData()) k1--;
						//if(Dummies(i).Exclude.IsMissing()) for(int k2=0; k2<K1; k2++) XXdN(Curr+k1,k2) += X(n,k2);
						if(Dummies(i).Exclude.IsMissing()) for(int k2=0; k2<K1; k2++) XXdN(Curr+k1,k2) += XTrans(k2,n);
						else                               if(Dummies(i).Dummy(n) != Dummies(i).Exclude) for(int k2=0; k2<K1; k2++) XXdN(Curr+k1,k2) += XTrans(k2,n);
					}
					for(int k1=Curr; k1<Curr+KCurr; k1++) for(int k2=0; k2<K1; k2++) XXdN(k1,k2) /= (double)N;
					Curr += KCurr;
				}
			
				/* DD Diag Part */
				Curr = K1;
				for(int i=0; i<Dummies.Size(); i++)
				{
					int KCurr;
					if(Dummies(i).Exclude.IsMissing()) KCurr = DummyList(i).Size();
					else							   KCurr = DummyList(i).Size() - 1;
						
					for(int k1=Curr; k1<Curr+KCurr; k1++) for(int k2=Curr; k2<=k1; k2++) XXdN(k1,k2) = Zero;
					
					for(int n=0; n<N; n++)
					{
						int k1 = BinarySearch(DummyList(i),Dummies(i).Dummy(n));
						if(k1 < 0)
						{
							Print("Error QQQ");
							HaltExecution();
						}
						if(!Dummies(i).Exclude.IsMissing()) if(Dummies(i).Dummy(n) > Dummies(i).Exclude.GetData()) k1--;
						if(Dummies(i).Exclude.IsMissing()) XXdN(Curr+k1,Curr+k1) += One;
						else                               if(Dummies(i).Dummy(n) != Dummies(i).Exclude) XXdN(Curr+k1,Curr+k1) += One;
					}
					for(int k1=Curr; k1<Curr+KCurr; k1++) XXdN(k1,k1) /= (double)N;
					Curr += KCurr;
				}
			}
			
			/* DD Off-Diag Part */
			{
				int Curr1 = K1;
				for(int i1=0; i1<Dummies.Size(); i1++)
				{
					int KCurr1;
					if(Dummies(i1).Exclude.IsMissing()) KCurr1 = DummyList(i1).Size();
					else							    KCurr1 = DummyList(i1).Size() - 1;
						
					int Curr2 = K1;
					for(int i2=0; i2<i1; i2++)
					{
						int KCurr2;
						if(Dummies(i2).Exclude.IsMissing()) KCurr2 = DummyList(i2).Size();
						else							    KCurr2 = DummyList(i2).Size() - 1;
					
						for(int k1=Curr1; k1<Curr1+KCurr1; k1++) for(int k2=Curr2; k2<Curr2+KCurr2; k2++) XXdN(k1,k2) = Zero;
						for(int n=0; n<N; n++)
						{
							int k1 = BinarySearch(DummyList(i1),Dummies(i1).Dummy(n));
							if(k1 < 0)
							{
								Print("Error QQQ");
								HaltExecution();
							}
							if(!Dummies(i1).Exclude.IsMissing()) if(Dummies(i1).Dummy(n) > Dummies(i1).Exclude.GetData()) k1--;
							
							int k2 = BinarySearch(DummyList(i2),Dummies(i2).Dummy(n));
							if(k2 < 0)
							{
								Print("Error QQQ");
								HaltExecution();
							}
							if(!Dummies(i2).Exclude.IsMissing()) if(Dummies(i2).Dummy(n) > Dummies(i2).Exclude.GetData()) k2--;
							
							if(Dummies(i1).Exclude.IsMissing())
							{
								if(Dummies(i2).Exclude.IsMissing())
								{
									XXdN(Curr1+k1,Curr2+k2) += One;
								}
								else
								{
									if(Dummies(i2).Dummy(n) != Dummies(i2).Exclude) XXdN(Curr1+k1,Curr2+k2) += One;
								}
							}
							else
							{
								if(Dummies(i2).Exclude.IsMissing())
								{
									if(Dummies(i1).Dummy(n) != Dummies(i1).Exclude) XXdN(Curr1+k1,Curr2+k2) += One;
								}
								else
								{
									if(Dummies(i1).Dummy(n) != Dummies(i1).Exclude && Dummies(i2).Dummy(n) != Dummies(i2).Exclude) XXdN(Curr1+k1,Curr2+k2) += One;
								}
							}
						}
						for(int k1=Curr1; k1<Curr1+KCurr1; k1++) for(int k2=Curr2; k2<Curr2+KCurr2; k2++) XXdN(k1,k2) /= (double)N;
						Curr2 += KCurr2;
					}
					Curr1 += KCurr1;
				}
			}
		}
		else
		{
			/* Add a Constant */
			if(Dummies.Size() > 0)
			{
				Print("Error in LinRegObj");
				HaltExecution("THIS PART NOT DONE YET!");
			}
			
			XYdN(0) = Zero;
			for(int n=0; n<N; n++) XYdN(0) += Y(n);
			XYdN(0) /= (double)N;
			for(int k1=1; k1<K; k1++)
			{
				XYdN(k1) = Zero;
				//for(int n=0; n<N; n++) XYdN(k1) += X(n,k1-1) * Y(n);
				XYdN(k1) += BlasDot(N,&XTrans(k1-1,0),1,Y.Begin(),1);
				XYdN(k1) /= (double)N;
			}

			XXdN(0,0) = One;
			for(int k1=1; k1<K; k1++)
			{
				XXdN(k1,0) = Zero;
				//for(int n=0; n<N; n++) XXdN(k1,0) += X(n,k1-1);
				XXdN(k1,0) += BlasSum(N,&XTrans(k1-1,0),1);
				XXdN(k1,0) /= (double)N;
			}
			for(int k1=1; k1<K; k1++) for(int k2=1; k2<=k1; k2++)
			{
				XXdN(k1,k2) = Zero;
				//for(int n=0; n<N; n++) XXdN(k1,k2) += X(n,k1-1) * X(n,k2-1);
				XXdN(k1,k2) += BlasDot(N,&XTrans(k1-1,0),1,&XTrans(k2-1,0),1);
				XXdN(k1,k2) /= (double)N;
			}
		}

		/* Invert X'*X / N */
		//Output("XYdN.dat",XYdN);
		//Output("XXdN.dat",XXdN);
		SymmetricMatrix<double> XXdNInv = XXdN.Copy();
		int Info = InvertCholesky(XXdNInv);
		if(Info)
		{
			Print("Error in LinRegObj");
			Print("Name",Name);
			Print("XXdN is singular");
			if(XXdN.Dim() <= 12) Print("XXdN",XXdN);
			double RankTol = 0.000000000001;
			Vector<string> BetaLabels(K);
			for(int k=0; k<K; k++) BetaLabels(k) = "Beta" + ToString(k);
			//if(XXdN.Dim() <= 200) PrintRowDependencies(XXdN,BetaLabels,RankTol);
			HaltExecution();
		}

		/* Calculate Beta */

		ProdAX(Beta,XXdNInv,XYdN);

		/* Compute Residuals */
		Pred  = Vector<double>(N);
		Resid = Vector<double>(N);
		for(int n=0; n<N; n++)
		{
			if(NoConst)
			{
				Pred(n) = Zero;
				//for(int k1=0; k1<K; k1++) Pred(n) += Beta(k1) * X(n,k1);
				for(int k1=0; k1<K; k1++) Pred(n) += Beta(k1) * XTrans(k1,n);
			}
			else
			{
				Pred(n) = Beta(0);
				//for(int k1=1; k1<K; k1++) Pred(n) += Beta(k1) * X(n,k1-1);
				for(int k1=1; k1<K; k1++) Pred(n) += Beta(k1) * XTrans(k1-1,n);
			}
			Resid(n) = Y(n) - Pred(n);
		}
		
		/* Calculate Sigma and R^2 */
		if(RobustSe)
		{
			SymmetricMatrix<double> B(K);

			if(NoConst)
			{
				for(int k1=0; k1<K; k1++) for(int k2=0; k2<=k1; k2++)
				{
					B(k1,k2) = Zero;
					//for(int n=0; n<N; n++) B(k1,k2) += X(n,k1) * X(n,k2) * Sqr(Resid(n));
					for(int n=0; n<N; n++) B(k1,k2) += XTrans(k1,n) * XTrans(k2,n) * Sqr(Resid(n));
					B(k1,k2) /= (double)N;
				}
			}
			else
			{
				B(0,0) = One;
				for(int k=1; k<K; k++)
				{
					B(k,0) = Zero;
					//for(int n=0; n<N; n++) B(k,0) += X(n,k-1);
					for(int n=0; n<N; n++) B(k,0) += XTrans(k-1,n);
					B(k,0) /= (double)N;
				}
				for(int k1=1; k1<K; k1++) for(int k2=1; k2<=k1; k2++)
				{
					B(k1,k2) = Zero;
					//for(int n=0; n<N; n++) B(k1,k2) += X(n,k1-1) * X(n,k2-1) * Sqr(Resid(n));
					for(int n=0; n<N; n++) B(k1,k2) += XTrans(k1-1,n) * XTrans(k2-1,n) * Sqr(Resid(n));
					B(k1,k2) /= (double)N;
				}
			}

			BetaVar = SymmetricMatrix<double>(K);
			ProdABA(BetaVar,XXdNInv,B);
			BetaVar /= (double)N;

			Sigma2 = Zero;
			Sigma2Se = Zero;
			for(int n=0; n<N; n++)
			{
				Sigma2 += Sqr(Resid(n)) / (N - (double)K);
				Sigma2Se += Sqr(Sqr(Resid(n))) / (double)N;
			}
			Sigma2Se = Sqrt(Sigma2Se + Sigma2) / Sqrt((double)N);

			double ybar = Zero;
			for(int n=0; n<N; n++) ybar += Y(n) / N;
			double ysq = Zero;
			for(int n=0; n<N; n++) ysq += Sqr(Y(n) - ybar);
			//RSquared = One - N * Sigma2 / ysq;
			RSquared = One - (N - K) * Sigma2 / ysq;

			/* Compute Standard Errors */
			BetaSe = Vector< DataPoint<double> >(K);
			BetaLower = Vector< DataPoint<double> >(K);
			BetaUpper = Vector< DataPoint<double> >(K);
			for(int k1=0; k1<K; k1++)
			{
				BetaSe(k1) = sqrt(BetaVar(k1,k1));
				BetaLower(k1) = Beta(k1) - 1.96 * BetaSe(k1);
				BetaUpper(k1) = Beta(k1) + 1.96 * BetaSe(k1);
			}
			BetaLabels = Vector<string>(K,"");
			
		}
		else
		{
			Sigma2 = Zero;
			Sigma2Se = Zero;
			Resid = Vector<double>(N);
			for(int n=0; n<N; n++)
			{
				Resid(n) = Y(n);
				if(NoConst)
				{
					//for(int k1=0; k1<K; k1++) Resid(n) -= X(n,k1) * Beta(k1);
					for(int k1=0; k1<K; k1++) Resid(n) -= XTrans(k1,n) * Beta(k1);
				}
				else
				{
					Resid(n) -= Beta(0);
					//for(int k1=1; k1<K; k1++) Resid(n) -= X(n,k1-1) * Beta(k1);
					for(int k1=1; k1<K; k1++) Resid(n) -= XTrans(k1-1,n) * Beta(k1);
				}
				Sigma2 += Sqr(Resid(n)) / (N - K);
				Sigma2Se += Sqr(Sqr(Resid(n))) / (double)N;
			}
			Sigma2Se = Sqrt(Sigma2Se + Sigma2) / Sqrt((double)N);
			double ybar = Zero;
			for(int n=0; n<N; n++) ybar += Y(n) / N;
			double ysq = Zero;
			for(int n=0; n<N; n++) ysq += Sqr(Y(n) - ybar);
			RSquared = One - N * Sigma2 / ysq;

			/* Compute Standard Errors */
			BetaSe = Vector< DataPoint<double> >(K);
			BetaLower = Vector< DataPoint<double> >(K);
			BetaUpper = Vector< DataPoint<double> >(K);
			for(int k1=0; k1<K; k1++)
			{
				BetaSe[k1] = sqrt(Sigma2 * XXdNInv(k1,k1) / N);
				BetaLower(k1) = Beta(k1) - 1.96 * BetaSe(k1);
				BetaUpper(k1) = Beta(k1) + 1.96 * BetaSe(k1);
			}
			BetaLabels = Vector<string>(K,"");
			BetaVar = Sigma2 * XXdNInv / (double)N;
		}
		LogLikelihood = -(double)N * (Half * log((((double)N - (double)K) / (double)N) * Sigma2) - Half);
		/* SHOULD THIS BE + 1/2 */

		return 0;
	}

	int Inference()
	{
		/* Nothing Here, Inference is Done Automatically */
		return 0;
	}
};

#endif