
#ifndef _ListValues
#define _ListValues


/*

Given a vector v of type T, create a list
of all the values found in v.

This algorithm copy the vector v to the
vector w, sorts the vector w, and ...

In the future, this algorithm should be
changed to via a binary search tree,
meaning that the vector will not need to be copied.

*/

template <class T>
Vector<T> ListValues(const GeneralVector<T> &v)
{
	int n = v.Size();
	Vector<T> w(n);
	if(n == 0) return w;
	Copy(w,v);
	Sort(w);

	Stack<T> s;
	T Curr = w(0);
	s.Push(Curr);

	for(int i=1; i<n; i++)
	{
		if(w(i) != Curr)
		{
			Curr = w(i);
			s.Push(Curr);
		}
	}

	Vector<T> Ret(s.Size());
	Ret.Name = "ListValues(" + v.Name + ")";
	//BlasCopy(s.Size(),&s[0],1,&Ret[0],1);
	Copy(Ret,s);
	return Ret;
}

template <class T>
Vector<T> ListValues(const GeneralVector<T> &v, bool (*LessThan)(const T &, const T &))
{
	int n = v.Size();
	Vector<T> w(n);
	if(n == 0) return w;

	Copy(w,v);
	Sort(w,LessThan);

	Stack<T> s;
	T Curr = w(0);
	s.Push(Curr);

	for(int i=1; i<n; i++)
	{
		if(w(i) != Curr)
		{
			Curr = w(i);
			s.Push(Curr);
		}
	}
	Vector<T> Ret(s.Size());
	Ret.Name = "ListValues(" + v.Name + ")";
	Copy(Ret,s);
	return Ret;
}

template <class T>
Vector<T> ListValues(const GeneralVector<T> &v, bool (*LessThan)(T, T))
{
	int n = v.Size();
	Vector<T> w(n);
	if(n == 0) return w;

	Copy(w,v);
	Sort(w,LessThan);

	Stack<T> s;
	T Curr = w(0);
	s.Push(Curr);

	for(int i=1; i<n; i++)
	{
		if(w(i) != Curr)
		{
			Curr = w(i);
			s.Push(Curr);
		}
	}
	Vector<T> Ret(s.Size());
	Ret.Name = "ListValues(" + v.Name + ")";
	Copy(Ret,s);
	return Ret;
}

/*
template <class T>
Vector<T> ListValues(const DataVector<T> &v)
{
	Stack<T> w;
	for(int n=0; n<v.NumObs(); n++) if(!v.Missing(n)) w.Push(v.Data(n));
	int n = w.Size();
	Sort(w);

	Stack<T> s(1);
	T Curr = w[0];
	s.Push(Curr);

	for(int i=1; i<n; i++)
	{
		if(w[i] != Curr)
		{
			Curr = w[i];
			s.Push(Curr);
		}
	}

	Vector<T> Ret(s.Size());
	BlasCopy(s.Size(),&s[0],1,&Ret[0],1);
	return Ret;
}
*/

/*
template <class T>
Vector<T> ListValues(const Vector<T> &v)
{
	int n = v.Size();
	Vector<T> w = v.Copy();
	Sort(w);

	Stack<T> s(1);
	T Curr = w[0];
	s.Push(Curr);

	for(int i=1; i<n; i++)
	{
		if(w[i] != Curr)
		{
			Curr = w[i];
			s.Push(Curr);
		}
	}

	Vector<T> Ret(s.Size());
	BlasCopy(s.Size(),&s[0],1,&Ret[0],1);
	return Ret;
}

template <class T>
Vector<T> ListValues(const Stack<T> &v)
{
	int n = v.Size();
	Stack<T> w = v.Copy();
	Sort(w);

	Stack<T> s(1);
	T Curr = w[0];
	s.Push(Curr);

	for(int i=1; i<n; i++)
	{
		if(w[i] != Curr)
		{
			Curr = w[i];
			s.Push(Curr);
		}
	}

	Vector<T> Ret(s.Size());
	BlasCopy(s.Size(),&s[0],1,&Ret[0],1);
	return Ret;
}
*/

template <class T>
Vector< Stack<T> > ListValues(const Matrix<T> &X)
{
	int N = X.Rows();
	int K = X.Cols();

	Vector< Stack<T> > Ret(K);
	Vector<T> v(N);		// Temporary vector

	for(int k=0; k<K; k++)
	{
		// For each variable..
		for(int n=0; n<N; n++) v[n] = X(n,k);
		Sort(v);
		//Ret[k] = Stack<T>(10);
		T Curr = v[0];
		Ret[k].Push(Curr);
		for(int n=1; n<N; n++)
		{
			if(v[n] != Curr)
			{
				Curr = v[n];
				Ret[k].Push(Curr);
			}
		}
	}
	return Ret;
}

/* List the values in a vector without a specified metric */
template <class T>
Vector<T> ListValuesSlow(const GeneralVector<T> &v)
{
	int n = v.Size();

	Stack<T> s(1);
	T Curr = v[0];
	s.Push(Curr);

	for(int i=1; i<n; i++)
	{
		int j = -1;
		for(j=i-1; j>=0; j--)	// Check if we've seen it before
		{
			if(v[i] == v[j])
			{
				break;
			}
		}
		if(j == -1) s.Push(v[i]);
	}

	Vector<T> Ret(s.Size());
	BlasCopy(s.Size(),&s[0],1,&Ret[0],1);
	return Ret;
}

#endif
