
#ifndef _GenMatUtil_Load
#define _GenMatUtil_Load

	// Convert a string to a Matrix
	template <class T> void Load(Matrix<T> &A, const string &s);
	
	/* Load values into LowerMatrix from a string of comma-separated entries */
//	template <class T> void Load(LowerMatrix<T> &A, const string &s);

	template <class T> SymmetricMatrix<T> LoadSymmetricMatrix(const string &s);



/* Load a Vector using a string */
template <class T>
void Load(Vector<T> &v, const string &s)
{
	int n = v.Size();
	BoundedStack<string> b(500);	// Bigger vectors will not be entered manually
	ParseBy(b,s,',');
	
	if(b.Size() != n)
	{
		Print("Error in Load(Vector,string)");
		Print("Incompadible Dimensions");
		Print("n",n);
		Print("b.Size()",b.Size());
		Print("s",s);
		HaltExecution();
	}
	
	for(int i=n-1; i>=0; i--) Convert(v[i],b.Pop());
}

	
// Convert a string to a Matrix
template <class T>
void Load(Matrix<T> &A, const string &s)
{
	int i,j;
	int m = A.Rows();
	int n = A.Cols();
	BoundedStack<string> b1(500); // Bigger vectors will not be entered manually
	BoundedStack<string> b2(500);
	ParseBy(b2,s,';');
	if(b2.Size() != m)
	{
		Print("Error in Load(Matrix,string)");
		Print("Incompadible Dimensions");
		Print("A.Rows() != b2.Size()");
		Print("A.Size()",A.Size());
		Print("b2.Size()",b2.Size());
		Print("s",s);
		HaltExecution();
	}
		
	for(i=m-1; i>=0; i--)
	{
		ParseBy(b1,b2.Pop(),',');
		if(b1.Size() != n)
		{
			Print("Error in Load(Matrix,String)");
			Print("Incompadible Dimensions");
			Print("A.Cols() != b1.Size()");
			Print("i",i);
			Print("A.Size()",A.Size());
			Print("b1.Size()",b1.Size());
			Print("s",s);
			HaltExecution();
		}
		else for(j=n-1; j>=0; j--) Convert(A(i,j),b1.Pop());
	}
}

/* Load values into LowerMatrix from a string of comma-separated entries */
//template <class T> void Load(LowerMatrix<T> &A, const string &s)
/*
template <class T> void Load(GeneralMatrix<T> &A, const string &s)
{
	int len = A.Length();
	BoundedStack<string> b(500); // Bigger vectors will not be entered manually
	ParseBy(b,s,',');
	
	if(b.Size() != len)
	{
		Print("Error in Load(LowerMatrix,string)");
		Print("Number of elements not equal to size of matrix");
		Print("len",len);
		Print("b.Size()",b.Size());
		Print("s",s);
		HaltExecution();
	}
	
	for(int k=len-1; k>=0; k--) Convert(A[k],b.Pop());
}
*/

template <class T> void Load(SymmetricMatrix<T> &A, const string &s)
{
	int n = A.Dim();
	Matrix<double> B(n,n);
	Load(B,s);
	
	if(!IsSymmetric(B))
	{
		Print("Error in Load(SymmetricMatrix,string)");
		Print("Matrix is not symmetric");
		Print("B",B);
		HaltExecution();
	}	
	
	Copy(A,B);
}


/* Load a Vector using a string */
template <class T>
Vector<T> LoadVector(const string &s)
{
	//BoundedStack<string> b(500);	// Bigger vectors will not be entered manually
	//ParseBy(b,s,',');
	Stack<string> b = ParseBy(s,',');
	int n = b.Size();
	Vector<T> v(n);
	for(int i=n-1; i>=0; i--)
	{
		string t = b.Pop();
		int Info = Convert(v[i],t);
		if(Info)
		{
			Print("Error in LoadVector");
			Print("Invalid Char");
			Print("t",t);
			HaltExecution();
		}
	}
	return v;	
}

/* Load a Vector using a string (and check for incompadible dimensions) */
template <class T>
Vector<T> LoadVector(int n, const string &s)
{
	Stack<string> b = ParseBy(s,',');
	
	if(b.Size() != n)
	{
		Print("Error in LoadVector(n,s)");
		Print("Incompadible Dimensions");
		Print("n",n);
		Print("b.Size()",b.Size());
		HaltExecution();
	}

	Vector<T> v(n);
	for(int i=n-1; i>=0; i--)
	{
		string t = b.Pop();
		int Info = Convert(v[i],t);
		if(Info)
		{
			Print("Error in LoadVector");
			Print("Invalid Char");
			Print("t",t);
			HaltExecution();
		}
	}
	return v;	
}
	
/* Load a Vector using a string (and check for incompadible dimensions) */
/*
template <class T>
VectorIncremented<T> LoadVectorIncremented(int n, int inc, const string &s)
{
	BoundedStack<string> b(500);	// Bigger vectors will not be entered manually
	ParseBy(b,s,',');
	
	if(b.Size() != n)
	{
		Print("Error in LoadVectorIncremented(n,s)");
		Print("Incompadible Dimensions");
		Print("n",n);
		Print("inc",inc);
		Print("b.Size()",b.Size());
		HaltExecution();
	}

	VectorIncremented<T> v(n,inc);
	BlasSet(n*abs(inc),Zero,v.Begin(),1);
	for(int i=n-1; i>=0; i--)
	{
		string t = b.Pop();
		int Info = Convert(v(i),t);
		if(Info)
		{
			Print("Error in LoadVectorIncremented");
			Print("Invalid Char");
			Print("t",t);
			HaltExecution();
		}
	}
	return v;	
}
*/

/* Convert a string to a Matrix */
template <class T>
Matrix<T> LoadMatrix(const string &s)
{
	
	Stack<string> b2 = ParseBy(s,';');
	
	int m = b2.Size();	// Number of rows
		
	Stack<string> b1 = ParseBy(b2(m-1),',');
	
	int n = b1.Size();	// Number of columns
	
	Matrix<T> A(m,n);
	
	for(int i=0; i<m; i++)
	{
		Stack<string> b1 = ParseBy(b2(i),',');
				
		if(b1.Size() != n)
		{
			Print("Error in LoadMatrix(str): Inconsistent Dimensions");
			HaltExecution();
		}
		
		for(int j=0; j<n; j++)
		{
			string t = b1(j);
			int Info = Convert(A(i,j),t);
			if(Info)
			{
				Print("Error in LoadMatrix(string)");
				Print("Invalid char");
				Print("t",t);
				HaltExecution();
			}
		}
	}
	
	return A;
}

/* Convert a string to a Matrix */
template <class T>
Matrix<T> LoadMatrix(int m, int n, const string &s)
{

	Stack<string> b2 = ParseBy(s,';');	// Get rows
	
	if(b2.Size() != m)
	{
		Print("Error in LoadMatrix(m,n,s)");
		Print("b2.Size() != m");
		Print("m",m);
		Print("b2.Size()",b2.Size());
		HaltExecution();
	}		
	
	Stack<string> b1 = ParseBy(b2.Pop(),',');
	
	Matrix<T> A(m,n);
	
	for(int j=n-1; j>=0; j--)
	{
		string t = b1.Pop();
		int Info = Convert(A(m-1,j),t);
		if(Info)
		{
			Print("Error in LoadMatrix(string)");
			Print("Invalid char");
			Print("t",t);
			HaltExecution();
		}
	}
		
	for(int i=m-2; i>=0; i--)
	{
		Stack<string> b1 = ParseBy(b2.Pop(),',');
		if(b1.Size() != n)
		{
			Print("Error in LoadMatrix(str): Inconsistent Dimensions");
			HaltExecution();
		}
		else
		{
			for(int j=n-1; j>=0; j--)
			{
				string t = b1.Pop();
				int Info = Convert(A(i,j),t);
				if(Info)
				{
					
					Print("Error in LoadMatrix(string)");
					Print("Invalid char");
					Print("t",t);
					HaltExecution();
				}
			}
		}					
	}
	
	return A;
}


template <class T>
LowerMatrix<T> LoadLowerMatrix(const string &s)
{
	Matrix<T> A = LoadMatrix<T>(s);
		
	if(!IsLower(A))
	{
		Print("Error in LoadLowerMatrix(string)");
		Print("Matrix is Not Lower Triangular");
		Print("A",A);
		HaltExecution();
	}
	
	int n = A.Rows();
	LowerMatrix<double> L(n);	
	Copy(L,A);
	return L;
}


template <class T>
SymmetricMatrix<T> LoadSymmetricMatrix(const string &s)
{
	Matrix<T> A = LoadMatrix<T>(s);
		
	if(!IsSymmetric(A))
	{
		Print("Error in LoadSymmetricMatrix(string)");
		Print("Matrix is Not Symmetric");
		Print("A",A);
		HaltExecution();
	}
	
	int n = A.Rows();
	SymmetricMatrix<double> B(n);	
	Copy(B,A);
	return B;
}

template <class T>
SymmetricMatrix<T> LoadSymmetricMatrix(int n, const string &s)
{
	Matrix<T> A = LoadMatrix<T>(s);
		
	if(!IsSymmetric(A))
	{
		Print("Error in LoadSymmetricMatrix(string)");
		Print("Matrix is Not Symmetric");
		Print("A",A);
		HaltExecution();
	}
	
	if(A.Rows() != n)
	{
		Print("Error in LoadSymmetricMatrix(int,string)");
		Print("Matrix is not confomant");
		Print("n",n);
		Print("A",A);
		HaltExecution();		
	}	
	
	SymmetricMatrix<double> B(n);	
	Copy(B,A);
	return B;
}

#ifdef _GenMat_LowerMatrixUN
	template <class T>
	LowerMatrixUN<T> LoadLowerMatrixUN(const string &s)
	{
		Matrix<T> A = LoadMatrix<T>(s);
			
		if(!IsLower(A))
		{
			Print("Error in LoadLowerMatrixUN(string)");
			Print("Matrix is Not Lower");
			Print("A",A);
			HaltExecution();
		}
		
		int n = A.Rows();
		LowerMatrixUN<T> B(n);	
		Copy(B,A);
		return B;
	}
#endif

#ifdef _GenMat_UpperMatrixUN
	template <class T>
	UpperMatrixUN<T> LoadUpperMatrixUN(const string &s)
	{
		Matrix<T> A = LoadMatrix<T>(s);
			
		if(!IsUpper(A))
		{
			Print("Error in LoadUpperMatrixUN(string)");
			Print("Matrix is Not Upper");
			Print("A",A);
			HaltExecution();
		}
		
		int n = A.Rows();
		UpperMatrixUN<T> B(n);	
		Copy(B,A);
		return B;
	}
#endif

#endif
