
#ifndef _Stat_MEst
#define _Stat_MEst

/*
 Lets Dance (InFiction String Remix)
Estimate a statistical model using constrained maximum
likelihood estiamtion. Only equality constraints are
supported.

The quasi-maximum likelihood estimator has the assymptotic
covariance matirx,

	C^-1 * B * C^-1

*/

namespace MEstSpace
{
	int N;
	int K;
	int R1E;
	int R1N;
	int R2E;
	int R2N;
	int JacOpt;
	double Fac;	// For finite sample identifiction problems

	int (*LogProb1)(const Vector<double> &, Vector<double> &);
	int (*LogProb2)(const Vector<double> &, Vector<double> &, Vector<double> &);
	int (*LogProb3)(const Vector<double> &, Vector<double> &, Matrix<double> &, int);
	int (*LogProb4)(const Vector<double> &, Vector<double> &, Vector<double> &, Matrix<double> &, Matrix<double> &, int);
	int LogProb2_(const Vector<double> &Theta, Vector<double> &CF);

	/* Eliminates constriants to jacobian */
	int LogProb2_(const Vector<double> &Theta, Vector<double> &PVec)
	{
		Vector<double> CF(R2E+R2N);
		return LogProb2(Theta,PVec,CF);
	}

	int Func1(const Vector<double> &Theta, double &F);
	int Func2(const Vector<double> &Theta, double &F, Vector<double> &CF);
	int Func3(const Vector<double> &Theta, double &F, Vector<double> &dF, int Info);
	int Func4(const Vector<double> &Theta, double &F, Vector<double> &dF, Vector<double> &CF, Matrix<double> &dCF, int Info);

	/* Conpute MEst Objective Function */
	int Func1(const Vector<double> &Theta, double &F)
	{
		/* Compute Log Likelihood */
		F = Zero;
		Vector<double> L(N);

		int Info;
		if(JacOpt == 0)
		{
			Info = LogProb1(Theta,L);
		}
		else
		{
			Matrix<double> dLP(N,K);
			Info = LogProb3(Theta,L,dLP,0);
		}

		if(Info)
		{
			F = Inf;
			return Info;
		}

		for(int n=0; n<N; n++)
		{
			if(IsNan(L[n]))
			{
				F = Inf;
				return N + n + 1;
			}
			F -= L[n] / N;
		}
		if(Fac != Zero)
		{
			for(int k=0; k<K; k++) F += Fac * Sqr(Theta(k));
		}
		if(F <= -Inf)
		{
			Print("Error in MEst::Func1");
			Print("F <= -Inf");
			Print("L",L);
			HaltExecution();
		}
		return 0;
	}

	int Func2(const Vector<double> &Theta, double &F, Vector<double> &CF)
	{
		/* Compute Log Likelihood */
		F = Zero;
		Vector<double> L(N);
		int Info = LogProb2(Theta,L,CF);
		if(Info)
		{
			F = Inf;
			return Info;
		}

		for(int n=0; n<N; n++)
		{
			if(IsNan(L[n]))
			{
				Print("Error in MEst::Func2");
				Print("Nan encoutered");
				Print("n",n);
				Print("L[n]",L[n]);
				HaltExecution();
			}
			F -= L[n] / N;
		}
		if(Fac != Zero)
		{
			for(int k=0; k<K; k++) F += Fac * Sqr(Theta(k));
		}
		if(F <= -Inf)
		{
			Print("Error in MEst::Func2");
			Print("F <= -Inf");
			Print("L",L);
			HaltExecution();
		}
		return 0;
	}

	/* Conpute MEst Objective Function */
	int Func3(const Vector<double> &Theta, double &F, Vector<double> &dF, int Info)
	{
		/* Compute Log Likelihood */
		F = Zero;
		for(int k=0; k<K; k++) dF[k] = Zero;
		Vector<double> L(N);
		Matrix<double> DL(N,K);
		int Info2 = LogProb3(Theta,L,DL,1);
		if(Info2)
		{
			F = Inf;
			return Info2;
		}

		for(int n=0; n<N; n++)
		{
			if(IsNan(L[n]))
			{
				F = Inf;
				return N + 1 + n;
			}
			F -= L[n] / N;
			for(int k=0; k<K; k++) dF[k] -= DL(n,k) / N;
		}
		if(Fac != Zero)
		{
			for(int k=0; k<K; k++) F += Fac * Sqr(Theta(k));
		}
		if(F <= -Inf)
		{
			Print("Error in MEst::Func3");
			Print("F <= -Inf");
			Print("L",L);
			HaltExecution();
		}
		return 0;
	}
	
	int Func4(const Vector<double> &Theta, double &F, Vector<double> &dF, Vector<double> &CF, Matrix<double> &dCF, int Info)
	{
		/* Compute Log Likelihood */
		F = Zero;
		Vector<double> L(N);
		Matrix<double> dL(N,K);
		int Info2 = LogProb4(Theta,L,CF,dL,dCF,1);
		if(Info2)
		{
			F = Inf;
			return Info2;
		}

		for(int n=0; n<N; n++)
		{
			if(IsNan(L[n]))
			{
				Print("Error in MEst::Func4");
				Print("Nan encoutered");
				Print("n",n);
				Print("L[n]",L[n]);
				HaltExecution();
			}
			F -= L[n] / N;
			for(int k=0; k<K; k++) dF[k] -= dL(n,k) / N;
		}
		if(Fac != Zero)
		{
			for(int k=0; k<K; k++) F += Fac * Sqr(Theta(k));
		}
		if(F <= -Inf)
		{
			Print("Error in MEst::Func2");
			Print("F <= -Inf");
			Print("L",L);
			HaltExecution();
		}
		return 0;
	}

	int DFunc(const Vector<double> &Theta, Vector<double> &dF);
	int DFunc(const Vector<double> &Theta, Vector<double> &dF)
	{
		/* Compute Log Likelihood */
		Set(dF,Zero);
		Vector<double> L(N);
		Matrix<double> dL(N,K);
		int Info = LogProb3(Theta,L,dL,1);
		if(Info) return Info;
		for(int n=0; n<N; n++) for(int k=0; k<K; k++) dF[k] -= dL(n,k) / N;
		return 0;
	}
}

class MEst
{
	public:

	string Name;

	int N;								// Number of data points
	int K;								// Number of parameters
	int R1E;							// Number of linear equality constraints
	int R1N;							// Number of linear inequality constraints
	int R2E;							// Number of non-linear equality constraints
	int R2N;							// Number of non-linear inequality constraints
	int Method;							// Method to use for optimizarion
	int JacOpt;							// Analytical jacobian provided?
	int PrintLevel;						// Level of output to print
	Matrix<double> A;
	Vector<double> b;
	//Matrix<double> Aeq;					// Lhs of linear equality constraints
	//Vector<double> beq;					// Rhs of linear equality constraints
	Vector<double> Theta;				// Parameters
	Vector< DataPoint<double> > ThetaSe;	// Standard errors of parameters (robust)
	Vector< DataPoint<double> > ThetaSeB;	// Standard errors of parameters (projected jacobian)
	Vector< DataPoint<double> > ThetaSeC;	// Standard errors of parametres (projected hessian)
	Vector<double> ThetaBias;
	Vector<double> ThetaCorr;			// Bias corrected theta
	SymmetricMatrix< DataPoint<double> > ThetaVar;	// Variance of parameter estimates
	SymmetricMatrix<double> ThetaVarProj;
	Vector<string> ThetaLabels;			// Lables for parameters
	Vector< DataPoint<double> > ThetaLower;
	Vector< DataPoint<double> > ThetaUpper;
	Vector< DataPoint<double> > PVal;
	Vector< DataPoint<int>    > ZeroTest;
	double Objective;			// Negative log-likelihood

	double Deriv1Step;					// Unormalized step for first derivatives
	int    Deriv1Type;					// Type for first derivatives
	int    Deriv1PrintLevel;
	double Deriv2Step;					// Unormalized step for second derivatives
	int    Deriv2Type;					// Type for second derivatives

	double MinNewtonTolD;
	double MinNewtonTolX;
	int    MaxIter;
	int	   MinNewtonPrintLevel;
	int	   MinNewtonBfgsUpdate;
	int	   MinNewtonRecompute;
	double MinNewtonMinDiff;

	double MinNelderMeadTol;
	double MinNelderMeadScale;
	int    MinNelderMeadPrintLevel;

	SymmetricMatrix<double> B;
	SymmetricMatrix<double> C;
	Matrix<double> ThetaZ;
	LowerMatrix<double> ThetaL;

	Vector<double> TypTheta;
	Vector<double> Lambda;

	double Alpha;	// Size of test
	double WaldStat;
	double LMStat;

	double GradCheckTol;

	bool JacCheck;
	bool IsMle;
	bool BiasCorrection;

	int ConstructorCode;
	
	double Fac;
	bool WarningsOpt;
	
	int Bandwidth;
	int InferenceOpt;
	int MeatOpt;
	int TimeSeriesOpt;
	Vector<string> Cluster;
	
	private:
	
	int Iter;
	
	public:

	// Compute vector containing log of probabilities
	int (*LogProb1)(const Vector<double> &, Vector<double> &);
	int (*LogProb2)(const Vector<double> &, Vector<double> &, Vector<double> &);
	int (*LogProb3)(const Vector<double> &, Vector<double> &, Matrix<double> &, int Info);
	int (*LogProb4)(const Vector<double> &, Vector<double> &, Vector<double> &, Matrix<double> &, Matrix<double> &, int Info);
	
	MEst() {}

	/* Constructor 1: No Non-linear Constraints */
	MEst(int (*LogProb1)(const Vector<double> &, Vector<double> &), int N, int K);

	/* Constructor 2: Non-linear Constraints */
	MEst(int (*LogProb2)(const Vector<double> &, Vector<double> &, Vector<double> &), int N, int K, int R1N, int R2N);

	/* Constructor 3: No Non-linear Constraints, Analytical Jacobian Provided */
	MEst(int (*LogProb3)(const Vector<double> &, Vector<double> &, Matrix<double> &, int), int N, int K);
	
	/* Constructor 4: Non-linear Constraints, Analytical Jacobian Provided */
	MEst(int (*LogProb4)(const Vector<double> &, Vector<double> &, Vector<double> &, Matrix<double> &, Matrix<double> &, int), int N, int K, int R1N, int R2N);

	void AddConstraints(const Matrix<double> &A, const Vector<double> &b, int R1E, int R2E);

	/* ... */
	void Initialize();

	/* Print the Constraints */
	void PrintConstraints();

	/* Compute Standard Errors */
	void Variance();

	/* Solve for the Optimal Parameter Value */
	int Solve();

	int SetClusters(const Vector<string> &Cluster);

	int Inference();

	/* ... */
	void ProjectedJacobian(Vector<double> &LP, Matrix<double> &dLP);

	void ComputeHigherOrderBias();

	DataPoint<bool> OneParameterTest(int k, double Val);

	/* Wald Test, Equality Constraints */
	bool WaldTest(const Matrix<double> &A, const Vector<double> &b);

	/* Wald Test, Inequality Constraints */
	bool WaldTestInequality(const Matrix<double> &A, const Vector<double> &b);

	/* Lagrange Multiplier Test, Equality Constraints */
	bool LMTest(const Vector<bool> &LinearConstraints, const Vector<bool> &NonlinearConstraints);

	Vector<double> Plot1D(int k, double Low, double High, int J);
	
	void OutputResults();
	void OutputResults(string FileName);
	
	int GetIter();
};

/* Constructor 1: No Constraints */
MEst::MEst(int (*LogProb1)(const Vector<double> &, Vector<double> &), int N, int K)
{
	this->LogProb1 = LogProb1;
	this->N = N;
	this->K = K;
	R1E = 0;
	R1N = 0;
	R2E = 0;
	R2N = 0;
	ConstructorCode = 1;
	JacOpt = 0;
	Initialize();
	Deriv1Step = MachEps2;
	Deriv1Type = 1;
	Deriv2Step = MachEps4;
	Deriv2Type = 3;
}

/* Constructor 2: Non-linear Constraints */
MEst::MEst(int (*LogProb2)(const Vector<double> &, Vector<double> &, Vector<double> &),
		  int N, int K, int R2E, int R2N)
{
	this->LogProb2 = LogProb2;
	this->N = N;
	this->K = K;
	R1E = 0;
	R1N = 0;
	this->R2E = R2E;
	this->R2N = R2N;
	ConstructorCode = 2;
	JacOpt = 0;
	Deriv1Step = MachEps2;
	Deriv1Type = 1;
	Deriv2Step = MachEps3;
	Deriv2Type = 1;
	Initialize();
}

/* Constructor 3: Analytical Derivatives Provided */
MEst::MEst(int (*LogProb3)(const Vector<double> &, Vector<double> &, Matrix<double> &, int), int N, int K)

{
	this->LogProb3 = LogProb3;
	this->N = N;
	this->K = K;
	R1E = 0;
	R1N = 0;
	R2E = 0;
	R2N = 0;
	ConstructorCode = 3;
	JacOpt = 1;
	Deriv1Step = MachEps2;
	Deriv1Type = 1;
	Deriv2Step = MachEps4;
	Deriv2Type = 3;
	Initialize();
}

/* Constructor 4: Non-linear Constraints, Analytical Jacobian Provided */
MEst::MEst(int (*LogProb4)(const Vector<double> &, Vector<double> &, Vector<double> &, Matrix<double> &, Matrix<double> &, int), int N, int K, int R1N, int R2N)
{
	this->LogProb4 = LogProb4;
	this->N = N;
	this->K = K;
	R1E = 0;
	R1N = 0;
	this->R2E = R2E;
	this->R2N = R2N;
	ConstructorCode = 4;
	JacOpt = 1;
	Deriv1Step = MachEps2;
	Deriv1Type = 1;
	Deriv2Step = MachEps4;
	Deriv2Type = 3;
	Initialize();
}

void MEst::AddConstraints(const Matrix<double> &A, const Vector<double> &b, int R1E, int R1N)
{
	this->A = A;
	this->b = b;
	this->R1E = R1E;
	this->R1N = R1N;
	if(b.Size() != R1E + R1N || A.Rows() != R1E + R1N || R1E < 0 || R1N < 0)
	{
		Print("Error in MEst::AddConstraints");
		Print("A.Size()",A.Size());
		Print("b.Size()",b.Size());
		Print("R1E",R1E);
		Print("R1N",R1N);
		HaltExecution();
	}
}

void MEst::Initialize()
{
	/* Allocate Memory */
	Theta       = Vector<double>(K,Zero);			Theta.Name       = "MEst_Theta";
	TypTheta    = Vector<double>(K,One);			TypTheta.Name    = "MEst_TypTheta";
	ThetaSe     = Vector< DataPoint<double> >(K);	ThetaSe.Name     = "MEst_ThetaSe";
	ThetaSeB    = Vector< DataPoint<double> >(K);	ThetaSeB.Name    = "MEst_ThetaSeB";
	ThetaSeC    = Vector< DataPoint<double> >(K);	ThetaSeC.Name    = "MEst_ThetaSeC";
	ThetaBias   = Vector<double>(K);				ThetaBias.Name   = "MEst_ThetaBias";
	ThetaCorr   = Vector<double>(K);				ThetaCorr.Name   = "MEst_ThetaCorr";
	ThetaLower  = Vector< DataPoint<double> >(K);	ThetaLower.Name  = "MEst_ThetaLower";
	ThetaUpper  = Vector< DataPoint<double> >(K);	ThetaUpper.Name  = "MEst_ThetaUpper";
	PVal        = Vector< DataPoint<double> >(K);	PVal.Name        = "MEst_PVal";
	ZeroTest    = Vector< DataPoint<int>    >(K);	ZeroTest.Name    = "MEst_ZeroTest";
	ThetaLabels = Vector<string>(K);				ThetaLabels.Name = "MEst_ThetaLabels";
	Cluster     = Vector<string>(N,"Cluster1");		Cluster.Name     = "MESt_Cluster";
	for(int k=0; k<K; k++) ThetaLabels(k) = "Theta" + ToString(k);
	Lambda.Name = "MEst::Lambda";
	A = Matrix<double>(0,K);
	b = Vector<double>(0);
	
	/* Default Parameter Values */
	Bandwidth = 0;
	InferenceOpt = 1;
	MeatOpt = 1;
	TimeSeriesOpt = 1;
	JacCheck = 1;
	IsMle = False;
	BiasCorrection = 0;
	Method = 1;
	PrintLevel = 1;
	Alpha = 0.05;
	WaldStat = Zero;
	LMStat = Zero;
	GradCheckTol = 0.00001;
	Name = "MEstUnnamed";
	Fac = Zero;
	WarningsOpt = True;
	Iter = Zero;
	MaxIter = 1000;
	MinNelderMeadTol = 0.000000000001;
	MinNelderMeadScale = 0.2;
	MinNelderMeadPrintLevel = 1;
	MinNewtonTolD = 0.000001;
	MinNewtonTolX = 0.000000000001;
	MinNewtonPrintLevel = 3;
	MinNewtonBfgsUpdate = 1;
	MinNewtonRecompute = 0;
	MinNewtonMinDiff = 0.00000001;
}

/* Compute the Robust Variance Matrix */
void MEst::Variance()
{
	/* Compute Variance Matrix */

}

void MEst::PrintConstraints()
{
	if(R1E + R1N == 0)
	{
		Print("There are no linear constriants for the MEst");
	}
	else
	{
		Print("Printing Constraints for Maximum Likelihood Estiamtor: " + Name);
		Print("");
		PrintLinearEquation(A,b,ThetaLabels);
		Print("");
	}
}

int MEst::Solve()
{
	MEstSpace::N = N;
	MEstSpace::K = K;
	MEstSpace::R1E = R1E;
	MEstSpace::R1N = R1N;
	MEstSpace::R2E = R2E;
	MEstSpace::R2N = R2N;
	MEstSpace::JacOpt = JacOpt;
	MEstSpace::Fac = Fac;

	     if(ConstructorCode == 1) MEstSpace::LogProb1 = LogProb1;
	else if(ConstructorCode == 2) MEstSpace::LogProb2 = LogProb2;
	else if(ConstructorCode == 3) MEstSpace::LogProb3 = LogProb3;
	else                          MEstSpace::LogProb4 = LogProb4;

	/* Print Some Information */
	if(PrintLevel >= 1) Print("Calling MEst");
	if(PrintLevel >= 1) Print("Constrained Maximum Likelihood Estimation");
	if(PrintLevel >= 1) Print("Number of Data Points",N);
	if(PrintLevel >= 1) Print("Number of Parameters",K);
	if(PrintLevel >= 1) Print("Number of Linear Equality Constraints",R1E);
	if(PrintLevel >= 1) Print("Number of Linear Inequality Constraints",R1N);
	if(PrintLevel >= 1) Print("Number of Non-linear Equality Constraints",R2E);
	if(PrintLevel >= 1) Print("Number of Non-linear Inequality Constraints",R2N);

	/* Try to evaluate function at first point */
	int Info;
	if(JacOpt == 0)
	{
		/* Use numerical derivatives */
		if(R2E + R2N == 0)
		{
			double F;
			Info = MEstSpace::Func1(Theta,F);
		}
		else
		{
			double F;
			Vector<double> CF(R2E+R2N);
			Info = MEstSpace::Func2(Theta,F,CF);
		}
	}
	else
	{
		/* Use analytical derivatives */
		if(R2E + R2N == 0)
		{
			double F;
			Vector<double> dF(K);
			Info = MEstSpace::Func3(Theta,F,dF,1);
		}
		else
		{
			double F;
			Vector<double> dF(K);
			Vector<double> CF(R2E+R2N);
			Matrix<double> dCF(R2E+R2N,K);
			Info = MEstSpace::Func4(Theta,F,dF,CF,dCF,1);
		}
	}

	if(Info)
	{
		Print("Error in MEst");
		Print("Cannot evaluate likelihood at initial point");
		Print("Theta",Theta);
		Print("Info",Info);

		Vector<double> LP(N);

		if(JacOpt == 0)
		{
			MEstSpace::LogProb1(Theta,LP);
		}
		else
		{
			Matrix<double> dLP(N,K);
			MEstSpace::LogProb3(Theta,LP,dLP,0);
		}

		int Obs;
		if(Info <= N)
		{
			Obs = Info - 1;
			Print("LogProb1 returned error in observation " + ToString(Obs));
			Print("Info",Info);
			Print("Obs",Obs);
			Print("N",N);
			Print("LP[Obs]",LP[Obs]);
		}
		else if(Info <= 2 * N)
		{
			Obs = Info - N - 1;
			Print("LogProb1 returned nan in observation " + ToString(Obs));
			Print("Info",Info);
			Print("Obs",Obs);
			Print("N",N);
			Print("LP[Obs]",LP[Obs]);
		}
		else
		{
			Print("Returned General Error");
			Print("Info",Info);
			Print("N",N);
			Print("Suggestion: You may have forgot to supply a return value in the LogProb function");
		}

		HaltExecution();
	}

	/* Analytical Gradient */
	if(JacCheck == 1 && JacOpt == 1)
	{
		Print("MEst::Checking Analytical Gradient");
		Vector< Vector<double> > Info = GradientCheck(MEstSpace::Func3,Theta,GradCheckTol);
		if(Sum(Info(3)) > 0)
		{
			Print("Error in MEst::Solve");
			Print("Check of Analytical Gradient Failed");
			Print("Info",Info);
			HaltExecution();
		}
	}

	/* Call Numerical Optimization Routine */
	if(R1N + R2E + R2N == 0)
	{
		/*

		Since there are no linear ineqaulity constrinat or non-linear
		constraints being imposed, call MinNewtonLin or MinNelderMead

		*/

		if(Method == 1)
		{
			if(JacOpt == 0)
			{
				/* Use MinNewtonLin to preform numerical optimization */
				if(PrintLevel >= 1) Print("Method = MinNewtonLin");
				MinNewtonLin m(MEstSpace::Func1,A,b,K,R1E);
				m.Deriv1Step  = Deriv1Step;
				m.Deriv1Type  = Deriv1Type;
				m.TolD       = MinNewtonTolD;
				m.TolX       = MinNewtonTolX;
				m.MaxIter    = MaxIter;
				m.PrintLevel = MinNewtonPrintLevel;
				m.BfgsUpdate = MinNewtonBfgsUpdate;
				m.Recompute  = MinNewtonRecompute;
				m.MinDiff    = MinNewtonMinDiff;
				Copy(m.x,Theta);
				m.Solve();
				Copy(Theta,m.x);
				Iter = m.GetIter();
				Objective = m.f;
				if(R1E > 0) ThetaZ = m.Z;
			}
			else
			{
				/* Use MinNewtonLin to preform numerical optimization */
				if(PrintLevel >= 1) Print("Method = MinNewtonLin");
				MinNewtonLin m(MEstSpace::Func3,A,b,K,R1E);
				m.Deriv1Step  = Deriv1Step;
				m.Deriv1Type  = Deriv1Type;
				m.TolD       = MinNewtonTolD;
				m.TolX       = MinNewtonTolX;
				m.MaxIter    = MaxIter;
				m.PrintLevel = MinNewtonPrintLevel;
				m.BfgsUpdate = MinNewtonBfgsUpdate;
				m.Recompute  = MinNewtonRecompute;
				m.MinDiff    = MinNewtonMinDiff;
				Copy(m.x,Theta);
				m.Solve();
				Copy(Theta,m.x);
				Iter = m.GetIter();
				Objective = m.f;
				if(R1E > 0) ThetaZ = m.Z;
			}
		}
		else if(Method == 2)
		{
			#ifdef _Opt_MinNelderMead
				if(R1E == 0)
				{
					/* Use MinNelderMead to Preform Numerical Optimization */
					if(PrintLevel >= 1) Print("Method = MinNelderMead");
					//MinNelderMead m(MEstSpace::Func1,Aeq,beq,K,R1);
					MinNelderMead m(MEstSpace::Func1,K);
					m.TolF       = MinNelderMeadTol;
					m.ScaleX     = MinNelderMeadScale;
					m.MaxIter    = MaxIter;
					m.PrintLevel = MinNelderMeadPrintLevel;
					Copy(m.x,Theta);
					m.Solve();
					Copy(Theta,m.x);
					Iter = m.GetIter();
					Objective = m.f;
				}
				else
				{
					/* Use MinNelderMead to Preform Numerical Optimization */
					if(PrintLevel >= 1) Print("Method = MinNelderMead");
					//MinNelderMead m(MEstSpace::Func1,Aeq,beq,K,R1);
					MinNelderMead m(MEstSpace::Func1,A,b,K,R1E,0);
					m.TolF       = MinNelderMeadTol;
					m.ScaleX     = MinNelderMeadScale;
					m.MaxIter    = MaxIter;
					m.PrintLevel = MinNelderMeadPrintLevel;
					Copy(m.x,Theta);
					m.Solve();
					Copy(Theta,m.x);
					Iter = m.GetIter();
					Objective = m.f;
					if(R1E > 0) ThetaZ = m.Z;
				}
			#else
				Print("Error in MEst");
				Print("MinNelderMead Off");
				HaltExecution();
			#endif
		}
		else
		{
			#ifdef _KNITRO
				if(JacOpt == 0)
				{
					/* Use MinKNitro to preform numerical optimization */
					if(PrintLevel >= 1) Print("Method = MinNewtonLin");
					MinKNitro m(MEstSpace::Func1,Vector<double>(K,-KTR_INFBOUND),Vector<double>(K,KTR_INFBOUND),A,b,K,R1E,0);
					//m.Deriv1Step  = Deriv1Step;
					m.Deriv1Type  = Deriv1Type;
					m.TolD       = MinNewtonTolD;
					//m.TolX       = MinNewtonTolX;
					m.MaxIter    = MaxIter;
					m.PrintLevel = MinNewtonPrintLevel;
					m.HessOpt = 6;
					
					//m.BfgsUpdate = MinNewtonBfgsUpdate;
					//m.Recompute  = MinNewtonRecompute;
					Copy(m.x,Theta);
					m.Solve();
					Print("m.x",m.x);
					Copy(Theta,m.x);
					
					/* Hack */
					if(PrintLevel >= 1) Print("Method = MinNewtonLin");
					MinNewtonLin m2(MEstSpace::Func1,A,b,K,R1E);
					m2.Deriv1Step  = Deriv1Step;
					m2.Deriv1Type  = Deriv1Type;
					m2.TolD       = MinNewtonTolD;
					m2.TolX       = MinNewtonTolX;
					m2.MaxIter    = MaxIter;
					m2.PrintLevel = MinNewtonPrintLevel;
					m2.BfgsUpdate = MinNewtonBfgsUpdate;
					m2.Recompute  = MinNewtonRecompute;
					m2.MinDiff    = MinNewtonMinDiff;
					Copy(m2.x,Theta);
					m2.Solve();
					Copy(Theta,m2.x);
					Iter = m2.GetIter();
					Objective = m2.f;
					if(R1E > 0) ThetaZ = m2.Z;
				}
				else
				{
					/* Use MinNewtonLin to preform numerical optimization */
					if(PrintLevel >= 1) Print("Method = MinNewtonLin");
					MinKNitro m(MEstSpace::Func3,Vector<double>(K,-KTR_INFBOUND),Vector<double>(K,KTR_INFBOUND),A,b,K,R1E,0);
					//m.Deriv1Step  = Deriv1Step;
					m.Deriv1Type  = Deriv1Type;
					m.TolD       = MinNewtonTolD;
					//m.TolX       = MinNewtonTolX;
					m.MaxIter    = MaxIter;
					m.PrintLevel = MinNewtonPrintLevel;
					//m.BfgsUpdate = MinNewtonBfgsUpdate;
					//m.Recompute  = MinNewtonRecompute;
					Copy(m.x,Theta);
					m.Solve();
					Print("m.x",m.x);
					Copy(Theta,m.x);
					
					/* Hack */
					if(PrintLevel >= 1) Print("Method = MinNewtonLin");
					MinNewtonLin m2(MEstSpace::Func3,A,b,K,R1E);
					m2.Deriv1Step  = Deriv1Step;
					m2.Deriv1Type  = Deriv1Type;
					m2.TolD       = MinNewtonTolD;
					m2.TolX       = MinNewtonTolX;
					m2.MaxIter    = MaxIter;
					m2.PrintLevel = MinNewtonPrintLevel;
					m2.BfgsUpdate = MinNewtonBfgsUpdate;
					m2.Recompute  = MinNewtonRecompute;
					m2.MinDiff    = MinNewtonMinDiff;
					Copy(m.x,Theta);
					m2.Solve();
					Copy(Theta,m.x);
					Iter = m2.GetIter();
					Objective = m2.f;
					if(R1E > 0) ThetaZ = m2.Z;
				}
			#else
				Print("Error in MEst");
				Print("KNitro Off");
				HaltExecution();
			#endif
		}
	}
	else if(R2E + R2N == 0)
	{
		/*

		Since there are non-linear constraints being
		imposed, call MinNewton or MinNelderMead

		*/

		if(Method == 1)
		{
			#ifdef _Opt_MinNewton
				/* Use MinNewton to Preform Numerical Optimization */
				if(PrintLevel >= 1) Print("Method = MinNewton");
				MinNewton m(MEstSpace::Func1,A,b,K,R1E,R1N);
				m.Deriv1Step  = Deriv1Step;
				m.Deriv1Type  = Deriv1Type;
				m.TolD       = MinNewtonTolD;
				m.TolX       = MinNewtonTolX;
				m.MaxIter    = MaxIter;
				m.PrintLevel = MinNewtonPrintLevel;
				Copy(m.x,Theta);
				m.Solve();
				Copy(Theta,m.x);
				Iter = m.GetIter();
				Objective = m.f;
				if(R1E > 0) ThetaZ = m.Z;
				Copy(Lambda,m.LambdaE2);
			#else
				Print("Error in MEst");
				Print("MinNewton Off");
				HaltExecution();
			#endif
		}
		else
		{
			/* Use MinPenalty to Preform Numerical Optimization */
			if(PrintLevel >= 1) Print("Method = MinPenalty");

			Print("ERROR: THIS PART IS NOT DONE YET");
			HaltExecution();

			/*
			MinNelderMead m(MEstSpace::Func,A,b,K,R);
			m.Tol        = MinNelderMeadTol;
			m.Scale      = MinNelderMeadScale;
			m.MaxIter    = MinNelderMeadMaxIter;
			m.PrintLevel = MinNelderMeadPrintLevel;
			Set(m.x,Theta);
			m.Solve();
			Set(Theta,m.x);
			Z = m.Z;
			*/
		}	
	}
	else
	{
		/*

		Since there are non-linear equality constraints being
		imposed, call MinNewton or MinPenalty

		*/

		if(JacOpt == 0)
		{
			if(Method == 1)
			{
				#ifdef _Opt_MinNewton
					/* Use MinNewton to Preform Numerical Optimization */
					if(PrintLevel >= 1) Print("Method = MinNewton");
					MinNewton m(MEstSpace::Func2,A,b,K,R1E,R1N,R2E,R2N);
					m.Deriv1Step  = Deriv1Step;
					m.Deriv1Type  = Deriv1Type;
					m.TolD       = MinNewtonTolD;
					m.TolX       = MinNewtonTolX;
					m.MaxIter    = MaxIter;
					m.PrintLevel = MinNewtonPrintLevel;
					Copy(m.x,Theta);
					m.Solve();
					Copy(Theta,m.x);
					Iter = m.GetIter();
					Objective = m.f;
					if(R1E > 0) ThetaZ = m.Z;
					Copy(Lambda,m.LambdaE2);
				#else
					Print("Error in MEst");
					Print("MinNewton Off");
					HaltExecution();
				#endif
			}
			else
			{
				/* Use MinPenalty to Preform Numerical Optimization */
				if(PrintLevel >= 1) Print("Method = MinPenalty");

				Print("ERROR: THIS PART IS NOT DONE YET");
				HaltExecution();

				/*
				MinNelderMead m(MEstSpace::Func,A,b,K,R);
				m.Tol        = MinNelderMeadTol;
				m.Scale      = MinNelderMeadScale;
				m.MaxIter    = MinNelderMeadMaxIter;
				m.PrintLevel = MinNelderMeadPrintLevel;
				Set(m.x,Theta);
				m.Solve();
				Set(Theta,m.x);
				Z = m.Z;
				*/
			}
		}
		else
		{
			if(Method == 1)
			{
				/* Use MinNewton to Preform Numerical Optimization */
				#ifdef _Opt_MinNewton
					if(PrintLevel >= 1) Print("Method = MinNewton");
					MinNewton m(MEstSpace::Func4,A,b,K,R1E,R1N,R2E,R2N);
					m.Deriv1Step  = Deriv1Step;
					m.Deriv1Type  = Deriv1Type;
					m.TolD       = MinNewtonTolD;
					m.TolX       = MinNewtonTolX;
					m.MaxIter    = MaxIter;
					m.PrintLevel = MinNewtonPrintLevel;
					Copy(m.x,Theta);
					m.Solve();
					Copy(Theta,m.x);
					Iter = m.GetIter();
					Objective = m.f;
					if(R1E > 0) ThetaZ = m.Z;
					Copy(Lambda,m.LambdaE2);
				#else
					Print("Error in MEst");
					Print("MinNewton Off");
					HaltExecution();
				#endif
			}
			else
			{
				/* Use MinPenalty to Preform Numerical Optimization */
				if(PrintLevel >= 1) Print("Method = MinPenalty");

				Print("ERROR: THIS PART IS NOT DONE YET (2)");
				HaltExecution();

				/*
				MinNelderMead m(MEstSpace::Func,A,b,K,R);
				m.Tol        = MinNelderMeadTol;
				m.Scale      = MinNelderMeadScale;
				m.MaxIter    = MinNelderMeadMaxIter;
				m.PrintLevel = MinNelderMeadPrintLevel;
				Set(m.x,Theta);
				m.Solve();
				Set(Theta,m.x);
				Z = m.Z;
				*/
			}
		}
	}

	return 0;
}

int MEst::SetClusters(const Vector<string> &Cluster)
{
	this->Cluster = Cluster;
	MeatOpt = 2;
}

int MEst::Inference()
{

	/* Stadard Stuff */
	if(R2E + R2N == 0)
	{

		/*

		Compute appoximaiton to information matrix based on
		projected jacobian, and check to make sure this
		approximaiton is symmetric positive definite

		*/

		if(InferenceOpt != 3)
		{
			Vector<double> g(N);
			Matrix<double> dg(N,K-R1E);
			
			if(JacOpt == 0)
			{
				if(R1E == 0) Jacobian(LogProb1,Theta,       g,dg,Deriv1Step,Deriv1Type,TypTheta);
				else         Jacobian(LogProb1,Theta,ThetaZ,g,dg,Deriv1Step,Deriv1Type,TypTheta);
			}
			else
			{
				Vector<double> LP(N);
				Matrix<double> dLP(N,K);
				int Info = LogProb3(Theta,LP,dLP,1);
				
				if(Info)
				{
					Print("Error in MEst::Solve");
					Print("3oilksdjfljsdf");
					HaltExecution();
				}
				
				if(R1E == 0) dg = dLP;
				else         dg = ThetaZ * dLP;
			}
			
			B = SymmetricMatrix<double>(K-R1E);
			B.Name = "MEst::B";
			
			if(MeatOpt == 1)
			{
				/* Standard Estimator: No Time-Series Dependence */
				for(int k1=0; k1<K-R1E; k1++) for(int k2=0; k2<=k1; k2++)
				{
					B(k1,k2) = Zero;
					for(int n=0; n<N; n++) B(k1,k2) += dg(n,k1) * dg(n,k2) / N;
				}
			}
			else if(MeatOpt == 2)
			{
				/* Cluster Standard Errors */
				Vector<string> ClusterList = ListValues(Cluster);
				const int G = ClusterList.Size();
				Vector<int> ClusterNum(G,0);
				for(int n=0; n<N; n++) ClusterNum(BinarySearch(ClusterList,Cluster(n)))++;
				Matrix<double> ClusterTemp(G,K,Zero);
				for(int n=0; n<N; n++)
				{
					int g = BinarySearch(ClusterList,Cluster(n));
					for(int k=0; k<K-R1E; k++) ClusterTemp(g,k) += dg(n,k);
				}
				for(int g=0; g<G; g++) for(int k=0; k<K-R1E; k++) ClusterTemp(g,k) /= (double)ClusterNum(g);
				for(int k1=0; k1<K-R1E; k1++) for(int k2=0; k2<=k1; k2++)
				{
					B(k1,k2) = Zero;
					for(int g=0; g<G; g++) B(k1,k2) += ClusterTemp(g,k1) * ClusterTemp(g,k2) / G;
				}			
			}
			else
			{
				/* Time-Series Standard Errors */
			
				/* Weights */
				Vector<double> W(N,Zero);
				
				if(Bandwidth <= 0 || Bandwidth > N)
				{
					Print("Error in MEst");
					Print("Bandwidth",Bandwidth);
					Print("N",N);
					HaltExecution();
				}
			
				if(TimeSeriesOpt == 1)
				{
					/* Fixed Bandwidth Estimator */
					for(int n=0; n<Bandwidth; n++) W(n) = One;
				}
				else if(TimeSeriesOpt == 2)
				{
					/* Bartlett */
					for(int n=0; n<Bandwidth; n++) W(n) = One - (double)n / Bandwidth;
				}
				else if(TimeSeriesOpt == 3)
				{
					/* Parzen */
					W(0) = One;
					for(int n=1; n<Bandwidth; n++)
					{
						double a = (double)n / Bandwidth;
						W(n) = (One - Six * Sqr(a) + Six * Pow3(a)) * (a <= Half) + Two * Pow3(One - a) * (a > Half);
					}
				}
				else if(TimeSeriesOpt == 4)
				{
					/* Tukey-Hanning */
					W(0) = One;
					for(int n=1; n<Bandwidth; n++)
					{
						double a = (double)n / Bandwidth;
						W(n) = (One + cos(Pi * a)) / Two;
					}
				}
				else
				{
					/* Quadratic Spectral */
					W(0) = One;
					for(int n=1; n<N; n++)
					{
						double a = (double)n / Bandwidth;
						double term = Six * Pi * a / Five;
						W(n) = 25.0 * (sin(term) / term - cos(term)) / (12.0 * Sqr(Pi * a));
					}
				}
	
				for(int k1=0; k1<K-R1E; k1++) for(int k2=0; k2<=k1; k2++)
				{
					B(k1,k2) = Zero;
					for(int n=0; n<N; n++) B(k1,k2) += W(0) * dg(n,k1) * dg(n,k2) / N;
					for(int b=1; b<Bandwidth; b++)
					{
						for(int n=b; n<N; n++) B(k1,k2) += W(b) * (dg(n-b,k1) * dg(n,k2) + dg(n,k1) * dg(n-b,k2)) / N;
					}
				}
			}
		}
		
		/*

		Compute estimate of information matrix based on
		projected hessian.

		*/
		
		if(InferenceOpt != 2)
		{
		
			Vector<double> df(K-R1E);
			SymmetricMatrix<double> ddf(K-R1E);
			Vector<double> cf(R2E+R2N);
			Matrix<double> dcf(R2E+R2N,K-R1E);
			Vector< SymmetricMatrix<double> > ddcf(R2E+R2N);
			for(int r=0; r<R2E+R2N; r++) ddcf[r] = SymmetricMatrix<double>(K-R1E);

			if(JacOpt == 0)
			{

				/* No Analytical Jacobian Provided */
				if(R1E == 0) Hessian(MEstSpace::Func1,Theta,       Objective,df,ddf,Deriv2Step,Deriv2Type,TypTheta);
				else         Hessian(MEstSpace::Func1,Theta,ThetaZ,Objective,df,ddf,Deriv2Step,Deriv2Type,TypTheta);
			}
			else
			{

				/* Analytical Jacobian Provided */

				Print("Grad Ceck",GradientCheck(MEstSpace::Func3,Theta,0.00001));

				Matrix<double> ddf2(K-R1E,K-R1E);
				ddf2.Name = "MEst::Solve::ddf2";
				if(R1E == 0) Jacobian(MEstSpace::DFunc,Theta,       df,ddf2,Deriv2Step,Deriv2Type,TypTheta);
				else         Jacobian(MEstSpace::DFunc,Theta,ThetaZ,df,ddf2,Deriv2Step,Deriv2Type,TypTheta);
				Copy(ddf,ddf2,1000.0);
			}

			C = SymmetricMatrix<double>(K-R1E);
			C.Name = "MEst::C";
			Copy(C,ddf);
			for(int r=0; r<R2E+R2N; r++) for(int i=0; i<K; i++) for(int j=0; j<=i; j++) C(i,j) += Lambda[r] * ddcf[r](i,j);
		}
	}
	else
	{

		/* Compute Estimate of Information Matrix Based on Projected Hessian */

		Vector<double> df(K-R1E);							df.Name   = "MEst::df";
		SymmetricMatrix<double> ddf(K-R1E);					ddf.Name  = "MEst::ddf";
		Vector<double> cf(R2E+R2N);							cf.Name   = "MEst::cf";
		Matrix<double> dcf(R2E+R2N,K-R1E);					dcf.Name  = "MEst::dcf";
		Vector< SymmetricMatrix<double> > ddcf(R2E+R2N);	ddcf.Name = "MEst::ddcf";
		for(int r=0; r<R2E+R2N; r++)
		{
			ddcf[r] = SymmetricMatrix<double>(K-R1E);
			ddcf[r].Name = "MEst::ddcf()" + ToString(r) + ")";
		}

		if(R1E == 0) Hessian(MEstSpace::Func2,Theta,       Objective,df,ddf,cf,dcf,ddcf,Deriv2Step,Deriv2Type,TypTheta);
		else         Hessian(MEstSpace::Func2,Theta,ThetaZ,Objective,df,ddf,cf,dcf,ddcf,Deriv2Step,Deriv2Type,TypTheta);

		C = SymmetricMatrix<double>(K-R1E);				C.Name    = "MEst::C";
		Copy(C,ddf);
		for(int r=0; r<R2E+R2N; r++) for(int i=0; i<K-R1E; i++) for(int j=0; j<=i; j++)
			C(i,j) -= Lambda[r] * ddcf[r](i,j);

		/*

		Compute appoximaiton to identity matrix based on
		projected jacobian, and check to make sure this
		approximaiton is symmetric positive definite

		*/

		Vector<double> g(N);			g.Name  = "MEst::g";
		Matrix<double> dg(N,K-R1E);		dg.Name = "MEst::dg";
		if(R1E == 0) Jacobian(MEstSpace::LogProb2_,Theta,       g,dg,Deriv1Step,Deriv1Type,TypTheta);
		else         Jacobian(MEstSpace::LogProb2_,Theta,ThetaZ,g,dg,Deriv1Step,Deriv1Type,TypTheta);

		/* Project Jacobian onto Nonlinear Constraints */
		for(int n=0; n<N; n++) for(int k=0; k<K-R1E; k++) for(int k2=0; k2<R2E+R2N; k2++)
			dg(n,k) -= Lambda[k2] * cf[k2];

		/* Form B Matrix */
		B = SymmetricMatrix<double>(K-R1E);	B.Name = "MEst::B";
		for(int k1=0; k1<K-R1E; k1++) for(int k2=0; k2<=k1; k2++)
		{
			B(k1,k2) = Zero;
			for(int n=0; n<N; n++) B(k1,k2) += dg(n,k1) * dg(n,k2) / N;
		}
	}

	/*

	Preform Diagnostics on C:
	Make Sure that C is Symmetric Positive Definite

	*/
	
	if(InferenceOpt == 1)
	{

		const double RankTol = 0.000001;

		EigenvaluesObj<double> eC(C);
		if(PrintLevel >= 1) Print("Eigenvalues of C",eC.GetEigenvalues());
		if(PrintLevel >= 1) Print("Cond(C)",eC.Cond());
		if(!eC.IsSpd(RankTol))
		{
			Print("Warning in MEst");
			Print("C is Not Symmetric Positive Definite");
			Print("Preforming Diagnostics");

			Print("C",C);
			Print("ThetaZ",ThetaZ);

			if(eC.Rank(RankTol) < K - R1E)
			{
				Print("C is Not of Full Rank");
				Print("Rank(C)",eC.Rank(RankTol));
				Print("Dim(C)",C.Dim());

				if(R1E == 0) PrintRowDependencies(C,       ThetaLabels,RankTol);
				else         PrintRowDependencies(C,ThetaZ,ThetaLabels,RankTol);

				if(WarningsOpt) PauseExecution();
			}
			else
			{
				Print("C is of Full Rank");
				Print("C Must Have Some Negative Eigenvalues (Check This)");
				Print("It is possible that we have converged to a minimum, but not likely");
				Print("A more likely cuase is that we are up against an unspecified bound");
				if(WarningsOpt) PauseExecution();
			}
		}
		else
		{
			if(PrintLevel >= 1) Print("Converged to a Minimum (C is Positive Definite)");
		}

		/*

		Preform Diagnostics on B:
		Make Sure that B is Symmetric Positive Definite

		*/

		EigenvaluesObj<double> eB(B);
		if(PrintLevel >= 1) Print("Eigenvalues of B",eB.GetEigenvalues());
		if(PrintLevel >= 1) Print("Cond(B)",eB.Cond());
		if(!eB.IsSpd(RankTol))
		{
			Print("Warning in MEst");
			Print("B is Not Symmetric Positive Definite");
			Print("Preforming Diagnostics");

			Print("B",B);

			if(eB.Rank(RankTol) < K - R1E)
			{
				Print("B is Not of Full Rank");
				Print("Rank(B)",eB.Rank(RankTol));
				Print("Dim(B)",B.Dim());

				if(R1E == 0) PrintRowDependencies(B       ,ThetaLabels,RankTol);
				else         PrintRowDependencies(B,ThetaZ,ThetaLabels,RankTol);

				if(WarningsOpt) PauseExecution();
			}
			else
			{
				Print("B is of Full Rank");
				Print("B Must Have Some Negative Eigenvalues (Check This)");
				Print("It is possible that we have converged to a minimum, but not likely");
				Print("A more likely cuase is that we are up against an unspecified bound");
				if(WarningsOpt) PauseExecution();
			}
		}

		/* Compute the Inverse of C */
		SymmetricMatrix<double> CInv = C.Copy();
		int Chk = Invert(CInv);
		if(Chk)
		{
			//Throw(  ToString("Error in MEst\n")
			//      + "C is not invertable\n"
			//      + "C: " + ToString(C));
			Print("Error in MEst");
			Print("C is not invertable");
			if(WarningsOpt) PauseExecution();			
			Set(ThetaSe,DataPoint<double>(Zero));
		}

		/* Check CInv */
		EigenvaluesObj<double> eCInv(CInv);
		if(PrintLevel >= 1) Print("Eigenvalues of CInv",eCInv.GetEigenvalues());
		if(PrintLevel >= 1) Print("Cond(CInv)",eCInv.Cond());

		Matrix<double> C2(K - R1E,K - R1E);
		Copy(C2,C);
		if(!eCInv.IsSpd())
		{
			Print("Warning in MEst");
			Print("CInv is not symmetric positive definite");

			if(eC.IsSpd())
			{
				Print("Since C was symmetric positive definite, this indicates that there");
				Print("was a numberical error inverting C");
				if(WarningsOpt) PauseExecution();
			}

			Print("CInv",CInv);
		}

		/* Compute the Inverse of B */
		SymmetricMatrix<double> BInv = B.Copy();
		Chk = Invert(BInv);
		if(Chk)
		{
			//Throw(  ToString("Error in MEst\n")
			//      + "B is not invertable\n"
			//      + "B: " + ToString(B));
			Print("Error in MEst");
			Print("B is not invertable");
			if(WarningsOpt) PauseExecution();			
			Set(ThetaSe,DataPoint<double>(Zero));
		}

		/* Check BInv */
		EigenvaluesObj<double> eBInv(BInv);
		if(PrintLevel >= 1) Print("Eigenvalues of BInv",eBInv.GetEigenvalues());
		if(PrintLevel >= 1) Print("Cond(BInv)",eBInv.Cond());
		if(!eBInv.IsSpd())
		{
			Print("Warning in MEst");
			Print("BInv is not symmetric positive definite");

			if(eC.IsSpd())
			{
				Print("Since B was symmetric positive definite, this indicates that there");
				Print("was a numberical error inverting B");
				if(WarningsOpt) PauseExecution();
			}

			Print("BInv",BInv);
		}

		/*

		Compute CBC = CInv * B * CInv

		*/
		/*
		SymmetricMatrix<double> CBC(K-R1);
		SymmetricMatrix<double> D(K-R1);
		ProdAB(D,B,CInv);
		ProdAB(CBC,CInv,D);
		*/

		/* Check CBC */
		/*
		EigenvalueSymmetric<double> eCBC(CBC);
		Print("Eigenvalues of CBC",eCBC.GetEigenvalues());
		if(!eCBC.IsSpd())
		{
			Print("Warning in MEst");
			Print("CBC is not symmetric positive definite");
			Print("CBC",BInv);
		}
		*/

		/*

		Compute CBC = CInv * B * CInv

		for numerical stability we compute this as,

		B = L * L^T

		D = C^-1 * L

		CBC = D * D^T

		this ensures that numerical instability will not cause CBC to
		fail to be symmetric positive definite

		*/

		try
		{
			CholeskyDecompObj<double> cdB(B);
			LowerMatrix<double> LB = cdB.GetL();
			LB.Name = "MEst::LB";
			Matrix<double> D(K-R1E,K-R1E);
			D.Name  = "MEst::D";
			ProdAB(D,CInv,LB);
			SymmetricMatrix<double> ThetaVarProj(K-R1E);
			ProdAAt(ThetaVarProj,D);
			ThetaVar = SymmetricMatrix< DataPoint<double> >(K);
			ThetaVar.Name = "MEst::ThetaVar";
			ThetaL = CholeskyDecompL(ThetaVarProj);
			if(R1E == 0) Copy(ThetaVar,ThetaVarProj);
			else         ProdABAt(ThetaVar,ToDataPointMatrix<double>(ThetaZ),ToDataPointSymmetricMatrix<double>(ThetaVarProj));

			/* Compute robust standard errors */
			if(PrintLevel >= 2)	Print("C",C);
			if(PrintLevel >= 2)	Print("CInv",CInv);
			if(PrintLevel >= 2)	Print("B",B);
			if(PrintLevel >= 2)	Print("BInv",BInv);
			if(PrintLevel >= 2)	Print("CInv * B * CInv",ThetaVarProj);

			if(R1E == 0)
			{
				if(MeatOpt != 2)
				{
					for(int k=0; k<K; k++) ThetaSe(k)  = sqrt(ThetaVarProj(k,k)  / N);
					for(int k=0; k<K; k++) ThetaSeB(k) = sqrt(BInv(k,k) / N);
					for(int k=0; k<K; k++) ThetaSeC(k) = sqrt(CInv(k,k) / N);
				}
				else
				{
					int G = ListValues(Cluster).Size();
					for(int k=0; k<K; k++) ThetaSe(k)  = sqrt(ThetaVarProj(k,k)  / G);
					for(int k=0; k<K; k++) ThetaSeB(k) = sqrt(BInv(k,k) / G);
					for(int k=0; k<K; k++) ThetaSeC(k) = sqrt(CInv(k,k) / G);
				}
			}
			else
			{
				Vector< DataPoint<double> > ThetaSeZ(K-R1E);
				Vector< DataPoint<double> > ThetaSeZB(K-R1E);
				Vector< DataPoint<double> > ThetaSeZC(K-R1E);

				if(MeatOpt != 2)
				{
					for(int k=0; k<K-R1E; k++) ThetaSeZ(k)  = sqrt(ThetaVarProj(k,k)  / N);
					for(int k=0; k<K-R1E; k++) ThetaSeZB(k) = sqrt(BInv(k,k) / N);
					for(int k=0; k<K-R1E; k++) ThetaSeZC(k) = sqrt(CInv(k,k) / N);
				}
				else
				{
					int G = ListValues(Cluster).Size();
					for(int k=0; k<K-R1E; k++) ThetaSeZ(k)  = sqrt(ThetaVarProj(k,k)  / G);
					for(int k=0; k<K-R1E; k++) ThetaSeZB(k) = sqrt(BInv(k,k) / G);
					for(int k=0; k<K-R1E; k++) ThetaSeZC(k) = sqrt(CInv(k,k) / G);
				}

				ProdAX(ThetaSe ,ToDataPointMatrix<double>(ThetaZ),ThetaSeZ );
				ProdAX(ThetaSeB,ToDataPointMatrix<double>(ThetaZ),ThetaSeZB);
				ProdAX(ThetaSeC,ToDataPointMatrix<double>(ThetaZ),ThetaSeZC);
			}
		}
		catch(int i)
		{
			Print("Warning in MEst");
			Print("Could Not Compute Standard Errors");
			if(WarningsOpt) PauseExecution();			
			Set(ThetaSe,DataPoint<double>(Zero));
		}

		if(PrintLevel >= 1)	Print("ThetaSe" ,ThetaSe );
		if(PrintLevel >= 1)	Print("ThetaSeB",ThetaSeB);
		if(PrintLevel >= 1)	Print("ThetaSeC",ThetaSeC);
	}
	else if(InferenceOpt == 2)
	{
		if(MeatOpt != 1)
		{
			Print("Error in MEst");
			Print("Do Not Use InferenceOpt == 2 w/ Non-Mle");
			HaltExecution();		
		}
		
		/* Compute the Inverse of B */
		SymmetricMatrix<double> ThetaVarProj = B.Copy();
		int Info = Invert(ThetaVarProj);
		if(Info)
		{
			Print("Error in MEst");
			Print("B is not invertable");
			Print("B",B);
			HaltExecution();
		}
		
		if(R1E == 0)
		{
			for(int k=0; k<K; k++) ThetaSe[k] = sqrt(ThetaVarProj(k,k) / N);
		}
		else
		{
			Vector< DataPoint<double> > ThetaSeZ(K-R1E);
			for(int k=0; k<K-R1E; k++) ThetaSeZ[k] = sqrt(ThetaVarProj(k,k) / N);
			ProdAX(ThetaSe,ToDataPointMatrix<double>(ThetaZ),ThetaSeZ);
		}
		if(PrintLevel >= 1)	Print("ThetaSe",ThetaSe);
	}
	else
	{
		if(MeatOpt != 1)
		{
			Print("Error in MEst");
			Print("Do Not Use InferenceOpt == 3 w/ Non-Mle");
			HaltExecution();		
		}
	
		/* Compute the Inverse of C */
		SymmetricMatrix<double> ThetaVarProj = C.Copy();
		int Info = Invert(ThetaVarProj);
		if(Info)
		{
			Print("Error in MEst");
			Print("C is not invertable");
			Print("C",C);
			HaltExecution();
		}
	
		if(R1E == 0)
		{
			for(int k=0; k<K; k++) ThetaSe[k] = sqrt(ThetaVarProj(k,k) / N);
		}
		else
		{
			Vector< DataPoint<double> > ThetaSeZ(K-R1E);
			for(int k=0; k<K-R1E; k++) ThetaSeZ[k] = sqrt(ThetaVarProj(k,k) / N);
			ProdAX(ThetaSe,ToDataPointMatrix<double>(ThetaZ),ThetaSeZ);
		}
		if(PrintLevel >= 1)	Print("ThetaSe",ThetaSe);		
	}

	/* Preform Significance Tests */
	for(int k=0; k<K; k++)
	{

		ThetaLower(k) = Theta(k) - 1.96 * ThetaSe(k);
		ThetaUpper(k) = Theta(k) + 1.96 * ThetaSe(k);

		if(ThetaSe(k).IsMissing())
		{
			PVal(k).SetToMissing();
			ZeroTest(k) = 0;	// Parameter is Fixed
		}
		else
		{
			if(ThetaSe(k).GetData() > Zero)
			{
				PVal(k) = Two * (One - NormalCdf(Abs(Theta(k) / ThetaSe(k))));
					 if(PVal(k).GetData() > 0.050) ZeroTest(k) = 0;	// Not Significant at 5% level
				else if(PVal(k).GetData() > 0.010) ZeroTest(k) = 1;	// Significant at 5% level
				else if(PVal(k).GetData() > 0.001) ZeroTest(k) = 2;	// Significant at 1% level
				else                               ZeroTest(k) = 3;	// Significant at 0.1% level
			}
			else
			{
				PVal(k) = Zero;
				ZeroTest(k) = 0;	// Parameter is Fixed
			}
		}
	}

	if(MeatOpt != 2)
	{
		ThetaVar /= ToDataPointDouble((double)N);
	}
	else
	{
		int G = ListValues(Cluster).Size();
		ThetaVar /= ToDataPointDouble((double)G);
	}

	/* Output the results */
	//Output(Name + "_Results.dat",ThetaLabels,Theta,ThetaSe,ThetaLower,ThetaUpper,PVal,ZeroTest);
	//Output(Name + "_Variance.dat",ThetaVar);
	//Output(Name + "_Z.dat",ThetaZ);
	//Output(Name + "_L.dat",ThetaL);
	//OutputResults(Name + "_ResultsTable.dat");

	return 0;
}

void MEst::ProjectedJacobian(Vector<double> &LP, Matrix<double> &dLP)
{

	if(LP.Size() != N || dLP.Rows() != N || dLP.Cols() != K  - R1E)
	{
		Print("Error in MEst::ProjectedJacobian");
		Print("Incompadible Dimensions");
		HaltExecution();
	}

	if(R1E == 0) Jacobian(LogProb1,Theta,       LP,dLP,Deriv1Step,Deriv1Type,TypTheta);
	else         Jacobian(LogProb1,Theta,ThetaZ,LP,dLP,Deriv1Step,Deriv1Type,TypTheta);
}

#ifdef _MEstHigherOrder
	void MEst::ComputeHigherOrderBias()
	{

		/* Compute the Bias of order O(1 / N) ... */



		/* Compute Q_TT and Q_TT_Inv */

		SymmetricMatrix<double> Q_TT     = C;
		SymmetricMatrix<double> Q_TT_Inv = Inverse(Q_TT);

		Print("Q_TT",Q_TT);

		/* Compute Q_TTT */



		if(R1E != 0)
		{
			Print("Error in MEst::ComputeHigherOrderBias()");
			Print("R1E",R1E);
			Print("This Part Note Done Yet");
			HaltExecution();
		}

		double f;
		MEstSpace::Func1(Theta,f);
		Print("Func1(Theta)",f);

		//if(R1 == 0) Deriv3(MEstSpace::Func1,Theta,       Q_TTT,Deriv2Step,Deriv2Type,TypTheta);
		//else        Deriv3(MEstSpace::Func1,Theta,ThetaZ,Q_TTT,Deriv2Step,Deriv2Type,TypTheta);

		/* Compute R_T and R_TT */

		Vector<double> LP(N);
		Vector< Vector<double> > R_T(N);
		Vector< SymmetricMatrix<double> > R_TT(N);

		for(int n=0; n<N; n++)
		{
			R_T[n]  = Vector<double>(K-R1E);			R_T[n].Name  = "MEst::R_T(" + ToString(n) + ")";
			R_TT[n] = SymmetricMatrix<double>(K-R1E);	R_TT[n].Name = "MEst::R_TT(" + ToString(n) + ")";
		}

		//if(R1 == 0) Hessian(LogProb1,Theta,       LP,R_T,R_TT,MachEps3,4,TypTheta);
		//else        Hessian(LogProb1,Theta,ThetaZ,LP,R_T,R_TT,Deriv2Step,Deriv2Type,TypTheta);

		Hessian(LogProb1,Theta,LP,R_T,R_TT,MachEps3,4,TypTheta);

		Matrix<double> R_Tc(N,K);
		for(int n=0; n<N; n++) for(int k=0; k<K; k++) R_Tc(n,k) = R_T(n)(k);
		Output("R_Tc.dat",R_Tc);

		Array3D<double> Q_TTT(K-R1E,K-R1E,K-R1E,-99999.0);
		
		if(!IsMle)
		{
			Deriv3(MEstSpace::Func1,Theta,Q_TTT,MachEps4,4,TypTheta);
		}
		else
		{
			/*

			If the estimator is a maximum likelihood estiamtorm, than we can compute
			Q_TTT using the second-order information eqaulity

			*/

			for(int k1=0; k1<K; k1++) for(int k2=0; k2<K; k2++) for(int k3=0; k3<K; k3++)
			{
				Q_TTT(k1,k2,k3) = Zero;
				for(int n=0; n<N; n++)
				{
					Q_TTT(k1,k2,k3) +=   (R_T(n)(k1) * R_T(n)(k2) * R_T(n)(k3) + 3.0 * R_TT(n)(k1,k2) * R_T(n)(k3))
					                   / (double)N;
				}
			}

			/*
			Print("Q_TTTa",Q_TTT);

			Deriv3(MEstSpace::Func1,Theta,Q_TTT,MachEps4,4,TypTheta);

			Print("Q_TTTb",Q_TTT);
			*/
		}

		Print("Q_TTT",Q_TTT);

		//for(int n=0; n<N; n++) R_TT[n] += Q_TT;

		//Print("R_T",R_T);
		//Print("R_TT",R_TT);

		Vector<double> E1(K-R1E,Zero);
		Vector<double> E2(K-R1E,Zero);

		Vector<double> Temp(K-R1E);

		Print("Q_TT_Inv",Q_TT_Inv);

		for(int n=0; n<N; n++)
		{

			/* Compute Temp = Q_TT^-1 * R_T */
			//Vector<double> Temp = Q_TT_Inv * R_T[n];
			ProdAX(Temp,Q_TT_Inv,R_T[n]);

			/* Compute E1 = E[ R_TT * Temp ] */
			for(int k1=0; k1<K-R1E; k1++) for(int k2=0; k2<K-R1E; k2++)
				E1[k1] += R_TT[n](k1,k2) * Temp[k2] / (double)N;

			/* Compute E2 = Q_TTT * E[ Temp (x) Temp ] */
			for(int k1=0; k1<K-R1E; k1++) for(int k2=0; k2<K-R1E; k2++) for(int k3=0; k3<K-R1E; k3++)
				E2[k1] += Q_TTT(k1,k2,k3) * Temp[k2] * Temp[k3] / (double)N;
		}

		Print("E1",E1);
		Print("E2",E2);

		Vector<double> Temp2(K-R1E);
		for(int k1=0; k1<K-R1E; k1++)
		{
			Temp2[k1] = Zero;
			for(int k2=0; k2<K-R1E; k2++) Temp2[k1] += Q_TT_Inv(k1,k2) * (E1[k2] - 0.5 * E2[k2]) / double(N);
		}

		//ProdAX(BiasTheta,ThetaZ,Temp2);
		Copy(ThetaBias,Temp2);

		Vector<double> Q_T(K);

		for(int k=0; k<K; k++)
		{
			Q_T[k] = Zero;
			for(int n=0; n<N; n++) Q_T[k] += R_T[n][k] / (double)N;
		}
		Print("Q_T",Q_T);

		Print("ThetaBias",ThetaBias);
		Print("N * ThetaBias",(double)N * ThetaBias);

		for(int k=0; k<K-R1E; k++) ThetaCorr(k) = Theta(k) - ThetaBias(k);

		SymmetricMatrix<double> Temp3(K-R1E,Zero);

		for(int n=0; n<N; n++) Temp3 += R_TT[n];

		for(int i=0; i<Temp3.Length(); i++) Temp3[i] /= (double)N;

		Print("Temp3",Temp3);
		Print("C",C);

		/*

		This code does not yet compute the higher order estimate for the
		variance

		*/
	}
#endif

DataPoint<bool> MEst::OneParameterTest(int k, double Val)
{
	DataPoint<double> NStat;
	if(!BiasCorrection) NStat = (Theta(k)                - Val) / ThetaSe(k);
	else                NStat = (Theta(k) - ThetaBias(k) - Val) / ThetaSe(k);

	double NCrit = NormalCdfInv(One - Half * Alpha);

	     if(NStat.IsMissing())            return DataPoint<bool>(); // Missing
	else if(Abs(NStat.GetData()) > NCrit) return True;	// Reject
	else                                  return False;	// Don't Reject
}

bool MEst::WaldTest(const Matrix<double> &A, const Vector<double> &b)
{

	/*

	Compute the Wald Test,

		A * Theta0 = b

	*/

	int J = A.Rows();	// Degree of Freedom

	/* Check for Incompadible Dimensions */
	if(A.Cols() != K || b.Size() != J)
	{
		Print("Error in MEst::WaldTest");
		Print("A * Theta0 = b");
		Print("Incompadible Dimensions");
		Print("K",K);
		Print("A.Size()",A.Size());
		Print("b.Size()",b.Size());
		HaltExecution();
	}

	Vector<double> v(J);
	SymmetricMatrix<double> M(J);

	/* Form v = Sqrt(N) * (A * Theta - b) */

	ProdAX(v,A,Theta);
	v -= b;
	v *= Sqrt((double)N);

	/* Form M = (A * V * A^T)^-1 */
	ProdABAt(M,A,ToSymmetricMatrix(ThetaVar));
	int Info = InvertCholesky(M);
	if(Info)
	{
		Print("Error in MEst::WaldTest");
		Print("M is not Invertable");
		HaltExecution();
	}

	WaldStat = ProdXtAX(v,M);

	Print("MEst::WaldTest::WaldStat",WaldStat);

	double WaldCrit = ChiSquareCdfInv(One - Alpha,J);

	Print("MEst::WaldTest::WaldCrit",WaldCrit);

	if(abs(WaldStat) > WaldCrit) return 1;	// Reject
	else                         return 0;	// Don't Reject
}

#ifdef _InequalityTests
	bool MEst::WaldTestInequality(const Matrix<double> &A, const Vector<double> &b)
	{

		/*

		Compute the wald test,

			A * Theta0 >= b

		*/

		int J = A.Rows();	// Degree of Freedom

		/* Check for Incompadible Dimensions */
		if(A.Cols() != K || b.Size() != J)
		{
			Print("Error in MEst::WaldTestInequality()");
			Print("A * Theta0 <= b");
			Print("Incompadible Dimensions");
			Print("K",K);
			Print("A.Size()",A.Size());
			Print("b.Size()",b.Size());
			HaltExecution();
		}

		Vector<double> v(J);
		SymmetricMatrix<double> M(J);

		/* Form v = Sqrt(N) * (A * Theta - b) */

		ProdAX(v,A,Theta);
		v -= b;
		v *= Sqrt((double)N);

		/* Trim v(k) = Min(v(k),0) */

		for(int j=0; j<J; j++) if(v(j) >= Zero) v(j) = Zero;

		/* Form M = (A * V * A^T)^-1 */
		ProdABAt(M,A,ToSymmetricMatrix(ThetaVar));
		int Info = InvertCholesky(M);
		if(Info)
		{
			Print("Error in MEst::WaldTestInequality()");
			Print("M is not Invertable");
			HaltExecution();
		}

		WaldStat = ProdXtAX(v,M);

		Print("MEst::WaldTestInequality::WaldStat",WaldStat);

		//double WaldCrit = ChiSquareCdfInv(One - Alpha,J);
		//Print("MEst::WaldTestInequality::WaldCrit",WaldCrit);
			
		//if(abs(WaldStat) > WaldCrit) return 1;	// Reject
		//else                         return 0;	// Don't Reject
		
		double pVal = One - ChiSquaredMixtureCdf(WaldStat,J);
		if(pVal < Alpha) return 1;
		else             return 0;
		
	}
#endif

#ifdef _InequalityTests
	bool MEst::LMTest(const Vector<bool> &LinearConstraints, const Vector<bool> &NonlinearConstraints)
	{

		/* Check for Incompadible Dimensions */
		if(LinearConstraints.Size() != R1E + R1N || NonlinearConstraints.Size() != R2E + R2N)
		{
			Print("Error in MEst::LMTest");
			Print("Incompadible Dimensions");
			Print("R1E",R1E);
			Print("R1N",R1N);
			Print("R2E",R2E);
			Print("R2N",R2N);
			Print("LinearConstraints.Size()",LinearConstraints.Size());
			Print("NonlinearConstraints.Size()",NonlinearConstraints.Size());
			HaltExecution();
		}

		/* Compute Number of Restrictions */
		int J1 = 0;
		int J2 = 0;
		for(int r=0; r<R1E; r++) if(LinearConstraints(r)) J1++;
		for(int r=0; r<R2E; r++) if(NonlinearConstraints(r)) J1++;
		for(int r=R1E; r<R1E+R1N; r++) if(LinearConstraints(r)) J2++; 
		for(int r=R2E; r<R2E+R2N; r++) if(NonlinearConstraints(r)) J2++; 
		int J = J1 + J2;

		/* Seperate the Constraints to be Tested Away from the Constraints to be Imposed */

		Matrix<double> A1(J,K);			A1.Name = "A1";		// Constraints to be Tested
		Matrix<double> A2(R1E+R1N-J,K);	A2.Name = "A2";		// Constraints to be Imposed

		int Curr1 = 0;
		int Curr2 = 0;
		for(int r=0; r<R1E+R2E; r++)
		{
			if(LinearConstraints(r))
			{
				for(int k=0; k<K; k++) A1(Curr1,k) = A(r,k);
				Curr1++;
			}
			else
			{
				for(int k=0; k<K; k++) A2(Curr2,k) = A(r,k);
				Curr2++;
			}
		}

		/* Compute Projection Matrix, Z2 */

		Matrix<double> Z2 = Projection(A2);

		/* Now, compute Q_T and Q_TT and V_Q */

		Vector<double> Phi(N);					Phi.Name   = "MEst::LMTest::Phi";
		Matrix<double> Phi_T(N,K-(R1E+R1N-J));	Phi_T.Name = "MEst::LMTest::Phi_T";

		Jacobian(LogProb1,Theta,Z2,Phi,Phi_T,Deriv1Step,Deriv1Type,TypTheta);

		SymmetricMatrix<double> V_Q(K-(R1E+R1N-J));
		V_Q.Name = "MEst::LMTest::V_Q";
		for(int k1=0; k1<K-(R1E+R1N-J); k1++) for(int k2=0; k2<=k1; k2++)
		{
			V_Q(k1,k2) = Zero;
			for(int n=0; n<N; n++) V_Q(k1,k2) += Phi_T(n,k1) * Phi_T(n,k2) / N;
		}

		Vector<double> Q_T(K-(R1E+R1N-J));
		Q_T.Name = "MEst::LMTest::Q_T";
		for(int k=0; k<K-(R1E+R1N-J); k++)
		{
			Q_T(k) = Zero;
			for(int n=0; n<N; n++) Q_T(k) += Phi_T(n,k) / sqrt(N);
		}

		double f;
		Vector<double> df(K-(R1E+R1N-J));				df.Name   = "MEst::LMTest::df";
		SymmetricMatrix<double> Q_TT(K-(R1E+R1N-J));	Q_TT.Name = "MEst::LMTest::Q_TT";

		Hessian(MEstSpace::Func1,Theta,Z2,f,df,Q_TT,Deriv2Step,Deriv2Type,TypTheta);

		int Info = Invert(Q_TT);
		if(Info)
		{
			Print("Error in MEst::LMTest");
			Print("Q_TT is not invertable");
			Print("Info",Info);
			Print("Q_TT(Info-1,Info-1)",Q_TT(Info-1,Info-1));
			HaltExecution();
		}

		SymmetricMatrix<double> AQA = ToSymmetricMatrix(Inverse((A1 * Z2) * Q_TT * V_Q * Q_TT * Transpose(A1 * Z2)),0.0001);
		Vector<double> v = A1 * Z2 * Q_TT * Q_T;

		/*

		Let P denote the a matrix that takes into account which
		linear constraints are to be used when forming the test statistic

		*/

		/*

		Form,

			v =     P    *     A    * Q_TT^-1 * Q_T

			    (J x R1)   (R1 x K)   (K x K)   (K x 1)

		*/

		LMStat = ProdXtAX(v,AQA);

		Print("LMStat",LMStat);

		//double LMCrit = ChiSquareCdfInv(One - Alpha,J);
		//Print("MEst::WaldTest::LMCrit",LMCrit);

		//if(abs(LMStat2) > LMCrit) return 1;	// Reject
		//else                      return 0;	// Don't Reject
		
		double PVal = One - ChiSquaredMixtureCdf(LMStat,J1,J2);
		
		if(PVal < Alpha) return 1;	// Reject
		else             return 0;	// Don't Reject
		
	}
#endif

Vector<double> MEst::Plot1D(int k, double Low, double High, int J)
{

	/* Fill Global Variables */
	if(ConstructorCode == 1)
	{
		MEstSpace::LogProb1 = LogProb1;
		MEstSpace::N = N;
		MEstSpace::K = K;
		MEstSpace::R2E = R2E;
		MEstSpace::R2N = R2N;
		MEstSpace::JacOpt = JacOpt;
	}
	else if(ConstructorCode == 2)
	{
		MEstSpace::LogProb2 = LogProb2;
		MEstSpace::N = N;
		MEstSpace::K = K;
		MEstSpace::R2E = R2E;
		MEstSpace::R2N = R2N;
		MEstSpace::JacOpt = JacOpt;
	}
	else if(ConstructorCode == 3)
	{
		MEstSpace::LogProb3 = LogProb3;
		MEstSpace::N = N;
		MEstSpace::K = K;
		MEstSpace::R2E = R2E;
		MEstSpace::R2N = R2N;
		MEstSpace::JacOpt = JacOpt;
	}
	else if(ConstructorCode == 4)
	{
		MEstSpace::LogProb4 = LogProb4;
		MEstSpace::N = N;
		MEstSpace::K = K;
		MEstSpace::R2E = R2E;
		MEstSpace::R2N = R2N;
		MEstSpace::JacOpt = JacOpt;
	}
	
	Vector<double> Theta1 = Theta.Copy();
	Vector<double> Grid1  = Grid(Low,High,J);
	Vector<double> F1(J);

	Print("Theta1",Theta1);

	double F;
	MEstSpace::Func1(Theta1,F);
	Print("FF",F);

	for(int j=0; j<J; j++)
	{
		Theta1(k) = Grid1(j);
		MEstSpace::Func1(Theta1,F1(j));
	}
	return F1;
}

void MEst::OutputResults(string FileName)
{
	if(Cluster.Size() == 0)
	{
		OutputStatTable(FileName,ThetaLabels,Theta,ThetaSe,ZeroTest,N,-1); // Unclustered standard errors
	}
	else
	{
		Vector<string> ClusterList = ListValues(Cluster);	
		OutputStatTable(FileName,ThetaLabels,Theta,ThetaSe,ZeroTest,N,ClusterList.Size()); // Clustered standard errors
	}
}

void MEst::OutputResults()
{
	OutputResults(Name + "_Results.dat");
}

int MEst::GetIter()
{
	return Iter;
}

#endif
