
#ifndef _GenMatUtil_Max
#define _GenMatUtil_Max

	/* Compute the maximum element of a matrix */
	template <class T> T Max(const GeneralVector<T>   &x);
	template <class T> T Max(const Matrix<T>          &A);
	template <class T> T Max(const SymmetricMatrix<T> &A);
	template <class T> T Max(const LowerMatrix<T>     &A);

template <class T>
T Max(const GeneralVector<T> &x)
{
	int n = x.Size();
	if(n < 1)
	{
		Print("Error in Max(Vector)");
		Print("n < 1");
		Print("n",n);
		HaltExecution();
	}
	T MaxVal = x[0];
	for(int i=1; i<n; i++) if(x[i] > MaxVal) MaxVal = x[i];
	return MaxVal;
}

template <class T>
T Max(const Stack<T> &x)
{
	int n = x.Size();
	if(n < 1)
	{
		Print("Error in Max(Stack)");
		Print("n < 1");
		Print("n",n);
	}
	T MaxVal = x[0];
	for(int i=1; i<n; i++) if(x[i] > MaxVal) MaxVal = x[i];
	return MaxVal;
}

template <class T>
T Max(const Matrix<T> &A)
{
	int m = A.Rows();
	int n = A.Cols();
	T Mx = A(0,0);
	for(int i=0; i<m; i++) for(int j=0; j<n; j++) if(A(i,j) > Mx) Mx = A(i,j);
	return Mx;
}

template <class T>
T Max(const GeneralMatrix<T> &A)
{
	int m = A.Rows();
	int n = A.Cols();
	T Mx = A(0,0);
	for(int i=0; i<m; i++) for(int j=0; j<n; j++) if(A(i,j) > Mx) Mx = A(i,j);
	return Mx;
}


/* Return the maximum value in a DiagonalMatrix */
#ifdef _GenMat_DiagonalMatrix
	template <class T>
	T Max(const DiagonalMatrix<T> &A)
	{
		int n = A.Dim();
		T Mx = A(0);
		for(int i=1; i<n; i++) if(A(i) > Mx) Mx = A(i);
		return Mx;
	}
#endif

/* Return the maximum value in a Matrix */
template <class T>
T Max(const SymmetricMatrix<T> &A)
{
	int n = A.Dim();
	T Mx = A(0,0);
	for(int i=0; i<m; i++) for(int j=0; j<=i; j++) if(A(i,j) > Mx) Mx = A(i,j);
	return Mx;
}

template <class T>
T Max(const LowerMatrix<T> &A)
{
	int n = A.Dim();
	T Mx = Zero;
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++) if(A(i,j) > Mx) Mx = A(i,j);
	return Mx;
}

#ifdef _GenMat_UpperMatrix
	template <class T>
	T Max(const UpperMatrix<T> &A)
	{
		int n = A.Dim();
		T Mx = Zero;
		for(int i=0; i<n; i++) for(int j=i; j<n; j++) if(A(i,j) > Mx) Mx = A(i,j);
		return Mx;
	}
#endif

template <class T>
Vector<double> MaxBy(const Vector<double> &x, const Vector<T> &y)
{
	int N = x.Size();
	
	Vector<T> List = ListValues(y);
	int K = List.Size();
	Sort(List);
	
	if(y.Size() != N)
	{
		Print("Error in MaxBy(x,y,List)");
		Print("Incompadible dimensions");
		Print("x.Name",x.Name);
		Print("y.Name",y.Name);
		Print("x.Size()",x.Size());
		Print("y.Size()",y.Size());
	}	
	
	Vector< Stack<double> > x_By_y(K);
	for(int n=0; n<N; n++)
	{
		int Curr = BinarySearch(List,y[n]);
		if(Curr < 0)
		{
			Print("Error in MaxBy");
			Print("Curr < 0");
			Print("Curr",Curr);
			HaltExecution();
		}
		x_By_y[Curr].Push(x[n]);
	}
	Vector<double> Ret(K);
	for(int k=0; k<K; k++) Ret[k] = Max(x_By_y[k]);
	return Ret;
}

#endif