
#ifndef _GenMatUtil_MiscUtil
#define _GenMatUtil_MiscUtil

// Utilities for manipulating matrices

// Function prototypes
	
	Matrix<double> PermutationMatrix(const Vector<int> &piv);
	
	int InvertSpd(Matrix<double> &A);
	Matrix<double> InverseSpd(const Matrix<double> &A);
	void Symmetrize(Matrix<double> &A);
	
	// Return the Lower Triangle of a Matrix
	Matrix<double> LowerTriangle(const Matrix<double> &A);
	
	Vector<double> NullVector();
	Matrix<double> NullMatrix();
	Vector<double> UnitVector(int n);

#ifdef _GenMat_BlockMatrix
	template <class T>
	BlockMatrix<T> IdentityBlockMatrix(const Vector<int> &b)
	{
		BlockMatrix<T> I(b);
		int m = I.Blocks();
		for(int i=0; i<m; i++) for(int j=0; j<I.Block(i); j++) for(int k=0; k<I.Block(i); k++)
			I(i,j,k) = One * (j == k);
		return I;
	}
#endif

template <class T>
T PosProd(const Vector<T> &x, const Vector<T> &y)
{
	T a = 0;
	int n = x.Size();
	for(int i=0; i<n; i++) if(x[i] * y[i] > 0) a += x[i] * y[i];
	return a;
}

template <class T>
T NegProd(const Vector<T> &x, const Vector<T> &y)
{
	T a = 0;
	int n = x.Size();
	for(int i=0; i<n; i++) if(x[i] * y[i] < 0) a += x[i] * y[i];
	return a;
} 

// Return the Unit Vector
Vector<double> UnitVector(int n)
{
	return Vector<double>(n,One);
}

// Return a Null Vector
Vector<double> NullVector()
{
	return Vector<double>(0);
}

template <class T>
int NonZeroElements(const Vector<T> &v)
{
	int Num = 0;
	for(int i=0; i<v.Size(); i++) if(v[i] != 0) Num++;
	return Num;
}

template <class T>
bool Equals(const Vector<T> &v1, const Vector<T> &v2)
{
	int n = v1.Size();
	if(v2.Size() != n)
	{
		Print("Error in b = Equals(v,v): Incompadible Dimensions");
		return 0;
	}
	for(int i=0; i<v1.Size(); i++) if(v1[i] != v2[i]) return 0;
	return 1;
}

// Return ..
/*
template <class T>
Vector<T> Pos(const Vector<T> &x)
{
	int n = x.Size();
	Vector<T> xPos(n);
	for(int i=0; i<n; i++)
	{
		if(x[i] >= 0) xPos[i] = x[i];
		else xPos[i] = 0;
	}
	return xPos;
}
*/

// Return ..
/*
template <class T>
Vector<T> Neg(const Vector<T> &x)
{
	int n = x.Size();
	Vector<T> xNeg(n);
	for(int i=0; i<n; i++)
	{
		if(x[i] <= 0) xNeg[i] = x[i];
		else xNeg[i] = 0;
	}
	return xNeg;
}
*/

template <class T>
Vector<T> Sqrt(const Vector<T> &x)
{
	int n = x.Size();
	Vector<T> xSqrt(n);
	for(int i=0; i<n; i++) xSqrt[i] = Sqrt(x[i]);
	return xSqrt;
}

template <class T>
Matrix<T> DiagMatrix(const Vector<T> &v)
{
	int i,j;
	int n = v.Size();
	Matrix<T> A(n,n);
	for(i=0; i<n; i++) for(j=0; j<n; j++) A(i,j) = (i == j) * v[i];
	return A;
}
		
/*
Matrix<double> IdentityMatrix(int n)
{
	int i,j;
	Matrix<double> I(n,n);
	for(i=0; i<n; i++) for(j=0; j<n; j++) I(i,j) = 1.0 * (i == j);
	return I;
}
*/

template <class T>
Matrix<T> OuterProduct(const Vector<T> &v1, const Vector<T> &v2)
{
	int i,j;
	int m = v1.Size();
	int n = v2.Size();
	Matrix<T> A(m,n);
	for(i=0; i<m; i++) for(j=0; j<n; j++) A(i,j) = v1[i] * v2[j];
	return A;
}


template <class T>
T MaxDiag(const Matrix<T> &A)
{
	int i;
	int n = A.Cols();
	T Mx = A(0,0);
	for(i=1; i<n; i++) if(A(i,i) > Mx) Mx = A(i,i);
	return Mx;
}

template <class T>
Vector<int> MaxIdx(const Matrix<T> &A)
{
	int i,j;
	double Mx;
	Vector<int> Idx(2);
	int m = A.Rows();
	int n = A.Cols();
	Idx[0] = 0;
	Idx[1] = 0;	
	for(i=0; i<m; i++) for(j=0; j<n; j++) if(A(i,j) > Mx)
	{
		Mx = A(i,j);
		Idx[0] = i;
		Idx[1] = j;
	}
	return Idx;
}

/*
Matrix<double> PermutationMatrix(const Vector<int> &piv)
{
	int n=piv.Size();
	Matrix<double> P(n,n,0.0);
	for(int i=0; i<n; i++) P(piv[i],i) = 1.0;
	return P;
}
*/

/*
Matrix<double> LowerTriangle(const Matrix<double> &A)
{
	int i,j;
	int m = A.Rows();
	int n = A.Cols();
	Matrix<double> B = A.Copy();
	for(i=0; i<m; i++) for(j=i+1; j<n; j++) B(i,j) = 0.0;
	return B;
}
*/

// Return a Null Matrix
/*
Matrix<double> NullMatrix()
{
	return Matrix<double>(0,0);
}
*/

/*
Compute the Inverse of a Symmetric Positive
Definite Matrix using the Cholesky Decomposition
*/

/*
int InvertSpd(Matrix<double> &A)
{
	CholeskyDecompObj<double> cd(A);
	if(!cd.IsSpd()) return 1;
	A = cd.Inverse();
	return 0;
}
*/

/*
Compute the Inverse of a Symmetric Positive
Definite Matrix using the Cholesky Decomposition
*/

/*
Matrix<double> InverseSpd(const Matrix<double> &A)
{
	CholeskyDecomp<double> cd(A);
	if(!cd.IsSpd()) Print("Error in SymetricInverse: Cholesky Factor Failed");
	return cd.Inverse();
}
*/

void Symmetrize(Matrix<double> &A)
{
	int i,j;
	double Temp;
	int n = A.Rows();
	if(A.Cols() != n)
	{
		Print("Error in Symmetrize: Cannot Symmetrize Non-Square Matrix");
		Print("Halting Execution");
		exit(0);
	}
	for(i=0; i<n; i++) for(j=0; j<i; j++)
	{
		Temp = 0.5 * A(i,j) + 0.5 * A(j,i);
		A(i,j) = Temp;
		A(j,i) = Temp;
	}
	return;
}


/* 

Compute y(i) = A(i,j) * x(i)

*/
 
template <class T>
inline Matrix<T> ProdByRows(const Matrix<T> &A, const Vector<T> &x)
{
	int i,j;
	int m = A.Rows();
	int n = A.Cols();
	if(x.Size() != m)
	{
		Print("Error in Matrix = ProdByRows(Matrix,Vector)");
		Print("Incompadible dimensions");
		Print("A.Name",A.Name);
		Print("x.Name",x.Name);
		Print("A.Size()",A.Size());
		Print("x.Size()",x.Size());
		HaltExecution();
	}

	Matrix<T> B(m,n);
	for(i=0; i<m; i++) for(j=0; j<n; j++) B(i,j) = A(i,j) * x[i];
	return B;
}

/* 

Compute y(i) = A(i,j) / x(i)

*/
 
template <class T>
inline Matrix<T> DivideByRows(const Matrix<T> &A, const Vector<T> &x)
{
	int i,j;
	int m = A.Rows();
	int n = A.Cols();
	if(x.Size() != m)
	{
		Print("Error in Matrix = DivideByRows(Matrix,Vector)");
		Print("Incompadible dimensions");
		Print("A.Name",A.Name);
		Print("x.Name",x.Name);
		Print("A.Size()",A.Size());
		Print("x.Size()",x.Size());
		HaltExecution();
	}

	Matrix<T> B(m,n);
	for(i=0; i<m; i++) for(j=0; j<n; j++) B(i,j) = A(i,j) / x[i];
	return B;
}

#ifdef _GenMat_SparseMatrixView
	SparseMatrixView FullSparseMatrixView(int m, int n)
	{
		SparseMatrixView S = SparseMatrixView(m,n,m*n);
		for(int i=0; i<m+1; i++) S.r[i] = n * i;
		int Curr = 0;
		for(int i=0; i<m; i++) for(int j=0; j<n; j++) S.c[Curr++] = j;
		Print("S.Size()",S.Size());
		return S;
	}
#endif

#endif
