
#ifndef _Data_Missing
#define _Data_Missing


	/* Function Prototypes */
	
	/* Eliminate the Missing Entries from a Vector */
	template <class T> Vector<T> DropMissing(const Vector<T> &X, const Vector<bool> &XMiss);
	template <class T> Vector<T> DropMissing(const GeneralVector< DataPoint<T> > &X);
	
	/* Eliminate the Missing Entries from a Matrix */
	template <class T> Matrix<T> DropMissing(const Matrix<T> &X, const Vector<bool> &XMiss);
	template <class T> Matrix<T> DropMissing(const Matrix<T> &X, const Matrix<bool> &XMiss);
	template <class T> Matrix<T> DropMissing(const Matrix<T> &X, const Vector<bool> &XMiss1, const Vector<bool> &XMiss2);
	template <class T> Array3D<T> DropMissing(const Array3D<T> &X, const Vector<bool> &XMiss1, const Vector<bool> &XMiss2);


/* Eliminate the Missing Entries from a Vector */
template <class T>
Vector<T> DropMissing(const Vector<T> &X, const Vector<bool> &XMiss)
{
	
	/* Check for Incompadible Dimensions */
	int N = X.Size();
	if(XMiss.Size() != N)
	{
		Print("Error in EliminateMissing(X,XMiss)");
		Print("Incompadible Dimensions");
		Print("X.Name",X.Name);
		Print("X.Size",X.Size());
		Print("XMiss.Name",XMiss.Name);
		Print("XMiss.Size()",XMiss.Size());
		HaltExecution();
	}

	/* Count Non-missing Entries */
	int N2 = N - NumTrue(XMiss);

	/* Eliminate Missing Entries */
	Vector<T> Y(N2);
	int Curr = 0;
	for(int n=0; n<N; n++) if(!XMiss[n]) Y[Curr++] = X[n];
	Y.Name = X.Name;
	return Y;

}

/* Eliminate the Missing Entries from a Vector */
template <class T>
Vector<T> DropMissing(const GeneralVector< DataPoint<T> > &X)
{
	int N = X.Size();
	
	/* Count Non-missing Entries */
	int N2 = 0;
	for(int n=0; n<N; n++) if(!X(n).IsMissing()) N2++;

	/* Eliminate Missing Entries */
	Vector<T> Y(N2);
	Y.Name = "DropMissing(" + X.Name + ")";
	int Curr = 0;
	for(int n=0; n<N; n++) if(!X(n).IsMissing()) Y(Curr++) = X(n).GetData();
	return Y;
}

/* Eliminate the Missing Entries from a Matrix */
template <class T>
Matrix<T> DropMissing(const Matrix<T> &X, const Vector<bool> &XMiss)
{
	
	/* Check for Incompadible Dimensions */
	int N = X.Rows();
	int K = X.Cols();
	if(XMiss.Size() != N)
	{
		Print("Error in EliminateMissing");
		Print("Incompadible dimensions");
		Print("X.Size",X.Size());
		Print("XMiss.Size()",XMiss.Size());
		HaltExecution();
	}

	/* Count Non-missing Entries */
	int N2 = N - NumTrue(XMiss);

	/* Eliminate Missing Entries */
	Matrix<T> Y(N2,K);
	int Curr = 0;
	for(int n=0; n<N; n++)
	{
		if(!XMiss(n))
		{
			for(int k=0; k<K; k++) Y(Curr,k) = X(n,k);
			Curr++;
		}
	}
	return Y;

}

/* Eliminate the Missing Entries from a Matrix */
template <class T>
Matrix<T> DropMissing(const Matrix<T> &X, const Matrix<bool> &XMiss)
{
	
	/* Check for Incompadible Dimensions */
	int N = X.Rows();
	int K = X.Cols();
	if(XMiss.Rows() != N || XMiss.Cols() != K)
	{
		Print("Error in EliminateMissing");
		Print("Incompadible Dimensions");
		Print("X.Size",X.Size());
		Print("XMiss.Size()",XMiss.Size());
		HaltExecution();
	}

	/* Count Non-missing Entries */
	int N2 = 0;
	for(int n=0; n<N; n++)
	{
		bool Miss = 0;
		for(int k=0; k<K; k++) if(XMiss(n,k)) Miss = 1;
		if(!Miss) N2++;
	}

	/* Eliminate Missing Entries */
	Matrix<T> Y(N2,K);
	int Curr = 0;
	for(int n=0; n<N; n++)
	{
		int Miss = 0;
		for(int k=0; k<K; k++) if(XMiss(n,k)) Miss = 1;
		if(!Miss == 0)
		{
			for(int k=0; k<K; k++) Y(Curr,k) = X(n,k);
			Curr++;
		}
	}
	return Y;
}

/* Eliminate the Missing Entries from a Matrix */
template <class T>
Matrix<T> DropMissing(const Matrix< DataPoint<T> > &X)
{
	
	int N = X.Rows();
	int K = X.Cols();
	
	/* Count Non-missing Entries */
	int N2 = 0;
	for(int n=0; n<N; n++)
	{
		bool Miss = 0;
		for(int k=0; k<K; k++) if(X(n,k).IsMissing()) Miss = 1;
		if(!Miss) N2++;
	}

	/* Eliminate Missing Entries */
	Matrix<T> Y(N2,K);
	int Curr = 0;
	for(int n=0; n<N; n++)
	{
		bool Miss = False;
		for(int k=0; k<K; k++) if(X(n,k).IsMissing()) Miss = True;
		if(!Miss)
		{
			for(int k=0; k<K; k++) Y(Curr,k) = X(n,k).GetData();
			Curr++;
		}
	}
	return Y;
}

/* Eliminate the Missing Entries from a Matrix */
template <class T>
Matrix<T> DropMissing(const Matrix<T> &X, const Vector<bool> &XMiss1, const Vector<bool> &XMiss2)
{

	/* Check for Incompadible Dimensions */
	int N = X.Rows();
	int K = X.Cols();
	if(XMiss1.Size() != N || XMiss2.Size() != K)
	{
		Print("Error in DropMissing(Matrix,Vector,Vector)");
		HaltExecution();
	}
	
	/* Count Non-missing Entries */
	int N2 = N - NumTrue(XMiss1);
	int K2 = K - NumTrue(XMiss2);

	/* Eliminate Missing Entries */
	Matrix<T> Y(N2,K2);
	int nCurr = 0;
	for(int n=0; n<N; n++)
	{
		if(!XMiss1(n))
		{
			int kCurr = 0;
			for(int k=0; k<K; k++)
			{
				if(!XMiss2(k))
				{
					Y(nCurr,kCurr) = X(n,k);
					kCurr++;
				}
			}
			nCurr++;
		}
	}
	return Y;
}

/* Eliminate the Missing Entries from a Matrix */
template <class T>
Array3D<T> DropMissing(const Array3D<T> &X, const Vector<bool> &XMiss)
{

	/* Check for Incompadible Dimensions */
	int N = X.Dim1();
	int K = X.Dim2();
	int J = X.Dim3();
	if(XMiss.Size() != N)
	{
		Print("Error in DropMissing(Array3D,Vector)");
		HaltExecution();
	}
	
	/* Count Non-missing Entries */
	int N2 = N - NumTrue(XMiss);

	/* Eliminate Missing Entries */
	Array3D<T> Y(N2,K,J);
	int Curr = 0;
	for(int n=0; n<N; n++)
	{
		if(!XMiss(n))
		{
			for(int k=0; k<K; k++) for(int j=0; j<J; j++) Y(Curr,k,j) = X(n,k,j);
			Curr++;
		}
	}
	return Y;
}


/* Eliminate the Missing Entries from a Matrix */
template <class T>
Array3D<T> DropMissing(const Array3D<T> &X, const Vector<bool> &XMiss1, const Vector<bool> &XMiss2)
{

	/* Check for Incompadible Dimensions */
	int N = X.Dim1();
	int K = X.Dim2();
	int J = X.Dim3();
	if(XMiss1.Size() != N || XMiss2.Size() != K)
	{
		Print("Error in DropMissing(ArrayD3,Vector,Vector)");
		HaltExecution();
	}
	
	/* Count Non-missing Entries */
	int N2 = N - NumTrue(XMiss1);
	int K2 = K - NumTrue(XMiss2);

	/* Eliminate Missing Entries */
	Array3D<T> Y(N2,K2,J);
	int nCurr = 0;
	for(int n=0; n<N; n++)
	{
		if(!XMiss1(n))
		{
			int kCurr = 0;
			for(int k=0; k<K; k++)
			{
				if(!XMiss2(k))
				{
					for(int j=0; j<J; j++) Y(nCurr,kCurr,j) = X(n,k,j);
					kCurr++;
				}
			}
			nCurr++;
		}
	}
	return Y;
}


/* Eliminate the Missing Entries from a Matrix */
template <class T>
Array3D<T> DropMissing(const Array3D< DataPoint<T> > &X)
{
	
	int N = X.Dim1();
	int J = X.Dim2();
	int K = X.Dim3();
	
	/* Count Non-missing Entries */
	int N2 = 0;
	for(int n=0; n<N; n++)
	{
		bool Miss = 0;
		for(int j=0; j<J; j++) for(int k=0; k<K; k++) if(X(n,j,k).IsMissing()) Miss = 1;
		if(!Miss) N2++;
	}

	/* Eliminate Missing Entries */
	Array3D<T> Y(N2,J,K);
	int Curr = 0;
	for(int n=0; n<N; n++)
	{
		bool Miss = False;
		for(int j=0; j<J; j++) for(int k=0; k<K; k++) if(X(n,j,k).IsMissing()) Miss = True;
		if(!Miss)
		{
			for(int j=0; j<J; j++) for(int k=0; k<K; k++) Y(Curr,j,k) = X(n,j,k).GetData();
			Curr++;
		}
	}
	return Y;
}


template <class T>
Vector<bool> MissingObs(const Vector< DataPoint<T> > &x)
{
	int N = x.Size();
	Vector<bool> Miss1(N,False);
	for(int n=0; n<N; n++) Miss1(n) = x(n).IsMissing();
	return Miss1;
}

template <class T>
Vector<bool> MissingObs(const Matrix< DataPoint<T> > &X)
{
	int N = X.Rows();
	int K = X.Cols();
	Vector<bool> Miss1(N,False);
	for(int n=0; n<N; n++) for(int k=0; k<K; k++) if(X(n,k).IsMissing()) Miss1(n) = True;
	return Miss1;
}

template <class T>
Vector<bool> MissingObs(const Array3D< DataPoint<T> > &X)
{
	int N = X.Dim1();
	int K1 = X.Dim2();
	int K2 = X.Dim3();
	Vector<bool> Miss1(N,False);
	for(int n=0; n<N; n++) for(int k1=0; k1<K1; k1++) for(int k2=0; k2<K2; k2++)
		if(X(n,k1,k2).IsMissing()) Miss1(n) = True;
	return Miss1;
}

Vector<bool> operator || (const Vector<bool> &x1, const Vector<bool> &x2)
{
	int N = x1.Size();
	if(x2.Size() != N)
	{
		Print("Error in Vector<bool> || Vector<bool>");
		Print("Incompadible Dimensions");
		Print("x1.Size()",x1.Size());
		Print("x2.Size()",x2.Size());
		HaltExecution();
	}
	Vector<bool> Ret(N);
	for(int n=0; n<N; n++) Ret(n) = x1(n) || x2(n);
	return Ret;
}

#endif
