
#ifndef _MatrixNorm
#define _MatrixNorm


	// Calculate the L1-Norm of a Vector
	template <class T> T NormL1(const GeneralMatrix<T> &x);

	// Calculate the L2-Norm of a Vector
	template <class T> T NormL2(const GeneralMatrix<T> &x);

	// Calculate the L-Inf-Norm of a Vector
	template <class T> T NormLInf(const GeneralMatrix<T> &A);

	//double NormLInf(const Vector<Complex> &x);


// Calculate the L1-Norm of a Vector
template <class T>
T NormL1(const GeneralVector<T> &x)
{
	int n = x.Size();
	T normx=0;
	for(int i=0; i<n; i++) normx += abs(x[i]);
	return normx;
}

// Calculate the L2-Norm of a Vector
template <class T>
T NormL2(const GeneralVector<T> &x)
{
	int n = x.Size();
	//T normx = 0;
	//for(int i=0; i<n; i++) normx += Sqr(x[i]);
	//normx = sqrt(normx);
	//return normx;
	return BlasNrm2(n,x.Begin(),x.Increment());
}

// Calculate the L-Inf-Norm of a Vector
/*
double NormLInf(const Vector<Complex> &x)
{
	int n = x.Size();
	double xNorm = 0;
	for(int i=0; i<n; i++) if(Abs(x[i]) > xNorm) xNorm = Abs(x[i]);
	return xNorm;
}
*/

// Calculate the L-Inf-Norm of a Vector
template <class T>
T NormLInf(const GeneralVector<T> &x)
{
	int n = x.Size();
	T xNorm = 0;
	for(int i=0; i<n; i++) if(Abs(x[i]) > xNorm) xNorm = Abs(x[i]);
	return xNorm;
}

// Calculate the L-Inf-Norm of a Vector
/*
template <class T>
T NormLInf(const VectorIncremented<T> &x)
{
	int n = x.Size();
	T xNorm = 0;
	for(int i=0; i<n; i++) if(Abs(x(i)) > xNorm) xNorm = abs(x(i));
	return xNorm;
}
*/

// Calculate the L1-Norm of a Vector
template <class T>
T NormL1(const GeneralMatrix<T> &x)
{
	int len = x.Length();
	T normx=0;
	for(int i=0; i<len; i++) normx += abs(x[i]);
	return normx;
}

// Calculate the L2-Norm of a Vector
template <class T>
T NormL2(const Matrix<T> &x)
{
	int len = x.Length();
	T normx = 0;
	for(int i=0; i<len; i++) normx += Sqr(x[i]);
	normx = sqrt(normx);
	return normx;
}

// Calculate the L-Inf-Norm of a GeneralMatrix
template <class T>
T NormLInf(const GeneralMatrix<T> &A)
{
	int m = A.Rows();
	int n = A.Cols();
	T Norm = 0;
	for(int i=0; i<m; i++) for(int j=0; j<n; j++) if(!(abs(A(i,j)) <= Norm)) Norm = abs(A(i,j));
	return Norm;
}

template <class T>
T NormLInf(const Matrix<T> &A)
{
	int len = A.Length();
	T NormA = 0;
	for(int i=0; i<len; i++) if(!(abs(A[i]) <= NormA)) NormA = abs(A[i]);
	return NormA;
}

template <class T>
T NormLInf(const SymmetricMatrix<T> &A)
{
	int len = A.Length();
	T NormA = 0;
	for(int i=0; i<len; i++) if(!(abs(A[i]) <= NormA)) NormA = abs(A[i]);
	return NormA;
}

template <class T>
T NormLInf(const Array3D<T> &A)
{
	int len = A.Length();
	T NormA = 0;
	for(int i=0; i<len; i++) if(!(abs(A[i]) <= NormA)) NormA = abs(A[i]);
	return NormA;
}

#ifdef _GenMatSymmetricBandToeplitzMatrix
	template <class T>
	T NormLInf(const SymmetricBandToeplitzMatrix<T> &A)
	{
		int k = A.Bands();
		T NormA = 0;
		for(int i=0; i<k; i++) if(!(abs(A(i)) <= normx)) normx = abs(A(i));
		return NormA;
	}
#endif

#endif