
#ifndef _Stat_OutputStatTable
#define _Stat_OutputStatTable


/* Function Prototypes */

	template <class T1, class T2, class T3>
	void OutputStatTable(const string &FileName, const Vector<string> &ThetaLabels, const Vector<T1> &Theta, const Vector<T2> &ThetaSe, const Vector<T3> &ZeroTest, int N, int G);




/*

Output estimates in a table
Used by Mle

*/

template <class T1, class T2, class T3>
void OutputStatTable(const string &FileName,
					 const Vector<string> &ThetaLabels,
                     const Vector<T1> &Theta,
                     const Vector<T2> &ThetaSe,
                     const Vector<T3> &ZeroTest,
                     int N,
                     int G)
{

	ofstream out1(FileName.c_str());

	out1 << "Table of Estimation Results (" + FileName + ")\n\n";
	
	int K = Theta.Size();
	
	if(ThetaSe.Size() != K || ZeroTest.Size() != K)
	{
		Print("Error in OutputStatTable");
		Print("Incompadible Dimensions");
		Print("FileName",FileName);
		Print("Theta.Size()",Theta.Size());
		Print("ThetaSe.Size()",ThetaSe.Size());
		Print("ZeroTest.Size()",ZeroTest.Size());
		HaltExecution();
	}
	
	out1.precision(3);
	out1.width(3);
	out1.flags(ios_base::fixed);
	
	for(int k=0; k<K; k++)
	{
		out1 << ThetaLabels(k) << "\t" << Theta(k);
		     if(ZeroTest(k) == 1) out1 << "*";
		else if(ZeroTest(k) == 2) out1 << "**";
		else if(ZeroTest(k) == 3) out1 << "***";
		out1 << " (" << ThetaSe(k) << ")\n";
	}
	
	if(N != -1) out1 << "N\t" << N << "\n";
	if(G != -1) out1 << "Number of Clusters\t" << G << "\n";
	out1.close();
}

#endif
