
#ifndef _Data_Permute
#define _Data_Permute



	template <class T> void Dim1Permute(GeneralVector<T> &x, const Vector<int> &Idx);
	template <class T> void Dim1Permute(Matrix<T>        &A, const Vector<int> &Idx);
	template <class T> void Dim2Permute(Matrix<T>        &A, const Vector<int> &idx);
	template <class T> void Dim1Permute(Array3D<T>       &X, const Vector<int> &Idx);
	template <class T> void Dim2Permute(Array3D<T>       &X, const Vector<int> &Idx);
	template <class T> void Dim3Permute(Array3D<T>       &X, const Vector<int> &Idx);
	
	void PermuteInverse(Vector<int> &InvIdx, const Vector<int> Idx);


template <class T>
void Dim1Permute(GeneralVector<T> &x, const Vector<int> &Idx)
{
	int n = x.Size();
	if(Idx.Size() != n)
	{
		Print("Error in Dim1Permute(v,v): Incompatible Dimensions");
		HaltExecution();
	}
	Vector<T> v(n);
	for(int i=0; i<n; i++) v(i) = x(Idx(i));
	for(int i=0; i<n; i++) x(i) = v(i);
}

template <class T>
void Dim1Permute(Matrix<T> &A, const Vector<int> &Idx)
{
	int m = A.Rows();
	int n = A.Cols();
	if(Idx.Size() != m)
	{
		Print("Error in Dim1Permute(m,v): Incompatible Dimensions");
		HaltExecution();
	}
	Vector<T> v(m);
	for(int j=0; j<n; j++)
	{
		for(int i=0; i<m; i++) v(i) = A(Idx(i),j);
		for(int i=0; i<m; i++) A(i,j) = v(i);
	}
}

template <class T>
void Dim2Permute(Matrix<T> &A, const Vector<int> &Idx)
{
	int m = A.Dim1();
	int n = A.Dim2();
	if(Idx.Size() != n)
	{
		Print("Error in Dim2Permute(m,v): Incompatible Dimensions");
		HaltExecution();
	}
	Vector<T> v(n);
	for(i=0; i<m; i++)
	{
		for(j=0; j<n; j++) v[j] = A(i,idx[j]);
		for(j=0; j<n; j++) A(i,j) = v[j];
	}
}

template <class T>
void Dim1Permute(Array3D<T> &X, const Vector<int> &Idx)
{
	int d1 = X.Dim1();
	int d2 = X.Dim2();
	int d3 = X.Dim3();
	if(Idx.Size() != d1)
	{
		Print("Error in Dim1Permute(Array3D,Vector)");
		Print("Incompatible Dimensions");
		Print("X.Size()",X.Size());
		Print("Idx.Size()",Idx.Size());
		HaltExecution();
	}
	Vector<T> v(d1);
	for(int i2=0; i2<d2; i2++) for(int i3=0; i3<d3; i3++)
	{
		for(int i1=0; i1<d1; i1++) v(i1) = X(Idx(i1),i2,i3);
		for(int i1=0; i1<d1; i1++) X(i1,i2,i3) = v(i1);
	}
}

template <class T>
void Dim2Permute(Array3D<T> &X, const Vector<int> &Idx)
{
	int d1 = X.Dim1();
	int d2 = X.Dim2();
	int d3 = X.Dim3();
	if(Idx.Size() != d2)
	{
		Print("Error in Dim2Permute(Array3D,Vector)");
		Print("Incompatible Dimensions");
		Print("X.Size()",X.Size());
		Print("Idx.Size()",Idx.Size());
		HaltExecution();
	}
	Vector<T> v(d2);
	for(int i1=0; i1<d1; i1++) for(int i3=0; i3<d3; i3++)
	{
		for(int i2=0; i2<d2; i1++) v(i2) = X(Idx(i2),i2,i3);
		for(int i2=0; i2<d2; i1++) X(i1,i2,i3) = v(i2);
	}
}

template <class T>
void Dim3Permute(Array3D<T> &X, const Vector<int> &Idx)
{
	int d1 = X.Dim1();
	int d2 = X.Dim2();
	int d3 = X.Dim3();
	if(Idx.Size() != d3)
	{
		Print("Error in Dim3Permute(Array3D,Vector)");
		Print("Incompatible Dimensions");
		Print("X.Size()",X.Size());
		Print("Idx.Size()",Idx.Size());
		HaltExecution();
	}
	Vector<T> v(d3);
	for(int i1=0; i1<d1; i1++) for(int i2=0; i2<d2; i2++)
	{
		for(int i3=0; i3<d3; i3++) v(i3) = X(Idx(i3),i2,i3);
		for(int i3=0; i3<d3; i3++) X(i1,i2,i3) = v(i3);
	}
}
	
void PermuteInverse(Vector<int> &InvIdx, const Vector<int> Idx)
{
	int n = Idx.Size();
	for(int i=0; i<n; i++) InvIdx(Idx(i)) = i;
}

#endif
