
#ifndef _PrintLinearEquation
#define _PrintLinearEquation

/*

Print the Linear Equation,

A * x = b

where the components of x are assigned the names given in Labels

*/

void PrintLinearEquation(const Matrix<double> &A, const Vector<double> &b, const Vector<string> &Labels);

void PrintLinearEquation(const Matrix<double> &A, const Vector<double> &b, const Vector<string> &Labels)
{
	int R = A.Rows();
	int C = A.Cols();

	if(b.Size() != R || Labels.Size() != C)
	{
		Print("Error in PrintLinearEquation(Matrix,Vector,Labels)");
		Print("A.Name",A.Name);
		Print("b.Name",b.Name);
		Print("Labels.Name",Labels.Name);
		Print("A.Size()",A.Size());
		Print("b.Size()",b.Size());
		Print("Labels.Size()",Labels.Size());
		HaltExecution();
	}

	int r,c;
	bool First;
	for(r=0; r<R; r++)
	{
		First = 1;
		for(c=0; c<C; c++)
		{
			if(A(r,c) > 0.0)
			{
				if(!First)
				{
					cout << " + ";
					fout << " + ";
				}
				
				if(A(r,c) == 1.0)
				{
					cout << Labels[c];
					fout << Labels[c];
				}
				else
				{
					cout << A(r,c) << "*" << Labels[c];
					fout << A(r,c) << "*" << Labels[c];
				}
							
				First = 0;
			}
			else if(A(r,c) < 0.0)
			{
				if(!First)
				{
					if(A(r,c) == -1.0)
					{
						cout << " - " << Labels[c];
						fout << " - " << Labels[c];
					}
					else
					{
						cout << " - " << -A(r,c) << "*" << Labels[c];
						fout << " - " << -A(r,c) << "*" << Labels[c];
					}
				}
				else
				{
					if(A(r,c) == -1.0)
					{
						cout << "-" << Labels[c];
						fout << "-" << Labels[c];
					}
					else
					{
						cout << A(r,c) << "*" << Labels[c];
						fout << A(r,c) << "*" << Labels[c];
					}
				}
				First = 0;
			}
		}
		cout << " = " << b[r] << "\n";
		fout << " = " << b[r] << "\n";
	}
}

void PrintLinearEquation(const Matrix<double> &A, const Vector<double> &b, const Vector<string> &Labels, double Tol);

void PrintLinearEquation(const Matrix<double> &A, const Vector<double> &b, const Vector<string> &Labels, double Tol)
{
	int R = A.Rows();
	int C = A.Cols();

	if(b.Size() != R || Labels.Size() != C)
	{
		Print("Error in PrintLinearEquation(Matrix,Vector,Labels,Tol)");
		Print("A.Name",A.Name);
		Print("b.Name",b.Name);
		Print("Labels.Name",Labels.Name);
		Print("A.Size()",A.Size());
		Print("b.Size()",b.Size());
		Print("Labels.Size()",Labels.Size());
		HaltExecution();
	}

	int r,c;
	bool First;
	for(r=0; r<R; r++)
	{
		First = 1;
		for(c=0; c<C; c++)
		{
			if(A(r,c) > Tol)
			{
				if(!First)
				{
					cout << " + ";
					fout << " + ";
				}
				cout << A(r,c) << "*" << Labels[c];
				fout << A(r,c) << "*" << Labels[c];
				First = 0;
			}
			else if(A(r,c) < -Tol)
			{
				if(!First)
				{
					cout << " - " << -A(r,c) << "*" << Labels[c];
					fout << " - " << -A(r,c) << "*" << Labels[c];
				}
				else
				{
					cout << A(r,c) << "*" << Labels[c];
					fout << A(r,c) << "*" << Labels[c];
				}
				First = 0;
			}
		}
		cout << " = " << b[r] << "\n";
		fout << " = " << b[r] << "\n";
	}
}


#endif


