
#ifndef _GenMatUtil_ProdAAt
#define _GenMatUtil_ProdAAt
	

template <class T>
void ProdAAt(Matrix<T> &B, const Matrix<T> &A)
{
	int m = A.Rows();
	int n = A.Cols();
	if(B.Rows() != m || B.Cols() != m)
	{
		Print("Error in ProdAAt(Matrix,Matrix)");
		Print("B = A * A^T");
		Print("Incompadible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	for(int i=0; i<m; i++) for(int j=0; j<m; j++)
		B(i,j) = BlasDot(n,&A(i,0),1,&A(j,0),1);
}

#ifdef _GenMat_BlasMatrix
	template <class T>
	void ProdAAt(Matrix<T> &B, const BlasMatrix<T> &A)
	{
		int m = A.Rows();
		int n = A.Cols();
		int ld = A.LeadingDimension();
		if(B.Rows() != m || B.Cols() != m)
		{
			Print("Error in ProdAAt(Matrix,Matrix)");
			Print("B = A * A^T");
			Print("Incompadible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}
		
		for(int i=0; i<m; i++) for(int j=0; j<m; j++)
			B(i,j) = BlasDot(n,&A[ld*i],1,&A[ld*j],1);
	}
#endif

#ifdef _GenMat_BlasMatrix
	template <class T>
	void ProdAAt(BlasMatrix<T> &B, const Matrix<T> &A)
	{
		int m = A.Rows();
		int n = A.Cols();
		int ld = B.LeadingDimension();
		if(B.Rows() != m || B.Cols() != m)
		{
			Print("Error in ProdAAt(Matrix,Matrix)");
			Print("B = A * A^T");
			Print("Incompadible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}
		
		for(int i=0; i<m; i++) for(int j=0; j<m; j++)
			B[ld*i+j] = BlasDot(n,&A(i,0),1,&A(j,0),1);
	}
#endif

#ifdef _GenMat_BlasMatrix
	template <class T>
	void ProdAAt(BlasMatrix<T> &B, const BlasMatrix<T> &A)
	{

		int m = A.Rows();
		int n = A.Cols();
		int lda = A.LeadingDimension();
		int ldb = B.LeadingDimension();
		if(B.Rows() != m || B.Cols() != m)
		{
			Print("Error in ProdAAt(Matrix,Matrix)");
			Print("B = A * A^T");
			Print("Incompadible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		//BlasGbemm(False,True,m,n,m,One,A.Begin(),lda,A.Begin(),lda,Zero,B.Begin(),ldb,128);
		for(int i=0; i<m; i++) for(int j=0; j<m; j++)
			B[ldb*i+j] = BlasDot(n,&A(i,0),1,&A(j,0),1);
	}
#endif

template <class T>
void ProdAAt(SymmetricMatrix<T> &B, const Matrix<T> &A)
{
	int m = A.Rows();
	int n = A.Cols();	
	if(B.Dim() != m)
	{
		Print("Error in ProdAAt(SymmetricMatrix,Matrix)");
		Print("B = A * A^T");
		Print("Incompadible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	for(int i=0; i<m; i++) for(int j=0; j<=i; j++)
	{
		B(i,j) = Zero;
		for(int k=0; k<n; k++) B(i,j) += A(i,k) * A(j,k);
	}
}


template <class T>
void ProdAAt(SymmetricMatrix<T> &B, const LowerMatrix<T> &A)
{
	int n = A.Dim();
	if(B.Dim() != n)
	{
		Print("Error in ProdAAt(SymmetricMatrix,LowerMatrix)");
		Print("B = A * A^T");
		Print("Incompadible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	/* Preform Calculations */
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++)
		B(i,j) = BlasDot(Min(i,j)+1,&A(i,0),1,&A(j,0),1);
}

#ifdef _GenMat_SymmetricMatrixLU
	template <class T>
	void ProdAAt(SymmetricMatrixLU<T> &B, const LowerMatrix<T> &A)
	{
		int n = A.Dim();
		if(B.Dim() != n)
		{
			Print("Error in ProdAAt(SymmetricMatrixLU,LowerMatrix)");
			Print("B = A * A^T");
			Print("Incompadible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		/* No Blas Routine Currertly Preforms this Calculation */	
		for(int i=0; i<n; i++) for(int j=0; j<=i; j++)
		{
			B(i,j) = Zero;
			for(int k=0; k<n; k++) B(i,j) += A(i,k) * A(j,k);
		}
	}
#endif

#if defined(_GenMat_SymmetricMatrixUU) && defined(_GenMat_LowerMatrixUN)
	template <class T>
	void ProdAAt(SymmetricMatrixUU<T> &B, const LowerMatrixUN<T> &A)
	{
		int n = A.Dim();
		if(B.Dim() != n)
		{
			Print("Error in ProdAAt(SymmetricMatrixUU,LowerMatrixUN)");
			Print("B = A * A^T");
			Print("Incompadible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		BlasSytr(False,True,n,n,One,A.Begin(),n,Zero,B.Begin(),n);
	}
#endif

#if defined(_GenMat_SymmetricBandMatrix) && defined(_GenMat_LowerBandMatrix)
	template <class T>
	void ProdAAt(SymmetricMatrixLU<T> &B, const LowerMatrixUN<T> &A)
	{
		int n = A.Dim();
		if(B.Dim() != n)
		{
			Print("Error in ProdAAt(SymmetricMatrixLU,LowerMatrixUN)");
			Print("B = A * A^T");
			Print("Incompadible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		BlasSytr(False,False,n,n,One,A.Begin(),n,Zero,B.Begin(),n);
	}
#endif

#if defined(_GenMat_SymmetricBandMatrix) && defined(_GenMat_LowerBandMatrix)
	template <class T>
	void ProdAAt(SymmetricBandMatrix<T> &B, const LowerBandMatrix<T> &A)
	{
		int n = A.Rows();
		int k = A.LowerBands();
		if(A.Cols() != n || B.Dim() != n || B.Bands() != k)
		{
			Print("Error in ProdAAt(SymmetricBandMatrix,LowerBandMatrix)");
			Print("B = A * A^T");
			Print("Incompadible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			Print("A.LowerBands()",A.LowerBands());
			Print("B.Bands()",B.Bands());
			HaltExecution();
		}

		/* No Blas Routine Currertly Preforms this Calculation */
		for(int i=0; i<n; i++) for(int j=0; j<=i; j++)
		{
			B(i,j) = Zero;
			for(int k=0; k<n; k++) B(i,j) += A(i,k) * A(j,k);
		}
	}
#endif

template <class T>
void ProdAAt(SymmetricMatrix<T> &B, const GeneralMatrix<T> &A)
{
	int m = A.Rows();
	int n = A.Cols();
	if(B.Rows() != m || B.Cols() != m)
	{
		Print("Error in ProdAAt(Matrix,GeneralMatrix)");
		Print("B = A * A^T");
		Print("Incompadible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	for(int i=0; i<m; i++) for(int j=0; j<=i; j++)
	{
		B(i,j) = Zero;
		for(int k=0; k<n; k++) B(i,j) += A(i,k) * A(j,k);
	}
}

#endif
