
#ifndef _GenMatUtil_MatrixProdAB
#define _GenMatUtil_MatrixProdAB


/* Multiply two matrices, C = A * B */
template <class T1, class T2, class T3>
void ProdAB(Matrix<T1> &C, const Matrix<T2> &A, const Matrix<T3> &B)
{
	int m = A.Rows();
	int n = B.Cols();
	int r = A.Cols();

	if(B.Rows() != r || C.Rows() != m || C.Cols() != n)
	{
		Print("Error in Prod(Matrix,Matrix,Matrix)");
		Print("C = A * B");
		Print("Incompadible Dimensions");
		Print("C.Name",C.Name);
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("C.Size()",C.Size());
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	//for(i=0; i<m; i++) for(j=0; j<n; j++)
	//{
	//	C(i,j) = 0.0;
	//	for(k=0; k<r; k++) C(i,j) += A(i,k) * B(k,j);
	//}
	
	BlasGemm(0,0,m,n,r,1.0,A.Begin(),r,B.Begin(),n,0.0,C.Begin(),n);
}

/*
template <class T>
void ProdAB(SymmetricMatrix<T> &C, const SymmetricMatrix<T> &A, const SymmetricMatrix<T> &B)
{
	int i,j,k;
	int n = A.Dim();

	if(B.Dim() != n || C.Dim() != n)
	{
		Print("Error in ProdAB(SymmetricMatrix,SymmetricMatrix,SymmetricMatrix)");
		Print("C = A * B");
		Print("Incompatible Dimensions");
		Print("C.Name",C.Name);
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("C.Size()",C.Size());
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	//for(i=0; i<n; i++) for(j=0; j<=i; j++)
	//{
	//	C(i,j) = 0.0;
	//	for(k=0; k<n; k++) C(i,j) += A(i,k) * B(k,j);
	//}
	
	BlasSet(n*(n+1)/2,0.0,C.Begin(),1);	
	for(i=0; i<n; i++)
	{
		for(j=0; j<=i; j++) C(i,j) = BlasDot(j+1,&A(i,0),1,&B(0,j),1);
		for(k=0; k<n; k++)
		{
			double Temp = A(i,k);
			BlasAxpy(Min(k,i+1),Temp,&B(k,0),1,&C(i,0),1);
		}
	}
}
*/

template <class T>
void ProdAB(LowerMatrix<T> &C, const LowerMatrix<T> &A, const LowerMatrix<T> &B)
{
	Print("ProdAB(LowerMatrix,LowerMatrix,LowerMatrix) is not doen yet!!");
	HaltExecution();

	int i,j,k;
	int n = A.Dim();

	if(B.Cols() != C.Rows() || A.Rows() != B.Rows() || A.Cols() != C.Cols())
	{
		Print("Error in Prod(LowerMatrix,LowerMatrix,LowerMatrix): Incompatible Dimensions");
		return;
	}

	for(i=0; i<I; i++) for(j=0; j<J; j++)
	{
		A(i,j) = 0.0;
		for(k=0; k<K; k++) A(i,j) += B(i,k) * C(k,j);
	}
}

template <class T>
void ProdAB(UpperMatrix<T> &A, const UpperMatrix<T> &B, const UpperMatrix<T> &C)
{
	Print("ProdAB(UpperMatrix,UpperMatrix,UpperMatrix) is not doen yet!!");
	HaltExecuton()

	int i,j,k;
	int I = A.Rows();
	int J = A.Cols();
	int K = B.Cols();

	if(B.Cols() != C.Rows() || A.Rows() != B.Rows() || A.Cols() != C.Cols())
	{
		Print("Error in Prod(UpperMatrix,UpperMatrix,UpperMatrix): Incompatible Dimensions");
		return;
	}

	for(i=0; i<I; i++) for(j=0; j<J; j++)
	{
		A(i,j) = 0.0;
		for(k=0; k<K; k++) A(i,j) += B(i,k) * C(k,j);
	}
	return;
}

template <class T>
void ProdAB(Matrix<T> &C, const Matrix<T> &A, const SymmetricMatrix<T> &B)
{
	int i,j,k;
	int m = A.Rows();
	int n = A.Cols();

	if(B.Dim() != n || C.Rows() != m || C.Cols() != n)
	{
		Print("Error in ProdAB(Matrix,Matrix,SymmetricMatrix)");
		Print("C = A * B");
		Print("Incompadible Dimensions");
		Print("C.Name",C.Name);
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("C.Size()",C.Size());
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	for(i=0; i<m; i++) for(j=0; j<n; j++)
	{
		C(i,j) = Zero;
		for(k=0; k<n; k++) C(i,j) += A(i,k) * B(k,j);
	}
}

template <class T>
void ProdAB(Matrix<T> &C, const SymmetricMatrix<T> &A, const Matrix<T> &B)
{
	int i,j,k;
	double Temp;
	int m = A.Dim();
	int n = B.Cols();

	if(B.Rows() != m || C.Rows() != m || C.Cols() != n)
	{
		Print("Error in ProdAB(Matrix,SymmetricMatrix,Matrix)");
		Print("C = A * B");
		Print("Incompadible Dimensions");
		Print("C.Name",C.Name);
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("C.Size()",C.Size());
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	//for(i=0; i<m; i++) for(j=0; j<n; j++)
	//{
	//	C(i,j) = 0.0;
	//	for(k=0; k<m; k++) C(i,j) += A(i,k) * B(k,j);
	//}
	BlasSet(C.Length(),Zero,C.Begin(),1);
	for(i=0; i<m; i++) for(k=0; k<m; k++)
	{
		Temp = A(i,k);
		BlasAxpy(n,Temp,&B(k,0),1,&C(i,0),1);
	}
}

/*

Compute the product, C = A * B

where,

C is a SymmetricMatrix
A is a Matrix
B is a Matrix

WARNING: Only use this routine if you are sure the result is
symmetric, as this routine will not check if this is the case

*/

template <class T1, class T2, class T3>
void ProdAB_(SymmetricMatrix<T1> &C, const Matrix<T2> &A, const Matrix<T3> &B)
{
	int m = A.Rows();
	int n = A.Cols();
	if(B.Rows() != n || B.Cols() != m || C.Dim() != m)
	{
		Print("Error in ProdAB(SymmetricMatrix,Matrix,Matrix)");
		Print("C = A * B");
		Print("Incompabible Dimensions");
		Print("C.Name",C.Name);
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("C.Size()",C.Size());
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	for(int i=0; i<m; i++) for(int j=0; j<=i; j++)
	{
		C(i,j) = Zero;
		for(int k=0; k<n; k++) C(i,j) += A(i,k) * B(k,j);
	}
}

// Multiply two matrices (efficient Routine)
template <class T1, class T2, class T3>
void ProdAB(Matrix<T1> &C, const SymmetricMatrix<T2> &A, const LowerMatrix<T3> &B)
{
	int i,j,k;
	int n = A.Dim();
	if(B.Dim() != n || C.Rows() != n || C.Cols() != n)
	{
		Print("Error in ProdAB(Matrix,SymmetricMatrix,LowerMatrix)");
		Print("C = A * B");
		Print("Incompadible Dimensions");
		Print("C.Name",C.Name);
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("C.Size()",C.Size());
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	for(i=0; i<n; i++) for(j=0; j<n; j++)
	{
		C(i,j) = Zero;
		for(k=0; k<n; k++) C(i,j) += A(i,k) * B(k,j);
	}
}

template <class T>
void ProdAB(DiagonalMatrix<T> &C, const DiagonalMatrix<T> &A, const DiagonalMatrix<T> &B)
{
	int n = A.Dim();
	if(B.Dim() != n || C.Dim() != n)
	{
		Print("Error in ProdAB(DiagonalMatrix,DiagonalMatrix,DiagonalMatrix)");
		Print("C = A * B");
		Print("Incompatible Dimensions");
		Print("C.Name",C.Name);
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("C.Size()",C.Size());
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	for(int i=0; i<n; i++) C[i] = A[i] * B[i];
}

/*

Mulity a BlockMatrix and a Matrix

C = A * B

  [ A1           ]   [ B1 ]   [ A1 * B1 ]
= [    A2        ] * [ B2 ] = [ A2 * B2 ]
  [       ...    ]   [ .. ]   [   ...   ]
  [           Am ]   [ Bm ]   [ Am * Bm ]

*/

#ifdef _GenMat_BlockMatrix
	template <class T>
	void ProdAB(Matrix<T> &C, const BlockMatrix<T> &A, const Matrix<T> &B)
	{
		int i,j,k,l;
		int r = A.Dim();
		int c = B.Cols();

		if(B.Rows() != r || C.Rows() != r || C.Cols() != c)
		{
			Print("Error in ProdAB(Matrix,BlockMatrix,Matrix)");
			Print("Inconsistent Dimensions");
			Print("C = A * B");
			Print("C.Name",C.Name);
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("C.Size()",C.Size());
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		Set(C,Zero);
		int m = A.Blocks();
		int sm = 0;
		for(i=0; i<m; i++)
		{
			int n = A.Block(i);
			for(j=0; j<n; j++) for(k=0; k<c; k++) for(l=0; l<n; l++) C(j+sm,k) += A(i,j,l) * B(l+sm,k);
			sm += n;
		}
	}
#endif

template <class T>
void ProdAB(Matrix<double> &C, const DiagonalMatrix<T> &A, const Matrix<T> &B)
{
	/* Check Dimensions */
	int m = A.Dim();
	int n = B.Cols();
	if(B.Rows() != m || C.Rows() != m || C.Cols() != n)
	{
		Print("ProdAB(Matrix,DiagonalMatrix,Matrix)");
		Print("C = A * B");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	/* Preform Calculations */
	Matrix<T> C(m,n);
	for(i=0; i<m; i++) for(j=0; j<n; j++) C(i,j) = A(i,i) * B(i,j);
	return C;
}

template <class T>
void ProdAB(Matrix<double> &C, const Matrix<T> &A, const DiagonalMatrix<T> &B)
{
	/* Check Dimensions */
	int m = A.Rows();
	int n = A.Cols();
	if(B.Dim() != n || C.Rows() != m || C.Cols() != n)
	{
		Print("Error in Matrix = Matrix * DiagonalMatrix");
		Print("C = A * B");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	/* Preform Calculations */
	Matrix<T> C(m,n);
	for(i=0; i<m; i++) for(j=0; j<n; j++) C(i,j) = A(i,j) * B(j,j);
	return C;
}

#ifdef _GenMat_SparseMatrix
	template <class T>
	void ProdAB(Matrix<T> &C, const SparseMatrix<T> &A, const Matrix<T> &B)
	{
		/* Check Dimensions */
		int m = A.Rows();
		int n = B.Cols();
		int r = A.Cols();
		if(B.Rows() != r || C.Rows() != m || C.Cols() != n)
		{
			Print("Error in ProdAB(Matrix,SparseMatrix,Matrix)");
			Print("C = A * B");
			Print("Incompadible Dimensions");
			Print("C.Name",C.Name);
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("C.Size()",C.Size());
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		/* Preform Calculations */
		Set(C,Zero);
		for(int i=0; i<m; i++)
		{
			for(int k=A.r_[i]; k<A.r_[i+1]; k++)
			{
				int Idx = A.c_[k];
				T Temp = A.v_[k];
				BlasAxpy(n,Temp,&B(Idx,0),1,&C(i,0),1);
			}
		}
	}

	template <class T>
	void ProdAB(Matrix<T> &C, const Matrix<T> &A, const SparseMatrix<T> &B)
	{
		/* Check Dimensions */
		int m = A.Rows();
		int n = B.Cols();
		int r = A.Cols();
		if(B.Rows() != r || C.Rows() != m || C.Cols() != n)
		{
			Print("Error in Prod(Matrix,Matrix,SparseMatrix)");
			Print("C = A * B");
			Print("Incompadible Dimensions");
			Print("C.Name",C.Name);
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("C.Size()",C.Size());
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		/* Preform Calculations */
		
		/* Notice the increments of n are used in the innermost loop.
		Can this be improved upon? */
		
		Set(C,Zero);
		for(int k=0; k<r; k++)
		{
			for(int l=B.r_[k]; l<B.r_[k+1]; l++)
			{
				int Idx = B.c_[l];
				T Temp = B.v_[l];
				BlasAxpy(m,Temp,&A(0,k),n,&C(0,Idx),n);
			}
		}
	}

	template <class T>
	void ProdAB(Matrix<T> &C, const SymmetricMatrix<T> &A, const SparseMatrix<T> &B)
	{
		/* Check Dimensions */
		int m = A.Dim();
		int n = B.Cols();
		if(B.Rows() != m || C.Rows() != m || C.Cols() != n)
		{
			Print("Error in ProdAB(Matrix,SymmetricMatrix,SparseMatrix)");
			Print("C = A * B");
			Print("Incompadible Dimensions");
			Print("C.Name",C.Name);
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("C.Size()",C.Size());
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		/* Preform Calculations */
		
		/* Notice the increments of n are used in the innermost loop.
		Can this be improved upon? */
		
		Set(C,Zero);
		for(int k=0; k<m; k++)
		{
			Print("k",k,"m",m);
			for(int l=B.r_[k]; l<B.r_[k+1]; l++)
			{
				int Idx = B.c_[l];
				T Temp = B.v_[l];
				//BlasAxpy(m,Temp,&A(0,k),n,&C(0,Idx),n);
				for(int i=0; i<m; i++) C(i,Idx) += Temp * A(i,k);	// This can be made faster!
			}
		}
	}


	template <class T>
	void ProdAB(Matrix<T> &C, const SparseMatrix<T> &A, const SymmetricMatrix<T> &B)
	{
		/* Check Dimensions */
		int m = A.Rows();
		int n = A.Cols();
		if(B.Dim() != n || C.Rows() != m || C.Cols() != n)
		{
			Print("Error in ProdAB(Matrix,SparseMatrix,SymmetricMatrix)");
			Print("C = A * B");
			Print("Incompadible Dimensions");
			Print("C.Name",C.Name);
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("C.Size()",C.Size());
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		/* Preform Calculations */
		/* Is there a better way to do this? */
		Set(C,Zero);
		for(int i=0; i<m; i++)
		{
			for(int k=A.r_[i]; k<A.r_[i+1]; k++)
			{
				int Idx = A.c_[k];
				T Temp = A.v_[k];
				//for(int j=0; j<n; j++) C(i,j) += Temp * B(Idx,j);
				BlasAxpy(Idx,Temp,&B(Idx,0),1,&C(i,0),1);
				for(int j=Idx; j<n; j++) C(i,j) += Temp * B(Idx,j);
			}
		}
	}

	template <class T1, class T2, class T3>
	void ProdAB_(SymmetricMatrix<T1> &C, const SparseMatrix<T2> &A, const Matrix<T3> &B)
	{
		/* Check Dimensions */
		int m = A.Rows();
		int n = B.Cols();
		int r = A.Cols();
		if(B.Rows() != r || C.Rows() != m || C.Cols() != n || m != n)
		{
			Print("Error in ProdAB_(SymmetricMatrix,SparseMatrix,Matrix)");
			Print("C = A * B");
			Print("Incompadible Dimensions");
			Print("C.Name",C.Name);
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("C.Size()",C.Size());
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		/* Preform Calculations */
		Matrix<T1> C_(m,m);
		Set(C_,Zero);
		for(int i=0; i<m; i++)
		{
			for(int k=A.r_[i]; k<A.r_[i+1]; k++)
			{
				int Idx = A.c_[k];
				T Temp = A.v_[k];
				BlasAxpy(n,Temp,&B(Idx,0),1,&C_(i,0),1);
			}
		}
		Copy(C,ToSymmetricMatrix(C_));
	}
#endif

#endif
