
#ifndef _GenMatUtil_ProdABA
#define _GenMatUtil_ProdABA


/*

Form the product,

	C = A * B * A
	
where A and B are symmetric matrices.
In this case, C is guarunteed to be symmetric

*/

template <class T>
void ProdABA(SymmetricMatrix<T> &C, const SymmetricMatrix<T> &A, const SymmetricMatrix<T> &B)
{
	int n = A.Dim();
	
	if(B.Dim() != n || C.Dim() != n)
	{
		Print("Error in ProdABA(SymmetricMatrix<T> C, SymmetricMatrix<T> A, SymmetricMatrix<T> B)");
		Print("C = A * B * A");
		Print("Incompadible Dimensions");
		Print("C.Name",C.Name);
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("C.Size()",C.Size());
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	/*
	
	Note:
	
	this code could be rewritten to be more efficent and to avoid
	temporary storage of a matrix
	
	*/

	Matrix<T> D = B * A;
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++)
	{
		C(i,j) = Zero;
		for(int k=0; k<n; k++) C(i,j) += A(i,k) * D(k,j);
	}
}


#endif