

#ifndef _GenMatUtil_ProdABAt
#define _GenMatUtil_ProdABAt


	template <class T> void ProdABAt(SymmetricMatrix<T> &C, const Matrix<T> &A, const SymmetricMatrix<T> &B);

/*
void ProdABAt(SymmetricMatrix<double> &C, const LowerMatrix<double> &A, const DiagonalMatrix<double> &B);

void ProdABAt(SymmetricMatrix<double> &C, const LowerMatrix<double> &A, const DiagonalMatrix<double> &B)
{
	int n = A.Dim();
	if(B.Dim() != n || C.Dim() != n)
	{
		Print("Error in ProdABAt(SymmetricMatrix,LowerMatrix,DiagonalMatrix)");
		Print("C = A * B * A^T");
		Print("Incompadible dimensions");
		Print("C.Name",C.Name);
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("C.Size()",C.Size());
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		Print("Halting execution");
		exit(0);
	}

	UpperMatrix<double> D(n);
	ProdABt(D,B,A);
	Prod(C,A,D);
}
*/

template <class T>
void ProdABAt(SymmetricMatrix<T> &C, const Matrix<T> &A, const SymmetricMatrix<T> &B)
{
	int m = A.Rows();
	int n = A.Cols();
	
	if(C.Dim() != m || B.Dim() != n)
	{
		Print("Error in ProdABAt");
		Print("C = A * B * A^T");
		Print("Incompadible Dimensions");
		Print("C.Name",C.Name);
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("C.Size()",C.Size());
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	Matrix<T> D(n,m);
	ProdABt(D,B,A);
	ProdAB_(C,A,D);
}

template <class T>
void ProdABAt(SymmetricMatrix<T> &C, const LowerMatrix<T> &A, const SymmetricMatrix<T> &B)
{
	int n = A.Dim();
	if(B.Dim() != n || C.Dim() != n)
	{
		Print("Error in ProdABAt");
		Print("C = A * B * A^T");
		Print("Incompadible Dimensions");
		Print("C.Name",C.Name);
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("C.Size()",C.Size());
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	//Matrix<double> D(n,n);
	//ProdABt(D,B,A);
	//ProdAB_(C,A,D);
	Matrix<T> D = A * B * Transpose(A);
	Copy(C,D);
}

#ifdef _GenMat_SparseMatrix
	template <class T>
	void ProdABAt(SymmetricMatrix<T> &C, const SparseMatrix<T> &A, const SymmetricMatrix<T> &B)
	{
		int m = A.Rows();
		int n = A.Cols();
		if(C.Dim() != m || B.Dim() != n)
		{
			Print("Error in ProdABAt");
			Print("C = A * B * A^T");
			Print("Incompadible Dimensions");
			Print("C.Name",C.Name);
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("C.Size()",C.Size());
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		Matrix<T> D(n,m);
		D.Name = "ProdABAt(SymmetricMatrix,SparseMatrix,SymmetricMatrix)";
		/*
		for(int i=0; i<n; i++) for(int j=0; j<m; j++)
		{
			D(m,n) = Zero;
			int Low  = A.r_[j];
			int High = A.r_[j+1];
			//for(int k=0; k<m; k++) D(i,j) += B(i,k) * A(j,k);
			for(int k=Low; k<High; k++) D(i,j) += B(i,A.c_[k]) * A.v_[k];
		}
		*/
		ProdABt(D,B,A);	
		ProdAB_(C,A,D);
	}

	template <class T>
	void ProdABAt_(SymmetricMatrix<T> &C, const SparseMatrix<T> &A, const Matrix<T> &B)
	{
		int m = A.Rows();
		int n = A.Cols();
		if(C.Dim() != m || B.Dim() != n)
		{
			Print("Error in ProdABAt_(SymmetricMatrix,SparseMatrix,Matrix)");
			Print("C = A * B * A^T");
			Print("Incompadible Dimensions");
			Print("C.Name",C.Name);
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("C.Size()",C.Size());
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		Matrix<T> D(n,m);
		D.Name = "ProdABAt_(SymmetricMatrix,SparseMatrix,Matrix)";
		/*
		for(int i=0; i<n; i++) for(int j=0; j<m; j++)
		{
			D(m,n) = Zero;
			int Low  = A.r_[j];
			int High = A.r_[j+1];
			//for(int k=0; k<m; k++) D(i,j) += B(i,k) * A(j,k);
			for(int k=Low; k<High; k++) D(i,j) += B(i,A.c_[k]) * A.v_[k];
		}
		*/
		ProdABt(D,B,A);	
		ProdAB_(C,A,D);
	}
#endif

#endif