
#ifndef _GenMatUtil_MatrixProdABOp
#define _GenMatUtil_MatrixProdABOp


/* Multiply two matrices, C = A * B */
template <class T>
Matrix<T> operator * (const Matrix<T> &A, const Matrix<T> &B)
{
	/* Check for Incompadible Dimensions */
	int m = A.Rows();
	int n = B.Cols();
	int r = A.Cols();
	if(B.Rows() != r)
	{
		Print("Error in Matrix = Matrix * Matrix");
		Print("C = A * B");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	/* Preform Calculations */
	Matrix<T> C(m,n);
	C.Name = "(" + A.Name + "*" + B.Name + ")";
	BlasGemm(0,0,m,n,r,GetOne<T>(),A.Begin(),r,B.Begin(),n,GetZero<T>(),C.Begin(),n);
	return C;
}

/*
template <class T>
SymmetricMatrix<T> operator * (const SymmetricMatrix<T> &A, const SymmetricMatrix<T> &B)
{
	int i,j,k;
	int n = A.Dim();

	/*

	This code is wrong !!!!!

	if(B.Dim() != n)
	{
		Print("Error in SymmetricMatrix = SymmetricMatrix * SymmetricMatrix");
		Print("C = A * B");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	SymmetricMatrix<T> C(n);
	BlasSet(n*(n+1)/2,0.0,C.Begin(),1);
	for(i=0; i<n; i++)
	{
		for(j=0; j<=i; j++) C(i,j) = BlasDot(j+1,&A(i,0),1,&B(0,j),1);
		for(k=0; k<n; k++)
		{
			double Temp = A(i,k);
			BlasAxpy(Min(k,i+1),Temp,&B(k,0),1,&C(i,0),1);
		}
	}
	return C;

	*/
/*
}
*/

/*
template <class T>
SymmetricMatrixUP<T> operator * (const SymmetricMatrixUP<T> &A, const SymmetricMatrixUP<T> &B)
{
	int i,j,k;
	int n = A.Dim();

	/* Check Dimensions */
/*	if(B.Dim() != n)
	{
		Print("Error in SymmetricMatrixUP = SymmetricMatrixUP * SymmetricMatrixUP");
		Print("C = A * B");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	SymmetricMatrixUP<T> C(n,n);
	for(i=0; i<n; i++) for(j=0; j<n; j++)
	{
		C(i,j) = 0.0;
		for(k=0; k<n; k++) C(i,j) += A(i,k) * B(k,j);
	}
	return C;
}
*/

#ifdef _GenMat_DiagonalMatrix
	template <class T>
	DiagonalMatrix<T> operator * (const DiagonalMatrix<T> &A, const DiagonalMatrix<T> &B)
	{
		/* Check for Incompadible Dimensions */
		int n = A.Dim();
		if(B.Dim() != n)
		{
			Print("Error in DiagonalMatrix = DiagonalMatrix * DiagonalMatrix")
			Print("C = A * B");
			Print"Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		/* Preform Calculations */
		DiagonalMatrix<T> C(n);
		C.Name = "(" + A.Name + "*" + B.Name + ")";

		/* Note: this part can be made faster using BlasSet and BlasDot */
		for(int i=0; i<n; i++) C[i] = A[i] * B[i];

		return C;
	}
#endif

template <class T>
LowerMatrix<T> operator * (const LowerMatrix<T> &A, const LowerMatrix<T> &B)
{
	/* Check for Incompadible Dimensions */
	int n = A.Dim();
	if(B.Dim() != n)
	{
		Print("Error in LowerMatrix = LowerMatrix * LowerMatrix");
		Print("C = A * B");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",A.Size());
		HaltExecution();
	}

	/* Preform Calculations */
	LowerMatrix<T> C(n);
	C.Name = "(" + A.Name + "*" + B.Name + ")";
	for(int i=0; i<n; i++) for(int j=0; j<n; j++)
	{
		C(i,j) = Zero;
		for(int k=0; k<n; k++) C(i,j) += A(i,k) * B(k,j);
	}
	return C;
}

template <class T>
UpperMatrix<T> operator * (const UpperMatrix<T> &A, const UpperMatrix<T> &B)
{
	/* Check for Incompadible Dimensions */
	int n = A.Dim();
	if(B.Dim() != n)
	{
		Print("Error in UpperMatrix = UpperMatrix * UpperMatrix");
		Print("C = A * B");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	/* Preform Calculations */
	UpperMatrix<T> C(n);
	C.Name = "(" + A.Name + "*" + B.Name + ")";
	for(int i=0; i<n; i++) for(int j=0; j<n; j++)
	{
		C(i,j) = Zero;
		for(int k=0; k<n; k++) C(i,j) += A(i,k) * B(k,j);
	}
	return C;
}


template <class T>
Matrix<T> operator * (const LowerMatrix<T> &A, const UpperMatrix<T> &B)
{
	/* Check Incompadible Dimensions */
	int n = A.Dim();
	if(B.Dim() != n)
	{
		Print("Error in Matrix = LowerMatrix * UpperMatrix");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	/* Note: This is not efficient yet! */
	Matrix<T> C(n,n);
	C.Name = "(" + A.Name + "*" + B.Name + ")";
	for(int i=0; i<n; i++) for(int j=0; j<n; j++)
	{
		C(i,j) = Zero;
		for(int k=0; k<n; k++) C(i,j) += A(i,k) * B(k,j);
	}	
	return C;
}

template <class T>
Matrix<T> operator * (const UpperMatrix<T> &A, const LowerMatrix<T> &B)
{
	/* Check Incompadible Dimensions */
	int n = B.Dim();
	if(B.Dim() != n)
	{
		Print("Error in Matrix = UpperMatrix * LowerMatrix");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	Matrix<T> C(n,n);
	C.Name = "(" + A.Name + "*" + B.Name + ")";
	for(i=0; i<n; i++) for(j=0; j<n; j++)
	{
		C(i,j) = Zero;
		for(k=0; k<n; k++) C(i,j) += A(i,k) * B(k,j);
	}
	return C;
}

template <class T>
Matrix<T> operator * (const Matrix<T> &A, const SymmetricMatrix<T> &B)
{
	/* Check Dimensions */
	int m = A.Rows();
	int n = A.Cols();
	if(B.Dim() != n)
	{
		Print("Error in Matrix = Matrix * SymmetricMatrix");
		Print("C = A * B");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	/* Preform Calculations */
	Matrix<T> C(m,n);
	C.Name = "(" + A.Name + "*" + B.Name + ")";
	Set(C,Zero);
	for(int i=0; i<m; i++) for(int k=0; k<n; k++)
	{
		T Temp = A(i,k);
		BlasAxpy(k,Temp,&B(k,0),1,&C(i,0),1);
	}
	for(int i=0; i<m; i++) for(int j=0; j<n; j++) C(i,j) += BlasDot(j+1,&A(i,0),1,&B(0,j),1);
	return C;
}

template <class T>
Matrix<T> operator * (const SymmetricMatrix<T> &A, const Matrix<T> &B)
{
	/* Check Dimensions */
	int m = A.Dim();
	int n = B.Cols();
	if(B.Rows() != m)
	{
		Print("Error in Matrix = SymmetricMatrix * Matrix");
		Print("C = A * B");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	/* Preform Calculations */
	Matrix<T> C(m,n);
	C.Name = "(" + A.Name + "*" + B.Name + ")";
	Set(C,Zero);
	for(int i=0; i<m; i++) for(int k=0; k<m; k++)
	{
		double Temp = A(i,k);
		BlasAxpy(n,Temp,&B(k,0),1,&C(i,0),1);
	}
	return C;
}

#ifdef _GenMat_DiagonalMatrix
	template <class T>
	Matrix<T> operator * (const DiagonalMatrix<T> &A, const Matrix<T> &B)
	{
		/* Check Dimensions */
		int m = A.Dim();
		int n = B.Cols();
		if(B.Rows() != m)
		{
			Print("Error in Matrix = DiagonalMatrix * Matrix");
			Print("C = A * B");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		/* Preform Calculations */
		Matrix<T> C(m,n);
		C.Name = "(" + A.Name + "*" + B.Name + ")";
		for(int i=0; i<m; i++) for(int j=0; j<n; j++) C(i,j) = A(i,i) * B(i,j);
		return C;
	}

	template <class T>
	Matrix<T> operator * (const Matrix<T> &A, const DiagonalMatrix<T> &B)
	{
		/* Check Dimensions */
		int m = A.Rows();
		int n = A.Cols();
		if(B.Dim() != n)
		{
			Print("Error in Matrix = Matrix * DiagonalMatrix");
			Print("C = A * B");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		/* Preform Calculations */
		Matrix<T> C(m,n);
		C.Name = "(" + A.Name + "*" + B.Name + ")";
		for(int i=0; i<m; i++) for(int j=0; j<n; j++) C(i,j) = A(i,j) * B(j,j);
		return C;
	}
#endif

#ifdef _GenMat_SymmetricMatrixUP
	template <class T>
	Matrix<T> operator * (const Matrix<T> &A, const SymmetricMatrixUP<T> &B)
	{
		/* Check for Incompadible Dimensions */
		int m = A.Rows();
		int n = B.Dim();
		if(A.Cols() != n)
		{
			Print("Error in Matrix = Matrix * SymmetricMatrixUP");
			Print("C = A * B");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		/* Preform Calculations */
		Matrix<T> C(m,n);
		C.Name = "(" + A.Name + "*" + B.Name + ")";
		for(int i=0; i<m; i++) for(int j=0; j<n; j++)
		{
			C(i,j) = Zero;
			for(int k=0; k<n; k++) C(i,j) += A(i,k) * B(k,j);
		}
		return C;
	}
#endif

#ifdef _GenMat_SymmetricMatrixLU
	template <class T>
	Matrix<T> operator * (const Matrix<T> &A, const SymmetricMatrixLU<T> &B)
	{
		/* Check for Incompadible Dimensions */
		int m = A.Rows();
		int n = B.Dim();
		if(A.Cols() != n)
		{
			Print("Error in Matrix = Matrix * SymmetricMatrixLU");
			Print("C = A * B");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		/* Preform Calculations */
		Matrix<T> C(m,n);
		C.Name = "(" + A.Name + "*" + B.Name + ")";
		for(int i=0; i<m; i++) for(int j=0; j<n; j++)
		{
			C(i,j) = Zero;
			for(int k=0; k<n; k++) C(i,j) += A(i,k) * B(k,j);
		}
		return C;
	}
#endif

template <class T>
Matrix<T> operator * (const GeneralMatrix<T> &A, const GeneralMatrix<T> &B)
{
	/* Check for Incompadible Dimensions */
	int m = A.Rows();
	int r = A.Cols();
	int n = B.Cols();
	if(B.Rows() != r)
	{
		Print("Error in Matrix = GeneralMatrix * GeneralMatrix");
		Print("C = A * B");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	/* Preform Calculations */
	Matrix<T> C(m,n);
	C.Name = "(" + A.Name + "*" + B.Name + ")";
	for(int i=0; i<m; i++) for(int j=0; j<n; j++)
	{
		C(i,j) = Zero;
		for(int k=0; k<r; k++) C(i,j) += A(i,k) * B(k,j);
	}
	return C;
}

/*
template <class T>
Matrix<T> operator * (const SparseMatrix<T> &A, const Matrix<T> &B)
{
	int i,j,k;
	int m = A.Rows();
	int r = A.Cols();
	int n = B.Cols();

	/* Check for Incompadible Dimensions */
/*	if(B.Rows() != r)
	{
		Print("Error in Matrix = GeneralMatrix * GeneralMatrix");
		Print("C = A * B");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	/* Preform Calculations */
/*	Matrix<T> C(m,n);
	C.Name = "(" + A.Name + "*" + B.Name + ")";
	for(i=0; i<m; i++) for(j=0; j<n; j++)
	{
		C(i,j) = Zero;
		for(k=0; k<r; k++) C(i,j) += A(i,k) * B(k,j);
	}
	return C;
}
*/


/*

AMUB performs the matrix product C = A * B.

Discussion:
	
	The column dimension of B is not needed.
	
Modified:

	08 January 2004

Author:

	Youcef Saad
	
Parameters:

	Input, integer NROW, the row dimension of the matrix.
	
	Input, integer NCOL, the column dimension of the matrix.
	
	Input, integer JOB, job indicator.  When JOB = 0, only the structure
	is computed, that is, the arrays JC and IC, but the real values
	are ignored.
	
	Input, real A(*), integer JA(*), IA(NROW+1), the matrix in CSR
	Compressed Sparse Row format.
	
	Input, b, jb, ib, matrix B in compressed sparse row format.
	
	Input, integer NZMAX, the length of the arrays c and jc.
	The routine will stop if the result matrix C  has a number
	of elements that exceeds exceeds NZMAX.
	
	on return:
	
	c,
	jc,
	ic    = resulting matrix C in compressed sparse row sparse format.
	
	ierr      = integer. serving as error message.
	          ierr = 0 means normal return,
	          ierr > 0 means that amub stopped while computing the
	          i-th row  of C with i = ierr, because the number
	          of elements in C exceeds nzmax.
	          
	work arrays:
	
	iw      = integer work array of length equal to the number of
	         columns in A.
	
Adapted from subroutine amub ( nrow, ncol, job, a, ja, ia, b, jb, ib, c, jc, ic, nzmax, &
  iw, ierr )

*/

#ifdef _GenMat_SparseMatrix
	template <class T>
	SparseMatrix<T> operator * (const SparseMatrix<T> &A, const SparseMatrix<T> &B)
	{
		int nrow = A.Rows();
		int ncol = B.Cols();
		if(A.Cols() != B.Rows())
		{
			Print("Error in ProdABOp");
			Print("Incompadible Dimensions");
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}
		
		Print("CheckFormat(A)",CheckFormat(A));
		Print("CheckFormat(B)",CheckFormat(B));
		PauseExecution();
		
		int len = 0;
		Vector<int> r(nrow+1);
		r.Name = "***r";
		r[0] = 0;
		Stack<int> c;
		c.Name = "***c";
		Stack<double> v;
		v.Name = "***v";
		
		/* Initialize IW. */
		Vector<int> iw(ncol,-1);
			
		for(int ii=0; ii<nrow; ii++)
		{
			/* Row I. */
			for(int ka=A.r[ii]; ka<A.r[ii+1]; ka++)
			{
				double scal = A.v[ka];
				int jj = A.c[ka];
				for(int kb=B.r[jj]; kb<B.r[jj+1]; kb++)
				{
					int jcol = B.c[kb];
					int jpos = iw[jcol];
					if(jpos == -1)
					{
						c.Push(jcol);
						iw[jcol] = len;
						v.Push(scal * B.v[kb]);
						len++;
					}
					else
					{
						v[jpos] += scal * B.v[kb];
					}
				}
			}		
			for(int k=r[ii]; k<len; k++) iw[c[k]] = -1;		
			r[ii+1] = len;
		}
		
		SparseMatrix<double> C(nrow,ncol,r,ToVector<int>(c),ToVector<double>(v));
		return C;
	}
#endif

#endif
