
#ifndef _GenMatUtil_ProbABt
#define _GenMatUtil_ProbABt



template <class T>
void ProdABt(Matrix<T> &C, const SymmetricMatrix<T> &A, const Matrix<T> &B)
{
	int m = A.Dim();
	int n = B.Rows();
	if(C.Rows() != m || C.Cols() != n || B.Cols() != m)
	{
		Print("Error in ProdABt(Matrix,SymmetricMatrix,Matrix)");
		Print("C = A * B^T");
		Print("Incompadible Dimensions");
		Print("C.Name",C.Name);
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("C.Size()",C.Size());
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	for(int i=0; i<m; i++) for(int j=0; j<n; j++)
	{
		C(i,j) = Zero;
		for(int k=0; k<m; k++) C(i,j) += A(i,k) * B(j,k);
	}
}

template <class T>
void ProdABt(UpperMatrix<T> &C, const DiagonalMatrix<T> &A, const LowerMatrix<T> &B)
{
	int n = A.Dim();
	
	if(B.Dim() != n || C.Dim() != n)
	{
		Print("Error in ProdABt(LowerMatrix,LowerMatrix,DiagonalMatrix)");
		Print("C = A * B^T");
		Print("Incompadible dimensions");
		Print("C.Name",C.Name);
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("C.Size()",C.Size());
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	for(int i=0; i<n; i++) for(int j=i; j<n; j++) C(i,j) = A(i) * B(j,i);
}

#ifdef _GenMat_SparseMatrix
	template <class T>
	void ProdABt(Matrix<T> &C, const SymmetricMatrix<T> &A, const SparseMatrix<T> &B)
	{
		int m = A.Dim();
		int n = B.Rows();
		if(C.Rows() != m || C.Cols() != n || B.Cols() != m)
		{
			Print("Error in ProdABt(Matrix,SymmetricMatrix,SparseMatrix)");
			Print("C = A * B^T");
			Print("Incompadible Dimensions");
			Print("C.Name",C.Name);
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("C.Size()",C.Size());
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}
		
		for(int i=0; i<m; i++) for(int j=0; j<n; j++)
		{
			C(i,j) = Zero;
			int Low  = B.r[j];
			int High = B.r[j+1];
			//for(int k=0; k<m; k++) C(i,j) += A(i,k) * B(j,k);
			for(int k=Low; k<High; k++) C(i,j) += A(i,B.c[k]) * B.v[k];
		}
	}
#endif

#endif
