
#ifndef _GenMatUtil_ProdAX
#define _GenMatUtil_ProdAX



/* Pre-multiply a Vector by a Matrix (y = A * x) */
template <class T>
void ProdAX(Vector<T> &y, const Matrix<T> &A, const Vector<T> &x)
{
	int m = A.Rows();
	int n = A.Cols();
	if(x.Size() != n || y.Size() != m)
	{
		Print("Error in ProdAX(Vector,Matrix,Vector)");
		Print("y = A * x");
		Print("Incompatible Dimensions");
		Print("y.Name",y.Name);
		Print("A.Name",A.Name);
		Print("x.Name",x.Name);
		Print("y.Size()",y.Size());
		Print("A.Size()",A.Size());
		Print("x.Size()",x.Size());
		HaltExecution();
	}
	BlasGemv(0,m,n,GetOne<T>(),A.Begin(),n,x.Begin(),1,GetZero<T>(),y.Begin(),1);
}

/*

Pre-multiply a Vector by a DiagonalMatrix

	y = A * x

*/

#ifdef _GenMat_DiagonalMatrix
	template <class T>
	void ProdAX(Vector<T> &y, const DiagonalMatrix<T> &A, const Vector<T> &x)
	{
		int n = A.Dim();
		
		if(x.Size() != n && y.Size() != n) 
		{
			Print("Error in ProdAX(Vector,DiagonalMatrix,Vector)");
			Print("y = A * x");
			Print("Incompatible Dimensions");
			Print("y.Name",y.Name);
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("y.Size()",y.Size());
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}
		
		for(int i=0; i<n; i++) y[i] = A(i) * x[i];
	}
#endif

/*

Pre-multiply a Vector by a SymmetricMatrix

	y = A * x

*/

template <class T>
void ProdAX(Vector<T> &y, const SymmetricMatrix<T> &A, const Vector<T> &x)
{
	int n = A.Dim();
	if(y.Size() != n || x.Size() != n)
	{
		Print("Error in ProdAX(Vector,SymmetricMatrix,Vector)");
		Print("y = A * x");
		Print("Incompatible Dimensions");
		Print("y.Name",y.Name);
		Print("A.Name",A.Name);
		Print("x.Name",x.Name);
		Print("y.Size()",y.Size());
		Print("A.Size()",A.Size());
		Print("x.Size()",x.Size());
		HaltExecution();
	}	
	
	for(int i=0; i<n; i++)
	{
		y[i] = Zero;
		for(int j=0; j<n; j++) y[i] += A(i,j) * x[j];
	}
}

/*

Pre-multiply a Vector by a LowerMatrix

	y = A * x

*/

template <class T>
void ProdAX(Vector<T> &y, const LowerMatrix<T> &A, const Vector<T> &x)
{
	int n = A.Dim();
	
	if(y.Size() != n || x.Size() != n)
	{
		Print("Error in ProdAX(Vector,LowerMatrix,Vector)");
		Print("y = A * x");
		Print("Incompatible Dimensions");
		Print("y.Name",y.Name);
		Print("A.Name",A.Name);
		Print("x.Name",x.Name);
		Print("y.Size()",y.Size());
		Print("A.Size()",A.Size());
		Print("x.Size()",x.Size());
		HaltExecution();
	}	
	
	for(int i=0; i<n; i++)
	{
		y[i] = Zero;
		for(int j=0; j<=i; j++) y[i] += A(i,j) * x[j];
	}
}

/*

Pre-multiply a Vector by a UpperMatrix

	y = A * x

*/

template <class T>
void ProdAX(Vector<T> &y, const UpperMatrix<T> &A, const Vector<T> &x)
{
	int n = A.Dim();
	
	if(y.Size() != n || x.Size() != m)
	{
		Print("Error in ProdAX(Vector,UpperMatrix,Vector)");
		Print("y = A * x");
		Print("Incompatible Dimensions");
		Print("y.Name",y.Name);
		Print("A.Name",A.Name);
		Print("x.Name",x.Name);
		Print("y.Size()",y.Size());
		Print("A.Size()",A.Size());
		Print("x.Size()",x.Size());
		HaltExecution();
	}	
	
	for(int i=0; i<n; i++)
	{
		y[i] = Zero;
		for(int j=i; j<n; j++) y[i] += A(i,j) * x[j];
	}
}

#ifdef _GenMat_SparseMatrix
	template <class T>
	void ProdAX(Vector<T> &y, const SparseMatrix<T> &A, const Vector<T> &x)
	{
		int m = A.Rows();
		int n = A.Cols();
		if(y.Size() != m || x.Size() != n)
		{
			Print("Error in ProdAX(Vector,SparseMatrix,Vector)");
			Print("y = A * x");
			Print("Incompatible Dimensions");
			Print("y.Name",y.Name);
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("y.Size()",y.Size());
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}	
		
		for(int i=0; i<m; i++)
		{
			y[i] = Zero;
			for(int j=A.r[i]; j<A.r[i+1]; j++) y[i] += A.v[j] * x[A.c[j]];
		}
	}
#endif

#ifdef _GenMat_SparseUpdatedMatrix
	template <class T>
	void ProdAX(Vector<T> &y, const SparseUpdatedMatrix<T> &A, const Vector<T> &x)
	{	
		/* Check for Incompadible Dimensions */
		int m = A.Rows();
		int n = A.Cols();
		int k = A.Updates();
		if(y.Size() != m || x.Size() != n)
		{
			Print("Error in ProdAX(Vector,SparseUpdatedMatrix,Vector)");
			Print("y = A * x");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}
		
		/* Preform Calculations */
		ProdAX(y,A.S,x);
		Vector<T> VtX(k,Zero);
		//for(int i=0; i<n; i++) for(int j=0; j<k; j++) VtX(j) += A.V(i,j) * x(i);
		BlasGemv(True,n,k,One,A.V.Begin(),k,x.Begin(),1,Zero,VtX.Begin(),1);
		for(int j=0; j<k; j++)
		{
			if(A.Pos(j))
			{
				//for(int i=0; i<n; i++) y(i) += A.U(i,j) * VtX(j);
				BlasAxpy(n,VtX(j),&A.U(0,j),k,y.Begin(),1);
			}
			else
			{
				//for(int i=0; i<n; i++) y(i) -= A.U(i,j) * VtX(j);
				BlasAxpy(n,-VtX(j),&A.U(0,j),k,y.Begin(),1);
			}
		}
	}
#endif

#ifdef _GenMat_MultipliedSparseMatrix
	template <class T>
	void ProdAX(Vector<T> &y, const MultipliedSparseMatrix<T> &A, const Vector<T> &x)
	{
		/* Check for incompadible dimensions */
		if(y.Size() != A.Cols() || x.Size() != A.Rows())
		{
			Print("Error in ProdAX(Vector,MultipliedSparseMatrix,Vector)");
			Print("y = A * x");
			Print("Incompatible Dimensions");
			Print("y.Name",y.Name);
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("y.Size()",y.Size());
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}	
		
		/* Preform calculations */
		Vector<T> xOld = x.Copy();
		Vector<T> xNew;
		for(int i=A.r-1; i>=0; i--)
		{
			xNew = A.v[i] * xOld;
			xOld = xNew;
		}	
		Copy(y,xNew);
	}
#endif

#ifdef _GenMat_SparseMatrixRcv
	template <class T>
	void ProdAX(Vector<T> &y, const SparseMatrixRcv<T> &A, const Vector<T> &x)
	{
		int m = A.Rows();
		int n = A.Cols();
		int len = A.Length();
		
		if(y.Size() != n || x.Size() != m)
		{
			Print("Error in ProdAX(Vector,SparseMatrixRcv,Vector)");
			Print("y = A * x");
			Print("Incompatible Dimensions");
			Print("y.Name",y.Name);
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("y.Size()",y.Size());
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}	
		
		Set(y,Zero);
		for(int i=0; i<len; i++) y[A.Row(i)] += A.Val(i) * x[A.Col(i)];
	}
#endif

// Pre-multiply a Vector x by a SparseMatrix A (efficient routine)
/*
template <class T>
void Prod(Vector<T> &y, SparseMatrix<T> &A, Vector<T> &x)
{
	int i,j;
	int n = x.Size();
	for(i=0; i<A.NumRows; i++) y[i] = 0.0;
	for(i=0; i<A.NumRows; i++) for(j=A.RowVec[i]; j<A.RowVec[i+1]; j++) y[i] += A.ValVec[j] * x[A.ColVec[j]];
	return;
}
*/

/*

Pre-multiply a Vector by a LowerMatrix

	y = A * x

*/

#ifdef _GenMat_LowerMatrixUN
	template <class T>
	void ProdAX(Vector<T> &y, const LowerMatrixUN<T> &A, const Vector<T> &x)
	{
		int n = A.Dim();
		
		if(y.Size() != n || x.Size() != m)
		{
			Print("Error in ProdAX(Vector,LowerMatrixUN,Vector)");
			Print("y = A * x");
			Print("Incompatible Dimensions");
			Print("y.Name",y.Name);
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("y.Size()",y.Size());
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}	
		
		for(int i=0; i<n; i++)
		{
			y[i] = Zero;
			for(int j=0; j<=i; j++) y[i] += A(i,j) * x[j];
		}
	}
#endif

#endif
