
#ifndef _GenMatUtil_ProdAXOp
#define _GenMatUtil_ProdAXOp



template <class T>
Vector<T> operator * (const Matrix<T> &A, const Vector<T> &x)
{
	/* Check for Incompadible Dimensions */
	int m = A.Rows();
	int n = A.Cols();
	if(x.Size() != n) 
	{
		Print("Error in Vector = Matrix * Vector");
		Print("y = A * x");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("x.Name",x.Name);
		Print("A.Size()",A.Size());
		Print("x.Size()",x.Size());
		HaltExecution();
	}
	
	/* Preform Calculations */
	Vector<T> y(m);
	ProdAX(y,A,x);
	return y;
}

#ifdef _GenMat_DiagonalMatrix
	template <class T>
	Vector<T> operator * (const DiagonalMatrix<T> &A, const Vector<T> &x)
	{
		/* Check for Incompadible Dimensions */
		int n = A.Dim();
		if(x.Size() != n)
		{
			Print("Error in Vector = DiagonalMatrix * Vector");
			Print("y = A * x");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}
		
		/* Preform Calculations */
		Vector<T> y(n);
		y.Name = "(" + A.Name + "*" + x.Name + ")";
		for(int i=0; i<n; i++) y[i] = A(i) * x[i];
		return y;
	}
#endif

template <class T1, class T2>
Vector<T1> operator * (const SymmetricMatrix<T1> &A, const Vector<T2> &x)
{
	/* Check for Incompadible Dimensions */	
	int n = A.Dim();
	if(x.Size() != n) 
	{
		Print("Error in Vector = SymmetricMatrix * Vector");
		Print("y = A * x");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("x.Name",x.Name);
		Print("A.Size()",A.Size());
		Print("x.Size()",x.Size());
		HaltExecution();
	}
	
	/* Preform Calculations */
	Vector<T1> y(n);
	y.Name = "(" + A.Name + "*" + x.Name + ")";
	for(int i=0; i<n; i++)
	{
		y[i] = Zero;
		for(int j=0; j<n; j++) y[i] += A(i,j) * x[j];
	}
	return y;
}

template <class T>
Vector<T> operator * (const LowerMatrix<T> &A, const Vector<T> &x)
{
	/* Check for Incompadible Dimensions */
	int n = A.Dim();
	if(x.Size() != n) 
	{
		Print("Error in Vector = LowerMatrix * Vector");
		Print("y = A * x");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("x.Name",x.Name);
		Print("A.Size()",A.Size());
		Print("x.Size()",x.Size());
		HaltExecution();
	}
	
	/* Preform Calculations */
	Vector<T> y(n);
	for(int i=0; i<n; i++)
	{
		y[i] = 0.0;
		for(int j=0; j<=i; j++) y[i] += A(i,j) * x[j];
	}
	return y;
}

template <class T>
Vector<T> operator * (const UpperMatrix<T> &A, const Vector<T> &x)
{
	/* Check for Incompadible Dimensions */
	int n = A.Dim();
	if(x.Size() != n) 
	{
		Print("Error in Vector = UpperMatrix * Vector");
		Print("y = A * x");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("x.Name",x.Name);
		Print("A.Size()",A.Size());
		Print("x.Size()",x.Size());
		HaltExecution();
	}
	
	/* Preform Calculations */
	Vector<T> y(n);
	for(int i=0; i<n; i++)
	{
		y[i] = Zero;
		for(int j=i; j<n; j++) y[i] += A(i,j) * x[j];
	}
	return y;
}

#ifdef _GenMat_BlockMatrix
	template <class T>
	Vector<T> operator * (const BlockMatrix<T> &A, const Vector<T> &x)
	{
		/* Check for incompadible dimensions */
		int n = A.Dim();
		if(x.Size() != n) 
		{
			Print("Error in Vector = BlockMatrix * Vector");
			Print("y = A * x");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}
		
		/* Preform calculations */
		/* Note: This is not efficeint yet */
		Vector<T> y(n);
		for(i=0; i<n; i++)
		{
			y[i] = 0.0;
			for(j=0; j<n; j++) y[i] += A(i,j) * x[j];
		}
		return y;
	}
#endif

#ifdef _GenMat_SparseMatrix
	template <class T>
	Vector<T> operator * (const SparseMatrix<T> &A, const Vector<T> &x)
	{
		/* Check for Incompadible Dimensions */
		int m = A.Rows();
		int n = A.Cols();
		if(x.Size() != n) 
		{
			Print("Error in Vector = SparseMatrix * Vector");
			Print("y = A * x");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}
		
		/* Preform Calculations */
		Vector<T> y(m);
		for(int i=0; i<m; i++)
		{
			y[i] = 0.0;
			for(int j=A.RowLoc(i); j<A.RowLoc(i+1); j++) y[i] += A.Val(j) * x[A.Col(j)];
		}
		return y;
	}
#endif

#ifdef _GenMat_SparseUpdatedMatrix
	template <class T>
	Vector<T> operator * (const SparseUpdatedMatrix<T> &A, const Vector<T> &x)
	{	
		/* Check for Incompadible Dimensions */
		int m = A.Rows();
		int n = A.Cols();
		int k = A.Updates();
		if(x.Size() != n) 
		{
			Print("Error in Vector = SparseUpdatedMatrix * Vector");
			Print("y = A * x");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}
		
		/* Preform Calculations */
		Vector<T> y = A.S * x;
		Vector<T> VtX(k,Zero);
		//for(int i=0; i<n; i++) for(int j=0; j<k; j++) VtX(j) += A.V(i,j) * x(i);
		BlasGemv(True,n,k,One,A.V.Begin(),k,x.Begin(),1,Zero,VtX.Begin(),1);
		for(int j=0; j<k; j++)
		{
			if(A.Pos(j))
			{
				//for(int i=0; i<n; i++) y(i) += A.U(i,j) * VtX(j);
				BlasAxpy(n,VtX(j),&A.U(0,j),k,y.Begin(),1);
			}
			else
			{
				//for(int i=0; i<n; i++) y(i) -= A.U(i,j) * VtX(j);
				BlasAxpy(n,-VtX(j),&A.U(0,j),k,y.Begin(),1);
			}
		}
		return y;
	}
#endif

/*
template <class T>
Vector<T> operator * (const SparseSymmetricMatrix<T> &A, const Vector<T> &x)
{
	int i,j;
	int n = A.Dim();
	
	/* Check for incompadible dimensions */		
/*	if(x.Size() != n) 
	{
		Print("Error in Vector = SparseSymmetricMatrix * Vector");
		Print("y = A * x");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("x.Name",x.Name);
		Print("A.Size()",A.Size());
		Print("x.Size()",x.Size());
		HaltExecution();
	}
	
	/* Preform calculations */
/*	Vector<T> y(n,0.0);
	for(i=0; i<n; i++) for(j=A.RowLoc(i); j<A.RowLoc(i+1); j++)
	{
		y[i]        += A.Val(j) * x[A.Col(j)];
		y[A.Col(j)] += A.Val(j) * x[i];
	}
	return y;
}
*/

#ifdef _GenMat_MultipliedSparseMatrix
	template <class T>
	Vector<T> operator * (const MultipliedSparseMatrix<T> &A, const Vector<T> &x)
	{
		/* Check for incompadible dimensions */
		if(x.Size() != A.Cols())
		{
			Print("Error in Vector = MultipliedSparseMatrix * Vector");
			Print("y = A * x");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}	
		
		/* Preform Calculations */
		Vector<T> xOld = x.Copy();
		Vector<T> xNew;
		for(int i=A.r-1; i>=0; i--)
		{
			xNew = A.v[i] * xOld;
			xOld = xNew;
		}	
		return xNew;
	}
#endif

#ifdef _GenMat_LowerMatrixUN
	template <class T>
	Vector<T> operator * (const LowerMatrixUN<T> &A, const Vector<T> &x)
	{
		/* Check for Incompadible Dimensions */
		int n = A.Dim();
		if(x.Size() != n) 
		{
			Print("Error in Vector = LowerMatrixUN * Vector");
			Print("y = A * x");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}
		
		/* Preform Calculations */
		Vector<T> y(n);
		for(i=0; i<n; i++)
		{
			y[i] = 0.0;
			for(j=0; j<=i; j++) y[i] += A(i,j) * x[j];
		}
		return y;
	}
#endif

#ifdef _GenMat_UpperMatrixUN
	template <class T>
	Vector<T> operator * (const UpperMatrixUN<T> &A, const Vector<T> &x)
	{
		/* Check for Incompadible Dimensions */
		int n = A.Dim();
		if(x.Size() != n) 
		{
			Print("Error in Vector = UpperMatrixUN * Vector");
			Print("y = A * x");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}
		
		/* Preform Calculations */
		Vector<T> y(n);
		for(i=0; i<n; i++)
		{
			y[i] = 0.0;
			for(j=i; j<n; j++) y[i] += A(i,j) * x[j];
		}
		return y;
	}
}
#endif

#ifdef _GenMat_BandMatrix
	template <class T>
	Vector<T> operator * (const BandMatrix<T> &A, const Vector<T> &x)
	{
		
		/* Check for Incompadible Dimensions */	
		int m = A.Rows();
		int n = A.Cols();
		int kl = A.LowerBands();
		int ku = A.UpperBands();
		/*
		if(x.Size() != n) 
		{
			Print("Error in Vector = GeneralMatrix * Vector");
			Print("y = A * x");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}
		*/
		
		/* Preform calculations */
		Vector<T> y(m);
		Set(y,Zero);
		
		/*
		for(i=0; i<m; i++)
		{
			y[i] = 0.0;
			for(j=0; j<n; j++) y[i] += A(i,j) * x[j];
		}
		*/
		
		BlasGbmv(false,m,n,kl,ku,1.0,A.Begin(),kl+ku+1,x.Begin(),1,0.0,y.Begin(),1);
			
		return y;
	}
#endif

template <class T>
Vector<T> operator * (const GeneralMatrix<T> &A, const Vector<T> &x)
{	
	/* Check for Incompadible Dimensions */
	int m = A.Rows();
	int n = A.Cols();
	if(x.Size() != n) 
	{
		Print("Error in Vector = GeneralMatrix * Vector");
		Print("y = A * x");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("x.Name",x.Name);
		Print("A.Size()",A.Size());
		Print("x.Size()",x.Size());
		HaltExecution();
	}
	
	/* Preform Calculations */
	Vector<T> y(m);
	for(int i=0; i<m; i++)
	{
		y[i] = Zero;
		for(int j=0; j<n; j++) y[i] += A(i,j) * x[j];
	}
	return y;
}

#endif
