
#ifndef _GenMatUtil_ProdCAOp
#define _GenMatUtil_ProdCAOp

/* Pre-Multiply a matrix by a constant, B = c * A */
template <class T>
Matrix<T> operator * (const T &c, const Matrix<T> &A)
{
	int m = A.Rows();
	int n = A.Cols();
	Matrix<double> B(m,n);
	for(int i=0; i<m; i++) for(int j=0; j<n; j++) B(i,j) = c * A(i,j);
	return B;
}	

/* Pre-Multiply a LowerMatrix by a constant, B = c * A */
template <class T>
SymmetricMatrix<T> operator * (const T &c, const SymmetricMatrix<T> &A)
{
	int i,j;
	int n = A.Dim();
	SymmetricMatrix<T> B(n);
	for(i=0; i<n; i++) for(j=0; j<=i; j++) B(i,j) = c * A(i,j);
	return B;
}

/* Pre-Multiply a LowerMatrix by a constant, B = c * A */
template <class T>
LowerMatrix<T> operator * (const T &c, const LowerMatrix<T> &A)
{
	int i,j;
	int n = A.Dim();
	LowerMatrix<T> B(n);
	for(i=0; i<n; i++) for(j=0; j<=i; j++) B(i,j) = c * A(i,j);
	return B;
}

// Pre-Multiply a Matrix by a constant
template <class T>
UpperMatrix<T> operator * (const T &b, const UpperMatrix<T> &C)
{
	int i,j;
	int n = C.Dim();
	UpperMatrix<T> A(n);
	for(i=0; i<n; i++) for(j=i; j<n; j++) A(i,j) = b * C(i,j);
	return A;
}

// Pre-Multiply a Matrix by a constant
#ifdef _GenMat_DiagonalMatrix
	template <class T>
	DiagonalMatrix<T> operator * (const T &b, const DiagonalMatrix<T> &C)
	{
		int n = C.Dim();
		DiagonalMatrix<T> A(n);
		for(int i=0; i<n; i++) A(i) = b * C(i);
		return A;
	}
#endif

#ifdef _GenMat_IdentityMatrix
	template <class T>
	DiagonalMatrix<T>  operator * (const T &c, const IdentityMatrix<T>  &A)
	{
		int n = A.Dim();
		DiagonalMatrix<T> B(n);
		for(int i=0; i<n; i++) B(i) = c;
		return B;
	}
#endif
	
#endif