
#ifndef _ProdXtAX
#define _ProdXtAX



	/* Function Prototypes */
	template <class T> T ProdXtAX(const Vector<T> &x, const SymmetricMatrix<T> &A);
	

/*

Compute the product,

c = x^T * A * x 

  =  x1^T * A1 * x1 + x2^T * A2 * x2 + ... + xm^T * Am * xm

where,

A is a BlockMatrix, x is a Vector

*/

#ifdef _GenMat_BlasMatrix
	template <class T>
	void ProdXtAX(T &y, const Vector<T> &x, const BlockMatrix<T> &A)
	{
		int i,j,k,n;
		int m = A.Blocks();
		int Sm = 0;
		y = 0;
		for(i=0; i<m; i++)
		{
			// For each block
			n = A.Block(i);
			for(j=0; j<n; j++) for(k=0; k<n; k++) y += A(i,j,k) * x[j+Sm] * x[k+Sm];
			Sm += n;
		}
		return;
	}
#endif

template <class T>
void ProdXtAX(T &c, const Vector<T> &x, const Matrix<T> &A)
{
	int i,j;
	int n = x.Size();
	
	if(A.Rows() != n || A.Cols() != n)
	{
		Print("Error in ProdAtAX(Real,Vector,Matrix)");
		Print("Incompadible Dimensions");
		HaltExecution();
	}
	
	c = 0;
	for(i=0; i<n; i++) for(j=0; j<n; j++) c += A(i,j) * x[i] * x[j];
}

/*
template <class T>
void ProdXtAX(T &c, const Vector<T> &x, const SymmetricMatrix<T> &A)
{
	int i,j;
	int n = x.Size();
	
	if(A.Dim() != n)
	{
		Print("Error in ProdAtAX(Real,Vector,SymmetricMatrix)");
		Print("Incompadible Dimensions");
		HaltExecution();
	}
	
	c = 0;
	for(i=0; i<n; i++) for(j=0; j<=i; j++) c += A(i,j) * x[i] * x[j];
}
*/

template <class T>
T ProdXtAX(const Vector<T> &x, const SymmetricMatrix<T> &A)
{
	int n = x.Size();
	
	if(A.Dim() != n)
	{
		Print("Error in Real = ProdXtAX(Vector,SymmetricMatrix)");
		Print("Incompadible Dimensions");
		Print("x.Name",x.Name);
		Print("A.Name",A.Name);
		Print("x.Size()",x.Size());
		Print("A.Size()",A.Size());
		HaltExecution();
	}
	
	T c = 0;
	for(int i=0; i<n; i++) for(int j=0; j<n; j++) c += A(i,j) * x[i] * x[j];
	return c;
}

#endif
