
#ifndef _Random
#define _Random


long IA = 16807;
long IM = 2147483647;
double AM = 1.0/IM;
long IQ = 127773;
long IR = 2836;
long MASK = 123459876;
const long NTAB = 32;
double NDIV = 1 + (IM - 1) / NTAB;
double EPS = 1.2e-7;
double RNMX = 1.0 - EPS;

long IM1 = 2147483563;
long IM2 = 2147483399;
double AM1 = 1.0 / IM1;
long IMM1 = IM1 - 1;
long IA1 = 40014;
long IA2 = 40692;
long IQ1 = 53668;
long IQ2 = 52774;
long IR1 = 12211;
long IR2 = 3791;
double NDIV1 = 1 + IMM1 / NTAB;

long MBIG = 1000000000;
long MSEED = 161803398;
long MZ = 0;
double FAC = 1.0 / MBIG;

void psdes(unsigned long *lword, unsigned long *irword);
double Random0(long &RandomSeed);
double Random1(long &RandomSeed);
double Random2(long &RandomSeed);
double Random3(long &RandomSeed);
double Random4(long &RandomSeed);
double RandomNormal0(long &RandomSeed);
double RandomNormal1(long &RandomSeed);
double RandomNormal2(long &RandomSeed);
double RandomNormal3(long &RandomSeed);
double RandomNormal4(long &RandomSeed);
double RandomNormal0(long &RandomSeed, double mu, double sigma);
double RandomNormal1(long &RandomSeed, double mu, double sigma);
double RandomNormal2(double Mu, double Sigma, long &RandomSeed);
double RandomNormal3(long &RandomSeed, double mu, double sigma);
double RandomNormal4(long &RandomSeed, double mu, double sigma);

int RandomInteger2(int low, int high, long &RandomSeed);
int RandomInteger3(int low, int high, long &RandomSeed);

double RandomExtremeValue2(long &RandomSeed);

double Random0(long &idum)
{
	long k;
	double ans;
	idum ^= MASK;
	k = idum / IQ;
	idum = IA * (idum - k * IQ) - IR * k;
	if(idum < 0) idum += IM;
	ans = AM * (idum);
	idum ^= MASK;
	return ans;
}

double Random1(long &idum)
{
	int j;
	long k;
	static long iy = 0;
	static long iv[NTAB];
	double temp;

	if(idum <= 0 || !iy)
	{
		if(-idum < 1) idum = 1; else idum = -idum;
		for(j=NTAB+7; j>=0; j--)
		{
			k = idum / IQ;
			idum = IA * (idum - k * IQ) - IR * k;
			if(idum < 0) idum += IM;
			if(j < NTAB) iv[j] = idum;
		}
		iy = iv[0];
	}
	k = idum / IQ;
	idum = IA * (idum - k * IQ) - IR * k;
	if(idum < 0) idum += IM;
	j = iy / NDIV;
	iy = iv[j];
	iv[j] = idum;
	if((temp = AM * iy) > RNMX) return RNMX; else return temp;
}

double Random2(long &idum)
{
	int j;
	long k;
	static long idum2 = 123456789;
	static long iy = 0;
	static long iv[NTAB];
	double temp;

	if(idum <= 0)
	{
		if(-idum < 1) idum = 1; else idum = -idum;
		idum2 = idum;
		for(j=NTAB+7; j>=0; j--)
		{
			k = idum / IQ1;
			idum = IA1 * (idum - k * IQ1) - k * IR1;
			if(idum < 0) idum += IM1;
			if(j < NTAB) iv[j] = idum;
		}
		iy = iv[0];
	}
	k = idum / IQ1;
	idum = IA1 * (idum - k * IQ1) - k * IR1;
	if(idum < 0) idum += IM1;
	k = idum2 / IQ2;
	idum2 = IA2 * (idum2 - k * IQ2) - k * IR2;
	if(idum2 < 0) idum2 += IM2;
	j = iy / NDIV;
	iy = iv[j] - idum2;
	iv[j] = idum;
	if(iy < 1) iy += IMM1;
	if((temp = AM1 * iy) > RNMX) return RNMX; else return temp;
}

double Random3(long &idum)
{
	static int inext,inextp;
	static long ma[56];
	static int iff=0;
	long mj,mk;
	int i,ii,k;

	if(idum < 0 || iff == 0)
	{
		iff = 1;
		mj = labs(MSEED - labs(idum));
		mj %= MBIG;
		ma[55] = mj;
		mk = 1;
		for(i=1; i<=54; i++)
		{
			ii = (21 * i) % 55;
			ma[ii] = mk;
			mk = mj - mk;
			if(mk < MZ) mk += MBIG;
			mj = ma[ii];
		}
		for (k=1; k<=4; k++) for(i=1; i<=55; i++)
		{
			ma[i] -= ma[1+(i+30)%55];
			if(ma[i] < MZ) ma[i] += MBIG;
		}
		inext = 0;
		inextp = 31;
		idum = 1;
	}
	if(++inext == 56) inext = 1;
	if(++inextp == 56) inextp = 1;
	mj = ma[inext] - ma[inextp];
	if(mj < MZ) mj += MBIG;
	ma[inext] = mj;
	return mj * FAC;
}

#define NITER 4

void psdes(unsigned long *lword, unsigned long *irword)
{
	unsigned long i,ia,ib,iswap,itmph=0,itmpl=0;
	static unsigned long c1[NITER]={
		0xbaa96887L, 0x1e17d32cL, 0x03bcdc3cL, 0x0f33d1b2L};
	static unsigned long c2[NITER]={
		0x4b0f3b58L, 0xe874f0c3L, 0x6955c5a6L, 0x55a7ca46L};

	for (i=0;i < NITER;i++) {
		ia=(iswap=(*irword)) ^ c1[i];
		itmpl = ia & 0xffff;
		itmph = ia >> 16;
		ib=itmpl*itmpl+ ~(itmph*itmph);
		*irword=(*lword) ^ (((ia = (ib >> 16) |
			((ib & 0xffff) << 16)) ^ c2[i])+itmpl*itmph);
		*lword=iswap;
	}
}

double Random4(long &RandomSeed)
{
	void psdes(unsigned long *lword, unsigned long *irword);
	unsigned long irword,itemp,lword;
	static long idums = 0;
#if defined(vax) || defined(_vax_) || defined(__vax__) || defined(VAX)
	static unsigned long jflone = 0x00004080;
	static unsigned long jflmsk = 0xffff007f;
#else
	static unsigned long jflone = 0x3f800000;
	static unsigned long jflmsk = 0x007fffff;
#endif

	if(RandomSeed < 0)
	{
		idums = -(RandomSeed);
		RandomSeed = 1;
	}
	irword = (RandomSeed);
	lword = idums;
	psdes(&lword,&irword);
	itemp = jflone | (jflmsk & irword);
	++(RandomSeed);
	return (*(double *)&itemp) - 1.0;
}

double RandomNormal2(long &RandomSeed)
{
	static int iset = 0;
	static double gset;
	double fac,rsq,v1,v2;

	if(RandomSeed < 0) iset = 0;
	if(iset == 0)
	{
		do
		{
			v1 = 2.0 * Random2(RandomSeed) - 1.0;
			v2 = 2.0 * Random2(RandomSeed) - 1.0;
			rsq = v1 * v1 + v2 * v2;
		}
		while(rsq >= 1.0 || rsq == 0.0);
		fac=sqrt(-2.0 * log(rsq) / rsq);
		gset = v1 * fac;
		iset = 1;
		return v2 * fac;
	}
	else
	{
		iset = 0;
		return gset;
	}
}

double RandomNormal3(long &RandomSeed)
{
	static int iset=0;
	static double gset;
	double fac,rsq,v1,v2;

	if(RandomSeed < 0) iset = 0;
	if(iset == 0)
	{
		do
		{
			v1 = 2.0 * Random3(RandomSeed) - 1.0;
			v2 = 2.0 * Random3(RandomSeed) - 1.0;
			rsq = v1 * v1 + v2 * v2;
		}
		while(rsq >= 1.0 || rsq == 0.0);
		fac = sqrt(-2.0 * log(rsq) / rsq);
		gset = v1 * fac;
		iset = 1;
		return v2 * fac;
	}
	else
	{
		iset = 0;
		return gset;
	}
}

double RandomNormal4(long &RandomSeed)
{
	static int iset = 0;
	static double gset;
	double fac,rsq,v1,v2;

	if(RandomSeed < 0) iset = 0;
	if(iset == 0)
	{
		do
		{
			v1 = 2.0 * Random4(RandomSeed) - 1.0;
			v2 = 2.0 * Random4(RandomSeed) - 1.0;
			rsq = v1 * v1 + v2 * v2;
		}
		while(rsq >= 1.0 || rsq == 0.0);
		fac = sqrt(-2.0 * log(rsq) / rsq);
		gset = v1 * fac;
		iset = 1;
		return v2 * fac;
	}
	else
	{
		iset = 0;
		return gset;
	}
}

double RandomNormal2(double Mu, double Sigma, long &RandomSeed)
{
	return Sigma * RandomNormal2(RandomSeed) + Mu;
}

/*
double RandomExtremeValue2(long &RandomSeed)
{
	return log(-log(Random2(RandomSeed)));
}
*/

int RandomInteger2(int low, int high, long &RandomSeed)
{
	return (low + floor((high - low + 1) * Random2(RandomSeed)));
}

int RandomInteger3(int low, int high, long &RandomSeed)
{
	return (low + floor((high - low + 1) * Random3(RandomSeed)));
}



#endif
