
#ifndef _SortBy
#define _SortBy

/*

Utility for sorting a number of vectors by the first vector
using sort function from c++ STL

*/
	
/* Sort by first of 2 vectors */
template <class Type1, class Type2>
class ByObject2
{
	public:
	Type1 T1;
	Type2 T2;
	ByObject2() {}		// Constructor
	~ByObject2() {}		// Destructor
};
		
template <class Type1, class Type2>
bool operator<(const ByObject2<Type1,Type2> &ByObj1,
	const ByObject2<Type1,Type2> &ByObj2)
{
	return ByObj1.T1 < ByObj2.T1;
}
	
template <class Type1, class Type2>
void SortBy(GeneralVector<Type1> &v1, GeneralVector<Type2> &v2)
{
	int n = v1.Size();
	Vector< ByObject2<Type1,Type2> > vBy(n);
	
	for(int i=0; i<n; i++)
	{
		vBy[i].T1 = v1[i];
		vBy[i].T2 = v2[i];
	}
	
	#ifndef _StableSort
		sort(vBy.Begin(),vBy.End());
	#else
		stable_sort(vBy.Begin(),vBy.End());
	#endif
	
	for(int i=0; i<n; i++)
	{
		v1[i] = vBy[i].T1;
		v2[i] = vBy[i].T2;
	}
}

/* Sort by first of 2 vectors */
template <class Type1, class Type2>
class ByObject2Reverse
{
	public:
	Type1 T1;
	Type2 T2;
	ByObject2Reverse() {}		// Constructor
	~ByObject2Reverse() {}		// Destructor
};

template <class Type1, class Type2>
bool operator<(const ByObject2Reverse<Type1,Type2> &ByObj1,
	const ByObject2Reverse<Type1,Type2> &ByObj2)
{
	return ByObj1.T1 > ByObj2.T1;
}
	

template <class Type1, class Type2>
void ReverseSortBy(GeneralVector<Type1> &v1, GeneralVector<Type2> &v2)
{
	int n = v1.Size();
	Vector< ByObject2Reverse<Type1,Type2> > vBy(n);
	
	for(int i=0; i<n; i++)
	{
		vBy[i].T1 = v1[i];
		vBy[i].T2 = v2[i];
	}
	
	#ifndef _StableSort
		sort(vBy.Begin(),vBy.End());
	#else
		stable_sort(vBy.Begin(),vBy.End());
	#endif
	
	for(int i=0; i<n; i++)
	{
		v1[i] = vBy[i].T1;
		v2[i] = vBy[i].T2;
	}
}

template <class Type1, class Type2>
void SortBy(GeneralVector<Type1> &v1, Array3D<Type2> &v2)
{
	int d1 = v1.Size();
	int d2 = v2.Dim2();
	int d3 = v2.Dim3();
	Vector< ByObject2<Type1, Matrix<Type2> > > vBy(d1);
	
	for(int i=0; i<d1; i++)
	{
		vBy[i].T1 = v1[i];
		vBy[i].T2 = Matrix<double>(d2,d3);
		for(int i2=0; i2<d2; i2++) for(int i3=0; i3<d3; i3++) vBy[i].T2(i2,i3) = v2(i,i2,i3);
	}
	
	#ifndef _StableSort
		sort(vBy.Begin(),vBy.End());
	#else
		stable_sort(vBy.Begin(),vBy.End());
	#endif
	
	for(int i=0; i<d1; i++)
	{
		v1[i] = vBy[i].T1;
		for(int i2=0; i2<d2; i2++) for(int i3=0; i3<d3; i3++) v2(i,i2,i3) = vBy[i].T2(i2,i3);
	}
}
	
/* Sort by first of 3 vectors */
template <class Type1, class Type2, class Type3> class ByObject3
{
	public:
	Type1 T1;
	Type2 T2;
	Type3 T3;
	ByObject3() {}		// Constructor
	~ByObject3() {}		// Destructor
};
		
template <class Type1, class Type2, class Type3>
bool operator<(const ByObject3<Type1,Type2,Type3> &ByObj1, 
	const ByObject3<Type1,Type2,Type3> &ByObj2)
{
	return ByObj1.T1 < ByObj2.T1;
}
		
template <class Type1, class Type2, class Type3>
void SortBy(GeneralVector<Type1> &v1, GeneralVector<Type2> &v2, GeneralVector<Type3> &v3)
{
	int n = v1.Size();
	Vector< ByObject3<Type1,Type2,Type3> > vBy(n);
	
	for(int i=0; i<n; i++)
	{
		vBy[i].T1 = v1[i];
		vBy[i].T2 = v2[i];
		vBy[i].T3 = v3[i];
	}
	
	#ifndef _StableSort
		sort(vBy.Begin(),vBy.End());
	#else
		stable_sort(vBy.Begin(),vBy.End());
	#endif
	
	for(int i=0; i<n; i++)
	{
		v1[i] = vBy[i].T1;
		v2[i] = vBy[i].T2;
		v3[i] = vBy[i].T3;
	}
}
		
/* Sort by first of 4 vectors */
template <class Type1, class Type2, class Type3, class Type4>
class ByObject4
{
	public:
	Type1 T1;
	Type2 T2;
	Type3 T3;
	Type4 T4;
	ByObject4() {}		// Constructor
	~ByObject4() {}		// Destructor
};
		
template <class Type1, class Type2, class Type3, class Type4>
	bool operator<(const ByObject4<Type1,Type2,Type3,Type4> &ByObj1,
	const ByObject4<Type1,Type2,Type3,Type4> &ByObj2)
{
	return ByObj1.T1 < ByObj2.T1;
}

template <class Type1, class Type2, class Type3, class Type4>
void SortBy(GeneralVector<Type1> &v1, GeneralVector<Type2> &v2, GeneralVector<Type3> &v3, GeneralVector<Type4> &v4)
{
	int n = v1.Size();
	Vector< ByObject4<Type1,Type2,Type3,Type4> > vBy(n);
	
	for(int i=0; i<n; i++)
	{
		vBy[i].T1 = v1[i];
		vBy[i].T2 = v2[i];
		vBy[i].T3 = v3[i];
		vBy[i].T4 = v4[i];
	}
	
	#ifndef _StableSort
		sort(vBy.Begin(),vBy.End());
	#else
		stable_sort(vBy.Begin(),vBy.End());
	#endif
	
	for(int i=0; i<n; i++)
	{
		v1[i] = vBy[i].T1;
		v2[i] = vBy[i].T2;
		v3[i] = vBy[i].T3;
		v4[i] = vBy[i].T4;
	}
}
	
/* Sort by first of 5 vectors */
template <class Type1, class Type2, class Type3, class Type4, class Type5>
class ByObject5
{
	public:
	Type1 T1;
	Type2 T2;
	Type3 T3;
	Type4 T4;
	Type5 T5;
	ByObject5() {}		// Constructor
	~ByObject5() {}		// Destructor
};
		
template <class Type1, class Type2, class Type3, class Type4, class Type5>
bool operator<(const ByObject5<Type1,Type2,Type3,Type4,Type5> &ByObj1,
	const ByObject5<Type1,Type2,Type3,Type4,Type5> &ByObj2)
{
	return ByObj1.T1 < ByObj2.T1;
}

template <class Type1, class Type2, class Type3, class Type4, class Type5>
void SortBy(GeneralVector<Type1> &v1, GeneralVector<Type2> &v2, GeneralVector<Type3> &v3, GeneralVector<Type4> &v4,
	GeneralVector<Type5> &v5)
{
	int n = v1.Size();
	Vector< ByObject5<Type1,Type2,Type3,Type4,Type5> > vBy(n);
	
	for(int i=0; i<n; i++)
	{
		vBy[i].T1 = v1[i];
		vBy[i].T2 = v2[i];
		vBy[i].T3 = v3[i];
		vBy[i].T4 = v4[i];
		vBy[i].T5 = v5[i];
	}
	
	#ifndef _StableSort
		sort(vBy.Begin(),vBy.End());
	#else
		stable_sort(vBy.Begin(),vBy.End());
	#endif
	
	for(int i=0; i<n; i++)
	{
		v1[i] = vBy[i].T1;
		v2[i] = vBy[i].T2;
		v3[i] = vBy[i].T3;
		v4[i] = vBy[i].T4;
		v5[i] = vBy[i].T5;
	}
}
		
/* Sort by first of 6 vectors */
template <class Type1, class Type2, class Type3, class Type4, class Type5, class Type6>
class ByObject6
{
	public:
	Type1 T1;
	Type2 T2;
	Type3 T3;
	Type4 T4;
	Type5 T5;
	Type6 T6;
	ByObject6() {}		// Constructor
	~ByObject6() {}		// Destructor
};

template <class Type1, class Type2, class Type3, class Type4, class Type5, class Type6>
bool operator<(const ByObject6<Type1,Type2,Type3,Type4,Type5,Type6> &ByObj1,
	const ByObject6<Type1,Type2,Type3,Type4,Type5,Type6> &ByObj2)
{
	return ByObj1.T1 < ByObj2.T1;
}

template <class Type1, class Type2, class Type3, class Type4, class Type5, class Type6>
void SortBy(GeneralVector<Type1> &v1, GeneralVector<Type2> &v2, GeneralVector<Type3> &v3, GeneralVector<Type4> &v4,
	GeneralVector<Type5> &v5, GeneralVector<Type6> &v6)
{
	int n = v1.Size();
	Vector< ByObject6<Type1,Type2,Type3,Type4,Type5,Type6> > vBy(n);
	
	for(int i=0; i<n; i++)
	{
		vBy[i].T1 = v1[i];
		vBy[i].T2 = v2[i];
		vBy[i].T3 = v3[i];
		vBy[i].T4 = v4[i];
		vBy[i].T5 = v5[i];
		vBy[i].T6 = v6[i];
	}
	
	#ifndef _StableSort
		sort(vBy.Begin(),vBy.End());
	#else
		stable_sort(vBy.Begin(),vBy.End());
	#endif
	
	for(int i=0; i<n; i++)
	{
		v1[i] = vBy[i].T1;
		v2[i] = vBy[i].T2;
		v3[i] = vBy[i].T3;
		v4[i] = vBy[i].T4;
		v5[i] = vBy[i].T5;
		v6[i] = vBy[i].T6;
	}
}

/* Sort by first of 7 vectors */	
template <class T1, class T2, class T3, class T4, class T5, class T6, class T7>
class ByObject7
{
	public:
	T1 t1;
	T2 t2;
	T3 t3;
	T4 t4;
	T5 t5;
	T6 t6;
	T7 t7;
	ByObject7() {}		// Constructor
	~ByObject7() {}		// Destructor
};
		
template <class T1, class T2, class T3, class T4, class T5, class T6, class T7>
bool operator<(const ByObject7<T1,T2,T3,T4,T5,T6,T7> &ByObj1,
	const ByObject7<T1,T2,T3,T4,T5,T6,T7> &ByObj2)
{
	return ByObj1.t1 < ByObj2.t1;
}

template <class T1, class T2, class T3, class T4, class T5, class T6, class T7>
void SortBy(GeneralVector<T1> &v1, GeneralVector<T2> &v2, GeneralVector<T3> &v3, GeneralVector<T4> &v4,
	GeneralVector<T5> &v5, GeneralVector<T6> &v6, GeneralVector<T7> &v7)
{
	int n = v1.Size();
	Vector< ByObject7<T1,T2,T3,T4,T5,T6,T7> > vBy(n);
	
	for(int i=0; i<n; i++)
	{
		vBy[i].t1 = v1[i];
		vBy[i].t2 = v2[i];
		vBy[i].t3 = v3[i];
		vBy[i].t4 = v4[i];
		vBy[i].t5 = v5[i];
		vBy[i].t6 = v6[i];
		vBy[i].t7 = v7[i];
	}
	
	#ifndef _StableSort
		sort(vBy.Begin(),vBy.End());
	#else
		stable_sort(vBy.Begin(),vBy.End());
	#endif
	
	for(int i=0; i<n; i++)
	{
		v1[i] = vBy[i].t1;
		v2[i] = vBy[i].t2;
		v3[i] = vBy[i].t3;
		v4[i] = vBy[i].t4;
		v5[i] = vBy[i].t5;
		v6[i] = vBy[i].t6;
		v7[i] = vBy[i].t7;
	}
}

/* Sort by first of 8 vectors */
template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8>
class ByObject8
{
	public:
	T1 t1;
	T2 t2;
	T3 t3;
	T4 t4;
	T5 t5;
	T6 t6;
	T7 t7;
	T8 t8;
	ByObject8() {}		// Constructor
	~ByObject8() {}		// Destructor
};
		
template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8>
bool operator<(const ByObject8<T1,T2,T3,T4,T5,T6,T7,T8> &ByObj1,
	const ByObject8<T1,T2,T3,T4,T5,T6,T7,T8> &ByObj2)
{
	return ByObj1.t1 < ByObj2.t1;
}

template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8>
void SortBy(GeneralVector<T1> &v1, GeneralVector<T2> &v2, GeneralVector<T3> &v3, GeneralVector<T4> &v4,
	GeneralVector<T5> &v5, GeneralVector<T6> &v6, GeneralVector<T7> &v7, GeneralVector<T8> &v8)
{
	int n = v1.Size();
	Vector< ByObject8<T1,T2,T3,T4,T5,T6,T7,T8> > vBy(n);
	
	for(int i=0; i<n; i++)
	{
		vBy[i].t1 = v1[i];
		vBy[i].t2 = v2[i];
		vBy[i].t3 = v3[i];
		vBy[i].t4 = v4[i];
		vBy[i].t5 = v5[i];
		vBy[i].t6 = v6[i];
		vBy[i].t7 = v7[i];
		vBy[i].t8 = v8[i];
	}
	
	#ifndef _StableSort
		sort(vBy.Begin(),vBy.End());
	#else
		stable_sort(vBy.Begin(),vBy.End());
	#endif
	
	for(int i=0; i<n; i++)
	{
		v1[i] = vBy[i].t1;
		v2[i] = vBy[i].t2;
		v3[i] = vBy[i].t3;
		v4[i] = vBy[i].t4;
		v5[i] = vBy[i].t5;
		v6[i] = vBy[i].t6;
		v7[i] = vBy[i].t7;
		v8[i] = vBy[i].t8;
	}
}

/* Sort by first of 9 vectors */
template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9>
class ByObject9
{
	public:
	T1 t1;
	T2 t2;
	T3 t3;
	T4 t4;
	T5 t5;
	T6 t6;
	T7 t7;
	T8 t8;
	T9 t9;
	ByObject9() {}		// Constructor
	~ByObject9() {}		// Destructor
};
		
template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9>
bool operator<(const ByObject9<T1,T2,T3,T4,T5,T6,T7,T8,T9> &ByObj1,
	const ByObject9<T1,T2,T3,T4,T5,T6,T7,T8,T9> &ByObj2)
{
	return ByObj1.t1 < ByObj2.t1;
}

template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9>
void SortBy(GeneralVector<T1> &v1, GeneralVector<T2> &v2, GeneralVector<T3> &v3, GeneralVector<T4> &v4,
	GeneralVector<T5> &v5, GeneralVector<T6> &v6, GeneralVector<T7> &v7, GeneralVector<T8> &v8, GeneralVector<T9> &v9)
{
	int n = v1.Size();
	Vector< ByObject9<T1,T2,T3,T4,T5,T6,T7,T8,T9> > vBy(n);
	
	for(int i=0; i<n; i++)
	{
		vBy[i].t1 = v1[i];
		vBy[i].t2 = v2[i];
		vBy[i].t3 = v3[i];
		vBy[i].t4 = v4[i];
		vBy[i].t5 = v5[i];
		vBy[i].t6 = v6[i];
		vBy[i].t7 = v7[i];
		vBy[i].t8 = v8[i];
		vBy[i].t9 = v9[i];
	}
	
	#ifndef _StableSort
		sort(vBy.Begin(),vBy.End());
	#else
		stable_sort(vBy.Begin(),vBy.End());
	#endif
	
	for(int i=0; i<n; i++)
	{
		v1[i] = vBy[i].t1;
		v2[i] = vBy[i].t2;
		v3[i] = vBy[i].t3;
		v4[i] = vBy[i].t4;
		v5[i] = vBy[i].t5;
		v6[i] = vBy[i].t6;
		v7[i] = vBy[i].t7;
		v8[i] = vBy[i].t8;
		v9[i] = vBy[i].t9;
	}
}

/* Sort by first of 10 vectors */
template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10>
class ByObject10
{
	public:
	T1 t1;
	T2 t2;
	T3 t3;
	T4 t4;
	T5 t5;
	T6 t6;
	T7 t7;
	T8 t8;
	T9 t9;
	T10 t10;
	ByObject10() {}		// Constructor
	~ByObject10() {}		// Destructor
};
		
template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10>
bool operator<(const ByObject10<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10> &ByObj1,
	const ByObject10<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10> &ByObj2)
{
	return ByObj1.t1 < ByObj2.t1;
}

template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10>
void SortBy(GeneralVector<T1> &v1, GeneralVector<T2> &v2, GeneralVector<T3> &v3, GeneralVector<T4> &v4,
	GeneralVector<T5> &v5, GeneralVector<T6> &v6, GeneralVector<T7> &v7, GeneralVector<T8> &v8, GeneralVector<T9> &v9,
	GeneralVector<T10> &v10)
{
	int n = v1.Size();
	Vector< ByObject10<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10> > vBy(n);
	
	for(int i=0; i<n; i++)
	{
		vBy[i].t1 = v1[i];
		vBy[i].t2 = v2[i];
		vBy[i].t3 = v3[i];
		vBy[i].t4 = v4[i];
		vBy[i].t5 = v5[i];
		vBy[i].t6 = v6[i];
		vBy[i].t7 = v7[i];
		vBy[i].t8 = v8[i];
		vBy[i].t9 = v9[i];
		vBy[i].t10 = v10[i];
	}
	
	#ifndef _StableSort
		sort(vBy.Begin(),vBy.End());
	#else
		stable_sort(vBy.Begin(),vBy.End());
	#endif
	
	for(int i=0; i<n; i++)
	{
		v1[i] = vBy[i].t1;
		v2[i] = vBy[i].t2;
		v3[i] = vBy[i].t3;
		v4[i] = vBy[i].t4;
		v5[i] = vBy[i].t5;
		v6[i] = vBy[i].t6;
		v7[i] = vBy[i].t7;
		v8[i] = vBy[i].t8;
		v9[i] = vBy[i].t9;
		v10[i] = vBy[i].t10;
	}
}

/* Sort by first of 11 vectors */
template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10, class T11>
class ByObject11
{
	public:
	T1 t1;
	T2 t2;
	T3 t3;
	T4 t4;
	T5 t5;
	T6 t6;
	T7 t7;
	T8 t8;
	T9 t9;
	T10 t10;
	T11 t11;
	ByObject11() {}		// Constructor
	~ByObject11() {}		// Destructor
};
		
template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10, class T11>
bool operator<(const ByObject11<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11> &ByObj1,
	const ByObject11<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11> &ByObj2)
{
	return ByObj1.t1 < ByObj2.t1;
}

template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10, class T11>
void SortBy(GeneralVector<T1> &v1, GeneralVector<T2> &v2, GeneralVector<T3> &v3, GeneralVector<T4> &v4,
	GeneralVector<T5> &v5, GeneralVector<T6> &v6, GeneralVector<T7> &v7, GeneralVector<T8> &v8, GeneralVector<T9> &v9,
	GeneralVector<T10> &v10, GeneralVector<T11> &v11)
{
	int n = v1.Size();
	Vector< ByObject11<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11> > vBy(n);
	
	for(int i=0; i<n; i++)
	{
		vBy[i].t1 = v1[i];
		vBy[i].t2 = v2[i];
		vBy[i].t3 = v3[i];
		vBy[i].t4 = v4[i];
		vBy[i].t5 = v5[i];
		vBy[i].t6 = v6[i];
		vBy[i].t7 = v7[i];
		vBy[i].t8 = v8[i];
		vBy[i].t9 = v9[i];
		vBy[i].t10 = v10[i];
		vBy[i].t11 = v11[i];
	}
	
	#ifndef _StableSort
		sort(vBy.Begin(),vBy.End());
	#else
		stable_sort(vBy.Begin(),vBy.End());
	#endif
	
	for(int i=0; i<n; i++)
	{
		v1[i] = vBy[i].t1;
		v2[i] = vBy[i].t2;
		v3[i] = vBy[i].t3;
		v4[i] = vBy[i].t4;
		v5[i] = vBy[i].t5;
		v6[i] = vBy[i].t6;
		v7[i] = vBy[i].t7;
		v8[i] = vBy[i].t8;
		v9[i] = vBy[i].t9;
		v10[i] = vBy[i].t10;
		v11[i] = vBy[i].t11;
	}
}

/* Sort by first of 12 vectors */
template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10, class T11, class T12>
class ByObject12
{
	public:
	T1 t1;
	T2 t2;
	T3 t3;
	T4 t4;
	T5 t5;
	T6 t6;
	T7 t7;
	T8 t8;
	T9 t9;
	T10 t10;
	T11 t11;
	T12 t12;
	ByObject12() {}		// Constructor
	~ByObject12() {}		// Destructor
};
		
template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10, class T11, class T12>
bool operator<(const ByObject12<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12> &ByObj1,
	const ByObject12<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12> &ByObj2)
{
	return ByObj1.t1 < ByObj2.t1;
}

template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10, class T11, class T12>
void SortBy(GeneralVector<T1> &v1, GeneralVector<T2> &v2, GeneralVector<T3> &v3, GeneralVector<T4> &v4,
	GeneralVector<T5> &v5, GeneralVector<T6> &v6, GeneralVector<T7> &v7, GeneralVector<T8> &v8, GeneralVector<T9> &v9,
	GeneralVector<T10> &v10, GeneralVector<T11> &v11, GeneralVector<T12> &v12)
{
	int n = v1.Size();
	Vector< ByObject12<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12> > vBy(n);
	
	for(int i=0; i<n; i++)
	{
		vBy[i].t1 = v1[i];
		vBy[i].t2 = v2[i];
		vBy[i].t3 = v3[i];
		vBy[i].t4 = v4[i];
		vBy[i].t5 = v5[i];
		vBy[i].t6 = v6[i];
		vBy[i].t7 = v7[i];
		vBy[i].t8 = v8[i];
		vBy[i].t9 = v9[i];
		vBy[i].t10 = v10[i];
		vBy[i].t11 = v11[i];
		vBy[i].t12 = v12[i];
	}
	
	#ifndef _StableSort
		sort(vBy.Begin(),vBy.End());
	#else
		stable_sort(vBy.Begin(),vBy.End());
	#endif
	
	for(int i=0; i<n; i++)
	{
		v1[i] = vBy[i].t1;
		v2[i] = vBy[i].t2;
		v3[i] = vBy[i].t3;
		v4[i] = vBy[i].t4;
		v5[i] = vBy[i].t5;
		v6[i] = vBy[i].t6;
		v7[i] = vBy[i].t7;
		v8[i] = vBy[i].t8;
		v9[i] = vBy[i].t9;
		v10[i] = vBy[i].t10;
		v11[i] = vBy[i].t11;
		v12[i] = vBy[i].t12;
	}
}

/* Sort by first of 13 vectors */
template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10, class T11, class T12, class T13>
class ByObject13
{
	public:
	T1 t1;
	T2 t2;
	T3 t3;
	T4 t4;
	T5 t5;
	T6 t6;
	T7 t7;
	T8 t8;
	T9 t9;
	T10 t10;
	T11 t11;
	T12 t12;
	T13 t13;
	ByObject13() {}		// Constructor
	~ByObject13() {}		// Destructor
};
		
template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10, class T11, class T12, class T13>
bool operator<(const ByObject13<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13> &ByObj1,
	const ByObject13<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13> &ByObj2)
{
	return ByObj1.t1 < ByObj2.t1;
}

template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10, class T11, class T12, class T13>
void SortBy(GeneralVector<T1> &v1, GeneralVector<T2> &v2, GeneralVector<T3> &v3, GeneralVector<T4> &v4,
	GeneralVector<T5> &v5, GeneralVector<T6> &v6, GeneralVector<T7> &v7, GeneralVector<T8> &v8, GeneralVector<T9> &v9,
	GeneralVector<T10> &v10, GeneralVector<T11> &v11, GeneralVector<T12> &v12,
	GeneralVector<T13> &v13)
{
	int n = v1.Size();
	Vector< ByObject13<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13> > vBy(n);
	
	for(int i=0; i<n; i++)
	{
		vBy[i].t1 = v1[i];
		vBy[i].t2 = v2[i];
		vBy[i].t3 = v3[i];
		vBy[i].t4 = v4[i];
		vBy[i].t5 = v5[i];
		vBy[i].t6 = v6[i];
		vBy[i].t7 = v7[i];
		vBy[i].t8 = v8[i];
		vBy[i].t9 = v9[i];
		vBy[i].t10 = v10[i];
		vBy[i].t11 = v11[i];
		vBy[i].t12 = v12[i];
		vBy[i].t13 = v13[i];
	}
	
	#ifndef _StableSort
		sort(vBy.Begin(),vBy.End());
	#else
		stable_sort(vBy.Begin(),vBy.End());
	#endif
	
	for(int i=0; i<n; i++)
	{
		v1[i] = vBy[i].t1;
		v2[i] = vBy[i].t2;
		v3[i] = vBy[i].t3;
		v4[i] = vBy[i].t4;
		v5[i] = vBy[i].t5;
		v6[i] = vBy[i].t6;
		v7[i] = vBy[i].t7;
		v8[i] = vBy[i].t8;
		v9[i] = vBy[i].t9;
		v10[i] = vBy[i].t10;
		v11[i] = vBy[i].t11;
		v12[i] = vBy[i].t12;
		v13[i] = vBy[i].t13;
	}
}

/* Sort by first of 15 vectors */
template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10, class T11, class T12, class T13, class T14, class T15>
class ByObject15
{
	public:
	T1 t1;
	T2 t2;
	T3 t3;
	T4 t4;
	T5 t5;
	T6 t6;
	T7 t7;
	T8 t8;
	T9 t9;
	T10 t10;
	T11 t11;
	T12 t12;
	T13 t13;
	T14 t14;
	T15 t15;
	ByObject15() {}		// Constructor
	~ByObject15() {}		// Destructor
};
		
template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10, class T11, class T12, class T13, class T14, class T15>
bool operator<(const ByObject15<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15> &ByObj1,
	const ByObject15<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15> &ByObj2)
{
	return ByObj1.t1 < ByObj2.t1;
}

template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10, class T11, class T12, class T13, class T14, class T15>
void SortBy(GeneralVector<T1> &v1, GeneralVector<T2> &v2, GeneralVector<T3> &v3, GeneralVector<T4> &v4,
	GeneralVector<T5> &v5, GeneralVector<T6> &v6, GeneralVector<T7> &v7, GeneralVector<T8> &v8, GeneralVector<T9> &v9,
	GeneralVector<T10> &v10, GeneralVector<T11> &v11, GeneralVector<T12> &v12,
	GeneralVector<T13> &v13, GeneralVector<T14> &v14, GeneralVector<T15> &v15)
{
	int n = v1.Size();
	Vector< ByObject15<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15> > vBy(n);
	
	for(int i=0; i<n; i++)
	{
		vBy[i].t1 = v1[i];
		vBy[i].t2 = v2[i];
		vBy[i].t3 = v3[i];
		vBy[i].t4 = v4[i];
		vBy[i].t5 = v5[i];
		vBy[i].t6 = v6[i];
		vBy[i].t7 = v7[i];
		vBy[i].t8 = v8[i];
		vBy[i].t9 = v9[i];
		vBy[i].t10 = v10[i];
		vBy[i].t11 = v11[i];
		vBy[i].t12 = v12[i];
		vBy[i].t13 = v13[i];
		vBy[i].t14 = v14[i];
		vBy[i].t15 = v15[i];
	}
	
	#ifndef _StableSort
		sort(vBy.Begin(),vBy.End());
	#else
		stable_sort(vBy.Begin(),vBy.End());
	#endif
	
	for(int i=0; i<n; i++)
	{
		v1[i] = vBy[i].t1;
		v2[i] = vBy[i].t2;
		v3[i] = vBy[i].t3;
		v4[i] = vBy[i].t4;
		v5[i] = vBy[i].t5;
		v6[i] = vBy[i].t6;
		v7[i] = vBy[i].t7;
		v8[i] = vBy[i].t8;
		v9[i] = vBy[i].t9;
		v10[i] = vBy[i].t10;
		v11[i] = vBy[i].t11;
		v12[i] = vBy[i].t12;
		v13[i] = vBy[i].t13;
		v14[i] = vBy[i].t14;
		v15[i] = vBy[i].t15;
	}
}

/* Sort by first of 15 vectors */
/*
template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10, class T11, class T12, class T13, class T14, class T15, class T16, class T17, class T18>
class ByObject18
{
	public:
	T1 t1;
	T2 t2;
	T3 t3;
	T4 t4;
	T5 t5;
	T6 t6;
	T7 t7;
	T8 t8;
	T9 t9;
	T10 t10;
	T11 t11;
	T12 t12;
	T13 t13;
	T14 t14;
	T15 t15;
	T16 t16;
	T17 t17;
	T18 t18;
	ByObject18() {}		// Constructor
	~ByObject18() {}		// Destructor
};
		
template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10, class T11, class T12, class T13, class T14, class T15, class T16, class T17, class T18>
bool operator<(const ByObject18<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18> &ByObj1,
	const ByObject18<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18> &ByObj2)
{
	return ByObj1.t1 < ByObj2.t1;
}

template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10, class T11, class T12, class T13, class T14, class T15, class T16, class T17, class T18>
void SortBy(GeneralVector<T1> &v1, GeneralVector<T2> &v2, GeneralVector<T3> &v3, GeneralVector<T4> &v4,
	GeneralVector<T5> &v5, GeneralVector<T6> &v6, GeneralVector<T7> &v7, GeneralVector<T8> &v8, GeneralVector<T9> &v9,
	GeneralVector<T10> &v10, GeneralVector<T11> &v11, GeneralVector<T12> &v12,
	GeneralVector<T13> &v13, GeneralVector<T14> &v14, GeneralVector<T15> &v15,
	GeneralVector<T16> &v16, GeneralVector<T17> &v17, GeneralVector<T18> &v18)
{
	int n = v1.Size();
	Vector< ByObject18<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18> > vBy(n);
	
	for(int i=0; i<n; i++)
	{
		vBy[i].t1 = v1[i];
		vBy[i].t2 = v2[i];
		vBy[i].t3 = v3[i];
		vBy[i].t4 = v4[i];
		vBy[i].t5 = v5[i];
		vBy[i].t6 = v6[i];
		vBy[i].t7 = v7[i];
		vBy[i].t8 = v8[i];
		vBy[i].t9 = v9[i];
		vBy[i].t10 = v10[i];
		vBy[i].t11 = v11[i];
		vBy[i].t12 = v12[i];
		vBy[i].t13 = v13[i];
		vBy[i].t14 = v14[i];
		vBy[i].t15 = v15[i];
		vBy[i].t16 = v16[i];
		vBy[i].t17 = v17[i];
		vBy[i].t18 = v18[i];
	}
	
	#ifndef _StableSort
		sort(vBy.Begin(),vBy.End());
	#else
		stable_sort(vBy.Begin(),vBy.End());
	#endif
	
	for(int i=0; i<n; i++)
	{
		v1[i] = vBy[i].t1;
		v2[i] = vBy[i].t2;
		v3[i] = vBy[i].t3;
		v4[i] = vBy[i].t4;
		v5[i] = vBy[i].t5;
		v6[i] = vBy[i].t6;
		v7[i] = vBy[i].t7;
		v8[i] = vBy[i].t8;
		v9[i] = vBy[i].t9;
		v10[i] = vBy[i].t10;
		v11[i] = vBy[i].t11;
		v12[i] = vBy[i].t12;
		v13[i] = vBy[i].t13;
		v14[i] = vBy[i].t14;
		v15[i] = vBy[i].t15;
		v16[i] = vBy[i].t16;
		v17[i] = vBy[i].t17;
		v18[i] = vBy[i].t18;
	}
}
*/


#endif
