
#ifndef _GenMat_Stack
#define _GenMat_Stack

/*

Class for Stacks in c++
Adapted form LEDA

*/
	
template <class T>
class Stack : public GeneralVector<T>
{	
	//private:
	public:
	
		LongInt len;	// Size of underlying array
		T* First;		// Start of array
		T* Stop;		// One position behind last element
		T* Free;		// Pointer to first free element
		int *RefCount;	// Counter to number of references
			
	public:
		
	/*
	
	Constructor:
	Creates a bounded stack that can hold up to n elements
	Stack is initialized to be empty
		
	*/

	Stack()
	{		
		len = 1;
		RefCount = new int;
		*RefCount = 1;
		Name = "NoName";
			
		try
		{
			First = new T[len];
			Free = First;
			Stop = First + len;			
		}
		catch(bad_alloc ba)
		{
			Print("Error in Stack::Stack()");
			Print("Unable to allocate memory");
			Print("len",len);
			Print("Memory Needed in Megabytes",len*sizeof(T)/1e6);
			HaltExecution();
		}
	}
	
	/*

	Copy constructor. Stack data is NOT copied, but shared.
	Thus, in Stack B(A), subsequent changes to A will
	be reflected in B.  For an indepent copy of A, use
	Stack B(A.Copy()), or B = A.Copy(), instead.

	*/
	
	Stack(const Stack &A)
	{
		len = A.len;
		First = A.First;
		Free = A.Free;
		Stop = A.Stop;
		RefCount = A.RefCount;
		(*RefCount)++;
		Name = "Copy of " + A.Name;
	}

	Stack(LongInt len)
	{
		this->len = len;
		RefCount = new int;
		*RefCount = 1;
		Name = "NoName";
		if(len < 1)
		{
			Print("Error in Stack::Stack(int)");
			Print("Inialize size must be positive");
			Print("len",len);
			HaltExecution();
		}
			
		try
		{
			First = new T[len];
			Free = First;
			Stop = First + len;			
		}
		catch(bad_alloc ba)
		{
			Print("Error in Stack::Stack(int)");
			Print("Unable to allocate memnory");
			Print("len",len);
			Print("Memory Needed in Megabytes",len*sizeof(T)/1e6);
			HaltExecution();
		}
	}
			
	/* Destructor */
	~Stack()
	{		
		(*RefCount)--;
		if(*RefCount < 1)
		{
			if(First != 0) delete[] First;
			if(RefCount != 0) delete RefCount;
		}
	}
			
	/* Returns the top element of the stack */
	T Top() const
	{
		if(Free == First)
		{
			Print("Error in Stack::Top");
			Print("The Stack is empty");
			HaltExecution();
		}
		
		return *(Free - 1);
	}
 
	/* Deletes and returns the top element of the stack */
	T Pop()
	{
		if(Free == First)
		{
			Print("Error in Stack::Pop");
			Print("The Stack is Empty");
			Print("Name",Name);
			HaltExecution();
		}
		
		return *--Free;
	}
	
	/* Add a new top element to the stack */
	void SetSize(LongInt len)
	{
		LongInt len0 = Stop - First;
		if(len < len0)
		{
			Print("Error in Stack");
			Print("Not Bigger");
			HaltExecution();
		}
		else if(len < 1)
		{
			Print("Error in Stack::SetSize");
			HaltExecution();		
		}
	
		try
		{
			T* Temp = new T[len];
			BlasCopy(len0,First,1,Temp,1);
			delete[] First;
			First = Temp;
			Free = First + len0;
			Stop = First + len;
		}
		catch(bad_alloc ba)
		{
			Print("Error in Stack::SetSize");
			Print("Unable to Allocate Memory");
			Print("len",len);
			Print("len0",len0);
			HaltExecution("Stack::SetSize::Insufficient Memory");		
		}
	}
	
	/* Add a new top element to the stack */
	void Reset(LongInt len)
	{
		if(len < 0)
		{
			Print("Error in Stack::Reset");
			Print("Not Bigger");
			HaltExecution();
		}
	
		try
		{
			if(*RefCount < 1) if(First != 0) delete[] (First);
			First = new T[len];
			Free = First;
			Stop = First + len;
			this->len = len;
		}
		catch(bad_alloc ba)
		{
			Print("Error in Stack::Reset");
			Print("Unable to Allocate Memory");
			Print("len",len);
			HaltExecution("Stack::SetSize::Insufficient Memory");		
		}
	}

	/* Add a new top element to the stack */
	void Push(const T &x)
	{
		if(Free == Stop)
		{
			/* Need to allocate more memory */
			try
			{
				T* Temp = new T[Max(2*len,(LongInt)1)];
				BlasCopy(len,First,1,Temp,1);
				delete[] First;
				First = Temp;
				Free = First + len;
				Stop = First + 2 * len;
				len *= 2;
			}
			catch(bad_alloc ba)
			{
				try
				{
					T* Temp = new T[Max((LongInt)(1.1*len),(LongInt)1)];
					BlasCopy(len,First,1,Temp,1);
					delete[] First;
					First = Temp;
					Free = First + len;
					Stop = First + (LongInt)(1.1*len);
					len = 1.1*len;
				}
				catch(bad_alloc ba)
				{
					Print("Error in Stack::Push(T)");
					Print("Unable to allocate memory");
					Print("1.1 * len",1.1 *len);
					Print("Memory Needed in Megabytes",1.1*len*sizeof(T)/1e6);
					Print("Name",Name);
					HaltExecution();
				}
			}			
		}
		*Free++ = x;		
	}
	
	/* Add a new top element to the stack */
	void Push(const GeneralVector<T> &x)
	{
		for(int i=0; i<x.Size(); i++) Push(x(i)); // Make more efficient later		
	}
	
	/* Clears the Stack */
	void Clear()
	{
		Free = First;
	}
	

	Stack<T> Copy() const
	{
		LongInt n = Free - First;
		Stack<T> A(n);
		A.Name = Name;
		//BlasCopy(n,A.Begin(),1,Begin(),1); // CAN"T BELIEVE THIS WAS WRONG FOR SOOOOO LONG (MP)
		BlasCopy(n,Begin(),1,A.Begin(),1);
		A.Free += n;
		return A;
	}
	
	Stack<T> & Ref(const Stack<T> &A)
	{	
		if(this != &A)
		{
			(*RefCount)--;
			if(*RefCount < 1)
			{
				if(First != 0) delete[] First;
				if(RefCount != 0) delete RefCount;
			}
			len = A.len;
			First = A.First;
			Free = A.Free;
			Stop = A.Stop;
			Name = A.Name;
			RefCount = A.RefCount;
			(*RefCount)++;
		}
		return *this;
	}

	/* B = A is shorthand notation for B.Ref(A) */
	Stack & operator=(const Stack &A)
	{
		return Ref(A);
	}

	/* Returns the size of the Stack */
	inline LongInt Size() const
	{
		return Free - First;
	}
	
	inline LongInt Length() const
	{
		return len;
	}
	
	inline int Increment() const
	{
		return 1;
	}
	
	inline int Type() const
	{
		return MatrixType::Stack;
	}

	/* Returns true if the stack is empty */
	bool Empty() const
	{
		return (Free == First) ? true : false;
	}
	
	void IncrementRefCount()
	{
		(*RefCount)++;
	}
	
	/*
	
	Note: I changed this code recently. Hopefully,
	this doesn't cause any side-effects
	
	T* End()
	{
		return Stop;	
	}
	*/
	
	
	inline T* Begin() const
	{
		return First;
	}
	
	
	inline T* End() const
	{
		return Free;
	}
	
	inline T& operator[](LongInt k)
	{
		return First[k];
	}
	
	inline const T& operator[](LongInt k) const
	{
		return First[k];
	}
	
	inline T& operator()(LongInt k)
	{
		/* Check for index out of bounds */
		if(k < 0 || k >= (Free - First))
		{
			Print("Error in Stack()");
			Print("Index Out of Bounds");
			Print("Name",Name);
			Print("k",k);
			Print("n",Free - First);
			Print("Name",Name);
			HaltExecution();
		}
		
		return First[k];
	}
	
	inline const T& operator()(LongInt k) const
	{
		/* Check for index out of bounds */
		if(k < 0 || k >= (Free - First))
		{
			Print("Error in Stack()");
			Print("Index Out of Bounds");
			Print("Name",Name);
			Print("k",k);
			Print("n",Free - First);
			Print("Name",Name);
			HaltExecution();
		}
		
		return First[k];
	}
	
	void Trim()
	{
		try
		{
			len = Free - First;
			T *Temp = new T[Max(len,(LongInt)1)];
			BlasCopy(len,First,1,Temp,1);
			delete[] First;
			First = Temp;
			Free = First + len;
			Stop = First + len;
		}
		catch(bad_alloc ba)
		{
			Print("Error in Stack::Trim()");
			Print("Unable to allocate memory");
			Print("len",len);
			Print("Memory Needed in Megabytes",len*sizeof(T)/1e6);
			Print("Name",Name);
			HaltExecution();
		}			
	}
	
	template <class T>
	bool Contains(const T &a)
	{
		LongInt K = Free - First;
		for(LongInt k=0; k<K; k++) if(First[k] == a) return True;
		return False;	
	}
	
	template <class T>
	LongInt Find(const T &a)
	{
		LongInt K = Free - First;
		for(LongInt k=0; k<K; k++) if(First[k] == a) return k;
		return -1;	
	}
		
	/*
	template <class T>
	Stack& Copy() const
	{
		lenA = Free - First; 
		Stack A(lenA);
		BlasCopy(lenA,A.First,1,First);
		A.Free = A.First + lenA;
		A.Stop = A.First + lenA;
		return A;
	}
	*/	
};

// Overload the cout operator for Stacks
template <class T>
ostream & operator << (ostream &os, const Stack<T> &s)
{
  	LongInt n = s.Size();
   	
   	if(n > 0)
	{
		/* Print Vector as comma-delimited */
		os << "[";
		for(LongInt i=0; i<n-1; i++) os << s[i] << ",";
		os << s[n-1] << "]";
	}
	else os << "[]";
	
	return os;
}

#endif
