
#ifndef _Stat_StanDev
#define _Stat_StanDev

	
	double StanDev(const GeneralVector<double> &X);
	Vector<double> StanDev(const Matrix<double> &X);
	Vector<double> StanDev(const Matrix<double> &X, const Matrix<bool> &XMiss);
	Vector<double> StanDev(const Vector< Vector<double> > &X);
	
	double StanDevWeighted(const GeneralVector<double> &X, const GeneralVector<double> &W);


double StanDev(const GeneralVector<double> &X)
{
	return Sqrt(Var(X));
}

DataPoint<double> StanDev(const GeneralVector< DataPoint<double> > &X)
{
	return Sqrt(Var(X));
}

/* Compute the Weighted Standeard Deviation for Scalar-Valued Data */
double StanDevWeighted(const GeneralVector<double> &X, const GeneralVector<double> &W)
{
	return Sqrt(VarWeighted(X,W));
	
	/*
	int N = X.Size();
	
	/* Check for Incompadible Dimensions */
	/*if(W.Size() != N)
	{
		Print("Error in StanDevWeighted(GeneralVector<double> X,GeneralVector<double> W)");
		Print("Incompadible Dimensions");
		Print("X.Name",X.Name);
		Print("W.Name",W.Name);		
		Print("X.Size()",X.Size());
		Print("W.Size()",W.Size());
		HaltExecution();
	}
	
	if(N < 2) return Nan;
	
	/* Compute Weighted Standeard Deviation */
	/*double Mu = MeanWeighted(X,W);
	
	double Num = Zero;
	double Den = Zero;
	for(int n=0; n<N; n++)
	{
		Num += W(n) * Sqr(X(n) - Mu);
		Den += W(n);
	}
	
	return sqrt(Num) / (Den - One);
	*/
}

Vector<double> StanDev(const Matrix<double> &X)
{
	int N = X.Rows();
	int K = X.Cols();
	Vector<double> Mu = Mean(X);
	Vector<double> StanDev1(K,Zero);
	for(int n=0; n<N; n++) for(int k=0; k<K; k++) StanDev1[k] += Sqr(X(n,k) - Mu[k]) / (double)(N - One);
	for(int k=0; k<K; k++) StanDev1[k] = sqrt(StanDev1[k]);
	return StanDev1;
}

Vector<double> StanDev(const Matrix<double> &X, const Matrix<bool> &XMiss)
{
	int N = X.Rows();
	int K = X.Cols();
	Vector<double> Mu = Mean(X,XMiss);
	Vector<double> StanDev1(K,Zero);
	Vector<double> Tot(K,Zero);
	for(int n=0; n<N; n++) for(int k=0; k<K; k++) if(!XMiss(n,k))
	{
		StanDev1[k] += Sqr(X(n,k) - Mu[k]);
		Tot[k] += One;
	}
	for(int k=0; k<K; k++) StanDev1[k] = sqrt(StanDev1[k] / (Tot[k] - One));
	return StanDev1;
}

Vector< DataPoint<double> > StanDev(const Matrix< DataPoint<double> > &X)
{
	int N = X.Rows();
	int K = X.Cols();
	Vector< DataPoint<double> > Mu = Mean(X);
	Vector< DataPoint<double> > StanDev1(K);
	StanDev1.Name = "StanDev(" + X.Name + ")";
	Vector<double> Tot(K,Zero);
	for(int k=0; k<K; k++) StanDev1(k) = Zero;
	for(int n=0; n<N; n++) for(int k=0; k<K; k++) if(!X(n,k).IsMissing() && !Mu(k).IsMissing())
	{
		StanDev1(k) += Sqr(X(n,k).GetData() - Mu(k).GetData());
		Tot(k) += One;
	}
	for(int k=0; k<K; k++)
	{
		if(Tot(k) >= Two) StanDev1(k) = Sqrt(StanDev1[k] / (Tot(k) - One));
		else              StanDev1(k).SetToMissing();
	}
	return StanDev1;
}

/* Compute the sample mean for scalar-valued data */
Vector<double> StanDev(const Vector< Vector<double> > &X)
{
	int K = X.Size();
	Vector<double> Sd(K);
	Sd.Name = "StanDev(" + X.Name + ")";
	for(int k=0; k<K; k++) Sd(k) = StanDev(X(k));
	return Sd;
}

template <class T>
Vector< DataPoint<double> > StanDevBy(const GeneralVector< DataPoint<double> > &X, const GeneralVector<T> &Y)
{
	int N = X.Size();
	if(Y.Size() != N)
	{
		Print("Error in StanDevBy(X,Y)");
		Print("Incompadible Dimensions");
		Print("X.Name",X.Name);
		Print("Y.Name",Y.Name);
		Print("X.Size()",X.Size());
		Print("Y.Size()",Y.Size());
	}	
	
	Vector<T> List = ListValues(Y);
	int K = List.Size();	
	Vector< Stack<double> > x_By_y(K);
	
	for(int n=0; n<N; n++)
	{
		if(!X(n).IsMissing())
		{
			int Curr = BinarySearch(List,Y(n));
			if(Curr < 0)
			{
				Print("Error in StanDevBy(X,Y)");
				Print("Curr < 0");
				Print("Curr",Curr);
				HaltExecution();
			}
			x_By_y(Curr).Push(X(n).Data);
		}
	}
	Vector< DataPoint<double> > Ret(K);
	for(int k=0; k<K; k++)
	{
		if(x_By_y(k).Size() > 1) Ret(k) = StanDev(x_By_y(k));
		else                     Ret(k).SetToMissing();
	}
	return Ret;
}

#endif
