
#ifndef _GenMatUtil_SubtractOp
#define _GenMatUtil_SubtractOp


/* Subtract a constant from a vector */
template <class T>
inline Vector<T> operator - (const Vector<T> &b, double c)
{
	int n = b.Size();
	Vector<T> a(n);
	for(int i=0; i<n; i++) a[i] = b[i] - c;
	return a;
}

/* Subrtact two vectors */
template <class T>
inline Vector<T> operator - (const GeneralVector<T> &x, const GeneralVector<T> &y)
{
	int n = x.Size();
	if(y.Size() != n)
	{
		Print("Error in z = x - y");
		Print("Incompatible Dimensions");
		Print("x.Name",x.Name);
		Print("y.Name",y.Name);
		Print("x.Size()",x.Size());
		Print("y.Size()",y.Size());
		HaltExecution();
	}
	Vector<T> z(n);
	for(int i=0; i<n; i++) z[i] = x[i] - y[i];
	z.Name = "[" + x.Name + " - " + y.Name + "]";
	return z;
}

// Subrtact Two Vectors (easy routine)
/*
template <class T>
inline VectorIncremented<T> operator - (const VectorIncremented<T> &x, const VectorIncremented<T> &y)
{
	int n = x.Size();
	int inc = x.Increment();
	if(y.Size() != n || y.Increment() != inc)
	{
		Print("Error in z = x - y: Incompatible Dimensions");
		HaltExecution();
	}
	VectorIncremented<T> z(n,inc);
	Set(z,Zero);
	for(int i=0; i<n; i++) z(i) = x(i) - y(i);
	return z;
}
*/

// Subtract two matrices (operator routine)
template <class T>
Matrix<T> operator - (const Matrix<T> &A, const Matrix<T> &B)
{
	int i,j;
	int m = A.Rows();
	int n = A.Cols();
	
	/* Check for incompadible dimensions */
	if(B.Rows() != m || B.Cols() != n)
	{
		Print("Error in Matrix = Matrix - Matrix");
		Print("C = A - B");
		Print("Incompadible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	/* Preform calculations */
	Matrix<T> C(m,n);
	for(i=0; i<m; i++) for(j=0; j<n; j++) C(i,j) = A(i,j) - B(i,j);
	return C;
}

// Subtract two matrices (operator routine)
#ifdef _GenMat_DiagonalMatrix
	template <class T>
	DiagonalMatrix<T> operator - (const DiagonalMatrix<T> &A, const DiagonalMatrix<T> &B)
	{
		int n = A.Dim();
		
		/* Check for incompadible dimensions */
		if(B.Dim() != n)
		{
			Print("Error in DiagonalMatrix = DiagonalMatrix - DiagonalMatrix");
			Print("C = A - B");
			Print("Incompatible Dimensions");
			Print("A.Name",B.Name);
			Print("B.Name",C.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}
		
		/* Preform calculations */
		DiagonalMatrix<T> C(n);
		for(int i=0; i<n; i++) C(i) = A(i) - B(i);
		return C;
	}
#endif

// Subtract two matrices (operator routine)
template <class T>
SymmetricMatrix<T> operator - (const SymmetricMatrix<T> &A, const SymmetricMatrix<T> &B)
{
	int i,j;
	int n = A.Dim();
	
	/* Check for incompadible dimensions */
	if(B.Dim() != n)
	{
		Print("Error in SymmetricMatrix = SymmetricMatrix - SymmetricMatrix");
		Print("C = A - B");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	/* Preform calculations */
	SymmetricMatrix<T> C(n);
	for(i=0; i<n; i++) for(j=0; j<=i; j++) C(i,j) = A(i,j) - B(i,j);
	return C;
}

// Subtract two matrices (operator routine)
template <class T>
LowerMatrix<T> operator - (const LowerMatrix<T> &A, const LowerMatrix<T> &B)
{
	int i,j;
	int n = A.Dim();
	
	/* Check for incompadible dimensions */
	if(B.Dim() != n)
	{
		Print("Error in LowerMatrix = LowerMatrix - LowerMatrix");
		Print("C = A - B");
		Print("Incompatible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	/* Preform calculations */
	LowerMatrix<T> C(n);
	for(i=0; i<n; i++) for(j=0; j<=i; j++) C(i,j) = A(i,j) - B(i,j);
	return C;
}

// Subtract two matrices (operator routine)
#ifdef _GenMat_UpperMatrix
	template <class T>
	UpperMatrix<T> operator - (const UpperMatrix<T> &A, const UpperMatrix<T> &B)
	{
		int i,j;
		int n = A.Dim();
		
		/* Check for incompadible dimensions */
		if(B.Dim() != n)
		{
			Print("Error in UpperMatrix = UpperMatrix - UpperMatrix");
			Print("C = A - B");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}
	
		/* Preform calculations */
		UpperMatrix<T> C(n);
		for(i=0; i<n; i++) for(j=i; j<n; j++) C(i,j) = A(i,j) - B(i,j);
		return A;
	}
#endif

template <class T>
Matrix<T> operator - (const LowerMatrix<T> &A, const Matrix<T> &B)
{
	int i,j;
	int n = A.Dim();
	
	/* Check for incompadible dimensions */
	if(B.Rows() != n || B.Cols() != n)
	{
		Print("Error in Matrix = LowerMatrix - Matrix");
		Print("C = A - B");
		Print("Incompadible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	/* Preform calculations */
	Matrix<T> C(n,n);	
	for(i=0; i<n; i++) for(j=0; j<n; j++) C(i,j) = A(i,j) - B(i,j);
	return C;
}

#ifdef _GenMat_UpperMatrix
	template <class T>
	Matrix<T> operator - (const UpperMatrix<T> &A, const Matrix<T> &B)
	{
		int i,j;
		int m = A.Rows();
		int n = A.Cols();
		
		/* Check for incompadible dimensions */
		if(B.Rows() != m || B.Cols() != n)
		{
			Print("Error in Matrix = UpperMatrix - Matrix");
			Print("C = A - B");
			Print("Incompadible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}
	
		/* Preform calculations */
		Matrix<T> C(m,n);	
		for(i=0; i<m; i++) for(j=0; j<n; j++) C(i,j) = A(i,j) - B(i,j);
		return C;
	}
#endif

template <class T>
Matrix<T> operator - (const SymmetricMatrix<T> &A, const Matrix<T> &B)
{
	int i,j;
	int n = A.Dim();
	
	/* Check for incompadible dimensions */
	if(B.Rows() != n || B.Cols() != n)
	{
		Print("Error in Matrix = SymmetricMatrix - Matrix");
		Print("C = A - B");
		Print("Incompadible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	/* Preform calculations */
	Matrix<T> C(n,n);	
	for(i=0; i<n; i++) for(j=0; j<n; j++) C(i,j) = A(i,j) - B(i,j);
	return C;
}

template <class T>
Matrix<T> operator - (const Matrix<T> &A, const SymmetricMatrix<T> &B)
{
	int i,j;
	int n = A.Rows();
	
	/* Check for incompadible dimensions */
	if(A.Cols() != n || B.Dim() != n)
	{
		Print("Error in Matrix = Matrix - SymmetricMatrix");
		Print("C = A - B");
		Print("Incompadible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	/* Preform calculations */
	Matrix<T> C(n,n);	
	for(i=0; i<n; i++) for(j=0; j<n; j++) C(i,j) = A(i,j) - B(i,j);
	return C;
}

#ifdef _GenMat_SymmetricMatrixUP
	template <class T>
	Matrix<T> operator - (const Matrix<T> &A, const SymmetricMatrixUP<T> &B)
	{
		int i,j;
		int n = A.Rows();
		
		/* Check for incompadible dimensions */
		if(A.Cols() != n || B.Dim() != n)
		{
			Print("Error in Matrix = Matrix - SymmetricMatrixUP");
			Print("C = A - B");
			Print("Incompadible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}
		
		/* Preform calculations */
		Matrix<T> C(n,n);	
		for(i=0; i<n; i++) for(j=0; j<n; j++) C(i,j) = A(i,j) - B(i,j);
		return C;
	}

	template <class T>
	SymmetricMatrix<T> operator - (const SymmetricMatrix<T> &A, const IdentityMatrix<T> &B)
	{
		int i,j;
		int n = A.Dim();
		
		/* Check for incompadible dimensions */
		if(B.Dim() != n)
		{
			Print("Error in SymmetricMatrix = SymmetricMatrix - IdentityMatrix");
			Print("C = A - B");
			Print("Incompadible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}
		
		/* Preform calculations */
		SymmetricMatrix<double> C(n);
		for(i=0; i<n; i++) for(j=0; j<n; j++) C(i,j) = A(i,j) - B(i,j);
		return C;
	}
#endif

#ifdef _GenMat_SymmetricMatrixUP
	template <class T>
	SymmetricMatrixUP<T> operator - (const SymmetricMatrixUP<T> &A, const IdentityMatrix<T> &B)
	{
		int i,j;
		int n = A.Dim();
		
		/* Check for incompadible dimensions */
		if(B.Dim() != n)
		{
			Print("Error in SymmetricMatrixUP = SymmetricMatrixUP - IdentityMatrix");
			Print("C = A - B");
			Print("Incompadible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}
		
		/* Preform calculations */
		SymmetricMatrixUP<double> C(n);
		for(i=0; i<n; i++) for(j=0; j<n; j++) C(i,j) = A(i,j) - B(i,j);
		return C;
	}
#endif

#ifdef _GenMat_IdentityMatrix
	template <class T>
	Matrix<T> operator - (const Matrix<T> &A, const IdentityMatrix<T> &B)
	{
		/* Check for incompadible dimensions */
		int n = A.Rows();
		if(A.Cols() != n || B.Dim() != n)
		{
			Print("Error in Matrix = Matrix - IdentityMatrix");
			Print("C = A - B");
			Print("Incompadible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}
	
		/* Preform calculations */
		Matrix<double> C(n,n);
		for(int i=0; i<n; i++) for(int j=0; j<n; j++) C(i,j) = A(i,j) - B(i,j);
		return C;
	}
#endif

#ifdef _GenMat_IdentityMatrix
	template <class T>
	Matrix<T> operator - (const IdentityMatrix<T> &A, const Matrix<T> &B)
	{
		int i,j;
		int n = A.Dim();
		
		/* Check for incompadible dimensions */
		if(B.Rows() != n || B.Cols() != n)
		{
			Print("Error in Matrix = IdentityMatrix - Matrix");
			Print("C = A - B");
			Print("Incompadible Dimensions");
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}
		
		/* Preform calculations */
		Matrix<double> C(n,n);
		for(i=0; i<n; i++) for(j=0; j<n; j++) C(i,j) = A(i,j) - B(i,j);
		return C;
	}
#endif

template <class T>
Matrix<T> operator - (const GeneralMatrix<T> &A, const GeneralMatrix<T> &B)
{
	int m = A.Rows();
	int n = A.Cols();
	
	/* Check for incompadible dimensions */
	if(B.Rows() != m || B.Cols() != n)
	{
		Print("Error in Matrix = GeneralMatrix - GeneralMatrix");
		Print("C = A - B");
		Print("Incompadible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	/* Preform calculations */
	Matrix<double> C(m,n);
	for(int i=0; i<m; i++) for(int j=0; j<n; j++) C(i,j) = A(i,j) - B(i,j);
	return C;
}

template <class T>
Array3D<T> operator - (const Array3D<T> &A, const Array3D<T> &B)
{
	int d1  = A.Dim1();
	int d2  = A.Dim2();
	int d3  = A.Dim3();
	int len = A.Length();

	/* Check for incompadible dimensions */
	if(d1 != B.Dim1() || d2 != B.Dim2() || d3 != B.Dim3())
	{
		Print("Error in Array3D = Array3D - Array3D");
		Print("C = A - B");
		Print("Incompadible Dimensions");
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	/* Preform calculations */
	Array3D<double> C(d1,d2,d3);
	BlasCopy(len,A.Begin(),1,C.Begin(),1);		// C = A
	BlasAxpy(len,-One,B.Begin(),1,C.Begin(),1);	// C = C - 1.0 * B
	return C;
}

#endif
