
#ifndef _MatrixSubtractProdAtX
#define _MatrixSubtractProdAtX

/*

Pre-multiply a Vector by a Matrix

	y -= A^T * x

*/

template <class T>
void SubtractProdAtX(Vector<T> &y, const Matrix<T> &A, const Vector<T> &x)
{
	int i,j;
	int m = A.Rows();
	int n = A.Cols();
	
	if(x.Size() != m || y.Size() != n)
	{
		Print("Error in SubtractProdAtX(Vector,Matrix,Vector)");
		Print("y -= A^T * x");
		Print("Incompatible Dimensions");
		Print("y.Name",y.Name);
		Print("A.Name",A.Name);
		Print("x.Name",x.Name);
		Print("y.Size()",y.Size());
		Print("A.Size()",A.Size());
		Print("x.Size()",x.Size());
		Print("Halting Execution");
		exit(0);
	}
	
	for(i=0; i<n; i++) for(j=0; j<m; j++) y[i] -= A(j,i) * x[j];
}

/*

Pre-multiply a Vector by a Matrix

	y -= A^T * x

*/

template <class T>
void SubtractProdAtX(Vector<T> &y, const SymmetricMatrix<T> &A, const Vector<T> &x)
{
	int i,j;
	int n = A.Dim();
	
	if(x.Size() != n || y.Size() != n)
	{
		Print("Error in SubtractProdAtX(Vector,SymmetricMatrix,Vector)");
		Print("y -= A^T * x");
		Print("Incompatible Dimensions");
		Print("y.Name",y.Name);
		Print("A.Name",A.Name);
		Print("x.Name",x.Name);
		Print("y.Size()",y.Size());
		Print("A.Size()",A.Size());
		Print("x.Size()",x.Size());
		Print("Halting Execution");
		exit(0);
	}
	
	for(i=0; i<n; i++) for(j=0; j<m; j++) y[i] -= A(j,i) * x[j];
}

#endif
