
#ifndef _GenMatUtil_Sum
#define _GenMatUtil_Sum

	/* Compute the sum of the elements in a vector */
	template <class T> T Sum(const GeneralVector<T> &v);

	/* Sum the elements of a matrix */
	template <class T> T Sum(const Matrix<T>          &A);
	template <class T> T Sum(const SymmetricMatrix<T> &A);
	template <class T> T Sum(const LowerMatrix<T>     &A);

	template <class T> Vector<T> SumByRows(const Matrix<T> &A);
	template <class T> Vector<T> SumByCols(const Matrix<T> &A);


/* Compute the sum of the elements in a vector */
template <class T> T Sum(const GeneralVector<T> &v)
{
	T t = 0;
	for(int i=0; i<v.Size(); i++) t += v[i];
	return t;
}


/* Sum the elements of a Matrix */
template <class T>
T Sum(const Matrix<T> &A)
{
	int m = A.Rows();
	int n = A.Cols();
	T t = 0;
	for(int i=0; i<m; i++) for(int j=0; j<n; j++) t += A(i,j);
	return t;
}

/* Sum the elements of a DiagonalMatrix */
#ifdef _GenMat_DiagonalMatrix
	template <class T>
	T Sum(const DiagonalMatrix<T> &A)
	{
		int n = A.Dim();
		T t = 0;
		for(int i=0; i<n i++) t += A(i);
		return t;
	}
#endif

/* Sum the elements of a SymmetricMatrix */
template <class T>
T Sum(const SymmetricMatrix<T> &A)
{
	int n = A.Dim();
	T t = 0;
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++) t += A(i,j);
	return t;
}

/* Sum the elements of a LowerMatrix */
template <class T>
T Sum(const LowerMatrix<T> &A)
{
	int n = A.Dim();
	T t = 0;
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++) t += A(i,j);
	return t;
}

/* Sum the elements of a UpperMatrix */
#ifdef _GenMat_UpperMatrix
	template <class T>
	T Sum(const UpperMatrix<T> &A)
	{
		int n = A.Dim();
		T t = 0;
		for(int i=0; i<n; i++) for(int j=i; j<n; j++) t += A(i,j);
		return t;
	}
#endif

/* Sum the elements of a BlockMatrix */
#ifdef _GenMat_BlockMatrix
	template <class T>
	T Sum(const BlockMatrix<T> &A)
	{
		int m = A.Blocks();
		T t = 0;
		for(int i=0; i<m; i++)
		{
			int n = A.Block(i);
			for(int j=0; j<n; j++) for(int k=0; k<n; k++) t += A(i,j,k);
		}
		return t;
	}
#endif

template <class T>
Vector<T> SumByRows(const Matrix<T> &A)
{
	int m = A.Rows();
	int n = A.Cols();
	Vector<T> v(n);
	for(int j=0; j<n; j++)
	{
		v[j] = 0;
		for(int i=0; i<m; i++) v[j] += A(i,j);
	}	
	return v;
}

template <class T>
Vector<T> SumByCols(const Matrix<T> &A)
{
	int m = A.Rows();
	int n = A.Cols();
	Vector<T> v(m);
	for(int i=0; i<m; i++)
	{
		v[i] = 0;
		for(int j=0; j<n; j++) v[i] += A(i,j);
	}	
	return v;
}

/* Compute the sum of the elements in a vector */
template <class T> T SumSub(const GeneralVector<T> &v, int i0, int i1)
{
	T t = 0;
	for(int i=i0; i<=i1; i++) t += v[i];
	return t;
}


#endif
