
#ifndef _GenMatUtil_SumProdAtX
#define _GenMatUtil_SumProdAtX



/* Compute the product, y += A^T * x */
template <class T>
void SumProdAtX(Vector<T> &y, const Matrix<T> &A, const Vector<T> &x)
{
	/* Check for Incompadible Dimensions */
	int m = A.Rows();
	int n = A.Cols();
	if(x.Size() != m || y.Size() != n) 
	{
		Print("Error in Vector = SumProdAtX(Matrix,Vector)");
		Print("y += A^T * x");
		Print("Incompatible Dimensions");
		Print("y.Name",y.Name);
		Print("A.Name",A.Name);
		Print("x.Name",x.Name);
		Print("y.Size()",y.Size());
		Print("A.Size()",A.Size());
		Print("x.Size()",x.Size());
		HaltExecution();
	}
	
	/* Preform Calculations */
	for(int i=0; i<n; i++) for(int j=0; j<m; j++) y[i] += A(j,i) * x[j];
}

/* Compute the product, y += A^T * x, where A is a SparseMatrix */
#ifdef _GenMat_SparseMatrix
	template <class T>
	void SumProdAtX(Vector<T> &y, const SparseMatrix<T> &A, const Vector<T> &x)
	{
		/* Check for Incompadible Dimensions */	
		int m = A.Rows();
		int n = A.Cols();
		if(x.Size() != m || y.Size() != n) 
		{
			Print("Error in SumProdAtX(Vector,SparseMatrix,Vector)");
			Print("y += A^T * x");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}
		
		/* Preform Calculations */
		for(int i=0; i<m; i++) for(int j=A.RowLoc(i); j<A.RowLoc(i+1); j++) y[A.Col(j)] += A.Val(j) * x[i];
	}
#endif

/* Compute the product, y += A^T * x, where A is a SparseMatrixRcv */ 
#ifdef _SparseMatrixRcv
	template <class T>
	void SumProdAtX(Vector<T> &y, const SparseMatrixRcv<T> &A, const Vector<T> &x)
	{
		/* Check for Incompadible Dimensions */
		int m   = A.Rows();
		int n   = A.Cols();
		int len = A.Length();
		if(y.Size() != m || x.Size() != n)
		{
			Print("Error in SumProdAtX(Vector,SparseMatrixRcv,Vector)");
			Print("y += A^T * x");
			Print("Incompatible Dimensions");
			Print("y.Name",y.Name);
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("y.Size()",y.Size());
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}	
		
		/* Preform Calculations */
		for(int i=0; i<len; i++) y[A.Col(i)] += A.Val(i) * x[A.Row(i)];
	}
#endif

/* Compute the product, y += A^T * x */
template <class T>
void SumProdAtX(Vector<T> &y, const GeneralMatrix<T> &A, const Vector<T> &x)
{
	/* Check for Incompadible Dimensions */
	int m = A.Rows();
	int n = A.Cols();
	if(x.Size() != m || y.Size() != n) 
	{
		Print("Error in Vector = SumProdAtX(GeneralMatrix,Vector)");
		Print("y += A^T * x");
		Print("Incompatible Dimensions");
		Print("y.Name",y.Name);
		Print("A.Name",A.Name);
		Print("x.Name",x.Name);
		Print("y.Size()",y.Size());
		Print("A.Size()",A.Size());
		Print("x.Size()",x.Size());
		HaltExecution();
	}
	
	/* Preform Calculations */
	for(int i=0; i<n; i++) for(int j=0; j<m; j++) y[i] += A(j,i) * x[j];
}

#endif
