
#ifndef _Stat_Survey
#define _Stat_Survey


/* Function Prototypes */

	Vector<double> CreateWeights(const Vector<int> &Idx, const Vector<double> &PopCell);
	
	Vector<int> Tensorize(const Vector<int> &x1, const Vector<int> &x2, const Vector<int> &x3);
	Vector<int> Tensorize(const Vector<int> &x1, const Vector<int> &x2, const Vector<int> &x3, const Vector<int> &x4);
	
	/* Raking Weights */
	Vector<double> Rake(const Matrix<int> &X, const Vector< Vector<double> > &Pop);
	
	
/*

Create Weights from Population Cells

Idx			-	The cell that observation n belongs to.
PopCell		-	The proportion of the population that belongs to cell k.

*/

Vector<double> CreateWeights(const Vector<int> &Idx, const Vector<double> &PopCell)
{

	int N = Idx.Size();
	int K = PopCell.Size();
	
	if(Min(Idx) < 0 || Max(Idx) > K - 1)
	{
		Print("Error in CreateWeights");
		Print("Min(Idx)",Min(Idx));
		Print("Max(Idx)",Max(Idx));
		HaltExecution();
	}
	
	if(abs(Sum(PopCell) - One) > 0.03)
	{
		Print("Error in CreateWeights");
		Print("Population Cells do not sum to one");
		Print("PopCell",PopCell);
		Print("Sum(PopCell)",Sum(PopCell));
		HaltExecution();
	}
	
	Vector<double> SampCell(K,Zero);
	for(int n=0; n<N; n++) SampCell(Idx(n)) += One / N;
	Print("SampCell",SampCell);
	
	
	double Sum1 = Sum(PopCell);
	
	Vector<double> Ret(N);
	for(int n=0; n<N; n++) Ret(n) = (PopCell(Idx(n)) / Sum1) / SampCell(Idx(n));
	
	return Ret;
}

Vector<double> CreateWeightsClasses(const Vector<int> &Idx, const Vector<int> &Resp, const Vector<double> &PopCell, int NMiss)
{

	int N = Idx.Size();
	int K = PopCell.Size();
	
	if(Min(Idx) < 0 || Max(Idx) > K - 1)
	{
		Print("Error in CreateWeightsClasses");
		Print("Min(Idx)",Min(Idx));
		Print("Max(Idx)",Max(Idx));
		HaltExecution();
	}
	
	if(abs(Sum(PopCell) - One) > 0.03)
	{
		Print("Error in CreateWeightsClasses");
		Print("Population Cells do not sum to one");
		Print("PopCell",PopCell);
		Print("Sum(PopCell)",Sum(PopCell));
		HaltExecution();
	}
	
	int R = Max(Resp);
	Matrix<double> Count1(K,2,Zero);
	Vector<double> Count2(2,Zero);
	for(int n=0; n<N; n++)
	{
		if(Resp(n) == R)
		{
			Count1(Idx(n),1) += One;
			Count2(1) += One;
		}
		else
		{
			Count1(Idx(n),0) += One;
			Count2(0) += One;
		}
	}
	Vector<double> Ret(N);
	for(int n=0; n<N; n++)
	{
		if(Resp(n) == R)
		{
			if(Count1(Idx(n),0) > Zero) Ret(n) = PopCell(Idx(n)) / (Count1(Idx(n),1) / (Count2(1) + NMiss));
			else                        Ret(n) = ((N + NMiss) / (Count2(1) + NMiss)) * PopCell(Idx(n)) / (Count1(Idx(n),1) / (Count2(1) + NMiss));          
		}
		else
		{
			if(Count1(Idx(n),1) > Zero) Ret(n) = PopCell(Idx(n)) / (Count1(Idx(n),0) /  Count2(0)         );
			else                        Ret(n) = ((N + NMiss) / Count2(0)) * PopCell(Idx(n)) / (Count1(Idx(n),0) /  Count2(0)         );
		}
	}
	
	return Ret;
}

/*

Create Population Weights from Cells

*/
/*
Vector<double> ComputeCells(const Vector<int> &Idx, const Vector<double> &W)
{

	int N = Idx.Size();
	
	if(W.Size() != N)
	{
		Print("Error in ComputeCells");
		Print("Idx.Size()",Idx.Size());
		Print("W.Size()",W.Size());
		HaltExecution();
	}
	
	if(Min(Idx) < 0)
	{
		Print("Error in ComputeCells");
		Print("Min(Idx)",Min(Idx));
		HaltExecution();
	}
	
	int K = Max(Idx) + 1;
	
	Vector<int> Count(K,0);
	for(int n=0; n<N; n++) Count(Idx(n))++;
	Print("Count",Count);
	
	Vector<double> Ret(K,Zero);
	//Vector<bool>   Done(K,False);
	
	for(int n=0; n<N; n++)
	{
		//int k = Idx(n);
		//if(!Done(k))
		//{
		//	Ret(k) = W(n);
		//	Done(k) = True;
		//}
		Ret(Idx(n)) += W(n) / N;
	}
	
	return Ret;
}
*/

/*

Create Weights Based on Marginals

*/

/*
Vector<double> CreateWeights(const Vector<int> &Idx, const Vector<double> &PopCell, const Vector<double> &SampCell)
{

	int N = Idx.Size();
	int K = PopCell.Size();
	
	if(SampCell.Size() != K)
	{
		Print("Error in CreateWeights");
		Print("Incompadible Dimensions");
		Print("PopCell.Size()",PopCell.Size());
		Print("SampCell.Size()",SampCell.Size());
		HaltExecution();
	}
	
	Vector<double> Ret(N);
	
	for(int n=0; n<N; n++) Ret(n) = (double)N * PopCell(Idx(n)) / SampCell(Idx(n));
	
	return Ret;
}
*/

/*
Vector<double> CreateWeights(const Vector<int> &Idx, const Vector<double> &PopCell)
{
	int N = Idx.Size();
	int K = PopCell.Size();
	Vector<double> SampCell(48,Zero);
	for(int n=0; n<N; n++) SampCell(Idx(n)) += One;
	Vector<double> W(N);
	for(int n=0; n<N; n++) W(n) = (double)N * PopCell(Idx(n)) / SampCell(Idx(n));
	return W;
}		
*/

Vector<int> Tensorize(const Vector<int> &x1)
{

	int n = x1.Size();
	Vector<int> List1 = ListValues(x1);		int n1 = List1.Size();
	Vector<int> Ret(n);
	for(int j=0; j<n; j++)
	{
		int Curr = 0;
		for(int i1=0; i1<n1; i1++)
		{
			if(x1(j) == List1(i1))
			{
				Ret(j) = Curr;
			}
			Curr++;
		}
	}
	return Ret;
}

Vector<int> Tensorize(const Vector<int> &x1, const Vector<int> &x2, const Vector<int> &x3)
{

	int n = x1.Size();
	Vector<int> List1 = ListValues(x1);		int n1 = List1.Size();
	Vector<int> List2 = ListValues(x2);		int n2 = List2.Size();
	Vector<int> List3 = ListValues(x3);		int n3 = List3.Size();
		
	Vector<int> Ret(n);
	
	for(int j=0; j<n; j++)
	{
		int Curr = 0;
		for(int i1=0; i1<n1; i1++)
		{
			for(int i2=0; i2<n2; i2++)
			{
				for(int i3=0; i3<n3; i3++)
				{
					if(x1(j) == List1(i1) && x2(j) == List2(i2) && x3(j) == List3(i3))
					{
						Ret(j) = Curr;
					}
					Curr++;
				}
			}
		}
	}
	
	return Ret;
}

Vector<int> Tensorize(const Vector<int> &x1, const Vector<int> &x2, const Vector<int> &x3, const Vector<int> &x4)
{

	int n = x1.Size();
	Vector<int> List1 = ListValues(x1);		int n1 = List1.Size();
	Vector<int> List2 = ListValues(x2);		int n2 = List2.Size();
	Vector<int> List3 = ListValues(x3);		int n3 = List3.Size();
	Vector<int> List4 = ListValues(x4);		int n4 = List4.Size();
	
	Vector<int> Ret(n);
	
	for(int j=0; j<n; j++)
	{
		int Curr = 0;
		for(int i1=0; i1<n1; i1++)
		{
			for(int i2=0; i2<n2; i2++)
			{
				for(int i3=0; i3<n3; i3++)
				{
					for(int i4=0; i4<n4; i4++)
					{
						if(x1(j) == List1(i1) && x2(j) == List2(i2) && x3(j) == List3(i3) && x4(j) == List4(i4))
						{
							Ret(j) = Curr;
						}
						Curr++;
					}
				}
			}
		}
	}
	
	return Ret;
}

/*
Vector<int> Tensorize(const Matrix<int> &X)
{
	int N = X.Rows();
	int K = X.Cols();
	Vector< Vector<int> > List(K);
	for(int k=0; k<K; k++) List(k) = ListValues(BlasVector<int>(N,K,&X(0,k)));
	Vector<int> Ret(n);
	for(int j=0; j<n; j++)
	{
		int Curr = 0;
		for(int i1=0; i1<n1; i1++)
		{
			for(int i2=0; i2<n2; i2++)
			{
				for(int i3=0; i3<n3; i3++)
				{
					if(x1(j) == List1(i1) && x2(j) == List2(i2) && x3(j) == List3(i3))
					{
						Ret(j) = Curr;
					}
					Curr++;
				}
			}
		}
	}
	
	return Ret;
}
*/

Vector<double> Rake(const Matrix<int> &X, const Vector< Vector<double> > &Pop)
{
	/* Parameters */
	const double MaxTol = 0.000001;
	const int MaxIter = 100;
	
	/* Check for Incompadible Dimensions */
	int N = X.Rows();
	int K = X.Cols();
	if(Pop.Size() != K)
	{
		Print("Error in Rake");
		Print("Incompadible Dimensions");
		Print("X.Size()",X.Size());
		Print("Pop.Size()",Pop.Size());
		HaltExecution();
	}
	
	/* Form List and Check Dimensions */
	Vector< Vector<int> > ListVals(K);
	for(int k=0; k<K; k++)
	{
		Vector<int> Temp(N);
		for(int n=0; n<N; n++) Temp(n) = X(n,k);
		ListVals(k) = ListValues(Temp);
		if(ListVals(k).Size() != Pop(k).Size())
		{
			Print("Error in Rake");
			Print("Incompadible Dimensions");
			Print("k",k);
			Print("ListVals(k)",ListVals(k));
			Print("Pop(k)",Pop(k));
			HaltExecution();
		}
	}
	
	int Iter = 1;
	double Tol = One;
	Vector<double> W(N,One); // Initial Weights
	Vector<double> WNew(N);
	while(Iter <= MaxIter && Tol >= MaxTol)
	{
		Copy(WNew,W);
		for(int k=0; k<K; k++)
		{
			Vector<double> Temp(Pop(k).Size(),Zero);
			for(int n=0; n<N; n++)
			{
				int Info = BinarySearch(ListVals(k),X(n,k));
				if(Info < 0)
				{
					Print("Error in Rake");
					Print("Info",Info);
					HaltExecution();
				}
				Temp(Info) += WNew(n) / (double)N;
			}
			
			for(int n=0; n<N; n++)
			{
				int Info = BinarySearch(ListVals(k),X(n,k));
				if(Info < 0)
				{
					Print("Error in Rake");
					Print("Info",Info);
					HaltExecution();
				}
				WNew(n) *= (Pop(k)(Info) / Temp(Info));
			}
		}
		
		Tol = MaxAbs(W - WNew);
		Copy(W,WNew);
		Print("Iter",Iter,"Tol",Tol);
		Iter++;
	}
	
	/* Check for Convergence Failure */
	if(Iter > MaxIter)
	{
		Print("Error in Rake");
		Print("Failed To Converge");
		HaltExecution();
	}
	
	return W;
}

#endif
