
#ifndef _GenMatUtil_ToSymmetricMatrix
#define _GenMatUtil_ToSymmetricMatrix


	template <class T> SymmetricMatrix<T> ToSymmetricMatrix(const GeneralMatrix<T> &A);
	template <class T> SymmetricMatrix<T> ToSymmetricMatrix(const Matrix<T> &A);
	template <class T> SymmetricMatrix<T> ToSymmetricMatrix(const Matrix<T> &A, const T &Tol);


template <class T1, class T2>
SymmetricMatrix<T2> To(const SymmetricMatrix<T1> &A)
{
	int n = A.Dim();
	SymmetricMatrix<T2> B(n);
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++)
	{
		bool Info = Convert(B(i,j),A(i,j));
		if(Info)
		{
			Print("Error in To");
			Print("Convert Failed");
			Print("A.Name",A.Name);
			Print("i",i);
			Print("j",j);
			Print("A(i,j)",A(i,j));
			Print("B(i,j)",B(i,j));
			Print("Info",Info);
			HaltExecution();
		}
	}
	return B;
}

template <class T>
SymmetricMatrix<T> ToSymmetricMatrix(const GeneralMatrix<T> &A)
{
	if(!IsSymmetric(A))
	{
		Print("Error in ToSymmetricMatrix(GeneralMatrix)");
		Print("B = ToSymmetricMatrix(A)");
		Print("A is Not Symmetric");
		Print("A.Name",A.Name);
		Print("A.Size()",A.Size());
		if(Max(A.Rows(),A.Cols()) <= 12) Print("A",A);
		HaltExecution();
	}

	int n = A.Rows();
	SymmetricMatrix<T> B(n);
	Copy(B,A);
	return B;
}

template <class T>
SymmetricMatrixLU<T> ToSymmetricMatrixLU(const GeneralMatrix<T> &A)
{
	if(!IsSymmetric(A))
	{
		Print("Error in SymmetricMatrixLU(GeneralMatrix)");
		Print("B = ToSymmetricMatrixLU(A)");
		Print("A is Not Symmetric");
		Print("A.Name",A.Name);
		Print("A.Size()",A.Size());
		if(Max(A.Rows(),A.Cols()) <= 12) Print("A",A);
		HaltExecution();
	}

	int n = A.Rows();
	SymmetricMatrixLU<T> B(n);
	Copy(B,A);
	return B;
}

#ifdef _GenMat_SymmetricMatrixUP
	template <class T>
	SymmetricMatrixUP<T> ToSymmetricMatrixUP(const GeneralMatrix<T> &A)
	{
		if(!IsSymmetric(A))
		{
			Print("Error in SymmetricMatrixUP(GeneralMatrix)");
			Print("B = ToSymmetricMatrixUP(A)");
			Print("A is Not Symmetric");
			Print("A.Name",A.Name);
			Print("A.Size()",A.Size());
			if(Max(A.Rows(),A.Cols()) <= 12) Print("A",A);
			HaltExecution();
		}

		int n = A.Rows();
		SymmetricMatrixUP<T> B(n);
		Copy(B,A);
		return B;
	}
#endif

template <class T>
SymmetricMatrix<T> ToSymmetricMatrix(const Matrix<T> &A)
{

	if(!IsSymmetric(A))
	{
		Print("Error in ToSymmetricMatrix(Matrix)");
		Print("B = ToSymmetricMatrix(A)");
		Print("A is Not Symmetric");
		Print("A.Name",A.Name);
		Print("A.Size()",A.Size());
		Print("SymmetricTol",SymmetricTol);
		if(A.Rows() == A.Cols())
		{
			T MaxAbsDiff = Zero;
			int n = A.Rows();
			for(int i=0; i<n; i++) for(int j=0; j<i; j++)
				if(Abs(A(i,j) - A(j,i)) >= MaxAbsDiff) MaxAbsDiff = Abs(A(i,j) - A(j,i));
			Print("MaxAbsDiff",MaxAbsDiff);		
		}
		if(Max(A.Rows(),A.Cols()) <= 12) Print("A",A);
		HaltExecution();
	}

	int n = A.Rows();
	SymmetricMatrix<T> B(n);
	Copy(B,A);
	return B;
}

template <class T>
SymmetricMatrix<T> ToSymmetricMatrix(const Matrix<T> &A, const T &Tol)
{

	if(!IsSymmetric(A,Tol))
	{
		Print("Error in ToSymmetricMatrix(Matrix)");
		Print("B = ToSymmetricMatrix(A)");
		Print("A is Not Symmetric");
		Print("A.Name",A.Name);
		Print("A.Size()",A.Size());
		if(A.Rows() == A.Cols())
		{
			Print("Tol",Tol);
			int n = A.Rows();
			double MaxAbsDiff = Zero;
			for(int i=0; i<n; i++) for(int j=0; j<i; j++)
				if(abs(A(i,j) - A(j,i)) > MaxAbsDiff) MaxAbsDiff = abs(A(i,j) - A(j,i));
			Print("MaxAbsDiff",MaxAbsDiff);
		}
		if(Max(A.Rows(),A.Cols()) <= 12) Print("A",A);
		HaltExecution();
	}

	int n = A.Rows();
	SymmetricMatrix<T> B(n);
	Copy(B,A,Tol);
	return B;
}

/* vech^-1 for Symmetric Matrices */
template <class T>
SymmetricMatrix<T> ToSymmetricMatrix(const Vector<T> &x)
{
	int len = x.Length();
	int n = std::round(Half * (-One + Sqrt(One + Eight * len)));
	if(n * (n + 1) / 2 != len)
	{
		Print("Error in ToSymmetricMatrix(Vector)");
		Print("Incorrect Dimensions");
		Print("len",len);
		Print("n",n);
		Print("n * (n + 1) / 2",n * (n + 1) / 2);
		HaltExecution();
	}
	SymmetricMatrix<T> A(n);
	int Curr = 0;
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++) A(i,j) = x(Curr++);
	return A;
}

#endif