
#ifndef _GenMatUtil_ToVector
#define _GenMatUtil_ToVector



template <class T1, class T2>
Vector<T2> To(const Vector<T1> &x)
{
	int n = x.Size();
	Vector<T2> y(n);
	for(int i=0; i<n; i++)
	{
		bool Info = Convert(y(i),x(i));
		if(Info)
		{
			Print("Error in To");
			Print("Convert Failed");
			Print("x.Name",x.Name);
			Print("i",i);
			Print("x(i)",x(i));
			Print("y(i)",y(i));
			Print("Info",Info);
			HaltExecution();
		}
	}
	return y;
}

template <class T>
Vector<T> ToVector(const GeneralVector<T> &x)
{
	int n = x.Size();
	Vector<T> y(n);
	for(int i=0; i<n; i++) y(i) = x(i);
	return y;
}

/*
template <class T1, class T2>
Vector<T2> ToVector(const Stack<T1> &x)
{
	int n = x.Size();
	Vector<T2> y(n);
	for(int i=0; i<n; i++) y(i) = (T2)x(i);
	return y;
}

template <class T>
Vector<T> ToVector(const Stack<T> &x)
{
	int n = x.Size();
	Vector<T> y(n);
	for(int i=0; i<n; i++) y(i) = x(i);
	return y;
}
*/

/*
Vector<double> ToVector(const VectorIncremented<double> &x)
{
	int n = x.Size();
	Vector<double> y(n);
	for(int i=0; i<n; i++) y(i) = x(i);
	return y;
}
*/

template <class T>
Vector<T> ToVector(const Matrix<T> &A)
{
	int n = A.Length();
	Vector<T> x(n);
	x.Name = "ToVector(" + A.Name + ")";
	for(int i=0; i<n; i++) x(i) = A[i];
	return x;
}

#ifdef _GenMat_SparseMatrix
	template <class T>
	Vector<T> ToVector(const SparseMatrix<T> &A)
	{
		int n = A.Length();
		Vector<T> x = (A.v).Copy();
		x.Name = "ToVector(" + A.Name + ")";
		return x;
	}
#endif

template <class T>
Vector<T> ToVector(const SymmetricMatrix<T> &A)
{
	int len = A.Length();
	Vector<T> x(len,(A.Copy()).Begin());
	return x;
}

template <class T>
Vector<T> ToVector(const LowerMatrix<T> &A)
{
	int n = A.Dim();
	int len = A.Length();
	Vector<T> x(len);
	int Curr = 0;
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++) x(Curr++) = A(i,j);
	return x;
}

template <class T>
Vector<T> ToVector(const DiagonalMatrix<T> &A)
{
	int n = A.Dim();
	Vector<T> x(n);
	for(int i=0; i<n; i++) x(i) = A(i);
	return x;
}

template <class T>
Vector<T> ToVector(const set<T> &s)
{
	Vector<T> Ret(s.size());
	std::copy(s.begin(),s.end(),Ret.Begin());
	return Ret;
}

/* Convert Vector of Vectors to Stack (SHOULD MERGE TO GLOBAL CODE) */
template <class T> Stack<T> ToStack(const Vector< Vector<T> > &x)
{
	Stack<T> y;
	y.Name = "ToStack(" + x.Name + ")";
	int m = x.Size();
	int len = 0;
	for(int i=0; i<m; i++)
	{
		int n = x(i).Size();
		for(int j=0; j<n; j++) y.Push(x(i)(j));
		len += n;
	}
	return y;
}

#endif
