
#ifndef _MatrixTranspose
#define _MatrixTranspose

	// Return the transpose of a Matrix
		
/* Return the transpose of a (column) Vector */
template <class T>
RowVector<T> Transpose(const Vector<T> &x)
{
	int n = x.Dim();
	RowVector<T> y(n);
	y.Name = "Transpose(" + x.Name + ")";
	for(int i=0; i<n; i++) y[i] = x[i];
	return y;
}

/* Return the transpose of a RowVector */
template <class T>
Vector<T> Transpose(const RowVector<T> &x)
{
	int n = x.Dim();
	Vector<T> y(n);
	y.Name = "Transpose(" + x.Name + ")";
	for(int i=0; i<n; i++) y[i] = x[i];
	return y;
}

/* Return the transpose of a Matrix */
template <class T>
Matrix<T> Transpose(const Matrix<T> &A)
{
	int n = A.Rows();
	int m = A.Cols();
	Matrix<T> B(m,n);
	B.Name = "Transpose(" + A.Name + ")";
	for(int i=0; i<n; i++) for(int j=0; j<m; j++) B(j,i) = A(i,j);
	return B;
}

/* Return the transpose of a BlasMatrix */
#ifdef _GenMat_BlasMatrix
	template <class T>
	Matrix<T> Transpose(const BlasMatrix<T> &A)
	{
		int n = A.Rows();
		int m = A.Cols();
		Matrix<T> B(m,n);
		B.Name = "Transpose(" + A.Name + ")";
		for(int i=0; i<n; i++) for(int j=0; j<m; j++) B(j,i) = A(i,j);
		return B;
	}
#endif

/* Return the transpose of a DiagonalMatrix */
#ifdef _GenMat_DiagonalMatrix
	template <class T>
	DiagonalMatrix<T> Transpose(const DiagonalMatrix<T> &A)
	{
		return A;
	}
#endif

/* Return the transpose of a SymmetricMatrix */
template <class T>
SymmetricMatrix<T> Transpose(const SymmetricMatrix<T> &A)
{
	return A;
}

/* Return the transpose of a UpperMatrix */
template <class T>
UpperMatrix<T> Transpose(const LowerMatrix<T> &L)
{
	int n = L.Dim();
	UpperMatrix<double> U(n);
	U.Name = "Transpose(" + L.Name + ")";
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++) U(j,i) = L(i,j);	
	return U;
}
	
/* Return the transpose of a LowerMatrix */
template <class T>
LowerMatrix<T> Transpose(const UpperMatrix<T> &U)
{
	int n = U.Dim();
	LowerMatrix<double> L(n);
	L.Name = "Transpose(" + U.Name + ")";
	for(int i=0; i<n; i++) for(int j=i; j<n; j++) L(j,i) = U(i,j);	
	return L;
}

/* Return the transpose of a UpperMatrix */
#if defined(_GenMat_LowerMatrixUN) && defined(_GenMat_UpperMatrixUN)
	template <class T>
	UpperMatrixUN<T> Transpose(const LowerMatrixUN<T> &L)
	{
		int n = L.Dim();
		UpperMatrixUN<double> U(n);
		U.Name = "Transpose(" + L.Name + ")";
		for(int i=0; i<n; i++) for(int j=0; j<=i; j++) U(j,i) = L(i,j);	
		return U;
	}
#endif

/* Return the transpose of a UpperMatrix */
#if defined(_GenMat_LowerMatrixUU) && defined(_GenMat_UpperMatrixUN)
	template <class T>
	UpperMatrixUN<T> Transpose(const LowerMatrixUU<T> &L)
	{
		int n = L.Dim();
		UpperMatrixUN<double> U(n);
		U.Name = "Transpose(" + L.Name + ")";
		for(int i=0; i<n; i++) for(int j=0; j<=i; j++) U(j,i) = L(i,j);	
		return U;
	}
#endif
	
/* Return the transpose of a LowerMatrix */
#if defined(_GenMat_LowerMatrixUN) && defined(_GenMat_UpperMatrixUN)
	template <class T>
	LowerMatrixUN<T> Transpose(const UpperMatrixUN<T> &U)
	{
		int n = U.Dim();
		LowerMatrixUN<double> L(n);
		L.Name = "Transpose(" + U.Name + ")";
		for(int i=0; i<n; i++) for(int j=i; j<n; j++) L(j,i) = U(i,j);	
		return L;
	}
#endif

/* Return the transpose of a Sparse Matrix */
#if defined(_GenMat_SparseMatrix) && defined(_GenMat_SparseMatrixCC)
	template <class T>
	SparseMatrixCC<T> Transpose(const SparseMatrix<T> &A)
	{
		int m = A.Rows();
		int n = A.Cols();
		int len = A.Length();
		SparseMatrixCC<double> ATrans(n,m,len);
		ATrans.Name = "Transpose(" + A.Name + ")";
		
		for(int i=0; i<len; i++)
		{
			ATrans.v[i] = A.v[i];
			ATrans.r[i] = A.c[i];
		}
		for(int i=0; i<m+1; i++) ATrans.c[i] = A.r[i];
		return ATrans;
	}
#endif

/* Return the transpose of a Matrix */
template <class T>
Matrix<T> Transpose(const GeneralMatrix<T> &A)
{
	int n = A.Rows();
	int m = A.Cols();
	Matrix<T> B(m,n);
	B.Name = "Transpose(" + A.Name + ")";
	for(int i=0; i<n; i++) for(int j=0; j<m; j++) B(j,i) = A(i,j);
	return B;
}

#endif
